# CGL.pm - The Object Class that provides a CGL Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: CGL.pm,v 1.16 2006/03/28 17:36:12 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::CGL;
use strict;
use Portal::Accounting::Investments::Main;
use Portal::Accounting::Investments::Objects::CGL;
use Portal::Accounting::Investments::Objects::Stock;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.16 $';

# global variables.
# modify to represent the commands this state provides.
my %commands = (
  display => "Capital Gains/Losses",
  search => "Capital Gains/Losses",
  searchForm => "Capital Gains/Losses",
);

=head1 NAME

CGL - Object used to build a CGL Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::CGL;
  my $obj = Portal::Accounting::CGL->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

CGL is a CGL class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::CGL module.
 See Portal::AppState(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_display
{
  my $self = shift;
  return $self->c_searchForm(@_);
}

sub c_searchForm
{
  my $self = shift;
  my $doc;
  my %args = ( command => $self->{input}->{command}, doc => undef, @_ );
  if (!defined $args{doc}) { $doc = $self->prepDoc; }
  my $command = $args{command};
  my $l = $self->{langObj};
  $self->{OBJECT} = Portal::Accounting::Investments::Objects::CGL->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode}, imageDir => $self->{imageDir});
  my $foo = $self->{OBJECT};
  if ($foo->error) { $doc->print($foo->errorMessage); return $doc;  }
  if ($command eq "search")
  { #validate user input then react
    if ($foo->populate(%{$self->{input}}) && $foo->validateForm(input => $self->{input}, formName => "main"))
    {
      if ($foo->get('format') ne 'html')
      {
        $doc = HTMLObject::Normal->new;
        $doc->setContentType('text/plain');
      }
      $doc->print($foo->printReport);
      if ($foo->error) { $doc->print($foo->errorMessage); }
      else
      {
        return $doc;
      }
    }
  }
  #display form
  my %form = $foo->printForm();
  if ($foo->error)
  {
    $self->error($foo->errorMessage);
  }
  if (defined %form)
  {
    $doc->print(%form);
  }
  else
  {
    $doc->print('error! form is undefined<br>');
    $doc->print("\nformObj:".$foo->{formObj}->error);
  }

  $doc->print($foo->debugString);
  return $doc;
}

sub c_search
{
  my $self = shift;
  return $self->c_searchForm(@_);
}

=back

=cut

1;
__END__

=head1 Exported FUNCTIONS (non-Inline POD)

  HTMLObject run(command)
    Runs the specified command and returns the resulting HTMLObject.


=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
