# CGL.pm - The Object Class that provides a CGL Object
#$Id: CGL.pm,v 1.17 2006/05/12 22:09:53 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::CGL;
use strict;
use Portal::Accounting::Investments::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.17 $';

=head1 NAME

Accounting::Investments::Objects::CGL

=head1 SYNOPSIS

  use Portal::Accounting::Objects::CGL;
  my $obj = Portal::Accounting::Objects::CGL->new();
  $obj->populate( );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

CGL is a class for gathering Capital gain loss data and presenting reports to the user.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::CGL module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::CGL", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "CGL";
  my $l = $self->{langObj};
  my $Stock = $l->map('Stock');
  my $Asset = $l->map('Asset');
  my $Account = $l->map('Account');
  my @date = $self->getDateArray();
  my $from;
  my $to;
  my $fy = $date[0];
  my $led = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  $led->set('code', $args{ledgerCode});
  $led->select;
  if ($led->error) { $self->error($led->errorMessage); }
  else
  {
    $fy = $led->workingYear();
    $from = $led->start(month => 1, fiscalYear => $fy);
    $to = $led->end(month => 12, fiscalYear => $fy);
  }
  my $collapse = $self->{imageDir}.'/minus.gif';
  my $expand = $self->{imageDir}.'/plus.gif';

  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "search" },
    "onload" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "from" => { -Label => $l->map("Start"), -Type => "date-picker", -DataType => "string",
      -year => $fy , -Value => $from },
    "to" => { -Label => $l->map("End"), -Type => "date-picker", -DataType => "string",
      -year => $fy , -Value => $to },
    "stock" => { -Label => $Stock, -Type => "multi-select", -DataType => "int",
      -Options => $self->{stocks}, size => 4,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "asset" => { -Label => $Stock, -Type => "multi-select", -DataType => "int",
      -Options => $self->{assets}, size => 4,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "stockOrder" => { -Label => $Stock." ".$l->map("Grouping"), -Type => "select", -DataType => "int",
      -Options => {
        names => [
          $Stock,
          "ST/LT, $Stock",
          "$Account, ST/LT, $Stock",
         ],
        values => [
          'sub_account',
          'account, sub_account',
          'invaccount, account, sub_account',
#          'invaccount, sub_account, account',
#          'account, sub_account, invaccount',
#          'account, invaccount, sub_account',  #not really helpful
#          'sub_account, invaccount, account', #not really helpful
#          'sub_account, account, invaccount',
        ]
      },
    },
    "order" => { -Label => $l->map("Sort"), -Type => "select", -DataType => "int",
      -Options => {
        names => [
          "ST/LT , $Asset",
          "$Asset, ST/LT",
         ],
        values => [
          'account_num, sub_account',
          'sub_account, account_num',
        ]
      },
    },
    "format" => { -Label => $l->map("Format"), -Type => "select", -DataType => "int",
      -Options => { names => [ 'html', 'xml', 'csv' ], values => [qw(html xml csv)] }, -Value => 'html',  },
  };
  $self->{data}->{report} = {
    "from" => { -Label => $l->map("From"), -Type => "text", -DataType => "string", -Value => $from, -ReadOnly => 1,
    },
    "to" => { -Label => $l->map("To"), -Type => "text", -DataType => "string", -Value => $to, -ReadOnly => 1,
    },
    "onload" => { -Type => "hidden", -onload => "var images = new Array(); images['field'] = 'label'; images['expand'] = '$expand'; images['collapse'] = '$collapse'; setLevelsByClass('cgl'); walkRowsByClass('cgl', 'dblclick', images);", },
  };
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  <table class="primary">
    <tr>
      <td>
        <table><tr>
          <td><b>#LABEL=from#</b> #FI=from#</td>
          <td><b>#LABEL=to#</b> #FI=to#</td>
        </tr></table>
      </td>
      <td>
        #LABEL=format# #FIELD=format#
      </td>
    </tr>
    <tr id="button">
      <td colspan="2">
        <input type="button" name="toggle" value="#LANG=Advanced# >>" onclick="hide('button'); document.getElementById('advanced').style.display = 'inline';"/>
      </td>
    </tr>
    <tr id="advanced" style="display: none;">
      <td valign="top">
        <br><b>#LABEL=stock#</b><br>#IF=stock#
        <br><b>#LABEL=stockOrder#</b>
        <br>#IF=stockOrder#
      </td>
      <td valign="bottom">
        <br><b>#LABEL=order# #LANG=Assets#</b>
        <br>#IF=order#
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{report} = <<"ENDOF";
  <div class="printHide" style="float: right;">
    <span onclick="tableShow(document.getElementById('cglstocks'), document.getElementById('level').value);">Show</span>
    <select id="level" name="level">
      <option value="2">Level 2</option>
      <option value="3">Level 3</option>
      <option value="4">Level 4</option>
      <option value="5">Level 5</option>
    </select>
    <span onclick="tableHide(document.getElementById('cglstocks'), document.getElementById('level').value);">Hide</span>
  </div>

  <div style="text-align: center; ">
    #FIELD=onload#
    #LABEL=from# #FIELD=from# #LABEL=to# #FIELD=to#
  </div>
  <div style="clear: both;"></div>
ENDOF
  $self->{css}->{report} = <<"ENDOF";
  table.cgl { width: 100%; }
  .spaceme { margin-top: 1em; }
  table.total { width: 50%; }
  th.label { text-align: left; }
  th.level { display: none; }
  th.total { display: none; }
  th.journal { display: none; text-align: left; }
  th.ref_num { display: none; }
  th.buy_date { text-align: left; }
  th.sell_date { text-align: left; }
  th.stlt_account { display: none; }
  th.stlt { text-align: left; }
  th.invaccount { text-align: left; }
  th.account_name { text-align: left; }
  th.account_num { display: none; }
  th.sub_account { display: none; }
  th.stock { text-align: left; display: none; }
  th.lot { text-align: left; display: none; }
  th.shares { text-align: right; }
  th.cost { text-align: right; }
  th.gain_loss { text-align: right; }
  th.proceeds { text-align: right; }
  th.amount { text-align: right; }
  td.label { text-align: left; }
  td.total { display: none; }
  td.level { display: none; }
  td.journal { display: none; text-align: left; }
  td.ref_num { display: none; }
  td.buy_date { text-align: left; }
  td.sell_date { text-align: left; }
  td.stlt_account { display: none; }
  td.stlt { text-align: left; }
  td.invaccount { text-align: left; }
  td.account_name { text-align: left; }
  td.account_num { display: none; }
  td.sub_account { display: none; }
  td.stock { text-align: left; display: none; }
  td.lot { text-align: left; display: none; }
  td.shares { text-align: right; }
  td.cost { text-align: right; }
  td.gain_loss { text-align: right; }
  td.proceeds { text-align: right; }
  td.amount { text-align: right; }
  .level1 { text-align: left; font-size: 1.1em; }
  .level2 { text-align: left; font-size: 1.05em; padding-left: 2em; }
  .level3 { text-align: left; font-size: 1em; padding-left: 4em; }
  .level4 { text-align: left; font-size: .9em; padding-left: 6em; }
  .level5 { text-align: left; font-size: .8em; padding-left: 9em; }
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( from to order stockOrder ) ],
    optional => [ qw( asset stock) ],
    constraints => {
         from => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
         to => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
         stock => qr/^\d+$/,
         stockOrder => qr/^(account(, )?|sub_account(, )?|invaccount(, )?)+$/,
         order => qr/^(account_num(, )?|sub_account(, )?)+$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( from to order stockOrder ) ],
    optional => [ qw( asset stock) ],
    constraints => { }
  };
  $self->{manualHeaders} = {
    level => $self->{langObj}->map('Level'),
    journal => $self->{langObj}->map('Journal'),
    ref_num => $self->{langObj}->map('ID'),
    sell_date => $self->{langObj}->map('Sold'),
    account_num => $self->{langObj}->map('Account'),
    account_name => $self->{langObj}->map('Account'),
    sub_account => $self->{langObj}->map('Sub Account'),
    sub_account_name => $self->{langObj}->map('Sub Account'),
    amount => $self->{langObj}->map('Amount'),
  };
  $self->{headers} = {
    level => $self->{langObj}->map('Level'),
    label => $self->{langObj}->map('Label'),  #based on ordering and subtotals
    journal => $self->{langObj}->map('Journal'),
    ref_num => $self->{langObj}->map('ID'),
    invaccount => $self->{langObj}->map('Account'),
    stlt_account => 'STLT',
#    st_lt => $self->{langObj}->map('STLT'),
    sub_account => $self->{langObj}->map('Sub Account'),
    stock => $self->{langObj}->map('Stock'),
    lot => $self->{langObj}->map('Lot'),
    buy_date => $self->{langObj}->map('Purchased'),
    sell_date => $self->{langObj}->map('Sold'),
    shares => $self->{langObj}->map('Shares'),
    cost => $self->{langObj}->map('Cost'),
    proceeds => $self->{langObj}->map('Proceeds'),
    gain_loss => $self->{langObj}->map('Gain/Loss'),
  };
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item @sqlStrings selectSQL

routine to generate sql neeeded to select data

=cut
sub selectSQL
{
  my $self = shift;
  my %to_char = (
    money => '999G999G990D00',
    shares => '999G999G999D000000',
    wholeShares => '999G999G999',
  );
  my $from = $self->get('from');
  my $to = $self->get('to');
#  my $asset = $self->get('asset');
  my $subAccountNum = $self->get('stock');
  my $ledgerCode = $self->get('ledgerCode');
#  my $accountNum = $self->get('account');
  my $stockOrder = $self->get('stockOrder');
  my $order = $self->get('order');   #ordring for non stock data
  use Date::Calc qw(Today_and_Now);
  my @date = Today_and_Now;
  my $view = "xiwa_cgl_".$ledgerCode."_".$date[0].$date[1].$date[2].$date[3].$date[4].$date[5];
  my $Total = $self->{langObj}->map('Total');

  #non stock cgl
  $self->{SELECT}[0] = <<"FOO";
  SELECT
    '2' AS level,
    '' AS label,
    j.journal,
    j.ref_num,
    d.trans_date::text AS sell_date,
    d.account_num,
    a.name AS account_name,
    d.sub_account_num AS sub_account,
    s.sub_acc_name AS sub_account_name,
    to_char(d.c_amount, '$to_char{money}') AS amount
  FROM journal_entry_tb AS j, debitcredit_tb AS d, account_tb AS a, sub_acc_tb AS s
  WHERE
    j.ref_num = d.parent_num
    AND a.number = d.account_num
    AND d.sub_account_num = s.sub_acc_number
    AND (
      d.account_num = 4200
      OR d.account_num = 4201
      OR d.account_num = 5200
      OR d.account_num = 5201
    )
    AND j.journal <> 'IS'
    AND j.ledger_code = '$ledgerCode'
    AND d.trans_date BETWEEN '$from' AND '$to'
  UNION
  SELECT
    '1' AS level,
    '$Total' AS label,
    '' AS journal,
    NULL AS ref_num,
    '' AS sell_date,
    NULL AS account_num,
    NULL AS account_name,
    NULL AS sub_account,
    NULL AS sub_account_name,
    CASE
      WHEN (SUM(d.c_amount) IS NULL) THEN '0.00'
      ELSE
        to_char(SUM(d.c_amount), '$to_char{money}')
      END
    AS amount
  FROM journal_entry_tb AS j, debitcredit_tb AS d
  WHERE j.ref_num = d.parent_num
    AND (
      d.account_num = 4200
      OR d.account_num = 4201
      OR d.account_num = 5200
      OR d.account_num = 5201
    )
    AND j.journal <> 'IS'
    AND j.ledger_code = '$ledgerCode'
    AND d.trans_date BETWEEN '$from' AND '$to'
  ORDER BY journal, level, $order
FOO

  $self->{SELECT}[5] = <<"FOO";
 SELECT
  to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
  to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
  to_char(
    sum(gain_loss) +
    CASE WHEN (
      SELECT
        d.c_amount AS amount
      FROM journal_entry_tb AS j, debitcredit_tb AS d
      WHERE j.ref_num = d.parent_num
        AND (
          d.account_num = 4200
          OR d.account_num = 4201
          OR d.account_num = 5200
          OR d.account_num = 5201
        )
        AND j.journal <> 'IS'
        AND j.ledger_code = '$ledgerCode'
        AND d.trans_date BETWEEN '$from' AND '$to'
      LIMIT 1
    ) IS NULL THEN
     0
    ELSE
    (
      SELECT
        SUM(d.c_amount) AS amount
      FROM journal_entry_tb AS j, debitcredit_tb AS d
      WHERE j.ref_num = d.parent_num
        AND (
          d.account_num = 4200
          OR d.account_num = 4201
          OR d.account_num = 5200
          OR d.account_num = 5201
        )
        AND j.journal <> 'IS'
        AND j.ledger_code = '$ledgerCode'
        AND d.trans_date BETWEEN '$from' AND '$to'
    ) END
    , '$to_char{money}'
  ) AS gain_loss
 FROM $view
FOO

  $self->{SELECT}[1] = <<"FOO";
  CREATE VIEW $view AS
  SELECT
    j.journal AS journal,
    j.ref_num,
    (
      SELECT event_date
      FROM lot_node_tb
      WHERE lot = l.buy_lot
        AND parent_id IS NULL
    )AS buy_date,
    d.trans_date AS sell_date,
    d.account_num AS stlt_account,
    (
      SELECT abrev FROM gl_abrev_tb WHERE id = d.account_num
    ) AS st_lt,
    (
      SELECT name FROM account_tb WHERE number =
      (
        SELECT account_num
        FROM debitcredit_tb
        WHERE parent_num = d.parent_num AND sub_account_num = '-1'
      )
    ) AS invaccount,
    d.sub_account_num AS sub_account,
    (
      SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = d.sub_account_num
    ) AS sub_account_name,
    l.lot AS lot,
    s.ns AS shares,
    (
      SELECT d_amount FROM debitcredit_tb
      WHERE parent_num = d.parent_num AND sub_account_num = -1
    ) AS proceeds,
    d.c_amount AS gain_loss
  FROM journal_entry_tb AS j, debitcredit_tb AS d, lot_node_tb AS l, sec_event_tb AS s
  WHERE j.ref_num = d.parent_num
    AND j.ref_num = l.journal_ref
    AND s.event_ref = l.event_ref
    AND (
      d.account_num = 4200
      OR d.account_num = 4201
      OR d.account_num = 5200
      OR d.account_num = 5201
    )
    AND j.ledger_code = '$ledgerCode'
    AND d.trans_date BETWEEN '$from' AND '$to'
FOO
  if ($subAccountNum ne "")
  {
    my @a = split('\0', $subAccountNum);
    $self->{SELECT}[1] .= "     AND d.sub_account_num IN (".join(",",@a).")";
  }
  $self->{SELECT}[1] .= <<"FOO";
  ORDER BY journal, stlt_account, sub_account_name, lot, sell_date
FOO

  $self->{SELECT}[2] = <<"FOO";
  DROP VIEW $view
FOO

  if ($self->get('stockOrder') eq 'sub_account')
  {
    $self->{template}->{report} = <<"ENDOF";
    <div class="printHide" style="float: right;">
      <span onclick="tableShow(document.getElementById('cglstocks'), document.getElementById('level').value);">Show</span>
      <select id="level" name="level">
        <option value="2">Stocks</option>
        <option value="3">Lots</option>
      </select>
      <span onclick="tableHide(document.getElementById('cglstocks'), document.getElementById('level').value);">Hide</span>
    </div>

    <div style="text-align: center; ">
      #FIELD=onload#
      #LABEL=from# #FIELD=from# #LABEL=to# #FIELD=to#
    </div>
    <div style="clear: both;"></div>
ENDOF
    $self->{headers}->{label} = $self->{langObj}->map("Stock")."/".$self->{langObj}->map("Lot");
    $self->{SELECT}[3] = <<"FOO";
  SELECT
    '3' AS level,
    lot::text AS label,
    journal,
    ref_num,
    sub_account,
    sub_account_name AS stock,
    lot,
    buy_date,
    sell_date::text,
    CASE WHEN shares - trunc(shares) < 0.0000001 THEN to_char(shares, '$to_char{wholeShares}') ELSE to_char(shares, '$to_char{shares}') END AS shares,
    to_char(proceeds - gain_loss, '$to_char{money}') AS cost,
    to_char(proceeds, '$to_char{money}') AS proceeds,
    to_char(gain_loss, '$to_char{money}') AS gain_loss,
    invaccount,
    stlt_account,
    st_lt
  FROM $view
  UNION ALL
  SELECT
    '2' AS level,
    sub_account_name AS label,
    journal,
    NULL AS ref_num,
    NULL AS sub_account,
    sub_account_name AS stock,
    -1 AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    CASE WHEN SUM(shares) - trunc(SUM(shares)) < 0.0000001 THEN to_char(SUM(shares), '$to_char{wholeShares}') ELSE to_char(SUM(shares), '$to_char{shares}') END AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss,
    NULL AS invaccount,
    NULL AS stlt_account,
    NULL AS st_lt
  FROM $view
  GROUP BY journal, sub_account_name
  UNION ALL
  SELECT
    '1' AS level,
    '$Total' AS label,
    journal,
    NULL AS ref_num,
    NULL AS sub_account,
    '' AS stock,
    -1 AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    '' AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss,
    NULL AS invaccount,
    NULL AS stlt_account,
    NULL AS st_lt
  FROM $view
  GROUP BY journal

  ORDER BY journal, stock, sell_date

FOO
  }
  elsif ($self->get('stockOrder') eq 'invaccount, account, sub_account')
  {
    $self->{template}->{report} = <<"ENDOF";
  <div class="printHide" style="float: right;">
    <span onclick="tableShow(document.getElementById('cglstocks'), document.getElementById('level').value);">Show</span>
    <select id="level" name="level">
      <option value="2">Accounts</option>
      <option value="3">ST/LT</option>
      <option value="4">Stocks</option>
      <option value="5">Lots</option>
    </select>
    <span onclick="tableHide(document.getElementById('cglstocks'), document.getElementById('level').value);">Hide</span>
  </div>

  <div style="text-align: center; ">
    #FIELD=onload#
    #LABEL=from# #FIELD=from# #LABEL=to# #FIELD=to#
  </div>
  <div style="clear: both;"></div>
ENDOF
    $self->{headers}->{label} = $self->{langObj}->map("Account")."/STLT/".$self->{langObj}->map("Stock");
    $self->{SELECT}[3] = <<"FOO";
  SELECT
    '5' AS level,
    lot::text AS label,
    journal,
    ref_num,
    invaccount,
    stlt_account,
    st_lt,
    sub_account,
    sub_account_name AS stock,
    lot,
    buy_date,
    sell_date::text,
    CASE WHEN shares - trunc(shares) < 0.0000001 THEN to_char(shares, '$to_char{wholeShares}') ELSE to_char(shares, '$to_char{shares}') END AS shares,
    to_char(proceeds - gain_loss, '$to_char{money}') AS cost,
    to_char(proceeds, '$to_char{money}') AS proceeds,
    to_char(gain_loss, '$to_char{money}') AS gain_loss
  FROM $view
  UNION ALL
  SELECT
    '4' AS level,
    sub_account_name::text AS label,
    journal,
    NULL AS ref_num,
    invaccount,
    stlt_account,
    st_lt,
    sub_account,
    sub_account_name AS stock,
    NULL AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    CASE WHEN SUM(shares) - trunc(SUM(shares)) < 0.0000001 THEN to_char(SUM(shares), '$to_char{wholeShares}') ELSE to_char(SUM(shares), '$to_char{shares}') END AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss
  FROM $view
  GROUP BY journal, invaccount, stlt_account, st_lt, sub_account, sub_account_name

  UNION ALL
  SELECT
    '3' AS level,
    st_lt::text AS label,
    journal,
    NULL AS ref_num,
    invaccount,
    stlt_account,
    st_lt AS st_lt,
    NULL AS sub_account,
    '' AS stock,
    NULL AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    NULL AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss
  FROM $view
  GROUP BY journal, invaccount, stlt_account, st_lt

  UNION ALL
  SELECT
    '2' AS level,
    invaccount::text AS label,
    journal,
    NULL AS ref_num,
    invaccount AS invaccount,
    NULL AS stlt_account,
    '' AS st_lt,
    NULL AS sub_account,
    '' AS stock,
    NULL AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    NULL AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss
  FROM $view
  GROUP BY journal, invaccount

  UNION ALL
  SELECT
    '1' AS level,
    '$Total' AS label,
    journal,
    NULL AS ref_num,
    '' AS invaccount,
    NULL AS stlt_account,
    NULL AS st_lt,
    NULL AS sub_account,
    '' AS stock,
    NULL AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    NULL AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(SUM(gain_loss), '$to_char{money}') AS gain_loss
  FROM $view
  GROUP BY journal

  ORDER BY journal, invaccount, st_lt, stock, sell_date
FOO
  }
  elsif ($self->get('stockOrder') eq 'account, sub_account')
  {
    $self->{template}->{report} = <<"ENDOF";
  <div class="printHide" style="float: right;">
    <span onclick="tableShow(document.getElementById('cglstocks'), document.getElementById('level').value);">Show</span>
    <select id="level" name="level">
      <option value="2">ST/LT</option>
      <option value="3">Stocks</option>
      <option value="4">Lots</option>
    </select>
    <span onclick="tableHide(document.getElementById('cglstocks'), document.getElementById('level').value);">Hide</span>
  </div>

  <div style="text-align: center; ">
    #FIELD=onload#
    #LABEL=from# #FIELD=from# #LABEL=to# #FIELD=to#
  </div>
  <div style="clear: both;"></div>
ENDOF
    $self->{headers}->{label} = "STLT/".$self->{langObj}->map("Stock")."/".$self->{langObj}->map("Lot");
    $self->{SELECT}[3] = <<"FOO";
  SELECT
    '4' AS level,
    lot::text AS label,
    journal,
    ref_num,
    stlt_account,
    st_lt,
    sub_account::text,
    sub_account_name AS stock,
    lot::text AS lot,
    buy_date,
    sell_date::text,
    CASE WHEN shares - trunc(shares) < 0.0000001 THEN to_char(shares, '$to_char{wholeShares}') ELSE to_char(shares, '$to_char{shares}') END AS shares,
    to_char(proceeds - gain_loss, '$to_char{money}') AS cost,
    to_char(proceeds, '$to_char{money}') AS proceeds,
    to_char(gain_loss, '$to_char{money}') AS gain_loss,
    invaccount
  FROM $view
  UNION ALL
  SELECT
    '3' AS level,
    sub_account_name AS label,
    journal,
    NULL AS ref_num,
    stlt_account,
    st_lt,
    sub_account::text,
    sub_account_name AS stock,
    '' AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    CASE WHEN SUM(shares) - trunc(SUM(shares)) < 0.0000001 THEN to_char(SUM(shares), '$to_char{wholeShares}') ELSE to_char(SUM(shares), '$to_char{shares}') END AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss,
    NULL AS invaccount
  FROM $view
  GROUP BY journal, stlt_account, sub_account, sub_account_name, st_lt

  UNION ALL
  SELECT
    '2' AS level,
    st_lt AS label,
    journal,
    NULL AS ref_num,
    stlt_account,
    st_lt,
    '' AS sub_account,
    '' AS stock,
    '' AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    NULL AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(SUM(gain_loss), '$to_char{money}') AS gain_loss,
    NULL AS invaccount
  FROM $view
  GROUP BY journal, stlt_account, st_lt

  UNION ALL
  SELECT
    '1' AS level,
    '$Total' AS label,
    journal,
    NULL AS ref_num,
    NULL AS stlt_account,
    '' AS st_lt,
    '' AS sub_account,
    '' AS stock,
    '' AS lot,
    NULL AS buy_date,
    '' AS sell_date,
    NULL AS shares,
    to_char(SUM(proceeds - gain_loss), '$to_char{money}') AS cost,
    to_char(SUM(proceeds), '$to_char{money}') AS proceeds,
    to_char(sum(gain_loss), '$to_char{money}') AS gain_loss,
    NULL AS invaccount
  FROM $view
  GROUP BY journal

  ORDER BY journal, st_lt, stock, sell_date
FOO

  }
  return @{$self->{SELECT}};
}

=item string printXML

  This routine uses the selectSQL to gather data in xml format.  If there is an error the return value will be the index of the sql array that had the error.  The manual sql is in index 0.  The html is returned when there are no errors.

=cut
sub printXML
{
  my $self = shift;
  my @sql = $self->selectSQL;
  $self->{dbHandle}->write(sql => $sql[1]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '1'; }
  my $xml = $self->{dbHandle}->readXML($sql[3]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '3'; }
  $self->{dbHandle}->write(sql => $sql[2]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '2'; }
  return $xml;
}

=item string printHTML

  This routine uses the selectSQL to gather manually entered Cap gains and losses (as opposed to stock data which is generated by the system) data in html format.  If there is an error the return value will be the index of the sql array that had the error.  The html is returned when there are no errors.

=cut
sub printForm
{
  my $self = shift;
  my %args = (form => 'main', @_);
  my %r;
  my $l = $self->langObj;
  if ($args{form} eq 'report')
  {
    %r = $self->SUPER::printForm(@_);

    if (!defined $self->{SELECT} || $self->{SELECT}[3] eq '') { $self->selectSQL; }
    if ($self->{formObj}->error) { $self->error($self->{formObj}->errorMessage); return %r; }
    $self->{dbHandle}->write(sql => $self->{SELECT}[1]);
    if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '1'; }
    my $t = $self->{dbHandle}->readHTML(sql => $self->{SELECT}[3], tableClass => 'cgl collapse spaceme', displayNULLAs => '', displayNumRows => 0, headers => $self->{headers});
    $t =~ s/class="cgl/id="cglstocks" class="cgl/;
    $self->{totals} = $self->{dbHandle}->readHTML(sql => $self->{SELECT}[5], tableClass => 'total', displayNULLAs => '', displayNumRows => 0, headers => {cost => $l->map('Total Cost'), gain_loss => $l->map('Total Gain/Loss'), proceeds => $l->map('Total Proceeds') } );
    if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '5'; }
    $r{body} .= $t;
    if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '3'; }
    $self->{dbHandle}->write(sql => $self->{SELECT}[2]);
    if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '2'; }
  }
  else
  {
    %r = $self->SUPER::printForm(@_);
  }
  return %r;
}

=item string printManualXML

  This routine uses the selectSQL to gather manually entered Capital gains data in xml format.  If there is an error the return value will be the index of the sql array that had the error.  The manual sql is in index 0.  The html is returned when there are no errors.

=cut
sub printManualXML
{
  my $self = shift;
  if (!defined $self->{SELECT} || $self->{SELECT}[0] eq '') { $self->selectSQL; }
  my $xml = $self->{dbHandle}->readXML($self->{SELECT}[0]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '0'; }
  return $xml;
}

=item string printManualHTML

  This routine uses the selectSQL to gather all data in html format.  If there is an error the return value will be the index of the sql array that had the error.  The manual sql is in index 0.  The html is returned when there are no errors.

=cut
sub printManualHTML
{
  my $self = shift;
  if (!defined $self->{SELECT} || $self->{SELECT}[0] eq '') { $self->selectSQL; }
  my $r;
  $r = $self->{dbHandle}->readHTML(sql => $self->{SELECT}[0], tableClass => 'cgl collapse spaceme', displayNULLAs => '', displayNumRows => 0, headers => $self->{manualHeaders});
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '0'; }
  return $r;
}

=item string readCSV

  This routine uses the selectSQL to gather all data in Comma Separated Value (CSV) format.  If there is an error the return value will be the index of the sql array that had the error.  There are 0-3 currently in $self->{SELECT}[].  The data is returned when there are no errors.

=cut
sub readCSV
{
  my $self = shift;
  if (!defined $self->{SELECT} || $self->{SELECT}[3] eq '') { $self->selectSQL; }
  $self->{dbHandle}->write(sql => $self->{SELECT}[1]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '1'; }
  my $csv = $self->{dbHandle}->readCSV($self->{SELECT}[3]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '3'; }
  $self->{dbHandle}->write(sql => $self->{SELECT}[2]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '2'; }
  $csv .= $self->{dbHandle}->readCSV($self->{SELECT}[0]);
  if ($self->dbHandle->error) { $self->error($self->{dbHandle}->errorMessage); return '0'; }
  return $csv;
}

sub printReport
{
  my $self = shift;
  $self->selectSQL;
  if ($self->error) { return undef; }
  my $format = $self->get('format');
  my %form;
  if ($format eq 'csv')
  {
    $form{body} = $self->readCSV();
    if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
  }
  elsif ($format eq 'xml')
  {
    $form{body} = $self->printXML();
    $form{body} .= $self->printManualXML();
  }
  else
  {
    %form = $self->printForm(form => 'report');
    $form{body} .= $self->printManualHTML(form => 'report');
    #!!!nasty hack
    # had to put total gathering in subroutines bc it relies on view to gather data
    #wanted to output it after all other output.
    # to do that must output here after all other data is done like manual, assets, etc.
    $form{body} .= $self->{totals};
    if ($self->get('stockOrder') eq 'invaccount, account, sub_account')
    {
      $form{css} .= "th.invaccount {display: none; }\ntd.invaccount { display: none; }\n";
      $form{css} .= "th.st_lt {display: none; }\ntd.st_lt { display: none; }\n";
    }
    elsif ($self->get('stockOrder') eq 'account, sub_account')
    {
      $form{css} .= "th.st_lt {display: none; }\ntd.st_lt { display: none; }\n";
    }
  }
  return %form;
}


1;
__END__

=back

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
