# LotNode.pm - The Object Class that provides a LotNode Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: LotNode.pm,v 1.15 2006/04/17 20:08:39 moreejt Exp $
#This package is released under the GNU General Public License version 2.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::LotNode;
use Portal::Accounting::Investments::Objects::Base;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.15 $';

=head1 NAME

Accounting::Investments::Objects::LotNode

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::LotNode;
  @ISA = qw(Portal::Accounting::Investments::Objects::LotNode Exporter AutoLoader);

=head1 DESCRIPTION

 LotNode - Object for deriving investment LotNode objects such as Buy, Sell, etc.

  #################NOTES#########
  #
  #  the user defined counter feature is currently not implemented
  #  1) front end code does not support it yet
  #  2) back end code is not complete
  #  3) lot balances become tied not to dates but to individual LotNodes
  #  4) retrieving lot balances is difficult enough already
  #  5) database tables are a mess--need rewrite
  #  6) in sec_LotNode_tb, date + counter is primary_key and LotNode_ref can be same because some LotNodes have two records --each having different counter
  #################################################

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

    Creates a new instance of Portal::Accounting::Investments::Objects::LotNode.
    See Portal::Objects::Base(3) for a listing of required arguments if the
    above args don't seem to work.

=cut

=item populate( id =>, ledgerCode =>, . . .)

  Data:
    required => [ qw( id ledgerCode event date parent
       account subAccount lot buyLot shares cost ) ],
    optional => [ qw( journalRef subEntity ) ],

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', @_);
  if ($self->error)
  {
    $self->error('');
    return $self;
  }

  $self->{'table'} = "account_tb";
  my $l = $self->{langObj};
  $self->{data}->{main} = {
    "id" => { -Label => $l->map("Id"), -Type => "text", -DataType => "integer" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "event" => { -Label => "Event", -Type => "text", -DataType => "integer" },
    "date" => { -Label => "Date", -Type => "date-picker", -DataType => "string" },
    "parent" => { -Label => "Parent", -Type => "text", -DataType => "integer" },
    "journalRef" => { -Label => "JournalEntry", -Type => "text", -DataType => "integer", -Value => -1, },
    "subEntity" => { -Label => "Sub Entity", -Type => "text", -DataType => "text" },
    "account" => { -Label => "Account", -Type => "text", -DataType => "integer" },
    "subAccount" => { -Label => "subAccount", -Type => "text", -DataType => "integer" },
    "lot" => { -Label => "Lot", -Type => "text", -DataType => "integer" },
    "buyLot" => { -Label => "Original Lot Purchased", -Type => "text", -DataType => "integer" },
    "shares" => { -Label => "Shares", -Type => "text", -DataType => "float", -Value => 0.0 },
    "cost" => { -Label => "Cost", -Type => "text", -DataType => "float", -Value => 0.0 },
  };
  $self->{profile}->{main} = {
    required => [ qw( ledgerCode event date account subAccount lot buyLot shares cost ) ],
    optional => [ qw( id parent journalRef subEntity ) ],
    constraints => {
      id => qr/^\d+$/,
      event => qr/^\d+$/,
      parent => qr/^\d+$/,
      journalRef => qr/^\d+|-1$/,
      account => qr/^\d+$/,
      subAccount => qr/^\d+$/,
      lot => qr/^\d+$/,
      buyLot => qr/^\d+$/,
      shares => qr/^\d+(\.?)\d*$/,
      cost => qr/^\d+(\.?)\d*$/,
      date => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
    },
  };

  $self->{'table'} = "lot_node_tb";
  $self->{'id'} = "node_id";
  $self->{INSERTorder}[0] = [ qw( node_id ledger_code event_ref event_date parent_id account_num sub_account_num lot buy_lot shares cost journal_ref sub_e_code ) ] ;  #insert order for db column names

  $self->{newOnly}[0] = {
    node_id => 'id',
    ledger_code => 'ledgerCode',
    event_date => 'date',
    parent_id => 'parent',
    event_ref => 'event',
    journal_ref => 'journalRef',
    sub_e_code => 'subEntity',
    account_num => 'account',
    sub_account_num => 'subAccount',
    lot => 'lot',
    buy_lot => 'buyLot',
    shares => 'shares',
    cost => 'cost'
  };
  $self->{db2data}[0] = {
    node_id => 'id',
    ledger_code => 'ledgerCode',
    event_date => 'date',
    parent_id => 'parent',
    event_ref => 'event',
    journal_ref => 'journalRef',
    sub_e_code => 'subEntity',
    account_num => 'account',
    sub_account_num => 'subAccount',
    lot => 'lot',
    buy_lot => 'buyLot',
    shares => 'shares',
    cost => 'cost'
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE node_id = #FIELD=id#";
  $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##) VALUES (##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE node_id = #FIELD=id#";
  $self->{DELETE}[0] = "DELETE FROM $self->{table} WHERE node_id = #FIELD=id#";

  return $self;
}

=item boolean isValid

 Returns 0 or 1 to indicate if the object is valid.

=cut
sub isValid
{
  my $self = shift;
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  # do validation code here.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }
  return 1;
}

=item scalar insert(commit => 0)

    Attempts to insert the contents of the object into the database.
    requires:
    optional: commit - defaults to not commit because lot nodes are usually entered as part of a larger transaction

=cut
sub insert
{
  my $self = shift;
  my %args = ( commit => 0, @_ );

  # check the validity of the transaction object
  if ($self->get('id') eq '')
  {
    my $sth = $self->{dbHandle}->read(sql => "SELECT nextval('lot_node_tb_node_id_seq')");
    if ($self->{dbHandle}->error())
    {
      $self->error('<p class="error">Failed to get the nextval() of sequence!</p>' . $self->{dbHandle}->errorMessage);
      return undef;
    }
    my @info = $sth->fetchrow_array;
    $self->set('id', $info[0]);
  }
  else
  {
    $self->error("<p>Error!  Tried to insert a transaction with id: ".$self->get('id').". Perhaps you need to update it instead.</p>\n");
    return 0;
  }
  return $self->SUPER::insert(@_, commit => $args{commit} );
}

sub isNew
{
  my $self = shift;
  return $self->{new};
}

=item scalar childNodes()

  requires: nothing
  optional:  date and counter and event ???!!! might add these to stop the process at a specified event

 returns: scalar pointing to hash of LotNodes that represent all child nodes of the current object.
   Does not traverse the tree any further than immediate children.  See tree() for full traversal.
   Caches the children in $self->{children} and returns them.

=cut
sub childNodes
{
  my $self = shift;
  my $parentId = $self->get('id');
  my %nodes;

  # query the database for all nodes that have parent_id = $parentId
  my $sql = "SELECT node_id FROM lot_node_tb WHERE parent_id = ? ORDER BY node_id";
  my $sth = $self->{dbHandle}->read(sql => $sql, plug => [ $parentId ]);
  if ($self->{dbHandle}->error)
  {
    $self->error("<p>DBIW Error in getChildLotNodes</p>" . $self->{dbHandle}->errorMessage);
    return %nodes;
  }

  while (my @info = $sth->fetchrow_array)
  {
    # lookup the node info for this node.
    my $nodeObj = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
    $nodeObj->set('id', $info[0]);
    $nodeObj->select;
    if ($nodeObj->error)
    {
      $self->error("<p>Lot Node with ID = '$info[0]' does not exist while getting children of node = '$parentId'.</p>");
    }
    else
    {
      # insert the node into the hash
      $nodes{$info[0]} = $nodeObj;
    }
  }
  $self->{children} = \%nodes;
  return \%nodes;
}

=item @nodes tree()

  optional:
  returns: list of LotNode IDs (integers)

  This routine, starts with the Lot Node Id in the given object and gathers
  all data for it as well as all nodes that are children of it.  It creates
  objects and stores them in $self->{children}.  The nodeId for each node is
  the key in the children hash.  Each child node also has a children hash if
  there are children under it.

  $self->{children}->{$id1}->{children}->{$id2} . . . and so on

  The return value is alist of the node IDs gathered during the process.

=cut
sub tree
{
  my $self = shift;
  my @new;
  my $id = $self->get('id');
  if ($id !~ /\d+/) { $self->error("<div>Invalid id in LotNode during call to tree: '$id'</div>"); return 0; }
  $self->select;
  if ($self->error) { return undef; }
  # now get all children nodes and cache them
  $self->childNodes();
  if ($self->error) { return undef; }
  foreach my $n (keys %{$self->{children}})
  {
    #get the children of the children
    @new = (@new, $self->{children}->{$n}->tree);
    if ($self->{children}->{$n}->error)
    {
      $self->error($self->{children}->{$n}->errorMessage);
    }
  }
  return ($id, @new);
}

=item LotNode balance(date =>, counter =>, lot => )

  required:
  optional: lot, date, counter
  returns: LotNode with proper data or with id of ''

  Routine which creates an object with the correct data based on the info given
  Uses internal lot setting if none passed in
  If no lot entry is found for the given data there will NOT be an error set but
  the returned node will have an id of ''.

=cut
sub balance
{
  my $self = shift;
  my %args = (lot => $self->get('lot'), @_);
  my $new = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
  if ($new->error) { $self->error($new->errorMessage); return undef; }
  my $id;
  #find node ID in table
  #set id $self->set('id', $d[0]);
  my $sql = "SELECT MAX(lot_node_tb.node_id) FROM lot_node_tb WHERE lot_node_tb.lot = ?";
  my $plug = [ $args{lot} ];
  my $date;
  my $counter;
  if (defined ($args{counter}))
  {
    $sql .= " AND ( lot_node_tb.event_date < ? OR (lot_node_tb.event_date = ? AND lot_node_tb.event_ref = sec_event_tb.event_ref AND sec_event_tb.counter <= ? ))";
    push(@{$plug}, $args{date}, $args{date}, $args{counter});
  }
  elsif (defined $args{date})
  {
    $sql .= " AND lot_node_tb.event_date <= ? ";
    push(@{$plug}, $args{date});
  }
  $sql .= " GROUP BY lot";
  my $sth = $self->dbHandle->read(sql => $sql, plug => $plug);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  if (my @d = $sth->fetchrow_array) { $id = $d[0]; }
  if ($id =~ /\d+/)
  {
    $new->set('id', $id);
    $new->select;
    if ($new->error) { $self->error($new->errorMessage); }
  }
  return $new;
}

=item @LotNode list(account => , stock =>, date =>, counter => )

  required: date
  optional: account, stock, counter
    for now only one account and stock can be sent at a time.

  summary: retrieves the latest lot nodes for each lot based on given search parameters.
    can be narrowed to given stock (subAccount), and/or account and/or date

=cut
sub list
{
  my $self = shift;
  my @return = ();
  my %args = (account => undef, stock => undef, date => undef, counter => undef, @_);
  my $l = $self->get('ledgerCode');
  my @plug = ( $l );
  my $date = $args{'date'};
  my $datesql;
  my $account = "";
  my $subAccount = "";
  if (defined $args{counter})
  {
    $datesql = " lot_node_tb.event_date < ? OR (lot_node_tb.event_date = ? AND sec_event_tb.counter < ?) ";
    @plug = (@plug, $date, $date, $args{counter});
  }
  else
  {
    $datesql = "lot_node_tb.event_date <= ? ";
    @plug = (@plug, $date);
  }
  if (defined $args{account})
  {
    $account = " AND lot_node_tb.account_num = ?";
    @plug = (@plug, $args{account});
  }
  if (defined $args{stock})
  {
    $subAccount = " AND lot_node_tb.sub_account_num = ?";
    @plug = (@plug, $args{stock});
  }

  my $sql = <<"ENDOF";
  SELECT max(lot_node_tb.node_id) AS id
  FROM lot_node_tb, sec_event_tb
  WHERE lot_node_tb.ledger_code = ?
    AND lot_node_tb.event_ref = sec_event_tb.event_ref
    AND (
      $datesql
    )
  $account
  $subAccount
  GROUP BY lot
ENDOF
#$self->error($sql.":".join(",",@plug)); return undef;
  my $sth = $self->dbHandle->read(sql => $sql, plug => \@plug );
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  while (my @id = $sth->fetchrow_array)
  {
    my $node = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $l);
    $node->set('id', $id[0]);
    $node->select;
    if ($node->error) { $self->error($node->errorMessage); }
    elsif($node->get('shares') > 0)
    {
      push(@return, $node);
    }
  }
  return @return;
}

=item string walktree

  outputs a visual repesentation of the entire tree of the given node.  if no node is given it defaults to self.

=cut
sub walktree
{
  my $self = shift;
  my $node = shift;
  if (!defined $node) { $node = $self; }
  my $r;
  $r .= "<div>". $node->get('id')."->p:".$node->get('parent').":".$node->get('subEntity')."\n";
  foreach my $n (keys %{$node->{children}})
  {
    $r .= $self->walktree($node->{children}->{$n});
  }
  $r .= "</div>\n";
  return $r;
}

=item integer delete(commit => 0)

  Routine calls all of the neccessary functions to get rid of this lot and all journal entries affected by it.

=cut
sub delete
{
  my $self = shift;
  my %args = (commit => 0, @_);
  my $j = $self->get('journalRef');
  if ($j > -1)
  {
    #delete the journal entry associated with this lot node
    use Portal::Accounting::Objects::JournalEntry;
    my $je = Portal::Accounting::Objects::JournalEntry->new(ledgerCode => $self->get('ledgerCode'), langObj => $self->langObj, dbHandle => $self->dbHandle );
    if ($je->error) { $self->error($je->errorMessage); return 0; }
    $je->set('id', $j);
    my $result = $je->delete(system => 1);
    if (! defined $result || $result == -1) { $self->error($je->errorMessage); return 0; }
  }
  return $self->SUPER::delete(@_, commit => $args{commit});
}

=item @integer deleteTree()

  for the current LotNode object, calls delete on it and all children to remove data from database.
  Returns an array of the lot nodes deleted.

=cut
sub deleteTree
{
  my $self = shift;
  if (!$self->isValid) { $self->error($self->genErrorString('all') ); return undef; }
  my @nodes = $self->deleteSubTree;
  if (!$self->error)
  {
    $self->delete;
    if (!$self->error)
    {
      return ($self->get('id'), @nodes);
    }
  }
  return @nodes;
}

=item integer deleteSubTree()

  for the current LotNode object, calls delete on and all children to remove data from database.
  Returns an array of the lot nodes deleted.

=cut
sub deleteSubTree
{
  my $self = shift;
  $self->childNodes;  #populate children
  if ($self->error)
  {
    if (defined $self->{children}) { $self->error("<div>".join(",", keys %{$self->{children} })."</div>"); }
    return undef;
  }
  my @deletes;
  foreach my $n (keys %{$self->{children}})
  {
    @deletes = (@deletes, $self->{children}->{$n}->deleteTree);
    if ($self->{children}->{$n}->error) { $self->error($self->{children}->{$n}->errorMessage); }
  }
  return @deletes ;
}

sub select
{
  my $self = shift;
#  $self->error("<p>FOO: ".$self->get('id')."</p>");
  my $r = $self->SUPER::select(@_);
  $self->isValid;
  return $r;
}

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut