# Investments::Search.pm - The Object Class that provides a Investments::Search Object
#$Id: Search.pm,v 1.12 2006/04/17 18:34:38 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Search;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use Portal::Accounting::Investments::Objects::Base;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.12 $';

=head1 NAME

Investments::Search - Object used to build a Investments::Search Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Search;
  my $Investments::SearchObj = Portal::Accounting::Investments::Objects::Search->new(   );
  if ($Investments::SearchObj->error())
  {
    die $Investments::SearchObj->errorMessage();
  }

=head1 DESCRIPTION

Investments::Search is a Investments::Search class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Investments::Objects::Search.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  my @date = Today();
  my $l = $self->{langObj};
  my $all = $l->map('ALL');
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  $acc->set('type',3);
#  my $sacc = Portal::Accounting::Objects::SubAccount->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $st = Portal::Accounting::Investments::Objects::Stock->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Investments::Journal" },
    "command" => { -Type => "hidden", -Value => "search" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "number" => { -Type => "select", -DataType => "string", -Value => 0,
      -Options => { names => [ qw( 10 30 ), $all ], values => [ qw( 10 30 0) ]},
    },
    "page" => { -Type => "hidden", -DataType => "int", -Value => 1,},
    "accounts" => { -Label => $l->map("Account"), -Type => "select", -DataType => "integer",
      -Options => $acc->selectOptionsByType(),
    },
    "stocks" => { -Label => $l->map("Stocks"), -Type => "select", -DataType => "integer",
      -Options => $st->selectOptions(),
    },
    "dateOperator1" => { -Type => "select", -DataType => "string", -Value => '=',
      -Options => { names => [ qw( = < <= > >= ) ],
      values => [ qw( = < <= > >= ) ]},
    },
    "dateOperator2" => { -Type => "select", -DataType => "string", -Value => '=',
      -Options => { names => [ qw( = < <= > >= ) ],
      values => [ qw( = < <= > >= ) ]},
    },
    "from" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] ,
    },
    "to" => { -Label => $l->map("Date"),   -Type => "date-picker", -DataType => "string",
      -year => $date[0] ,
    },
    "lots" => { -Type => "text", -DataType => "int", -Value => '', onfocus => 'this.blur();',},
  };
  $self->{data}->{pager} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "JournalEntry" },
    "command" => { -Type => "hidden", -Value => "search" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "number" => { -Type => "hidden", -DataType => "string", -Value => 0,},
    "page" => { -Type => "hidden", -DataType => "int", -Value => 1,},
    "accounts" => { -Type => "hidden", -DataType => "int",    },
    "stocks" => { -Type => "hidden", -DataType => "int",    },
    "lots" => { -Type => "hidden", -DataType => "int",    },
    "dateOperator1" => { -Type => "hidden", -DataType => "string",    },
    "dateOperator2" => { -Type => "hidden", -DataType => "string",    },
    "from" => { -Type => "hidden", -DataType => "string",    },
    "to" => { -Type => "hidden", -DataType => "string",    },
  };
   $self->{template}->{main} = <<"ENDOF";
<h4>#LANG=Search#</h4>
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=page#
  <table class="primary" cellpadding="5">
    <tr>
      <td>
        #LABEL=from#
        #FIELD=dateOperator1# #FI=from#
      </td>
      <td>
        #LABEL=to#
        #FIELD=dateOperator2# #FI=to#
      </td>
    </tr>
    <tr>
      <td>
        #LABEL=accounts#<br>
        #FI=accounts#
      </td>
      <td>
        #LABEL=stocks#<br>
        #FI=stocks#
      </td>
    </tr>
    <tr>
      <td colspan="2">#LANG=Transactions# #LANG=per page# #FI=number#
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( dateOperator1 dateOperator2 accounts stocks) ],
    optional => [ qw( number page from to accounts lots) ],
    constraints => {
      from => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
      to => qr/^\d{4}-\d{2}-\d{2}$/,
      dateOperator1 => qr/^[<>=]+$/,
      dateOperator2 => qr/^[<>=]+$/,
      accounts => qr/^\d+$/,
      lots => qr/^\d+$/,
      stocks => qr/^\d+$/,
      number => qr/^\d+$/,
      page => qr/^\d+$/,
    },
  };
  $self->{profile}->{pager} = {
    required => [ qw( dateOperator1 dateOperator2 accounts stocks) ],
    optional => [ qw( number page from to accounts lots) ],
    constraints => {
      from => qr/^\d{4}-\d{2}-\d{2}$/,
      to => qr/^\d{4}-\d{2}-\d{2}$/,
      dateOperator1 => qr/^[<>=]+$/,
      dateOperator2 => qr/^[<>=]+$/,
      accounts => qr/^\d+$/,
      lots => qr/^\d+$/,
      stocks => qr/^\d+$/,
      number => qr/^\d+$/,
      page => qr/^\d+$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

sub isNew
{
  return 1;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub selectSQL
{
  my $self = shift;
  #0 will be the count sql and 1 will be the data sql with paging
  $self->{LIST}[0] = "SELECT COUNT(DISTINCT(event_ref)) FROM sec_event_tb WHERE ";
  $self->{LIST}[1] = "SELECT DISTINCT(event_ref), trans_date FROM sec_event_tb WHERE ";
  my $l = $self->get('ledgerCode');
  my $a = $self->get('accounts');
  my $s = $self->get('stocks');
  #buy, sell, adjs and sxfr use account, subaccount format
  #rtcp uses * for ledger
  #--spin off, swap, acqs uses * for ledger and a for second stock
  #--memo uses * OR ledgerCode and only stock is filled in
  my $sql = <<"FOO";
  (
    ledger_code = '$l'
    AND S = $s
    AND A = $a
    AND (
      event_code = 'sell' OR event_code = 'sxfr' OR event_code = 'adjs' OR event_code = 'buy'
    )
  )
  OR (
    (event_code = 'acqs' OR event_code = 'spof')
    AND (S = $s OR A = $s )
  )
  OR (
    (event_code = 'splt' OR event_code = 'rtcp' OR event_code = 'memo' )
    AND S = $s
  )
FOO
  if ($self->get('from') ne '')
  {
    $sql .= " AND calc_date ".$self->get('dateOperator1')." ".$self->get('from');
  }
  if ($self->get('to') ne '')
  {
    $sql .= " AND calc_date ".$self->get('dateOperator2')." ".$self->get('to');
  }
  $self->{LIST}[0] .= $sql;
  $self->{LIST}[1] .= $sql . " ORDER BY trans_date ";
  my $num = $self->get('number');
  if ($num > 0)
  {
    $self->{LIST}[1] .= " LIMIT $num";
    my $page = $self->get('page');
    if ($page > 1)
    {
      $self->{LIST}[1] .= " OFFSET " . ($num * ($page - 1));
    }
  }
  return @{$self->{LIST}};
}

=item boolean selectArray

# requires: startIndex, numEntriesToDisplay, ledgerCode, showEvents
# optional: accounts, stocks, actions, startDate, endDate
# returns: InvestmentJournalViewObject or undef if error
# summary: Builds up the SQL statement to gather SecurityEvents from the database based upon the possible accounts, stocks, actions, and dates to
# search across.  accounts, stocks and actions are comma (,) separated strings of the numbers that represent the various items.  If nothing is specified other
# than the ledgerCode, startIndex and numEntriesToDisplay, then all security event entries will be searched in the specified ledger.  The number of rows
# that would have been returned before limiting will be calculated and then the actual data will be gathered and stored in the InvestmentJournalViewObject.

=cut
sub selectArray
{
  my $self = shift;
  my $event = Portal::Accounting::Investments::Objects::Event->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode') );
  if ($event->error) { $self->error($event->errorMessage); return undef; }
  my @events;
  $self->{dbHandle}->write("SET DATESTYLE='ISO'");

  my @sql = $self->selectSQL;
  my $sth = $self->{dbHandle}->read(sql => $sql[0]);
  if ($self->{dbHandle}->error)
  {
    $self->error($self->{dbHandle}->errorMessage);
    return undef;
  }
  my @info = $sth->fetchrow_array;
  if (! defined $info[0])
  {
    $self->error('<div class="error">No COUNT returned!</div>');
    return undef;
  }
  $self->{count} = $info[0];
  if ($self->{count} == 0)
  {
    # exit early as there is nothing to gather.
    return ();
  }

  # now gather all event_refs of the rows that are supposed to be returned.
  my $sth = $self->{dbHandle}->read(sql => $sql[1]);
#$self->error($sql[1]);
  if ($self->{dbHandle}->error)
  {
    $self->error($self->{dbHandle}->errorMessage);
    return undef;
  }
  # create the Security Event Objects to display
  while (my @info = $sth->fetchrow_array)
  {
    push(@events, $event->getEvent($info[0]) );
  }
  if (scalar @events == 0)
  {
    $self->error("<div class=\"error\">No Events returned but the database claimed '$self->{count}' entries existed!</div>");
    return undef;
  }

  return @events;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
