# Sell.pm - The Object Class that provides a Sell Object
#$Id: Sell.pm,v 1.24 2006/05/12 14:35:45 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Sell;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.24 $';

=head1 NAME

Sell

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Sell;
  my $obj = Portal::Accounting::Investments::Objects::Sell->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  Sell is the class used to manage Sell security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting Sell events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::Sell module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "sell";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  my $Shares = $l->map("Shares");
  my $Price = $l->map("Price");
  my $Total = $l->map("Total");
  my $Comm = $l->map("Commission");
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $today,
      onchange => "fetchShares(this.form);"
    },
    "account" => { -Label => $l->map("Account"), -Type => "select", -DataType => "integer",
      -Options => $self->{accounts},
      -NoSelectByDefault => 1,
      onchange => "fetchShares(this.form);"
    },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks},
      -NoSelectByDefault => 1,
      onchange => "fetchShares(this.form);"
    },
    "lots" => { -Type => "hidden", -DataType => "text" },
    "lot" => { -Type => "text", -ReadOnly => 1, -DataType => 'integer', },
    "master_ref" => { -Type => "text", -ReadOnly => 1, -DataType => 'integer', },
    "counter" => { -Type => "hidden", -DataType => "integer" },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "shares" => { -Type => "text", -DataType => "float",
      onfocus => "this.form.price.focus();",
      tabindex => '-1',
      -Label => $Shares, size => 8,
#      onchange => "setCommission(this); totalCalc(this);"
    },
    "price" => { -Type => "text", -DataType => "float", -Label => $Price, size => 8, onchange => "totalCalc(this);" },
    "commission" => { -Type => "text", -DataType => "float", -Label => $Comm, size => 8, onchange => "setCommShares; totalCalc(this);" },
    "total" => { -Type => "text", -DataType => "float", -Label => $Total, size => 8, onchange => "totalCalc(this);" },
    "commPerShare" => { -Type => "text", -DataType => "float", -Label => $l->map("Commission")." ".$l->map("Per Share"), size => 8, onchange => "setCommission(this); totalCalc(this);" },
    "subEntity" => { -Type => "hidden", -DataType => "int" },
  };
  $self->{data}->{report} = {
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "lot" => { -Type => "text", -ReadOnly => 1, },
    "master_ref" => { -Type => "text", -ReadOnly => 1, -DataType => 'integer', },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly => 1, },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $today, -ReadOnly => 1, },
    "account" => { -Label => $l->map("Account"), -Type => "select", -DataType => "int",
      -Options => $self->{accounts} , -ReadOnly => 1,  },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "subEntity" => { -Label => $l->map("Sub Entity"), -Type => "select", -DataType => "int",
      -Options => $self->{subEntities}, , -ReadOnly => 1,},
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment"), -ReadOnly => 1, },
    "shares" => { -Type => "text", -DataType => "float", -Label => $Shares, -ReadOnly => 1,},
    "price" => { -Type => "text", -DataType => "float", -Label => $Price, -ReadOnly => 1,},
    "commission" => { -Type => "text", -DataType => "float", -Label => $Comm, -ReadOnly => 1,},
    "total" => { -Type => "text", -DataType => "float", -Label => $Total, -ReadOnly => 1, -Value => 0,},
  };

  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FI=app#
  #FI=state#
  #FI=command#
  #FI=ledgerCode#
  #FIELD=id#
  <h4>#LABEL=type##FI=type#</h4>
  <table class="primary">
    <tr>
      <td>#LABEL=date#</td><td>#FI=date#</td>
      <td colspan="2">&nbsp;</td>
    </tr>
    <tr>
      <td>#LABEL=account#</td><td>#FI=account#</td>
      <td>
        #LABEL=stock#</td><td>#FI=stock#
      </td>
    </tr>
    <tr>
      <td>#LABEL=shares#</td>
      <td>
        #FI=lots# <input type="button" name="lotsButton" value="*" onclick="fetchShares(this.form); " title="#LANG=SelectLots#">
        #FI=shares#
      </td>
      <td>#LABEL=price#</td><td>#FI=price#</td>
    </tr>
    <tr>
      <td>#LABEL=commPerShare#</td><td>#FI=commPerShare#</td>
      <td>#LABEL=commission#</td><td>#FI=commission#</td>
    </tr>
    <tr>
      <td>#LABEL=total#</td><td>#FI=total#</td>
      <td>#LABEL=comment#</td><td>#FI=comment#</td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" onclick="this.disabled = true;">
  &nbsp;&nbsp;&nbsp;&nbsp;AutoChange:
    <input type="radio" name="autoChange" value="price">$Price
    <input type="radio" name="autoChange" value="commission" CHECKED>$Comm
    <input type="radio" name="autoChange" value="total">$Total<br>
  <input type="hidden" name="focusField" value="price" />
</form>
<p>#FORMREQUIREDSTRING#</p>
<div id="frame">
  <iframe name="sharesselection" align="top" frameborder="0" width="100%" scrolling="yes" Height="600" >
  </iframe>
</div>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=comment#</td>
    <td class="price">#FIELD=price#</td>
    <td class="shares">#FIELD=shares#</td>
    <td class="cost">#FIELD=total#</td>
    <td class="subEntity">#VALUE=subEntity#</td>
    <td class="lot">#FIELD=lot#</td>
$self->{deleteString}
  </tr>
</table>
ENDOF

  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date shares price total stock account) ],
    optional => [ qw( subEntity commission counter lots lot) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         shares => 'RE_num_real',
         price => 'RE_num_real',
         total => 'RE_num_real',
         stock => qr/^\d+$/,
         account => qr/^\d+$/,
         commission => 'RE_num_real',
         counter => qr/^\d+$/,
         lots => qr/^(\d+=\d+(\.)?(\d+)?)(\;\d+=\d+(\.)?(\d+)?)*$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date shares price total stock account ) ],
    optional => [ qw( subEntity commission counter lots lot) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         shares => 'RE_num_real',
         price => 'RE_num_real',
         total => 'RE_num_real',
         stock => qr/^\d+$/,
         account => qr/^\d+$/,
         commission => 'RE_num_real',
         counter => qr/^\d+$/,
       },
  };

  $self->{INSERTorder}[0] = [ qw( event_code  trans_date  counter  sub_e_code  a  s  l  os  ns  cc  ledger_code  event_ref  comment master_ref ) ] ;  #insert order for db column names

  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    sub_e_code => 'subEntity',
    a => 'account',
    s => 'stock',
    l => 'lot',
    os => 'price',
    ns => 'shares',
    cc => 'commission',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  if ($self->get('commission') eq '') { $self->set('commission', 0.00); }
  # validate our parameters.
  if ($self->get('lots') eq '')
  {
    if ($self->get('lot') eq '') { $self->missing('lot', 'Must specify either lot or lots.'); }
  }
  my $total = sprintf("%.2f", $self->get('total'));
  $self->set('total', $total);
  my $calculate = sprintf("%.2f", $self->get('price') * $self->get('shares') - $self->get('commission'));
  if ($calculate ne $total)
  {
    $self->invalid('total',$self->get('total'), "P * S + C = $calculate != $total");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  This routine selects a record from the database and performs any transforms on the data to fill out the object with fields that are not stored in the database but are needed for display and calculations.

=cut
sub select
{
  my $self = shift;
  my $r = $self->SUPER::select(@_);
  if (!$self->error) { $self->set('total', sprintf("%.02f", $self->get('price') * $self->get('shares') - $self->get('commission')) ); }
  return $r;
}

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.

=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }
  my $totalShares = 0;
  my @lots = split(';', $self->get('lots'));
  my $mRef = $self->get('id');
  my $commission = $self->get('commission');
  my $records = 0;
  my $lot;
  my $shares;
  my @events;
  if (scalar @lots == 0)
  {
    #we are not in an object used for forms.  just work with the data in $self->get('lot')
    $lot = $self->get('lot');
    $shares = $self->get('shares');
    my $counter = $self->nextSecurityEventCounter($self->get('date'));
    my $eventRef = $self->nextSecurityEventRef();
    $self->set('counter',$counter);
    $self->set('id',$eventRef);

    # retrieve the lot balance entry from the lot_node_tb table for the "old" lot
    my $lotbalobj = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode') );
    $lotbalobj = $lotbalobj->balance(date => $self->get('date'), counter => $counter, lot => $lot );
    if (! defined $lotbalobj)
    {
      $self->error('<div class="error">Failed to retrieve lot balance! obj undefined for '.$lot."! </div>");
    }
    elsif ($lotbalobj->error())
    {
      $self->error($lotbalobj->errorMessage);
    }
    elsif ($lotbalobj->get('id') == -1)
    {
      $self->error('<div class="error">No balance found for lot '.$lot." as of '".$self->get('date')."'!</div>");
    }
    elsif ($lotbalobj->get('shares') < $shares)
    {
      $self->error('<div class="error">Lot '."$lot has only ".$lotbalobj->get('shares')." which is less than the shares to transfer: $shares</div>");
    }
    else
    {
      $self->set('stock', $lotbalobj->get('subAccount'));
      $self->set('account', $lotbalobj->get('account'));
      $self->insertEvent($lotbalobj);
      if (!$self->error)
      {
        $self->{dbHandle}->write(
          sql => "INSERT INTO sec_event_tb (event_code, master_ref, trans_date, counter, A, S, L, OS, NS, CC, ledger_code, event_ref, comment, sub_e_code) "
            . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
          plug => [
            $self->get('type'),
            $self->get('master_ref'),
            $self->get('date'),
            $self->get('counter'),
            $self->get('account'),
            $self->get('stock'),
            $self->get('lot'),
            $self->get('price'),
            $self->get('shares'),
            $self->get('commission'),
            $self->get('ledgerCode'),
            $self->get('id'),
            $self->get('comment'),
            $lotbalobj->get('subEntity'),
          ]
        );
        return 1;
      }
    }
    return -1;
  }
  #split the lots into individual events
  foreach my $l (@lots)
  {
    ($lot, $shares) = split('=',$l);
    my $event = Portal::Accounting::Investments::Objects::Sell->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
    $event->populate(
      lot => $lot,
      shares => $shares,
      date => $self->get('date'),
      account => $self->get('account'),
      comment => $self->get('comment'),
      stock => $self->get('stock'),
      price => $self->get('price'),
      commission => $commission,
      total => ($self->get('price') * $shares - $commission),
      master_ref => $mRef,
    );
    if ($event->error) { $self->error($event->errorMessage); last; }
    $event->insert;
    if ($event->error) { $self->error($event->errorMessage); last; }
    push(@events, $event);
    $totalShares += $shares;
    if ($commission != 0) { $commission = 0; }  #only use the commission on the first entry
  }
  if (!$self->error && $self->get('shares') != $totalShares)
  {
    $self->error("<div>shares do not match<br>".$self->get('shares')." != $totalShares</div>");
  }

  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->dbHandle->rollback; }
  return $mRef;
}

=item boolean propagate(LotNode)

  See Investments::Objects::Event::propagate

=cut
sub propagate
{
  my $self = shift;
  my $lotNode = shift;
  my $date = $self->get('date');
  my $lc = $self->get('ledgerCode');
  #calculate new lot node data from old lot and data from this eventRef

  # use the node's values as the current balance of cost and shares for the lot
  # calculate the sell value and the sell cost to determine if the sell results in a loss or a gain

  my $capitalGainAccount;
  my $capitalLossAccount;
  #initialize vars as if all of this node will be sold
  my $sharesLeft = 0;
  my $costBalance = 0.00;
  my $sellCost = sprintf("%.2f", $lotNode->get('cost') );
  if ($self->get('shares') > $lotNode->get('shares'))
  {
    $self->error('<div class="error">Trying to sell more shares ('.$self->get('shares').') than available ('.$lotNode->get('shares').') in lot "'.$lotNode->get('lot').'"</div>');
    return undef;
  }
  elsif ($self->get('shares') < $lotNode->get('shares')) #calculate new balances for new node
  {
    $sellCost = sprintf("%.2f", $lotNode->get('cost') / $lotNode->get('shares') * $self->get('shares') );
    $sharesLeft = $lotNode->get('shares') - $self->get('shares');
    $costBalance = $lotNode->get('cost') - $sellCost;
  }
  my $sellValue = sprintf("%.2f", ($self->get('price') * $self->get('shares') ) - $self->get('commission'));
  my $gainLoss = sprintf("%.2f", abs($sellValue - $sellCost));

  # determine if the sell is occurring after one year that the security has been held in order to assign long or short
  # term gains or losses
  my $buydate;
  my $sql = "SELECT MIN(buy_date) from sec_event_tb WHERE l = (SELECT buy_lot FROM lot_node_tb WHERE node_id = ? LIMIT 1) AND buy_date IS NOT NULL";
  my $sth = $self->{dbHandle}->read(sql => $sql, plug => [$lotNode->get('id')] );
  if ($self->{dbHandle}->error())
  {
    $self->error($self->{dbHandle}->errorMessage);
    return 0;
  }
  my @info = $sth->fetchrow_array;
  $buydate = $info[0];

  my @ltst = $self->shortTermLongTerm($date, $buydate);
  $capitalGainAccount = $ltst[0];
  $capitalLossAccount = $ltst[1];

  my $transobj = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $lc);
  $transobj->set('journal', $self->{Journal}); # hardcoded investments journal code
  $transobj->setDate($date);
  $transobj->addRow(
    subEntity => $lotNode->get('subEntity'),
    account => $self->get('account'),
    subAccount => $self->get('stock'),
    dorc => "false",
    amount => sprintf("%.2f", $sellCost),
    comment => $self->get('comment'),
  );
  $transobj->addRow(
    subEntity => $lotNode->get('subEntity'),
    account => $self->get('account'),
    dorc => "true",
    amount => sprintf("%.2f",$sellValue),
    comment => $self->get('comment'),
  );
  my $account = $capitalGainAccount;
  my $dorc = 'false';
  if ($sellValue < $sellCost)
  {
    # sold for less than what it cost; loss (use capital loss account)
    $account = $capitalLossAccount;
    $dorc = 'true';
  }
  $transobj->addRow(
    subEntity => $lotNode->get('subEntity'),
    account => $account,
    subAccount => $self->get('stock'),
    dorc => $dorc,
    amount => sprintf("%.2f",$gainLoss),
    comment => $self->get('comment'),
  );
  if ($transobj->error)
  {
    $self->error($transobj->errorMessage);
    return 0;
  }
  # insert the transaction into the database
  my $refNum = $transobj->insert(system => 1);
  if ($transobj->error())
  {
    $self->error($transobj->errorMessage);
    return 0;
  }

  # insert a new lot node into the database
  my $newLotNodeObj = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $lc);
  $newLotNodeObj->populate(
    date => $self->get('date'),
    parent => $lotNode->get('id'),
    event => $self->get('id'),
    journalRef => $transobj->get('id'),
    subEntity => $lotNode->get('subEntity'),
    account => $self->get('account'),
    subAccount => $self->get('stock'),
    lot => $lotNode->get('lot'),
    buyLot => $lotNode->get('buyLot'),
    cost => sprintf("%.2f", $costBalance),
    shares => sprintf("%.6f", $sharesLeft),
  );
  if ($newLotNodeObj->error) { $self->error($newLotNodeObj->errorMessage); return undef; }

  #old node tree was deleted in Event->insert
  $newLotNodeObj->insert;
  if ($newLotNodeObj->error) { $self->error($newLotNodeObj->errorMessage); return undef; }

  return $self->SUPER::propagate($newLotNodeObj);
}

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.

=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut