# Portfolio.pm - The Object Class that provides a Portfolio Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Portfolio.pm,v 1.24 2006/04/13 21:33:07 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Portfolio;
use strict;
use HTMLObject::Widgets;
use Portal::Accounting::Investments::Main;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Investments::Objects::Portfolio;
use POSIX qw(tmpnam);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.24 $';

# global variables.
# modify to represent the commands this state provides.
my %commands;
%commands = (
  display => "Portfolio",
  search => "Portfolio",
  searchForm => "Portfolio",
  execute => "Portfolio",
  update => "Portfolio",
  print => "Portfolio",
  executeFile => "Portfolio"
);

=head1 NAME

Portfolio - Object used to build a Portfolio Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Portfolio;
  my $obj = Portal::Accounting::Portfolio->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Portfolio is a Portfolio class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::Portfolio module.
 See Portal::AppState(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;
  $self->{widgets} = HTMLObject::Widgets->new();
  if ($self->{widgets}->error)
  {
    $self->error($self->{widgets}->errorMessage);
    return $self;
  }

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_display
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Investments::Objects::Portfolio->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  return $self->SUPER::c_searchForm(@_);
}

sub c_searchForm
{
  my $self = shift;
  my $doc;
  my %args = ( command => $self->{input}->{command}, doc => undef, @_ );
  if (!defined $args{doc}) { $doc = $self->prepDoc; }
  my $command = $args{command};
  my $l = $self->{langObj};
  my $foo = Portal::Accounting::Investments::Objects::Portfolio->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode =>
$self->{input}->{ledgerCode}, imageDir => $self->{imageDir});
  $self->{OBJECT} = $foo;
  if ($foo->error) { $doc->print($foo->errorMessage); return $doc;  }
  if ($command eq "search")
  { #validate user input then react
    if ($foo->populate(%{$self->{input}}) && $foo->validateForm(input => $self->{input}, formName => "main"))
    {
#       $doc->print('<div id="xmlDiv">');
#       $doc->print("\n".'<textarea id="stocks" rows="20" cols="150" style="width: 100%;">'.$foo->printXML."</textarea>\n");
#       $doc->print('</div>');
      if ($foo->get('format') eq 'csv')
      {
        $doc = HTMLObject::Normal->new();
        $doc->setContentType('text/plain');
      }
      $doc->print($foo->printReport());
      my $bar = $foo->debugString;
      if ($bar ne '')
      {
        $doc->print($bar);
        $doc->print("<div>$foo->{SELECT}[0]</div>\n");
        $doc->print("<div>$foo->{SELECT}[1]</div>\n");
      }
      if ($foo->error) { $doc->print($foo->errorMessage); }
      else
      {
        return $doc;
      }
    }
  }
  #display form
  my %form = $foo->printForm();
  if ($foo->error)
  {
    $self->error($foo->errorMessage);
  }
  if (defined %form)
  {
    $doc->print(%form);
  }
  else
  {
    $doc->print('error! form is undefined<br>');
    $doc->print("\nformObj:".$foo->{formObj}->error);
  }

  $doc->print($foo->debugString);
  return $doc;
}

sub c_search
{
  my $self = shift;
  return $self->c_searchForm(@_);
}

1;
__END__

=back

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

JT at Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
