# Base.pm - The Object Class that provides a Base Object
#$Id: BaseL.pm,v 1.7 2006/03/24 21:43:54 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::BaseL;
use strict;
use Portal::Accounting::Objects::Base;;

use vars qw($AUTOLOAD $VERSION @ISA @EXPORT %Elements);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.7 $';
%Elements = ();

=head1 NAME

BaseL - Base Object for deriving other objects. Derives from Objects::Base but also checks for ledgerCode.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::BaseL;
  @ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);

=head1 DESCRIPTION

BaseL is a Base class for Accounting Objects such as
Objects::SubEntity.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Base.
    See Portal::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", @_ );
  $self->{name} = "Portal::Accounting::Objects::BaseL";
  if ($args{ledgerCode} eq "") { $self->error("<p>LedgerCode must be specified for this object!</p>\n");  }
  if ($self->error)
  {
    return $self;
  }

  # instantiate anything unique to this module
  $self->{data}->{main} = {
    ledgerCode => { -Type => "hidden", -DataType => "string", -Value => $args{ledgerCode} }
  };
  # dont do validation because most objects will start as an empty template
  # do anything else you might need to do.
  return $self;
}

=item bool isValid(void)

 Returns 0 for invalid or 1 for valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub printList
{
  my $self = shift;
  return $self->SUPER::printList(ledgerCode => $self->get("ledgerCode"));
}

sub selectArrayHash
{
  my $self = shift;
  my $which = shift;
  my $object = ref($self);
  if (defined $self->{object} && $self->{object} ne '') { $object = $self->{object}; eval("use $object;"); }
  if ($which ne "array" && $which ne "hash") { $self->error("<p>Invalid option give to selectArrayHash:$which</p>\n"); return undef; }
  my $return;
  if (! defined $self->{LIST}) { $self->error("<p>LIST sql not defined but subroutine is trying to use it</p>\n"); return undef; }
  my ($sqls, $plugs) = $self->substitute(sql => "LIST");

  my $result = $self->{dbHandle}->read($sqls->[0]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef;  }
  while (my @d = $result->fetchrow_array)
  {
    my $o;
    my $e = '$o = '. $object .'->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => "'.$self->get('ledgerCode').'", @_);' ;
    eval($e);
    if (!defined $o) { $self->error("<p>Could not create object needed of type: ".$object."<br>$e</p>\n");  return undef;  }
    if ($o->error) { $self->error("<p>Could not create object needed of type: ".$object."<br>$e<br>".$o->errorMessage."</p>\n");  return undef;  }
    $o->set($o->{id}, $d[1]);
    if (! $o->error)  {  $o->select;  }
    if ($o->error) { $self->error($o->errorMessage);  return undef; }
    if ($which eq "hash")
    {
      $return->{$d[1]} = $o;
    }
    else
    {
      push @{$return}, $o;
    }
  }
  if ($which eq "hash")
  {
    if (not defined $return) { return (); }
    return %{$return};
  }
  if (not defined $return) { return (); }
  return @{$return};
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Base(3)

=cut
