#$Id: Base.pm,v 1.15 2006/05/31 19:30:14 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::JournalEntry::Base;

use strict;
use Portal::Accounting::Objects::JournalEntry;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::JournalEntry Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.15 $';
=head1 NAME

JournalEntry::Base - Object used as a parent for data entry screen objects.

=head1 SYNOPSIS

  use JournalEntry::Base;
  @ISA = qw(Portal::Accounting::Objects::JournalEntry::Base Exporter AutoLoader);

=head1 DESCRIPTION

JournalEntry::Base is a class used to derive other classes for data entry screens
It provides a container for a common data and methods across all screens.

=head1 Exported FUNCTIONS

=item scalar new(langObj =>, ledger => )

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  $self->{javascript}->{main} .= <<"ENDOF";
  langObj['acctWarning']      = "#LANG=acctWarning#";
  langObj['Activating']       = "#LANG=Activating#";
  langObj['already exists']   = "#LANG=already exists#";
  langObj['areUsure']         = "#LANG=areUsure#";
  langObj['CannotCreateSubs'] = "#LANG=CannotCreateSubs#";
  langObj['comment']          = "#LANG=comment#";
  langObj['Code']             = "#LANG=Code#";
  langObj['codePrompt']       = "#LANG=codePrompt#";
  langObj['Create SubEntity'] = "#LANG=Create SubEntity#";
  langObj['Delete all rows?'] = "#LANG=Delete all rows?#";
  langObj['Delete row']       = "#LANG=Delete row#";
  langObj['differentJournal'] = "#LANG=differentJournal#";
  langObj['differentLedger']  = "#LANG=differentLedger#";
  langObj['for this session'] = "#LANG=for this session#";
  langObj['Insert Failed']    = "#LANG=Insert Failed#";
  langObj['Insert']           = "#LANG=Insert#";
  langObj['Invalid']          = "#LANG=Invalid#";
  langObj['Invalid Account']  = "#LANG=Invalid Account#";
  langObj['Invalid Date']     = "#LANG=Invalid Date#";
  langObj['Invalid Page']     = "#LANG=Invalid Page#";
  langObj['is inactive']      = "#LANG=is inactive#";
  langObj['journalChange']    = "#LANG=journalChange#";
  langObj['MetaPrompt']       = "#LANG=MetaPrompt#";
  langObj['minimumRows']      = "#LANG=minimumRows#";
  langObj['Name']             = "#LANG=Name#";
  langObj['namePrompt']       = "#LANG=namePrompt#";
  langObj['New Account']      = "#LANG=New Account#";
  langObj['New Sub Account']  = "#LANG=New Sub Account#";
  langObj['noTransaction']    = "#LANG=noTransaction#";
  langObj['notExist']         = "#LANG=notExist#";
  langObj['numbers only']     = "#LANG=numbers only#";
  langObj['reconciledRow']    = "#LANG=reconciledRow#";
  langObj['rightClick']       = "#LANG=rightClick#";
  langObj['row']              = "#LANG=row#";
  langObj['rowWarning']       = "#LANG=rowWarning#";
  langObj['Stop checking?']   = "#LANG=Stop checking?#";
  langObj['subAccountAssignWarning'] = "#LANG=subAccountAssignWarning#";
  langObj['Success']          = "#LANG=Success#!";
  langObj['taxPrompt']        = "#LANG=taxPrompt#";
  langObj['Taxable']          = "#LANG=Taxable#";
  langObj['Type']             = "#LANG=Type#";
  langObj['typePrompt']       = "#LANG=typePrompt#";
  langObj['Update']           = "#LANG=Update#";
  PAYEESTRING                 = "#LANG=Description#";

function submitWrapper()
{
  if (checkPage()) { return submitData(); }
  alert(langObj['Invalid Page']);
  return false;
}
ENDOF

  $self->{css}->{main} = <<"ENDOF";
.editByCheck { display: none; }
ENDOF

  $self->{headingDiv} = <<"END";
      <div class="heading">
        <div class="subEntity"><span title="#LANG=SubEntity#">#LANG=SE#</span></div>
        <div class="account">#LANG=Account#</div>
        <div class="subAccount">&nbsp;#LANG=Sub Account#</div>
        <div class="amount" style="text-align: center;">#LANG=Amount#</div>
        <div class="comment">#LANG=Comment#</div>
        <div class="delete" title="#LANG=Delete#">X</div>
        <div style="clear: both;"></div>
      </div>
END
  $self->{buttonsDiv} = <<"END";
      <div style="clear: both; padding-top: 1em; ">
        <input style="float:left;" name="submitDataButton" onclick="return submitWrapper();" value="#LANG=Insert#" type="button">
        <input style="float:left;" onclick="javascript: return addRow(this.form.view.value,'insertRowHere');" name="add" value="#LANG=Add Row#" type="button">
        <div style="float:right;" style="text-align:right;">
          <a title="#LANG=editHelp#">#</a>
          <input type="text" name="editID" size="4" title="#LANG=editHelp#" />
          <input type="button" name="edit" Value="#LANG=Edit# By ID" onclick="document.main.id.value=document.main.editID.value; init(); " title="#LANG=editHelp#" />
        </div>
        <div class="editByCheck" style="float:right; " style="text-align:right;">
          <input type="text" name="dorc" id="dorc" />
          <input type="text" id="bank" />
          <a title="#LANG=editHelp#">#</a>
          <input type="text" name="editIDCheck" size="4" title="#LANG=editHelp#" />
          <input type="button" name="edit" Value="#LANG=Edit# by Check" onclick="editTransaction(false, this.form.editIDCheck.value); " title="#LANG=editHelp#" />
        </div>
        <div style="text-align: center; ">
          <input type="button" value="#LANG=Reset#" onclick="return resetForm();" />
          #LABEL=id#:<span id="refNumSpan">&nbsp;</span>
        </div>
      </div>
END
  $self->{optionsDiv} = <<"END";
  <div style="margin: 1em; clear: both;">
    <div id="optionsA" name="optionsA" >
      <input type="button" name="hideA" value="#LANG=Show Options# &gt;&gt;" onclick="hideDiv('optionsA'); showDiv('optionsB');">
    </div>
    <div id="optionsB" name="optionsB" class="secondary" style="display: none;" >
      <input type="button" name="hideB" value="&lt;&lt;#LANG=Hide#" onclick="hideDiv('optionsB'); showDiv('optionsA');"><br>
      <p><input type="checkbox" name="confirmDelete" checked="1"> #LANG=Confirm Deletes#?</p>
      <p>#LANG=When creating new rows#:<br>
      #LANG=Show accounts by#
      <select name="accountSwitch">
        <option value="name">Names</option>
        <option value="code">Codes</option>
      </select>
      <br>
      #LANG=Show subaccounts by#
      <select name="subAccountSwitch">
        <option value="name">Names</option>
        <option value="code">Codes</option>
      </select>
      </p>
    </div>
  </div>
END
  $self->{requiredDivs} = <<"END";
<div id="pleaseDiv" style="display: none; text-align: center;">
  #LANG=Please Wait#
  <img src="$self->{imageDir}/hourglass.gif" />
</div>
<div id="popupDiv" style="display: none;">
  <form name="popupForm">
    <input type="hidden" name="what" />
    <input type="button" name="popupFinished" value="#LANG=Cancel#" onclick="cancel(); myFocus(this.form.what.value); " />
  </form>
  <iframe id="popupFrame" name="popupFrame" src="" style="width: 100%; height: 30em; border: 0; " ></iframe>
</div>
<div id="newDiv" style="display: none; text-align: center; background-color: #008f00; color: #ffffff;">
  #LANG=New#
</div>
<div id="editDiv" style="display: none; text-align:center; background-color: #a10048; color: #ffffff; ">
  #LANG=Edit#
</div>
END
  my $close = $self->get('lastClose');
  if ($close ne '') { $self->{requiredDivs} .= "<div style=\"display: none;\" id=\"lastClose\">$close</div>\n"; }
  return $self;
}

=item %hash printForm()

=cut
sub printForm
{
   my $self = shift;
   my %form = $self->SUPER::printForm(@_, name => 'main');
   $form{javascript} = $self->langSubstitute($self->{javascript}->{main});
   $form{css} = $self->{css}->{main};
   return %form;
}

=item string PAYEES(journal =>, style =>, days =>)

  This routine creates a javascript hash using payees as keys
  and transaction id (refNum) as values.  Mostly this is
  for use in a recall searchbox. It will accept an optional
  journal Code (G, CP, CSR, etc) to limit the payees.
  Also the days parameter will limit the search to the most
  recent transactions.  Defaults to 365.

  optional Styles: G, memo

=cut
sub PAYEES
{
  my $self = shift;
  my %args = (days => 365, style => 'G', journal => '', @_);
  my %sql;
  my $l = $self->get("ledgerCode");
  my $r = '';
  my $days = $args{days};
  if ($days !~ /^\d+$/) { $days = 365; }
  my $jW = "";
  my $j = $args{journal};
  if ($j ne "" && exists $self->{journals}->{$j})
  {
    $jW = " AND j.journal = '$j'";
  }
  $sql{G} = <<"SELECT";
  SELECT
   j.payee_payer AS payee,
   j.ref_num AS id
  FROM journal_entry_tb AS j
  WHERE j.ledger_code = '$l' AND j.payee_payer <> '' AND j.remember = true AND j.trans_date + $days > NOW() $jW
   UNION SELECT
     j.payee_payer AS payee,
     MAX(j.ref_num) AS id
  FROM journal_entry_tb AS j
  WHERE j.ledger_code = '$l' AND j.payee_payer <> '' AND j.trans_date + $days > NOW() $jW
  GROUP BY payee ORDER BY payee
SELECT
  $sql{memo} = <<"SELECT";
 SELECT
   j.ref_num AS id,
   '*'|| j.payee_payer || '   ('|| (SELECT comment FROM debitcredit_tb WHERE ref_num = parent_num AND counter = 0)||')' AS payee
  FROM journal_entry_tb AS j
  WHERE j.ledger_code = '$l' AND j.payee_payer <> '' AND j.remember = true AND j.trans_date + $days > NOW() $jW
 UNION SELECT
    MAX(j.ref_num) AS id,
    j.payee_payer || '    (' || debitcredit_tb.comment || ')' AS payee
  FROM journal_entry_tb AS j, debitcredit_tb
  WHERE j.payee_payer <> '' AND j.trans_date + $days > NOW() $jW
    AND debitcredit_tb.parent_num =
      (SELECT max(ref_num) FROM journal_entry_tb WHERE payee_payer = j.payee_payer)
    AND debitcredit_tb.counter = 0
  GROUP BY j.payee_payer, debitcredit_tb.comment
 ORDER BY payee
SELECT

  if (not defined $sql{$args{style}}) { $args{style} = "G"; }
#  $r .= "/*".$sql{$args{style}}."*/\n";
  my $sth = $self->{dbHandle}->read($sql{$args{style}});
  if ($self->{dbHandle}->error)
  {
    $self->error($self->{dbHandle}->errorMessage."for code: $j");
    return undef;
  }
  while (my $foo = $sth->fetchrow_hashref)
  {
    $r .= "PAYEES[\"$foo->{payee}\"] = '$foo->{id}';\n";
  }
  return $r."/*\n".$sql{$j}."\n*/\n";
}

1;
__END__

=pod

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1), JournalEntry::Base(3), DBIWrapper(3)

=cut
