#$Id: Check.pm,v 1.16 2006/05/31 18:59:35 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

Check - Object used to hold forms for cash purchases view (Check)

=head1 SYNOPSIS

  use Portal::Accounting::Objects::JournalEntry::Check;
  my $transobj = Portal::Accounting::Objects::JournalEntry::Check->new( . . .);
  if ($transobj->error())
  {
    die $transobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $transobj->set("id", "1234");
  my $form = $transobj->printForm();

=head1 DESCRIPTION

Check is a class that stores data, methods, and forms for a
cash purchases data entry view.  It derives from JournalEntry::Base
so it provides methods for writing to the database or updating an existing record in the database.
It's main purpose is to generate a Check like form for the user to enter data into the system.

=head1 Exported FUNCTIONS

=cut

package Portal::Accounting::Objects::JournalEntry::Check;

use strict;
use Portal::Accounting::Objects::JournalEntry::Base;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::SubEntity;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::JournalEntry::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.16 $';

=item scalar new(langObj =>, dbHandle =>, ledgerCode =>)

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => "", @_);
  my $act = Portal::Accounting::Objects::Account->new(@_);
  my $sact = Portal::Accounting::Objects::SubAccount->new(@_);
  my $se = Portal::Accounting::Objects::SubEntity->new(@_);
  if ($act->error) { $self->error($act->error); }
  if ($sact->error) { $self->error($sact->error); }
  if ($se->error) { $self->error($se->error); }
  my $l = $self->{langObj};

  # use .= for javascript bc the langObj mappings are in base class
  $self->{javascript}->{main} .= $self->PAYEES(days => 400, journal => 'CP') . "\n";
  $self->{javascript}->{main} .= $act->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $sact->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $se->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
#  $self->{javascript}->{main} .= "DEBUG = true;\n";

  #use form object to get calculator, date and some non repeating fields
  $self->{data}->{main} = {
    "id" => { -Label => $l->map("Id"), -Type => "hidden", -Value => "-1", -DataType => "int", },
    "view" => { -Type => "hidden", -Value => "CP", -DataType => "string" },
    "journal" => { -Type => "hidden", -Value => "CP", -DataType => "string", },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Journal" },
    "command" => { -Type => "hidden", -Value => "display" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "dorc" => { -Type => "hidden", -Value => "false", -DataType => "int" },
    "bank" => { -Type => "hidden", -DataType => "string" },
    "date" => { -Type => "date-picker", size => "7", -Label => $l->map("Date"),
      -Value => $self->getCurrentDate(),
      -year => $self->{data}->{main}->{workingYear}->{-Value} ,
    },
    "payee" => { -Type => "searchBox", -Label => $l->map("Description"), class => 'payee',
      -shareData => "PAYEES", -isAA => 1, -isSorted => 0,
      -onblur => 'if (RECALL && payeeChange() && checkPayee()) { recallData(); }' ,
      -onload => 'init();',
    },
    "memo" => { -Type => "hidden", -DataType => "string", },
    "remember" => { -Type => "checkbox", },
    "subEntity_0" => { -Type => "hidden", -Label => $l->map("Sub Entity"), },
    "subEntity_Display0" => { -Label => $l->map("Sub Entity"),
      -Type => "text", -DataType => "int", style => 'width: 3em;',
      -onload => <<"JS",
  result = searchBox(document.main.subEntity_Display0,
    document.main.subEntity_0,
    SUBENTITIES,
    false, true, true, false,
    '',
    'RECALL = false;   if (! subEntityCheck(document.main.subEntity_0)) { document.main.subEntity_Display0.select(); }'
  );
JS
      },
    "account_0" => { -Type => "hidden", -Label => $l->map("Account"), },
    "account_Display0" => { -Label => $l->map("Account"),
      -Type => "text", -DataType => "int",
      -onload => <<"JS",
  result = searchBox(
    document.main.account_Display0
    , document.main.account_0
    , BANKACCOUNTS
    , false, true, true, true
    , ''
    , 'if (!bankAccountCheck(0)) { alert(langObj["mustChoose"]); } setCheckNumber(0); '
  );
JS
      },
    "subAccount_Display0" => { -Type => "text", -Label => $l->map('Check Number'), style => 'text-align: right; width: 3em; ', onchange => 'this.form.subAccount_0.value = "="+this.value;' },
    "subAccount_0" => { -Type => "hidden"},
    "comment_0" => { -Type => "text", -Label => $l->map('Memo'), class => 'memo', maxlength => "255" },
    "amount_0" => { -Type => "text", -Label => $l->map('Amount'), style => "width: 5em;",
      onchange => "RECALL = false; this.value = floatify(this.value,2); convertMoneyToText(this.value); if (!twoRowCheck(0) ) { lastRowCheck(); }",
    },
  };
  $self->{css}->{main} .= <<"ENDOF";
    .row { clear: both; }
    .heading { font-style: bold; }
    .account { border: none; width: 10em; float: left; margin:0; margin-right: 1px; }
    .subAccount { border: none; width: 10em; float: left; margin: 0; margin-right: 1px; margin-bottom: 1px;}
    .subEntity { border: none; width: 3em; float: leftt; margin: 0; margin-right: 1px;}
    .amount { border: none; width: 7em; text-align:right; float: left; margin: 0; margin-right: 1px;}
    .subEntity { border: none; float: left; margin: 0; margin-right: 1px;}
    .comment { border: none; width: 20em; float: left; margin: 0; margin-right: 1px;}
    .memo { width: 20em; }
    .payee { width: 15em; }
    .delete {float: left; text-align: center; }
    .padme { padding: .5em; }
    #optionsB { padding: 1em; }
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<div id='CP' style="display: none;" class="row">
  <div class="subEntity">
    <input type="text" id="subEntity_Display" name="subEntity_Display" value="" class="subEntity" onChange="clearMe(this);" />
  </div>
  <div style="float: left;">
    <input type="text" id="account_Display" name="account_Display" value="" class="account" onChange="clearMe(this); " />
  </div>
  <div class="subAccount">
    <input type="text" id="subAccount_Display" name="subAccount_Display" value="" class="subAccount" onChange="clearMe(this); " />
  </div>
  <div class="amount">
    <input type="text" id="amount_" name="amount_" value="" class="amount" onChange="clearMe(this); RECALL = false; calculateFormula(this, false); this.value = floatify(this.value,2); lastRowCheck();" />
  </div>
  <div class="comment">
    <input type="text" id="comment_" name="comment_" value="" class="comment" maxlength="255" />
  </div>
  <div class="delete">
    <input type="button" id="delete_" name="delete_" value="X" onclick="return deleteRow(this);" style="border: none;" />
  </div>

  <input type="text" id="account_"    name="account_"    class="debug account" />
  <input type="text" id="subAccount_" name="subAccount_" class="debug subAccount" />
  <input type="text" id="subEntity_"  name="subEntity_"  class="debug subEntity" />
  <input type="hidden" id="cleared_" name="cleared_" value="_" style="clear: both;">
  <div style="clear:both;"></div>
</div>

$self->{requiredDivs}

<div id="mainDiv">
  <form #FORMARGS#>
    #FIELD=id#
    #FIELD=view#
    #FIELD=app#
    #FIELD=state#
    #FIELD=command#
    #FIELD=ledgerCode#
    #FIELD=journal#
    #FIELD=memo#
    <input type="hidden" name="cleared_0" value="_" />
    <div class="ledgerBGColor" style="padding: 5px; margin: .5em; width: 50em; "><div style="border-style: double; border-width: 6px;">
      <div class="padme" style="float: left;">
          #LABEL=account_0#:
          #FI=account_0##FI=account_Display0#
      </div>
      <div class="padme"  style="float: right;">
          <span onclick="setCheckNumber(0, true);" title="Click Here to autofill">#</span>:&nbsp;#FI=subAccount_0# #FI=subAccount_Display0#
      </div>

      <div class="padme"  style="clear: both; float: right;">
        #LABEL=date#:&nbsp;#FI=date#
      </div>
      <div class="padme"  style="clear: both; float: left;">
          <small>#LANG=Pay to the order of#</small>&nbsp;#FI=payee#
      </div>
      <div class="padme"  style="float: right; text-align: right;">
          \$&nbsp;#FI=amount_0#
      </div>
      <div class="padme" id="textAmount" style="clear: both;" >
        &nbsp;
      </div>
      <div class="padme"  style="clear: both; float: left;">
        #LABEL=comment_0#:&nbsp;#FI=comment_0#
      </div>
      <div style="float: right;" title="#LABEL=remember#">#LANG=R# #FI=remember#</div>
      <div class="padme"  style="float: right;">
        #LABEL=subEntity_0#
        #FI=subEntity_0##FI=subEntity_Display0#
      </div>
      <div style="clear: both;"></div>
    </div></div>
    <div class="primary">

$self->{headingDiv}

      <div id="insertRowHere"></div>

$self->{buttonsDiv}

      <div style="clear: both;"></div>
    </div>

$self->{optionsDiv}

  </form>
</div>

<div class="tertiary" style="padding: .5em;">
  <span style="font-weight: bold; border-bottom-style: solid; border-weight: thin;">#LANG=Help#</span>
  #LANG=checkHelp#
  #LANG=otherTips#
</div>
ENDOF
  if ($act->error) { $self->error($act->errorMessage); }
  if ($sact->error) { $self->error($sact->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }

  $self->{profile}->{main} = {
    required => [  ],
    optional => [  ],
    constraints => {
    },
  };
  $self->{defaultForm} = "main";

  return $self;
}

=item boolean isValid

  This routine will always return true because the double entry
  object is just a placeholder for the screen used to edit and
  enter a double entry transaction.

=cut
sub isValid
{
  return 1;
}

1;
__END__

=pod

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

JT Moree

=head1 SEE ALSO

perl(1), JournalEntry(3), DBIWrapper(3)

=cut
