#$Id: DoubleEntry.pm,v 1.12 2006/05/17 01:35:54 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

JournalEntry::DoubleEntry - Object used to hold forms for JournalEntry::DoubleEntry and manipulate a DE transaction

=head1 SYNOPSIS

  use JournalEntry::DoubleEntry;
  my $transobj = JournalEntry::DoubleEntry->new(langObj =>);
  if ($transobj->error())
  {
    die $transobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $transobj->set("id", "1234");
  my $form = $transobj->printForm();

=head1 DESCRIPTION

JournalEntry::DoubleEntry is a class that stores data, methods, and forms for a
General Journal double entry view.  It derives from JournalEntry so it provides methods
to write to the database or update an existing record in the database.
It's main purpose is to generate a Double Entry form for the user to enter data into the system.

=head1 Exported FUNCTIONS

=cut

package Portal::Accounting::Objects::JournalEntry::DoubleEntry;

use strict;
use Portal::Accounting::Objects::JournalEntry::Base;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::SubEntity;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::JournalEntry::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.12 $';

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => "", @_);
  my $act = Portal::Accounting::Objects::Account->new(@_);
  my $sact = Portal::Accounting::Objects::SubAccount->new(@_);
  my $se = Portal::Accounting::Objects::SubEntity->new(@_);
  if ($act->error) { $self->error($act->error); }
  if ($sact->error) { $self->error($sact->error); }
  if ($se->error) { $self->error($se->error); }
  my $l = $self->{langObj};
  my $y = $self->{ledger}->workingYear;
  if ($self->{ledger}->error) { $self->error($self->{ledger}->error); }
  #use form object to get calculator, date and some non repeating fields
  $self->{data}->{main} = {
    "id" => { -Label => $l->map('Transaction'), -Type => "hidden", -Value => -1, -DataType => "int", },
    "view" => { -Type => "hidden", -Value => "G", -DataType => "string" },
    "journal" => { -Type => "hidden", -Value => "G", -DataType => "string", },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Journal" },
    "command" => { -Type => "hidden", -Value => "display" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "date" => { -Type => "date-picker", size => "7", -Label => $l->map("Date"),
      -Value => $self->getCurrentDate(),
      -year => $y,
    },
####calculator fields do not get onchange code that is required for this screen to work.  until JAMES fixes it, cannot use them directly.  leave these here as placeholders
    "debit_" => { -Type => "calculator", -Label => $l->map("Debit"), class => "amount",
      -WidgetOptions => { calcButton => 0, undoButton => 0, helpLink => 0 },
      onchange => "RECALL = false; clearMe(this); this.value = floatify(this.value,2); debitCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }",
    },
    "credit_" => { -Type => "calculator", -Label => $l->map("Credit"), class => "amount",
      -WidgetOptions => { calcButton => 0, undoButton => 0, helpLink => 0 },
      onchange => "RECALL = false; clearMe(this); this.value = floatify(this.value,2); creditCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }",
    },
    "payee" => { -Type => "searchBox", -Label => $l->map("Description"),
      -shareData => "PAYEES", -isAA => 1, -isSorted => 0, class => "ledgerBorderColor",
      -onblur => 'if (RECALL && payeeChange() && checkPayee()) { recallData(); }' ,
      -onload => 'init(document.main.id.value);',
    },
    "remember" => { -Type => "checkbox", },
    "memo" => { -Type => 'text', -DataType => 'string', -Label => 'Memo', class => 'memo', },
  };
  $self->{css}->{main} .= <<"ENDOF";
    .row { clear: both; }
    .heading { font-style: bold; margin-top: 1em; }
    .account { border: none; width: 10em; float: left; margin:0; margin-right: 1px; }
    .subAccount { border: none; width: 10em; float: left; margin: 0; margin-right: 1px; margin-bottom: 1px;}
    .subEntity { border: none; width: 3em; float: leftt; margin: 0; margin-right: 1px;}
    .amount { border: none; width: 7em; text-align:right; float: left; margin: 0; margin-right: 1px;}
    .subEntity { border: none; float: left; margin: 0; margin-right: 1px;}
    .comment { border: none; width: 20em; float: left; margin: 0; margin-right: 1px;}
    .memo { width: 50em; }
    .payee { width: 15em; }
    .delete {float: left; }
    #optionsB { padding: 1em; }
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<div id='G' style="display: none;" class="row">
  <div class="subEntity">
    <input type="text" id="subEntity_Display" name="subEntity_Display" value="" class="subEntity" onChange="clearMe(this);" />
  </div>
  <div style="float: left;">
    <input type="text" id="account_Display" name="account_Display" value="" class="account" onChange="clearMe(this); " />
  </div>
  <div class="subAccount">
    <input type="text" id="subAccount_Display" name="subAccount_Display" value="" class="subAccount" onChange="clearMe(this); " />
  </div>
  <div class="amount">
    <input type="text" id="debit_" name="debit_" value="" class="amount" onChange="RECALL = false; calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); debitCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }" />
  </div>
  <div class="amount">
    <input type="text" id="credit_" name="credit_" value="" class="amount" onChange="RECALL = false; calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); creditCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }" />
  </div>
  <div class="comment">
    <input type="text" id="comment_" name="comment_" value="" class="comment" maxlength="255" />
  </div>
  <div class="delete">
    <input type="button" id="delete_" name="delete_" value="X" onclick="return deleteRow(this);" style="border: none;" />
  </div>

  <input type="text" id="account_"    name="account_"    class="debug account" />
  <input type="text" id="subAccount_" name="subAccount_" class="debug subAccount" />
  <input type="text" id="subEntity_"  name="subEntity_"  class="debug subEntity" />
  <input type="hidden" id="cleared_" name="cleared_" value="_" style="clear: both;">
  <div style="clear:both;"></div>
</div>

$self->{requiredDivs}

<form name="main" target="_blank" method="post" action="">
  <div id="mainDiv" class="primary">
    <div>
      #FIELD=view#
      #FIELD=app#
      #FIELD=state#
      #FIELD=command#
      #FIELD=ledgerCode#
      #FIELD=journal#
      <a href="" title="#LANG=Remembered#" onClick="javascript: return false;" disabled="1">#LANG=R#</a> #FIELD=remember#
      #LABEL=date# #FIELD=date#
      #LABEL=payee# #FIELD=payee#
    </div>
    <div>#LABEL=memo#&nbsp;#FIELD=memo#</div>
    <div class="heading">
      <div class="subEntity"><span title="#LANG=SubEntity#">#LANG=SE#</span></div>
      <div class="account">#LANG=Account#</div>
      <div class="subAccount">&nbsp;#LANG=Sub Account#</div>
      <div class="amount" style="text-align: center;">#LANG=Debit#</div>
      <div class="amount" style="text-align: center;">#LANG=Credit#</div>
      <div class="comment">#LANG=Comment#</div>
      <div class="delete"><a title="#LANG=Delete#">D</a></div>
      <div style="clear: both;"></div>
    </div>
<div style="display: none;" class="row"> <!-- this can never be used because the other views treat it as a special case.  here as a placeholder -->
  <input type="text" id="account_Display" name="account_Display0" value="" class="account" onChange="clearMe(this); " />
  <input type="text" id="subAccount_Display" name="subAccount_Display0" value="" class="subAccount" onChange="clearMe(this); " />
  <input type="text" id="debit_0" name="debit_" value="" class="amount" onChange="RECALL = false; calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); debitCheck(this); if (! twoRowCheck(0)) { lastRowCheck(); }" />
  <input type="text" id="credit_0" name="credit_" value="" class="amount" onChange="RECALL = false; calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); creditCheck(this); if (! twoRowCheck(0)) { lastRowCheck(); }" />
  <input type="text" id="comment_0" name="comment_" value="" class="comment" />
  <input type="text" id="subEntity_Display0" name="subEntity_Display" value="" class="subEntity" onChange="clearMe(this);" />

  <input type="hidden" name="cleared_0" value="_">
  <input type="button" name="delete_0" value="X" onclick="return deleteRow(this);" style="border: none;" />

  <input type="text" size="3" name="account_0"    style="display: none;" />
  <input type="text" size="3" name="subAccount_0" style="display: none;" />
  <input type="text" size="3" name="subEntity_0"  style="display: none;" />

</div>
    <div id="insertRowHere"><img width="1" height="5"></div>

    <div style="clear: both; padding-top: 1em; ">
      <input style="float:left;" name="submitDataButton" onclick="return submitWrapper();" value="#LANG=Insert#" type="button">
      <input style="float:left;" onclick="javascript: return addRow(this.form.view.value,'insertRowHere');" name="add" value="#LANG=Add Row#" type="button">
      <input style="float:right;" type="button" value="#LANG=Reset#" onclick="return resetForm();" />
      <div style="float:right;" style="text-align:right;">
        <a title="#LANG=editHelp#">#</a>
        #FIELD=id#<span id="refNumSpan">&nbsp;</span>
        <input type="text" name="editID" size="4" title="#LANG=editHelp#" />
        <input type="button" name="edit" Value="#LANG=Edit#" onclick="document.main.id.value=document.main.editID.value; init(); " title="#LANG=editHelp#" />
      </div>
    </div>

    <div style="clear: both;"></div>
  </div>

$self->{optionsDiv}

</form>
<div class="tertiary" style="padding: .5em;">
  <span style="font-weight: bold; border-bottom-style: solid; border-weight: thin;">#LANG=Help#</span>
  #LANG=doubleHelp#
  #LANG=otherTips#
</div>
ENDOF
  $self->{javascript}->{main} .= $self->PAYEES(days => 400, journal => 'G') . "\n";
  $self->{javascript}->{main} .= $act->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $sact->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $se->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  if ($act->error) { $self->error($act->errorMessage); }
  if ($sact->error) { $self->error($sact->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }

  $self->{profile}->{main} = {
    required => [  ],
    optional => [  ],
    constraints => {
    },
  };
  $self->{defaultForm} = "main";

  return $self;
}

=item boolean isValid

  This routine will always return true because the double entry
  object is just a placeholder for the screen used to edit and
  enter a double entry transaction.

=cut
sub isValid
{
  return 1;
}

1;
__END__

=pod

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

JT Moree

=head1 SEE ALSO

perl(1), JournalEntry(3), DBIWrapper(3)

=cut
