# Register.pm - The Object Class that provides a Register Object
#$Id: Register.pm,v 1.42 2006/05/04 17:55:13 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Register;
use strict;
use Portal::Accounting::Objects::Base;
use Portal::Accounting::Objects::Account;
use Date::Calc qw (Today_and_Now Add_Delta_YMD);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

my $Revision = '$Revision: 1.42 $';
($VERSION = $Revision) =~ s/^Revision:\s//;

=head1 NAME

Register - Object used to build a Register Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Register;
  my $RegisterObj = Portal::Accounting::Objects::Register->new(   );
  if (! $RegisterObj->populate(%{$self->{input}}) || ! $RegisterObj->validate())
  {
    print $RegisterObj->printForm();
  }
  else
  {
    if ($RegisterObj->select)
    {
      print $RegisterObj->printHTML;
    }
    else
    {
      print $RegisterObj->errorMessage;
    }
  }

=head1 DESCRIPTION

Register is a Register class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Register.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  my %args = (ledgerCode => "", id => "", @_ );
  my $l = $self->{langObj};
  my $Names = $l->map("Names");
  my $Codes = $l->map("Codes");
  if ($args{ledgerCode} eq "") { $self->error("LedgerCode must be specified<br>\n"); return $self;  }
  if ($args{id} eq "") { $self->error("User id must be specified<br>\n"); return undef;  }
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});

  $self->{usingForms} = 1;
  $self->{detail} = [];
  $self->{id} = '';

  my @date = Today_and_Now();
  my $desc = "U: $args{id} $date[0]-$date[1]-$date[2] $date[3]:$date[4]:$date[5]";
  if ($date[1] == 1)
  {
    $date[1] = 12;
    $date[0]--;
  }
  else
  {  $date[1]--;  }
  $date[2] = "01";
  my $from = "$date[0]-$date[1]-$date[2]";
  @date = Add_Delta_YMD($date[0], $date[1], $date[2], 0, 1, -1);
  my $to = "$date[0]-$date[1]-$date[2]";

  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  my $a = $acc->selectOptions('name');
  if ($acc->error) { $self->error("Failed to get accounts<br>".$acc->errorMessage."\n"); return $self;  }
  if (! defined $a) { $a = {names => [], values => [] }; }
  my $b = $acc->selectOptions('code');
  if ($acc->error) { $self->error("Failed to get accounts<br>".$acc->errorMessage."\n"); return $self;  }
  if (! defined $b) { $b = {names => [], values => [] }; }
  my $c = $self->getCommonAccounts($args{ledgerCode});
  if (! defined $c) { $c = {names => [], values => [] }; }
  $self->{data}->{main} = {
    "account" => { -Type => "hidden", -DataType => "int" },
    "common" => { -Label => $l->map("Most Used Accounts"),
      -Type => "select", size => 4,
      onchange => <<"JS",
      var account = this.options[this.selectedIndex].value;
      this.form.account.value = account;
      this.form.accountNames.selectedIndex = findAccount(account, this.form.accountNames);
      this.form.accountCodes.selectedIndex = findAccount(account, this.form.accountCodes);
      return setDate(this.form);
JS
      -onload => "this.selectedIndex = -1;",
      -Options => $c
      },
    "accountNames" => { -Label => $l->map("All Accounts"),
      -Type => "select", -DataType => "int",
      onchange => <<"JS",
      this.form.common.selectedIndex = -1;
      var account = this.options[this.selectedIndex].value;
      this.form.account.value = account;
      this.form.accountCodes.selectedIndex = findAccount(account, this.form.accountCodes);
JS
      -onload => "this.selectedIndex = -1;",
      -Options => $a
      },
    "accountCodes" => { -Label => $l->map("All Accounts"),
      -Type => "select", -DataType => "int",
      onchange => <<"JS",
      this.form.common.selectedIndex = -1;
      var account = this.options[this.selectedIndex].value;
      this.form.account.value = account;
      this.form.accountNames.selectedIndex = findAccount(account, this.form.accountNames);
JS
      -onload => "this.selectedIndex = -1;",
      -Options => $b
      },
    "from" => { -Label => $l->map("From"),
      -Type => "date-picker", -DataType => "string",
      -Value => $from,
      -year => $date[0] ,
      },
    "to" => { -Label => $l->map("To"),
      -Type => "date-picker", -DataType => "string",
      -Value => $to,
      -year => $date[0] ,
      },
    "switchAcc" => { -Type => "radio", -Options => { names => [ qw( Names Codes ) ], values => [ qw( names codes ) ] },
      -Value => 'names', onchange => 'javascript: toggleAccount(this.form); ',
    },
    "reconcile" => { -Label => $l->map("Reconcile"), -Type => "checkbox", -Value => "1", checked => 1, -DataType => "int"},
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Register" },
    "command" => { -Type => "hidden", -Value => "displayRegister" },
    "includeR" => { -Label => $l->map("Include")." r", -Type => "checkbox", -Value => "1", checked => 1},
    "beginStatement" => { -Type => "hidden",},
    "type" => { -Type => "hidden",},
  };

  $self->{template}->{main} = <<"ENDOF";
  <form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  <table class="primary">
    <tbody>
    <tr>
      <td>
        #LABEL=common#<br>
        #FIELD=common#
      </td>
      <td align="right">
        #LABEL=from#: #FIELD=from#<br>
        #LABEL=to#: #FIELD=to#
      </td>
    </tr>
    <tr>
      <td>
        #LABEL=accountNames#<br>
        <div id="names">
          #FIELD=accountNames#
        </div>
        <div id="codes" style="display: none;">
          #FIELD=accountCodes#
        </div>
        #FIELD=account#
        #FIELD=switchAcc#
        </td>
        <td align="right">
          #FIELD=reconcile# #LABEL=reconcile#
      </td>
    </tr>
    <tr>
      <td align="right" colspan="2">
        <input type="submit" name="submit" value="#LANG=Submit#">
      </td>
    </tr>
    </tbody>
  </table>
  </form>
ENDOF

  $self->{profile}->{main} = {
    required => [ qw( account from to ) ],
    constraints  =>
        {
            from   => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
            to   => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
            account  => qr/^\d+$/,
        },
  };

  $self->{data}->{reconcile} = {
    "beginStatement" => { -Type => "hidden", -Value => ""},
    "account" => { -Type => "hidden", -Value => "", -DataType => "int"},
    "accountName" => { -Type => "hidden", -Value => "", -DataType => "int"},
    "accountCode" => { -Type => "hidden", -Value => "", -DataType => "int"},
    "balance" => { -Type => "hidden", -Value => "", -DataType => "int"},
    "rows" => { -Type => "hidden", -Value => "", -DataType => "int"},
    "type" => { -Type => "hidden", -Value => "", -DataType => "int"},
    "note" => { -Type => "text", -DataType => "string", -Value => "", -DataType => "string" },
    "from" => { -Label => $l->map("From"),
      -Type => "date-picker", -DataType => "string",
      -Value => $from,
      -year => $date[0] ,
      },
    "to" => { -Label => $l->map("To"),
      -Type => "date-picker", -DataType => "string",
      -Value => $to,
      -year => $date[0] ,
      },
    "reconcile" => { -Label => $l->map("Reconcile"), -Type => "checkbox", -Value => "1", checked => 1, -DataType => "int" },
    "includeR" => { -Label => $l->map("Include")." r", -Type => "checkbox", -Value => "1", checked => 1},
    "command" => { -Type => "hidden", -Value => "saveReconcile" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode} },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Register" },
    "HTMLObjectFormInternalVariable" => { -Type => "hidden", -Value => "", -DataType => "int", },
  };

  $self->{newOnly}[0] = {};
  $self->{INSERTorder}[0] = [ qw( date_of account ledger_code note ) ];
  $self->{db2data}[0] = { date_of => "to", ledger_code => "ledgerCode" };
  $self->{data2db}[0] = reverse %{$self->{db2data}[0]};
  $self->{UPDATE}[0] = "INSERT INTO reconcile_tb (description, ##COLUMNS##)";
  $self->{UPDATE}[0] .= " VALUES ('$desc', ##VALUES## )";

  $self->{newOnly}[1] = {};
  $self->{db2data}[1] = { date_of => "to" };
  $self->{data2db}[1] = reverse %{$self->{db2data}[1]};
  $self->{INSERTorder}[1] = [ ];
  $self->{UPDATE}[1] = "UPDATE debitcredit_tb SET cleared = 'r', reconcile = (SELECT id FROM reconcile_tb WHERE description = '$desc') WHERE cleared = 'x' AND trans_date <= '#FIELD=to#'";

  $self->{debug} = 0;
  return $self;
}

=item bool populate(%input)

  Checks for existence of the {rows} var.  If exists it dynamically creates
  a data->{reconcile} element for each row. so that when
  the parent populate method is called it will pull the data from
  the input hash

  If no rows exists then the form is main instead of reconcile and nothing extra is done.

=cut
sub populate
{
  my $self = shift;
  my %args = (rows => 0, @_);
  if ($args{rows} =~ /^\d+$/)
  {
    for (my $i = 0; $i < $args{rows}; $i++)
    {
      $self->{data}->{reconcile}->{"ref_$i"} = { -Type => "hidden", -DataType => "int" };
      $self->{data}->{reconcile}->{"counter_$i"} = { -Type => "hidden", -DataType => "int" };
      $self->{data}->{reconcile}->{"cleared_$i"} = { -Type => "hidden", -DataType => "string" };
      $self->{data}->{main}->{"ref_$i"} = { -Type => "hidden", -DataType => "int" };
      $self->{data}->{main}->{"counter_$i"} = { -Type => "hidden", -DataType => "int" };
      $self->{data}->{main}->{"cleared_$i"} = { -Type => "hidden", -DataType => "string" };
    }
  }
  #set name, code, and type based on given account #
#  $self->selectAccountData(account => $args{account} , from => $args{from} ) ;
  if ($self->error) { return undef;  }
  return $self->SUPER::populate(@_);
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

#   if ($self->numInvalid() > 0 || $self->numMissing() > 0)
#   {
#     $self->error($self->genErrorString("all"));
#     return 0;
#   }

  return 1;
}

sub outstanding
{
  my $self = shift;
  my $total = 0;
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    $self->{data}->{reconcile}->{cashFlow}->{-Value} += $self->{detail}->[$row]->[10];
    if ($self->{detail}->[$row]->[2] ne "")  {  $total += $self->{detail}->[$row]->[10];  }
  }
  return $total;
}

=item print

  This routine looks at the internal reconcile flag and calls the appropriate routine.

=cut
sub printHTML
{
  my $self = shift;
  if ($self->{data}->{main}->{reconcile}->{checked})
  {  return $self->printReconcile(@_);  }
  return $self->printPlain(@_);
}

=item string columnMarkup(text, row, column)

  This routine adds html that is appropriate to for each column in the register.
  Column is a integer starting from 0, text is the text to markup.
  The columns are based on the type of register
               0          1          2               3         4               5          6                  7                  8              9       10
  Bank Acct:   "ref_num", "counter", "$Outstanding", "$Date", "$Description", "$Comment", "$Chk"           , "$Dep"           , "$SubAccount", "$Cld", "$Amount"
  Invest Acct: "ref_num", "counter", "$Outstanding", "$Date", "$Description", "$Comment", ""               , ""               , "$SubAccount", "$Cld", "$Amount"
  All others:  "ref_num", "counter", "$Outstanding", "$Date", "$Description", "$Comment", "$SubAccountName", "$SubAccountCode", "$SubAccount", "$Cld", "$Amount"

=cut
sub columnMarkup
{
  my $self = shift;
  my $text = shift;
  my $row = shift;
  my $col = shift;
  my $return = "";
  my $parms = "app=Accounting&state=JournalEntry&command=view&ledgerCode=".$self->{data}->{reconcile}->{ledgerCode}->{-Value}."&id=".$self->{detail}->[$row]->[0];
  if ($text eq "") { $text = "&nbsp;"; }
  if ($col == 9 && $self->{data}->{main}->{reconcile}->{checked})
  {
    #do button values get submitted?  is that why we have a hidden field?
    #perhaps we could change to text fields that were not editable?
    $return .= <<"ENDOF";
<input class="button" type="button" value="$text" name="button_$row" onclick="cyclebutton(this.form.cleared_$row, this); calculateBalance(this.form);">
<input type="hidden" name="cleared_$row" value="$text">
ENDOF
  }
  elsif ($col == 0 && $self->{data}->{main}->{reconcile}->{checked})
  {
    $return .= <<"ENDOF";
<input type="hidden" value="$text" name="ref_$row">$text
ENDOF
  }
  elsif ($col == 1 && $self->{data}->{main}->{reconcile}->{checked})
  {
    $return .= <<"ENDOF";
<input type="hidden" value="$text" name="counter_$row">$text
ENDOF
  }
  elsif ($col == 10)
  {
    $return .= "<input type=\"hidden\" name=\"amount_$row\" value=\"$text\">";
    my $class = 'positive';
    if ($text < 0)
    {
      $class = 'negative';
    }
    $return .= '<span onclick="'."myGet('_new', '', '$parms', 'toolbar=no'); return false;".'" class="'.$class.'">'.$text."</span>";
  }
  elsif ($col == 11)
  {
    my $class = 'positive';
    if ($text < 0)
    {
      $class = 'negative';
    }
    $return .= '<span class="'.$class.'">'.$text.'</span>';
  }
  elsif ($col == 6 && $self->{data}->{reconcile}->{type}->{-Value} == 0) #check
  {
    #this code sucks! hard to understand... need to rewrite markChecks routine to cleanup.
    my $value = $text;
    if ($text eq "&nbsp;") {  $value = "*"; }
    else
    {
      $return .= <<"ENDOF";
  <a style="" href="?$parms" target="_new" onclick="myGet('_new', '', '$parms', 'toolbar=no'); return false;">$text</a>
ENDOF
    }
    $return .= "<input type=\"hidden\" name=\"check_$row\" value=\"$value\">&nbsp;";
  }
  elsif ($col == 7 && $self->{data}->{reconcile}->{type}->{-Value} == 0) #dep
  {
    my $value = $text;
    if ($text eq "&nbsp;") {  $value = "*"; }
    else
    {
      $return .= <<"ENDOF";
  <a style="" href="?$parms" target="_new" onclick="myGet('_new', '', '$parms', 'toolbar=no'); return false;">$text</a>
ENDOF
    }
    $return .= "<input type=\"hidden\" name=\"deposit_$row\" value=\"$value\">&nbsp;";
  }
  elsif ($col == 4)
  {
    $return .= <<"ENDOF";
  <a href="?$parms" target="_new" onclick="myGet('_new', '', '$parms', 'toolbar=no'); return false;">$text</a>
ENDOF
  }
  else #($text ne "")
  {
    $return .= $text;
  }

  return $return;
}

sub setupStyles
{
  my $self = shift;
  my $l = $self->{langObj};

  my $Date = $l->map("Date");
  my $Comment = $l->map("Comment");
  my $Outstanding = $l->map("Outstanding");
  my $SubAccountName = $l->map("Sub Account")." ". $l->map("Name");
  my $SubAccountCode = $l->map("Sub Account")." ". $l->map("Code");
  my $SubAccount = $l->map("Sub Account");
  my $SACode = $l->map("SA")." ".$l->map("Code");
  my $SAName = $l->map("SA")." ".$l->map("Name");
  my $Debit = $l->map("Debit");
  my $Amount = $l->map("Amount");
  my $Credit = $l->map("Credit");
  my $Cleared = $l->map("Cleared");
  my $Cld = $l->map("Cld");
  my $Balance = $l->map("Balance");
  my $Description = $l->map("Description");
  my $Chk = $l->map("Chk");
  my $Dep = $l->map("Dep");
  my %return;
  my $type = $self->{data}->{reconcile}->{type}->{-Value};
  if ($self->debug) { $self->debug("<p>Type: $type</p>\n");  }
  if ($type eq "")
  {
    $return{css} .= "// Error setting up styles.  No type given to setupStyles. \n";
    $return{headers} .= "<!-- Error setting up headings.  No type given. -->\n";
  }
  elsif ($type == 0)
  {
    $return{headers} .= <<"ENDOF";
      <th class="active-column-0">ref#</th>
      <th class="active-column-1">#</th>
      <th class="active-column-2"><a title="$Outstanding">O</a></th>
      <th class="active-column-3">$Date</th>
      <th class="active-column-4">$Description</th>
      <th class="active-column-5">$Comment</th>
      <th class="active-column-6">$Chk </th>
      <th class="active-column-7">$Dep </th>
      <th class="active-column-8">$SACode</th>
      <th class="active-column-9"><a title="$Cleared"><span style="color: #000000;">$Cld</span></a></th>
      <th class="active-column-10">$Amount</th>
      <th class="active-column-11">$Balance</th>
ENDOF
    $return{css} .= <<"ENDOF";
  .active-column-6 {text-align: right;} /* chk */
  .active-column-7 {text-align: right;} /* dep */
  .active-column-8 {display: none;}    /* sub account */
ENDOF
  }
  elsif ($type == 3) #investment accounts
  {
    $return{headers} .= <<"ENDOF";
      <th class="active-column-0">ref#</th>
      <th class="active-column-1">#</th>
      <th class="active-column-2"><a title="$Outstanding">O</a></th>
      <th class="active-column-3">$Date</th>
      <th class="active-column-4">$Description</th>
      <th class="active-column-5">$Comment</th>
      <th class="active-column-6">&nbsp;</th>
      <th class="active-column-7">&nbsp;</th>
      <th class="active-column-8">$SACode</th>
      <th class="active-column-9"><a title="$Cleared"><font color="#000000">$Cld</font></a></th>
      <th class="active-column-10">$Amount</th>
      <th class="active-column-11">$Balance</th>
ENDOF
    $return{css} .= <<"ENDOF";
  .active-column-6 {display: none;} /* sub account */
  .active-column-7 {display: none;} /* sub account */
  .active-column-8 {display: none;} /* sub account */
ENDOF
  }
  else #all others
  {
    $return{headers} .= <<"ENDOF";
      <th class="active-column-0">ref#</th>
      <th class="active-column-1">#</th>
      <th class="active-column-2"><a title="$Outstanding">O</a></th>
      <th class="active-column-3">$Date</th>
      <th class="active-column-4">$Description</th>
      <th class="active-column-5">$Comment</th>
      <th class="active-column-6"><a title="$SubAccountName"><font color="#000000">$SAName</font></a></th>
      <th class="active-column-7"><a title="$SubAccountCode"><font color="#000000">$SACode</font></a></th>
      <th class="active-column-8"><a title="$SubAccountCode"><font color="#000000">$SubAccount</font></a></th>
      <th class="active-column-9"><a title="$Cleared"><font color="#000000">$Cld</font></a></th>
      <th class="active-column-10">$Amount</th>
      <th class="active-column-11">$Balance</th>
ENDOF
    $return{css} = <<"ENDOF";
  .active-column-6 {text-align: left;}  /* sub account */
  .active-column-7 {display: none; text-align: right;} /* sub account */
  .active-column-8 {display: none;}     /* sub account */
ENDOF
  }
  my $outstanding = "display: none;";
  my $balance = "display: none;";
  if ($self->{data}->{main}->{reconcile}->{checked})  { $outstanding = "width: 20px;"; }
  else { $balance = "text-align: right;"; }
  $return{css} = <<"ENDOF";
  body, html {margin:0px; padding: 0px; }
  input.button { font-size: 9; border-style: none; }
  input.text { border: 2px solid; }

  .active-controls-grid {font: menu;}
  .active-grid-column {border-right: 1px solid threedlightshadow;}
  .active-grid-row {border-bottom: 1px solid threedlightshadow;}
  .negative { color: #ff0000; }
  .positive { color: #000000; }
  a.negative { color: #ff0000; }
  a.positive { color: #000000; }
  a.negative:link { color: #ff0000; }
  a.positive:link { color: #000000; }

  .active-column-0 {display: none; text-align: right;}      /* ref */
  .active-column-1 {display: none; text-align: left;}      /* counter */
  .active-column-2 {$outstanding}      /* outstanding */
  .active-column-3 {text-align: left;}   /* date */
  .active-column-4 {text-align: left;}   /* payee */
  .active-column-5 {text-align: left;}   /* comment */
$return{css}
  .active-column-9 {width: 40px; text-align: center;}  /* cleared */
  .active-column-10 {text-align: right;} /* amount */
  .active-column-11 {$balance}  /* balance // */
  .active-column-12 {display:none}  /* subCode // */
ENDOF
  return %return;
}

=item %hash printPlain()

  This is a register view with a running balance.  But no reconciliation forms.

=cut
sub printPlain
{
  my $self = shift;
  my %return;
  my %setup = $self->setupStyles();
  my $BalanceForward = $self->{langObj}->map("Balance")." ".$self->{langObj}->map("Forward");
  my $balance = $self->{data}->{reconcile}->{balance}->{-Value};
  $return{css} = $setup{css};
  $return{body} .= <<"ENDOF";
  <div style="text-align: center;">$self->{data}->{reconcile}->{accountName}->{-Value} - $self->{data}->{reconcile}->{accountCode}->{-Value} - $self->{data}->{main}->{from}->{-Value} - $self->{data}->{main}->{to}->{-Value}</div>
  <table class="register" border="1" width="100%" cellpadding="0" cellspacing="0">
  <thead class="primary">
  $setup{headers}
  </thead>
  <tbody>
  <tr>
    <td class="active-column-3">$self->{from}</td>
    <td class="active-column-4">$BalanceForward</td>
    <td class="active-column-5">&nbsp;</td>
    <td class="active-column-6">&nbsp;</td>
    <td class="active-column-7">&nbsp;</td>
    <td class="active-column-8">&nbsp;</td>
    <td class="active-column-9">&nbsp;</td>
    <td class="active-column-10">$balance</td>
    <td class="active-column-11">$balance</td></tr>
ENDOF
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    $return{body} .= "  <tr>\n ";
    my $col;
    for ($col = 3; $col < scalar @{$self->{detail}->[$row]}; $col++)
    {
      $return{body} .= "    <td class=\"active-column-$col\">".$self->columnMarkup($self->{detail}->[$row]->[$col], $row, $col)."</td>\n";
    }
    $balance += $self->{detail}->[$row]->[$col - 1];
    $return{body} .= "    <td class=\"active-column-$col\">".$self->columnMarkup(sprintf("%.02f",$balance))."</td>\n";
    $return{body} .= "  </tr>\n ";
  }
  $return{body} .= "  </tbody>\n</table>\n ";
  return %return;
}

=item %hash printReconcile()

  This register view contains forms for editing line items and marking entries as cleared.

=cut
sub printReconcile
{
  my $self = shift;
  my %args = (app => "Accounting", state => "Register", @_);
  $self->{data}->{reconcile}->{app}->{-Value} = $args{app};
  $self->{data}->{reconcile}->{state}->{-Value} = $args{state};
  my %return;
  my $l = $self->{langObj};
  my $Finished = $l->map("Finished");
  my $Save = $l->map("Save");
  my $Reset = $l->map("Reset");
  my %setup = $self->setupStyles();
  my $headers;
  my $styles;
  my $rows = scalar @{$self->{detail}};
  my $columns = 0;
  if (defined $self->{detail}->[0])
  {
    $columns = scalar @{$self->{detail}->[0]};
  }
  my $Mark = $l->map("Automark");
  my $From = $l->map("From");
  my $To = $l->map("To");
  my $Dep = $l->map("Dep");
  my $Chk = $l->map("Chk");
  my $usethis = $l->map("usethis");
  my $from = $self->{data}->{main}->{from}->{-Value};
  my $to = $self->{data}->{main}->{to}->{-Value};
  my $account = $self->{data}->{main}->{account}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  my $balance = $self->commify(sprintf("%.02f", $self->{data}->{reconcile}->{balance}->{-Value} - $self->outstanding) );
  my $book = $self->commify(sprintf("%.02f",$self->{data}->{reconcile}->{balance}->{-Value} + $self->{data}->{reconcile}->{cashFlow}->{-Value}) );
  my $WhenFinished = $l->map("BalanceShouldMatch");
  my $EndStatementBalance = $l->map("End")." ".$l->map("Statement")." ".$l->map("Balance")." ".$l->map("Calculation");
  my $Differ = $l->map("Difference between statement and calculations");
  my $EnterYour = $l->map("EndStatementBalanceLong");
  my $Begin = $l->map("Begin")." ".$l->map("Statement")." ".$l->map("Balance")." ".$l->map("Calculation");
  my $IfThis = $l->map("BalanceNoMatch");
  my $Balance = $l->map("Balance");
  my $Book = $l->map("Book");
  my $recHelp = $l->map("reconcileHelp");
  my $inc = $l->map("Include")." r ".$l->map("in")." ".$l->map("Automark");
  my $ClearAll = $l->map("Clear All");

  $return{css} = $setup{css};
  $return{javascript} .= <<"ENDOF";
ENDOF
  $return{body} .= <<"ENDOF";
<div style="text-align: center;">$self->{data}->{reconcile}->{accountName}->{-Value} - $self->{data}->{reconcile}->{accountCode}->{-Value} : $from - $to</div>
<form name="mainForm" method="post">
  <input type="hidden" name="HTMLObjectFormInternalVariable" value="1">
  <input type="hidden" name="app" value="$self->{data}->{reconcile}->{app}->{-Value}">
  <input type="hidden" name="state" value="$self->{data}->{reconcile}->{state}->{-Value}">
  <input type="hidden" name="command" value="reconcile">
  <div class="primary controls">
    <div style="float: left;">
      <input type="hidden" name="book" value="$book">
      <input class="active-column-10" type="text" name="manualEndStatement" onchange="calculateBalance(this.form); return true;" style="border: 1px solid; width: 9em;" > $EnterYour<br>
      <input class="active-column-10" type="text" name="beginStatement" onfocus="this.form.checks.focus();" style="border: medium none; width: 9em;" value="$balance" tabIndex="-1">
        <a title="$IfThis.">$Begin</a><br>
      <input class="active-column-10" type="text" name="endStatement" onfocus="this.form.checks.focus();" style="border: medium none ; width: 9em; " value="$balance">
        <a title="$WhenFinished.">$EndStatementBalance</a><br>
      <input class="active-column-10" type="text" name="difference" onfocus="this.form.checks.focus();" style="border: medium none ; width: 9em;" tabIndex="-1"> $Differ
   </div>
   <div style="float: right;">
ENDOF
  if ($self->{data}->{reconcile}->{type}->{-Value} == 0) {
  $return{body} .= <<"ENDOF";
      <table style="border:0; text-align:right;">
        <tr><th><font style="color: blue; font-size: large;"><input type="button" name="checks" value="$Mark" onclick="mark(this.form);" title="$usethis"></font></th><th>$From</th><th>$To</th></tr>
        <tr><th>$Chk</th><td><input type="text" size="5" name="chkFrom"></td> <td><input type="text" size="5" name="chkTo"></td></tr>
        <tr><th>$Dep</th><td><input type="text" size="5" name="depFrom"></td> <td><input type="text" size="5" name="depTo"></td></tr>
      </table>
ENDOF
  }
  else {
  $return{body} .= <<"ENDOF";
  <input type="button" name="checks" value="$Mark" onclick="mark(this.form);" title="$usethis">
  <input type="hidden"  name="chkFrom"><input type="hidden"  name="chkTo">
  <input type="hidden"  name="depFrom"><input type="hidden"  name="depTo">
ENDOF
  }
  $return{body} .= <<"ENDOF";
    </div>
    <div style="clear:both;"></div>
  </div>
  <input type="hidden" name="reconcile" value="1">
  <input type="hidden" name="rows" value="$rows">
  <input type="hidden" name="ledgerCode" value="$lc">
  <input type="hidden" name="account" value="$account">
  <input type="hidden" name="from" value="$from">
  <input type="hidden" name="to" value="$to">

  <div class="secondary controls" style="clear: both;">
    <div style="float: left;">
      <input type="submit" name="save" value="$Save" onclick="return checkSubmit(this.form);">
    </div>
    <div style="float: right;">
      <input type="submit" name="Submit" value="$Finished" onclick="this.form.command.value = 'finishReconcile'; return checkSubmit(this.form);">
    </div>
    <div style="margin:0 auto; width: 25em;">
      <input type="button" name="clearAll" value="$ClearAll" onclick="clearButtons(this.form);" /> <input type="checkbox" name="includeR" checked="1">$inc
    </div>
  </div>
ENDOF
  $return{body} .= <<"ENDOF";
  <table class="register" border="1" width="100%" cellpadding="0" cellspacing="0" style="clear: both;">
  $setup{headers}
ENDOF
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    $return{body} .= "  <tr>\n ";
    my $col;
    for ($col = 0; $col < scalar @{$self->{detail}->[$row]}; $col++)
    {
      $return{body} .= "    <td class=\"active-column-$col\">".$self->columnMarkup($self->{detail}->[$row]->[$col], $row, $col)."</td>\n";
    }
    $return{body} .= "  </tr>\n ";
  }
  $return{body} .= <<"ENDOF";
  </table>
  <div class="secondary controls">
    <div style="float: left;">
      <input type="submit" name="save" value="$Save" onclick="return checkSubmit(this.form);">
    </div>
    <div style="float: right;">
      <input type="submit" name="Submit" value="$Finished" onclick="this.form.command.value = 'finishReconcile'; return checkSubmit(this.form);">
    </div>
    <div style="margin:0 auto; width: 25em;">
      <input type="button" name="clearAll" value="$ClearAll" onclick="clearButtons(this.form);" /> <input type="checkbox" name="includeR" checked="1">$inc
    </div>
  </div>
  <p>$recHelp<br>
  </p>
</form>
ENDOF
  $return{body} = $self->langSubstitute($return{body});
  return %return;
}

sub printOutstanding
{
  my $self = shift;
  my $l = $self->{langObj};
  my $debits;
  my $credits;
  my $sub = $l->map("Sub Account");
  my $Date = $l->map("Date");
  my $desc = $l->map("Description");
  my $amount = $l->map("Amount");
  my $Total = $l->map("SubTotal");
  my $Balance = $l->map("SubBalance");
  my $Outstanding = $l->map("Outstanding");
  my $cBalance = 0;
  my $dBalance = 0;
  my $type = $self->{data}->{reconcile}->{type}->{-Value};

  if ($type == 0) { $sub = $l->map("Chk");  }
  $debits .= <<"ENDOF";
<table border="1" class="register">
  <tr>
    <th>$Date</th>
    <th>$desc</th>
    <th>$sub</th>
    <th>$amount</th>
  </tr>
ENDOF
  if ($type == 0) { $sub = $l->map("Dep");  }
  $credits .= <<"ENDOF";
<table border="1" class="register">
  <tr>
    <th>$Date</th>
    <th>$desc</th>
    <th>$sub</th>
    <th>$amount</th>
  </tr>
ENDOF
  for (my $i = 0; $i < scalar @{$self->{detail}}; $i++)
  {
    if ($self->{detail}->[$i]->[2] == "*")
    { #entry is outstanding
      my @detail = @{$self->{detail}->[$i]};
      if ($detail[10] < 0)
      {
        $credits .= <<"ENDOF";
  <tr><td>$detail[3]</td><td>$detail[4]</td><td>$detail[6]</td><td>$detail[10]</td>
ENDOF
        $cBalance += $detail[10];
      }
      else
      {
        $debits .= <<"ENDOF";
  <tr><td>$detail[3]</td><td>$detail[4]</td><td>$detail[7]</td><td>$detail[10]</td>
ENDOF
        $dBalance += $detail[10];
      }
    }
  }
  my $o = sprintf ("%.02d", $dBalance + $cBalance);
  $dBalance = sprintf ("%.02d", $dBalance);
  $debits .= <<"ENDOF";
  <tr><td align="right">$Total</td><td align="right" colspan="3">$dBalance</td></tr>
</table>
ENDOF
  $cBalance = sprintf ("%.02d", $cBalance);
  $credits .= <<"ENDOF";
  <tr><td align="right">$Total</td><td align="right" colspan="3">$cBalance</td></tr>
</table>
<p>Summary<br>
$Balance:$self->{data}->{reconcile}->{balance}->{-Value}<br>
$Outstanding:$o
</p>
ENDOF
  return $debits.$credits;
}

=item boolean selectAccountData

=cut
sub selectAccountData
{
  my $self = shift;
  my %args = (account => "", from => "", @_);
  my $account = $args{account};
  my $from = $args{from};
  my $lc = $self->get('ledgerCode');
  my $accountSql = <<"FOO";
 SELECT name, code, type,
    (SELECT SUM(d_amount) FROM debitcredit_tb
      WHERE account_num = '$account'
      AND trans_date < '$from'
      AND ledger_code = '$lc'
    ) AS balance
    FROM account_tb WHERE number = '$account'
FOO
  if ($self->debug) { $self->debug($accountSql."<br>\n"); }
  my $sth = $self->{dbHandle}->read(sql => $accountSql);
  if ($self->{dbHandle}->error)
  {
    $self->error("$accountSql".$self->{dbHandle}->errorMessage);
    return undef;
  }
  if (my @a = $sth->fetchrow_array)
  {
    $self->{data}->{reconcile}->{accountName}->{-Value} = $a[0];
    $self->{data}->{reconcile}->{accountCode}->{-Value} = $a[1];
    $self->{data}->{reconcile}->{type}->{-Value} = $a[2];
    $self->{data}->{reconcile}->{balance}->{-Value} = $a[3];
    if ($self->{data}->{reconcile}->{balance}->{-Value} eq "") { $self->{data}->{reconcile}->{balance}->{-Value} = '0.00';  }
  }
  else
  {
    $self->error("No account found for '$self->{data}->{main}->{account}->{-Value}'");
    return undef;
  }
  return 1;
}

=item integer select

  returns: undef on error, # of records selected on success

=cut
sub select
{
  my $self = shift;
  my @account;
  my $sql;
  my $rows = 0;
  my $to = $self->get('to');
  my $from = $self->get('from');
  my $account = $self->get('account');
  my $ledgerCode = $self->get('ledgerCode');
  $self->selectAccountData(account => $self->{data}->{main}->{account}->{-Value}, from => $self->{data}->{main}->{from}->{-Value});
  if ($self->error) { return undef;  }
  if ($self->{data}->{reconcile}->{type}->{-Value} == 0)  #if the type is a bank account do not select sub account names
  {
    $sql  = <<"FOO";
 SELECT
  d.parent_num AS id,
  d.counter,
  '' AS outstanding,
  d.trans_date AS date
  , j.payee_payer AS payee
  , SUBSTRING(d.comment, 1, 15) AS comment
  , checknum(d.dorc, d.sub_account_num) AS check
  , depositnum(d.dorc, d.sub_account_num) AS deposit
  , d.sub_account_num
  , d.cleared
  , d.d_amount
 FROM debitcredit_tb AS d, journal_entry_tb AS j
 WHERE
   d.ledger_code = '$ledgerCode'
   AND d.parent_num = j.ref_num
   AND d.account_num = '$account'
   AND d.trans_date >= '$from'
   AND d.trans_date <= '$to'
FOO
    if ($self->{data}->{main}->{reconcile}->{checked})
    {
      $sql .= <<"MYSQL"
  UNION ALL
  SELECT
    d.parent_num AS id,
    d.counter,
    '*' AS oustanding,
    d.trans_date AS date
    , j.payee_payer AS payee
    , SUBSTRING(d.comment, 1, 15) AS comment
    , checknum(d.dorc, d.sub_account_num) AS check
    , depositnum(d.dorc, d.sub_account_num) AS deposit
    , d.sub_account_num
    , d.cleared
    , d.d_amount
  FROM debitcredit_tb AS d, journal_entry_tb AS j
  WHERE
    d.ledger_code = '$ledgerCode'
    AND j.ref_num = d.parent_num
    AND d.account_num = '$account'
    AND d.trans_date < '$from'
    AND cleared <> 'r'
MYSQL
    }
    $sql .= " ORDER BY date, sub_account_num";
  }
  elsif ($self->{data}->{reconcile}->{type}->{-Value} == 3)  #if the type is a stock account do not select sub accounts.  Only show cash transactions
  {
    $sql  = "SELECT parent_num, counter, '' AS outstanding, trans_date ";
    $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num) AS payee";
    $sql .= ", SUBSTRING(comment, 1, 15)";
    $sql .= ", '', '', sub_account_num, cleared, d_amount";
    $sql .= " FROM debitcredit_tb";
    $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
    $sql .= " AND trans_date >= '$self->{data}->{main}->{from}->{-Value}' AND trans_date <= '$self->{data}->{main}->{to}->{-Value}' AND sub_account_num = '-1'";
    if ($self->{data}->{main}->{reconcile}->{checked})
    {
      $sql .= " UNION ALL ";
      $sql .= "SELECT parent_num, counter, '*' AS oustanding, trans_date ";
      $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num) AS payee";
      $sql .= ", SUBSTRING(comment, 1, 15)";
      $sql .= ", '', '', sub_account_num, cleared, d_amount";
      $sql .= " FROM debitcredit_tb";
      $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
      $sql .= " AND trans_date < '$self->{data}->{main}->{from}->{-Value}' AND cleared <> 'r' AND sub_account_num = '-1'";
    }
    $sql .= " ORDER BY trans_date, payee";
  }
  else
  {
    $sql  = "SELECT parent_num, counter, '' AS outstanding, trans_date ";
    $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num)";
    $sql .= ", SUBSTRING(comment, 1, 15)";
    $sql .= ", (SELECT sub_acc_name from sub_acc_tb WHERE sub_acc_number = sub_account_num) AS name";
    $sql .= ", (SELECT code FROM sub_acc_tb WHERE sub_acc_number = sub_account_num) AS code";
    $sql .= ", sub_account_num, cleared, d_amount";
    $sql .= " FROM debitcredit_tb";
    $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
    $sql .= " AND trans_date >= '$self->{data}->{main}->{from}->{-Value}' AND trans_date <= '$self->{data}->{main}->{to}->{-Value}'";
    if ($self->{data}->{main}->{reconcile}->{checked})
    {
      $sql .= " UNION ALL ";
      $sql .= "SELECT parent_num, counter, '*' AS oustanding, trans_date ";
      $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num)";
      $sql .= ", SUBSTRING(comment, 1, 15)";
      $sql .= ", (SELECT sub_acc_name from sub_acc_tb WHERE sub_acc_number = sub_account_num) AS name";
      $sql .= ", (SELECT code FROM sub_acc_tb WHERE sub_acc_number = sub_account_num) AS code";
      $sql .= ", sub_account_num, cleared, d_amount";
      $sql .= " FROM debitcredit_tb";
      $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
      $sql .= " AND trans_date < '$self->{data}->{main}->{from}->{-Value}' AND cleared <> 'r'";
    }
    $sql .= " ORDER BY trans_date, sub_account_num";
  }
#$self->error($sql); return undef;
  my $sthData = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->error)
  {
    $self->error("Error occured calling read<br>$sql".$self->{dbHandle}->errorMessage);
    return undef;
  }
  if (my $data = $sthData->fetchall_arrayref)
  {
   ############!~~~~~~ uncomment to run over data set and output with error
#     $self->debug("Debug on <br>$sql<br>".scalar @$data."<hr>\n");
#     for (my $i = 0; $i < scalar @$data ; $i++)
#     {
#       $self->debug("start:");
#       for (my $j = 0; $j < scalar @{@$data[$i]} ; $j++)
#       {
#         $self->debug("$i,$j:".@$data[$i]->[$j]."\t");
#       }
#       $self->debug("<br>");
#     }
#     return undef;
    ##########################
    $self->{detail} = $data;
    $rows = $sthData->rows;
  }
  return $rows;
}

=item integer insert

  This routine inserts the internal data reprenting a reconciliation.
  returns:  undef on error, number of records inserted on success

=cut
sub insert
{
  my $self = shift;
  my $sql = "";
  my $result = 0;
  #take each row and update the record with the character
  for (my $i = 0; $i < $self->{data}->{reconcile}->{rows}->{-Value}; $i++)
  {
    $sql .= <<"ENDOF";
UPDATE debitcredit_tb SET cleared = '$self->{data}->{reconcile}->{"cleared_$i"}->{-Value}' WHERE parent_num = '$self->{data}->{reconcile}->{"ref_$i"}->{-Value}' AND counter = '$self->{data}->{reconcile}->{"counter_$i"}->{-Value}';
ENDOF
  }
  if ($sql ne "")
  {
    $result = $self->{dbHandle}->write($sql);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage);  return undef;  }
    $self->{dbHandle}->commit;
  }
  return $result;
}

=item @ getCommonAccounts(ledgerCode)

  this returns a 2D array with values for
  name, number, from, to

=cut
sub getCommonAccounts
{
  my $self = shift;
  my $l = shift;
  my %data;
  my $sql = <<"SQL";
SET DATESTYLE = 'ISO';
SELECT account,
  (SELECT name FROM account_tb WHERE number = account) AS name,
  MAX(date_of) + 1 AS f,
  MAX(date_of) + 30 AS t
FROM reconcile_tb
WHERE ledger_code = '$l'
GROUP BY account ORDER BY f ASC, account
SQL
  if ($self->debug) { $self->debug($sql."<br>");  }
  my $sth = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error)
  {
    $self->prefixError($self->{dbHandle}->errorMessage);
    return undef;
  }
  my $index = 0;
  while (my $d = $sth->fetchrow_hashref)
  {
    push @{$data{names}}, $d->{name};
    push @{$data{values}}, $d->{account};
    $self->{javascript}->{main} .= "FROM[".$d->{account}."] = '$d->{f}' ;\n";
    $self->{javascript}->{main} .= "TO[".$d->{account}."] = '$d->{t}' ;\n";
  }
  return \%data;
}

=item %HTMLObject recSummary

pre:  select should be called before running this routine.
returns:  hash to be given to HTMLObject print method.
summary:  uses internal data to create a summary of the reconciltion just finished.

=cut
sub recSummary
{
  my $self = shift;
  my %return;
  my $l = $self->{langObj};
  my %outst = (debits => 0, credits => 0);  #holds running totals for outstanding debits and credits
  my %OUTST = (debits => "", credits => "");  #holds strings of individual line items for outst section
  my %clear = (debits => 0, credits => 0);  #holds running totals for cleared debits and credits
  my $totalOut = 0;
  my $num6;
  my $num7;
  #walk data
  #if a row is _ add to %outst{debits} {credits}, (debits and credits)
  #else add to $debits or $credits and to clearBal
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    if ($self->{data}->{reconcile}->{type}->{-Value} == 0)
    {
      $num6 = $self->{detail}->[$row]->[6];
      $num7 = $self->{detail}->[$row]->[7];
    }
    elsif ($self->{data}->{reconcile}->{type}->{-Value} == 3)
    {
      $num6 = "&nbsp;";
      $num7 = "&nbsp;";
    }
    else
    {
      $num6 = $self->{detail}->[$row]->[6]."-".$self->{detail}->[$row]->[7];
      $num7 = $self->{detail}->[$row]->[6]."-".$self->{detail}->[$row]->[7];
    }
    if ($self->{detail}->[$row]->[9] eq "_") #oustanding
    {
      $totalOut = sprintf "%.2f", $totalOut + $self->{detail}->[$row]->[10];
      if ($self->{detail}->[$row]->[10] > 0) #debit
      {
        $outst{debits} += $self->{detail}->[$row]->[10];
        $OUTST{debits} .= <<"ENDOF";
  <tr><td align="left">$self->{detail}->[$row]->[3]</td><td align="left">$num7</td><td align="right">$self->{detail}->[$row]->[10]</td></tr>
ENDOF
      }
      else #credit
      {
        $outst{credits} += $self->{detail}->[$row]->[10];
        $OUTST{credits} .= <<"ENDOF";
  <tr><td align="left">$self->{detail}->[$row]->[3]</td><td align="left">$num6</td><td align="right">$self->{detail}->[$row]->[10]</td></tr>
ENDOF
      }
    }
    else #cleared
    {
      if ($self->{detail}->[$row]->[10] > 0) #debit
      {
        $clear{debits} += $self->{detail}->[$row]->[10];
      }
      else
      {
        $clear{credits} += $self->{detail}->[$row]->[10];
      }
    }
  }
  #print the bank balance from before
  #total out
  #total in
  #ending statement balance

  #outstanding debits
  #outst credits
  #book balance
  #format numbers to have 2 dec
  $clear{debits} = sprintf "%.2f", $clear{debits};
  $clear{credits} = sprintf "%.2f", $clear{credits};
  $outst{debits} = sprintf "%.2f", $outst{debits};
  $outst{credits} = sprintf "%.2f", $outst{credits};
  my $outstanding = $self->outstanding;  #original outstanding?  is this different than $outst{debits} + $oust{credits}
  my $beginbalance = sprintf("%.2f", $self->{data}->{reconcile}->{balance}->{-Value} - $outstanding);
  my $clearBal = sprintf("%.2f",$beginbalance + $clear{debits} + $clear{credits} );
  my $bookBal = sprintf( "%.2f", $clearBal + $outst{credits} + $outst{debits});
  my $Notes = $l->map("Notes");
  my $BookBalance = $l->map("Book")." ".$l->map("Balance");
  my @date = Today_and_Now;
  if (! $self->{data}->{reconcile}->{balance}->{-Value} + $self->{data}->{reconcile}->{cashFlow}->{-Value} == $bookBal) { $return{body} .= "<p>Mismatch in ending balances</p>"; }
  $return{body} = <<"ENDOF";
$self->{debugString}
<h4>$self->{data}->{reconcile}->{accountName}->{-Value} - $self->{data}->{reconcile}->{accountCode}->{-Value} : $self->{data}->{reconcile}->{from}->{-Value} - $self->{data}->{reconcile}->{to}->{-Value} </h4>
<table border="0" width="100%" class="register">
  <tr>
    <td>
      <table cellpadding="5" cellspacing="0" border="1">
      <tr><td align="left">#LANG=Begin Statement Balance#</td><td align="right">$beginbalance</td></tr>
      <tr><td align="left">#LANG=Total# #LANG=Debits#</td><td align="right">$clear{debits}</td></tr>
      <tr><td align="left">#LANG=Total# #LANG=Credits#</td><td align="right">$clear{credits}</td></tr>
      <tr><td align="left"><b>#LANG=End Statement Balance#</b></td><td align="right"><b>$clearBal</b></td></tr>
      </table>
    </td>
    <td>
      <span style="display: none;"><b>$Notes</b></span> $self->{data}->{reconcile}->{note}->{-Value}
    </td>
  </tr>
</table>
<p><b>#LANG=Outstanding Transactions#</b></p>
<table cellpadding="5" cellspacing="0" border="1">
<tr><th align="left">#LANG=Date# </th><th align="left">#/#LANG=Sub Account#</th><th align="right">Amount</th></tr>
$OUTST{debits}
<tr><td colspan="2" align="right"><i>#LANG=Total# #LANG=Debits#</i></td><td align="right"><i>$outst{debits}</i></td></tr>
$OUTST{credits}
<tr><td colspan="2" align="right"><i>#LANG=Total# #LANG=Credits#</i></td><td align="right"><i>$outst{credits}</i></td></tr>
<tr><td colspan="2" align="left"><b>#LANG=Total# #LANG=Outstanding#</b></td><td align="right"><b>$totalOut</b></td></tr>
<tr><td colspan="2" align="left"><b>#LANG=Book Balance#</b> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td><td align="right"><b>$bookBal</b></td></tr>
</table>
<p>$date[0]-$date[1]-$date[2] $date[3]:$date[4]:$date[5]</p>
ENDOF
  $return{body} = $self->langSubstitute($return{body});
  return %return;
}

sub isNew
{
  my $self = shift;
  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
