# SubAccount.pm - The Object Class that provides a SubAccount Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: SubAccount.pm,v 1.31 2006/05/12 22:09:54 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::SubAccount;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.31 $';

=head1 NAME

SubAccount - Object used to create, edit, save an SubAccount.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::SubAccount;
  my $obj = Portal::Accounting::Objects::SubAccount->new();
  $obj->populate(
    name => "test", type => 1, code => "1234", );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

SubAccount is a SubAccount class.  This represents a sub account in the
accounting system.  Right now number is used internally. Name and code
are user defined.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::SubAccount module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( type => 50, name => '', code => '', embed => 0, @_ );
  my $code = $args{code};
  if ($code eq '') { $code = $self->suggestCode($args{type}); }
  if (! defined $args{embed} ) { $args{embed} = 0; }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  $self->selectTypes();
  $self->{usingForms} = 1;
  my $l = $self->{langObj};
  my $Submit = $l->map("Save");
  $self->{data}->{main} = {
    "id" => { -Label => $l->map("Id"), -Type => "hidden", -DataType => "int" },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "SubAccounts" },
    "embed" => { -Type => "hidden", -Value => $args{embed}, -DataType => "int" },
    "command" => { -Type => "hidden", -Value => "save" },
    "name" => { -Label => $l->map("Name"), -Type => "text", size => 15, maxlength => 30, -DataType => "string", -onload => 'this.select();', -Value => $args{name}, onchange => 'if (this.form.code.value == "" ) { this.form.code.value = suggestCode(this.value, this.form.type.value); }', },
    "code" => { -Label => $l->map("Code"), -Type => "text", size => 10, maxlength => 30, -Value => $code, -DataType => "string" },
    "type" => { -Label => $l->map("Type"), -Value => $args{type}, -Type => "hidden", -DataType => "int" },
    "typeName" => { -Label => $self->{metaTypes}->{$args{type}}, -Type => "hidden", -DataType => "string" },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  #FIELD=embed#
  <table class="primary">
    <tr>
      <td>#LABEL=name#</td><td>#FI=name#</td>
    </tr><tr>
      <td>#LABEL=code#</td><td>#FI=code#</td>
    </tr><tr>
      <td>#LABEL=type#</td><td>#LABEL=typeName# #FI=type#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code ) ],
    optional => [ qw(id embed app state command) ],
    constraints => {
      id => qr/^\d\d+$/,
      name => qr/^[\d\w][\d\s\w&-,]+$/,
      code => qr/^[\d\w.-]+$/,
    },
  };
#   $self->{formObj} = HTMLObject::Form->new();
  $self->{defaultForm} = "main";
#   $self->{action} = "";
#   $self->{method} = "post";
  ##db infrastructure
  $self->{id} = "id";
  $self->{table} = "sub_acc_tb";
  $self->{INSERTorder}[0] = [ qw( sub_acc_name code type ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { sub_acc_number => "", type => "" };
  $self->{db2data}[0] = { sub_acc_number => 'id', sub_acc_name => 'name' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM sub_acc_tb WHERE sub_acc_number = '#FIELD=$self->{id}#'";
  $self->{INSERT}[0] = "INSERT INTO sub_acc_tb (sub_acc_number, ##COLUMNS##) VALUES (next_sub_account(), ##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE sub_acc_tb SET ##UPDATE## WHERE sub_acc_number = '#FIELD=$self->{id}#' AND NOT type < 50";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM sub_acc_tb WHERE sub_acc_number = #FIELD=$self->{id}# AND NOT type < 50";
  #find out ifis this being referenced from other tables before deleting
  $self->{preDELETE}[0] = "SELECT debitcredit_tb.parent_num FROM debitcredit_tb WHERE debitcredit_tb.sub_account_num = '#FIELD=$self->{id}#' ";
  my $Reserved = $l->map("Reserved Type");
  my $cannot = $l->map("Cannot Delete");
  $self->{preDELETE}[1] = "SELECT '$Reserved'||':'||'$cannot' WHERE '#FIELD=type#' < 50";
  #what should we output as the value of the thing just deleted?
  $self->{postDELETE}[1] = "SELECT sub_acc_name||', '||code FROM sub_acc_tb WHERE sub_acc_number = '#FIELD=$self->{id}#'";
  $self->{LIST}[0] = "SELECT sub_acc_name AS name, sub_acc_number AS id FROM sub_acc_tb WHERE type = #FIELD=type# ORDER BY type, sub_acc_name";

  $self->{types} = {
    50 => "Nml.",
    55 => "Ast.",
    57 => "Bnd.",
    60 => "PS.",
    70 => "St.",
    100 => "Sup.",
    110 => "Cl.",
    120 => "Emp.",
    140 => "Inv.",
  };
  return $self;
}

=item string suggestCode()

=cut
sub suggestCode
{
  my $self = shift;
  my $foo = shift;
  if (defined $self->{types}->{$foo}) { return $self->{types}->{$foo}; }
  return '';
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean selectTypes()

  Routine loads types from sub account type tb into
  object under $self->{metaTypes}

=cut
sub selectTypes
{
  my $self = shift;
  my $records = 0;
  my $l = $self->{langObj};
  $self->{metaTypes} = {  };
  my $sql = "SELECT name, num FROM sub_acct_type_tb";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef;  }
  while (my @data = $result->fetchrow_array)
  {
    $records++;
    $self->{metaTypes}->{$data[1]} = $l->map($data[0]);
  }
  return $self->{metaTypes};
}

sub select
{
  my $self = shift;
  if (!$self->SUPER::select(@_))
  {
    $self->prefixError();
    return $self;
  }
  $self->{data}->{main}->{typeName}->{-Value} = $self->{metaTypes}->{$self->{data}->{main}->{type}->{-Value}};
}

=item string typeName

  Returns the name in the correct language for the type of this sub account.

=cut
sub typeName
{
  my $self = shift;
  return $self->{metaTypes}->{$self->{data}->{main}->{type}->{-Value}};
}

=item integer type

  Returns the type for this object.

=cut
sub type
{
  my $self = shift;
  return $self->{data}->{main}->{type}->{-Value};
}

=item javascriptOverhead(ledgerCode)

  this routine returns a string with all of the javascript overhead needed
  for managing sub accounts such as names, names

=cut
sub javascriptOverhead
{
  my $self = shift;
  my $j;
  my $names =  "SELECT name, num FROM sub_acct_type_tb ORDER BY num;";
  my $sql = "SELECT sub_acc_name AS name, code, sub_acc_number AS number, type FROM sub_acc_tb ORDER BY type, number";
  my $subs;
  my $result = $self->{dbHandle}->read($names);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  while (my $d = $result->fetchrow_hashref)
  {
    $j .= "SUBACCOUNTTYPENAMES['$d->{num}'] = '$d->{name}';\n";
    $subs .= "SUBACCOUNTS['name']['$d->{num}'] = new SubAccount();\n";
    $subs .= "SUBACCOUNTS['code']['$d->{num}'] = new SubAccount();\n";
  }
  $j .= $subs;
  $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  while (my $d = $result->fetchrow_hashref)
  {
    $j .= "SUBACCOUNTS['name']['$d->{type}'].data['$d->{name}'] = '$d->{number}';\n";
    $j .= "SUBACCOUNTS['code']['$d->{type}'].data['$d->{code}'] = '$d->{number}';\n";
  }
  $j.="\n";
  return $j;
}

sub insert { my $self = shift; return $self->insertUpdate('insert', @_); }
sub update { my $self = shift; return $self->insertUpdate('update', @_); }
sub insertUpdate
{
  my $self = shift;
  my $which = shift;
  if ($self->{data}->{main}->{type}->{-Value} < 50)
  {
    $self->error("<p>".$self->{langObj}->map("subAccountType50")."</p>\n".$self->{data}->{main}->{type}->{-Value});
    $self->invalid("type",$self->get('type'), $self->{langObj}->map("subAccountType50")."</p>\n");
    return -1;
  }
  my $id = $self->get('id');
  if ($id ne '') { $id = " AND sub_acc_number <> '$id'"; }
  my $sql = "SELECT sub_acc_name AS name FROM sub_acc_tb WHERE code = '".$self->get('code')."'". $id;
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->invalid('database', $sql, $self->{dbHandle}->errorMessage); return -1; }
  if (my @data = $result->fetchrow_array)
  {
    $self->invalid('code', $self->get('code'), $self->langObj->map('subAccountDuplicate').$data[0]);
    return -1;
  }
  my $return;
  eval('$return = $self->SUPER::'.$which.'(@_);');
  if (!$self->error && $self->get('id') eq '')
  {
    $sql = "SELECT sub_acc_number AS id FROM sub_acc_tb WHERE code = '".$self->get('code')."'";
    $result = $self->dbHandle->read($sql);
    if ($self->{dbHandle}->error) { $self->invalid('database', $sql, $self->{dbHandle}->errorMessage); return -1; }
    if (my $row = $result->fetchrow_hashref)
    {
      $self->set('id', $row->{id});
    }
  }
  return $return;
}

sub selectArrayByType
{
  my $self = shift;
  my $temp = $self->{LIST}[0];
  $self->{LIST}[0] = "SELECT sub_acc_name AS name, sub_acc_number AS id FROM sub_acc_tb WHERE type = #FIELD=type#";
  my @foo = $self->SUPER::selectArray(@_);
  $self->{LIST}[0] = $temp;
  return @foo;
}

sub selectOptionsAllByType
{
  my $self = shift;
  $self->{LIST}[0] = "SELECT sub_acc_name AS name, sub_acc_number AS id FROM sub_acc_tb WHERE type <> 15 AND type <> 70 AND type <> 25 ORDER BY type";
  return $self->SUPER::selectOptions(@_);
}

sub isNew
{
  my $self = shift;
  if ($self->get('id') eq '') { return 1; }
  return 0;
}

sub delete
{
  my $self = shift;
  my $p = $self->langObj;
  my $debitcredit = "SELECT debitcredit_tb.parent_num AS id FROM debitcredit_tb, account_tb WHERE debitcredit_tb.sub_account_num = '".$self->get('id')."' AND debitcredit_tb.account_num = account_tb.number AND NOT account_tb.type IN (0,3,10) ";
  my $result = $self->{dbHandle}->read($debitcredit);
  if ($self->{dbHandle}->error) { $self->invalid('database', $debitcredit, $self->{dbHandle}->errorMessage); return -1; }
  if (my @data = $result->fetchrow_array)
  {
    $self->invalid($p->map('Sub Account'), $self->get('name'), $p->map('in use') );
    return -1;
  }
  return $self->SUPER::delete(@_);
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
