# Payroll.pm - The Object Class that provides a Payroll Object
#$Id: Main.pm,v 1.3 2006/05/15 22:33:29 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::Payroll::Main;
use strict;
use Portal::Accounting::BaseL;
use Portal::Accounting::Payroll::Objects::Form;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.3 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Payroll",
  new => "Payroll",
  save => "Payroll",
);

=head1 NAME

Payroll - Object used to build a Payroll for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Payroll;
  my $obj = Portal::Accounting::Payroll->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This module contains payroll tools.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Payroll module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Payroll State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::Payroll::Main::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }
  return 1;
}

sub prepDoc
{
  my $s = shift;
  my $doc = $s->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $s->{jsDir}."Journal.js");
  return $doc;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  my %args = ( command => "display", message => "", @_ );
  my $command = $args{command};
  my $l = $self->{langObj};
  my $message = $args{message};
  my $doc = $self->prepDoc;

  $doc->setFocus("body");
  my $Employees = $l->map("Employees");
  my $fed = $l->map("Federal 941 Deposit");
  my $fed941 = $l->map("Federal 941");
  my $s941 = $l->map("Missouri 941");
  my $pr = $doc->encodeString('Payroll::Report');
  my $rc = $doc->encodeString('Reports::Customize');
  $doc->print(<<"ENDOF");
  <p>
  <a href="?app=$self->{app}&state=SubAccounts&command=list&ledgerCode=$self->{input}->{ledgerCode}&type=120">$Employees</a><br>
  <a href="?app=$self->{app}&state=Payroll::State&command=display&ledgerCode=$self->{input}->{ledgerCode}">Configure $Employees State Parameters</a><br>
  <a href="?app=$self->{app}&state=JournalEntry&command=federalDepositForm&ledgerCode=$self->{input}->{ledgerCode}">$fed</a><br>
  <a href="?app=$self->{app}&state=$self->{state}&command=new&ledgerCode=$self->{input}->{ledgerCode}">Run Payroll</a><br>
  </p>

  <p><b>Payroll Reports</b><br>
  <a href="?app=$self->{app}&state=$rc&ledgerCode=$self->{input}->{ledgerCode}&command=display&id=16">W2</a><br>
  <a href="?app=$self->{app}&state=$pr&command=form&report=Fed941&ledgerCode=$self->{input}->{ledgerCode}">$fed941</a><br>
  <a href="?app=$self->{app}&state=$pr&command=form&report=MO941&ledgerCode=$self->{input}->{ledgerCode}">$s941</a>
  </p>
  
ENDOF
  return $doc;
}

sub c_save
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Payroll::Objects::Form->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode} );
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef; }
  my $doc = $self->SUPER::c_save(@_);
  if (!$self->error)
  {
    if (defined $self->{OBJECT}->{JE} && scalar @{$self->{OBJECT}->{JE}} > 0)
    {
      my @jes = @{$self->{OBJECT}->{JE}};
      my $count = scalar @jes;
      $doc->print("<div>$count Payroll Entries made.  You should edit these as needed.  (i.e. to add check numbers)</div>");
      for (my $i = 0 ; $i < $count; $i++)
      {
        my $je = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode} );
        if ($i == 0) { $doc->print($je->printHeaders); }
        $je->set('id', $jes[$i]);
        $je->select;
        if ($je->error) { $self->error($je->errorMessage); }
        else
        {
          $doc->print($je->printForm);
        }
      }
    }
    else
    {
      $doc->print("<div>No Payroll Entries made</div>");
    }
    $doc->print('<div><a href="?'.$self->getInfo.'"></div>');
  }
  $doc->setOnload(replace => 1, code => '');
  return $doc;
}

sub c_new
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Payroll::Objects::Form->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode} );
  my $doc = $self->prepDoc;
  if ($self->{OBJECT}->error) { $doc->print($self->{OBJECT}->errorMessage); }
  $doc->print($self->{OBJECT}->printForm);
  if ($self->{OBJECT}->error) { $doc->print($self->{OBJECT}->errorMessage); }
#  $doc->print($self->{OBJECT}->printHTML);
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut