# Employee.pm - The Object Class that provides a Employee Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Employee.pm,v 1.6 2006/05/12 22:09:55 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Payroll::Objects::Employee;
use strict;
use Portal::Accounting::Objects::SubAccount;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::SubAccount Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.6 $';

=head1 NAME

Employee - Object used to build a Employee Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Payroll::Objects::Employee;
  my $obj = Portal::Accounting::Payroll::Objects::Employee->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Employee is a Employee class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::Objects::Employee module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', count => '', @_ );

  if ($self->error)
  {
    $self->error();
    return $self;
  }

  # instantiate anything unique to this module
  my $count = $args{count};
  $self->{name} = "Portal::Accounting::Payroll::Objects::Employee";
  my $l = $self->{langObj};
  $self->{data}->{main}->{ledgerCode} = { -Value => $args{ledgerCode}, -Type => "hidden"};
  $self->{data}->{main}->{name}->{-Label} = $l->map("Initials");
  $self->{data}->{main}->{fullname} = { -Label => $l->map("Name"), -Type => "text", -DataType => "text", -onload => 'this.select();', maxlength => 50, };
  $self->{data}->{main}->{gross} = { -Label => $l->map("Normal Gross Pay"), -Type => "text", -DataType => "float" };
  $self->{data}->{main}->{type} = { -Label => $l->map("Employee"), -Value => 120, -Type => "hidden"};
  $self->{data}->{main}->{married} = { -Label => $l->map("Married"), -Type => "checkbox", -Value => 1, -Boolean => 1, -DataType => 'boolean',};
  $self->{data}->{main}->{active} = { -Label => $l->map("Active"), -Type => "checkbox", -Value => 1, checked => 1, -Boolean => 1, -DataType => 'boolean',};

  # do anything else you might need to do.
  my $Submit = $l->map("Save");
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  <div class="title">#LABEL=type#</div>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  #FIELD=ledgerCode#
  #FI=type#
  <div class="primary">
    <div>#LABEL=fullname##FIR=fullname#</div>
    <div>#LABEL=name##FIR=name#</div>
    <div>#LABEL=code##FIR=code#</div>
    <div>#LABEL=gross##FIR=gross#</div>
    <div>#LABEL=married##FIR=married#</div>
    <div>#LABEL=active##FIR=active#</div>
    <input type="submit" name="Submit" value="$Submit">
  </div>
</form>
#FORMREQUIREDSTRING#
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code type fullname) ],
    optional => [ qw( gross married active ) ],
    constraints => {
      id => qr/^\d\d+$/,
      type => qr/^120$/,
      name => qr/^[\d\w][\d\s\w&,-]*$/,
      code => qr/^[\d\w.-]+$/,
      gross => qr/^\d*\.\d{2}$/,
    },
  };

  $self->{data}->{payroll}->{id} = { -Type => "hidden"};
  $self->{data}->{payroll}->{name} = { -Label => $l->map("Initials"), -Type => "text", -DataType => "text" };
  $self->{data}->{payroll}->{fullname} = { -Label => $l->map("Name"), -Type => "text", -DataType => "text" };
  $self->{data}->{payroll}->{code} = { -Label => $l->map("Code"), -Type => "text", -DataType => "text" };
  $self->{data}->{payroll}->{ledgerCode} = { -Value => $args{ledgerCode}, -Type => "hidden"};
  $self->{data}->{payroll}->{gross} = { -Label => $l->map("Normal Gross Pay"), -Type => "text", -DataType => "float" };
  $self->{data}->{payroll}->{type} = { -Label => $l->map("Employee"), -Value => 120, -Type => "hidden"};
  $self->{data}->{payroll}->{married} = { -Label => $l->map("Married"), -Type => "checkbox", -Value => 1, -Boolean => 1, -DataType => 'boolean',};
  $self->{data}->{payroll}->{cache} = { -Label => $l->map("Cache"), -Type => "checkbox", -Value => 1, -Boolean => 1, -DataType => 'boolean',};

  $self->{defaultForm} = "main";
  ##db infrastructure
  $self->{INSERTorder}[@{$self->{INSERTorder}}] = [ qw( ledger_code fullname gross married active ) ] ;  #insert order for db column names
  $self->{newOnly}[@{$self->{newOnly}}] = { id => "id" };
  $self->{db2data}[@{$self->{db2data}}] = { ledger_code => 'ledgerCode' };  #map database columns to ->data hash, not specified assumes same name
  my $index = (scalar @{$self->{data2db}}) - 1;
  $self->{data2db}[$index] = {reverse %{$self->{db2data}[$index]}};

  $self->{SELECT}[@{$self->{SELECT}}] = "SELECT ##COLUMNS## FROM employees_tb WHERE employees_tb.id = #FIELD=$self->{id}#";
  $self->{INSERT}[@{$self->{INSERT}}] = "INSERT INTO employees_tb (id, ##COLUMNS##) VALUES ((SELECT sub_acc_number FROM sub_acc_tb WHERE sub_acc_name = '#FIELD=name#' AND code = '#FIELD=code#'), ##VALUES##)";
  $self->{UPDATE}[@{$self->{UPDATE}}] = "UPDATE employees_tb SET ##UPDATE## WHERE id = '#FIELD=$self->{id}#'";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[@{$self->{DELETE}}] = "DELETE FROM employees_tb WHERE id = #FIELD=$self->{id}#";
  $self->{DELETE}[@{$self->{DELETE}}] = "DELETE FROM state_payroll WHERE employee = #FIELD=$self->{id}#";
  #find out ifis this being referenced from other tables before deleting
  $self->{LIST}[0] = "SELECT s.sub_acc_name AS name, s.sub_acc_number AS id FROM sub_acc_tb AS s, employees_tb AS e WHERE s.sub_acc_number = e.id AND type = #FIELD=type# AND ledger_code = '$args{ledgerCode}' ORDER BY s.sub_acc_name";

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  if ($self->get('ledgerCode') eq '') { $self->missing('ledgerCode'); }
  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub delete
{
  my $self = shift;
  #check to see if this sub is used in debitcredit_tb
  my $p = $self->langObj;
  my $sql = "SELECT parent_num AS id FROM debitcredit_tb WHERE sub_account_num = ". $self->get('id');
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->invalid('database', $sql, $self->{dbHandle}->errorMessage); return -1; }
  if (my @data = $result->fetchrow_array)
  {
    $self->invalid($p->map('Employee'), $self->get('name'), $self->langObj->map('in use').":".$data[0] );
    return -1;
  }
  my $foo = $self->SUPER::delete(@_);
  return $foo;
}

sub insert
{
  my $self = shift;
  #check for code already existing
  #dont worry about name
  return $self->SUPER::insert(@_);
}

=back

=cut

1;
__END__

=head1 NOTE

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
