#$Id: Form.pm,v 1.2 2006/05/12 14:35:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::Payroll::Objects::Form;
use strict;
use Portal::Accounting::Payroll::Objects::EmployeePayroll;
use Portal::Accounting::Objects::SubEntity;
use Portal::Accounting::Objects::Account;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.2 $';

=head1 NAME

Form - Object used to build a Form Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Payroll::Objects::Form;
  my $obj = Portal::Accounting::Payroll::Objects::Form->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Form is a Form class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(ticker, name, number, type, market)

 Creates a new instance of the Portal::Accounting::Objects::Form module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', @_ );

  if ($self->error)
  {
    $self->error();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{name} = "Portal::Accounting::Payroll::Objects::Form";
  $self->{new} = 1;  #after populate it is no longer new
  $self->{company} = 10; #company sub account for matching
  my $l = $self->{langObj};

  $self->{SUBENTITY} = Portal::Accounting::Objects::SubEntity->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $args{ledgerCode});
  $self->{ACCOUNT} = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $args{ledgerCode});
  my $subEntity = $self->{SUBENTITY}->selectOptions;
  $self->{ACCOUNT}->set('type', '0');
  my $banks = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '55');
  my $salary = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '56');
  my $ficaE = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '57');
  my $medicareE = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '26');
  my $fica = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '27');
  my $medicare = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '25');
  my $sitw = $self->{ACCOUNT}->selectOptionsByType;
  $self->{ACCOUNT}->set('type', '24');
  my $fitw = $self->{ACCOUNT}->selectOptionsByType;
  if ($self->{ACCOUNT}->error) { $self->error($self->{ACCOUNT}->errorMessage); return $self; }
  if ($self->{SUBENTITY}->error) { $self->error($self->{SUBENTITY}->errorMessage); return $self; }
  my @date = $self->getDateArray;
  my $period = {
    names => [qw(annual semiannual quarterly monthly semimonthly biweekly weekly daily)],
    values => [qw(annual semiannual quarterly monthly semimonthly biweekly weekly daily)],
  };
  my $states = {
    names => [qw(MO)],
    values => [qw(MO)],
  };
  my @today = Today;
  my $today = "$today[0]-".sprintf("%02d", $today[1])."-".sprintf("%02d", $today[2]);
  # do anything else you might need to do.
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Payroll::Main" },
    "command" => { -Type => "hidden", -Value => "save" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "date" => { -Type => "date-picker", size => "7", -Label => $l->map("Date"),
      -Value => $today,   -year => $date[0], -DataType => 'string',
    },
    "start" => { -Type => "date-picker", size => "7", -Label => $l->map("Period Start Date"),
      -Value => '',   -year => $date[0], -DataType => 'string',
    },
    "end" => { -Type => "date-picker", size => "7", -Label => $l->map("Period End Date"),
      -Value => '',   -year => $date[0], -DataType => 'string',
    },
    "round" => { -Type => "checkbox", -Value => 1, -DataType => 'boolean', -Label => 'Round off cents where possible', checked => 1},
    "subEntity" => { -Type => 'select', -Options => $subEntity, -DataType => 'string', -Label => 'SubEntity',},
    "fica" => { -Type => 'select', -Options => $fica, -DataType => "integer", -Label => 'FICA(SS) Liability Account',},
    "medicare" => { -Type => 'select', -Options => $medicare, -DataType => "integer", -Label => 'Medicare Liability Account', },
    "ficaExp" => { -Type => 'select', -Options => $ficaE, -DataType => "integer", -Label => 'FICA(SS) Expense Account',},
    "medicareExp" => { -Type => 'select', -Options => $medicareE, -DataType => "integer", -Label => 'Medicare Expense Account', },
    "fitw" => { -Type => 'select', -Options => $fitw, -DataType => "integer",  -Label => 'Federal Withholding Account',},
    "sitw" => { -Type => 'select', -Options => $sitw, -DataType => "integer", -Label => 'State Withholding Account', },
    "salary" => { -Type => 'select', -Options => $salary, -DataType => "integer", -Label => 'Salary Expense Account', },
    "bank" => { -Type => 'select', -Options => $banks, -DataType => "integer", -Label => 'Checking Account', },
    "states" => { -Type => 'select', -Options => $states, -DataType => "text", -Label => 'State', },
    "period" => { -Type => 'select', -Options => $period, -DataType => "integer", -Label => 'Period', -Value => $self->defaultPeriod($args{ledgerCode}), },
    "submitButton" => { -Type => 'button', -Label => 'Run Payroll', -Value => 'Run Payroll', onclick => 'this.disabled = true; this.form.submit();'},
    "rows" => {-Type => 'hidden', -DataType => 'int', -Value => 0 },
  };
  $self->{css}->{main} .= <<"ENDOF";
div { padding: .3em; }
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIR=app#
  #FIR=state#
  #FIR=command#
  #FIR=ledgerCode#
  <div class="primary">
    <div style="float: right;">#FIELD=submitButton#
      <table>
HEADERS
        <tbody>
BODY
        </tbody>
      </table>
      <input type="hidden" name="rows" value="COUNT" />
      #FIELD=submitButton#
    </div>
    <div>#FORMREQUIREDSTRING#</div>
    <div>#LABEL=date# #FIR=date#</div>
    <div>#LABEL=period# #FIR=period#</div>
    <div>#LABEL=start# #FIR=start#</div>
    <div>#LABEL=end# #FIR=end#</div>
    <div>#LABEL=subEntity# #FIR=subEntity#</div>
    <div>#LABEL=bank# #FIR=bank#</div>
    <div>#LABEL=salary# #FIR=salary#</div>
    <div>#LABEL=ficaExp# #FIR=ficaExp#</div>
    <div>#LABEL=medicareExp# #FIR=medicareExp#</div>
    <div>#LABEL=fica# #FIR=fica#</div>
    <div>#LABEL=medicare# #FIR=medicare#</div>
    <div>#LABEL=fitw# #FIR=fitw#</div>
    <div>#LABEL=sitw# #FIR=sitw#</div>
    <div>#LABEL=states# #FIR=states#</div>
  </div>
</form>  
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date period fica medicare fitw sitw states start end) ],
    optional => [ qw( ) ],
    constraints => {
      date => qr/^\d{4}-\d{2}-\d{2}$/,
      start => qr/^\d{4}-\d{2}-\d{2}$/,
      end => qr/^\d{4}-\d{2}-\d{2}$/,
    },
  };
  
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  my $date = $self->get('date');
  if ($date =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/)
  {
    $self->set('date', "$1-".sprintf("%02d",$2)."-".sprintf("%02d",$3) );
  }
  my $start = $self->get('start');
  if ($start =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/)
  {
    $self->set('start', "$1-".sprintf("%02d",$2)."-".sprintf("%02d",$3) );
  }
  my $end = $self->get('end');
  if ($end =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/)
  {
    $self->set('end', "$1-".sprintf("%02d",$2)."-".sprintf("%02d",$3) );
  }
  
  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  if ($self->get('ledgerCode') eq '') { $self->missing('ledgerCode'); }
  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub getEmployees
{
  my $self = shift;
  my $o = Portal::Accounting::Payroll::Objects::EmployeePayroll->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
  if ($o->error) { $self->error($o->errorMessage); return 0; }
  my %list = $o->selectHash;
  $self->{EMPLOYEES} = \%list;
  if ($o->error) { $self->error($o->errorMessage); return 0; }
  my @list = sort keys %list;  #use keys from hash to reference object but use for loop to start index at 0 and go to number of rows
  if (!$self->isNew)
  {
    #has user already changed data that goes with this employee?  if so update object.
    #walk all rows in main and push data into objects
    for (my $j = 0; $j < $self->get('rows'); $j++)
    {
      my $id = $self->get("id_$j");
      $self->{EMPLOYEES}->{$id}->set("gross", $self->get("gross_$j"));
      if ($self->{EMPLOYEES}->{$id}->error) { $self->error($self->{EMPLOYEES}->{$id}->errorMessage); }
    }
  }
  return 1;  
}

sub printForm
{
  my $self = shift;
  my %form = $self->SUPER::printForm(@_);
  if ($self->error) { return %form; }
  $self->getEmployees;
  my $count = 0;
  my $foo;
  my $last;
  foreach my $key (sort keys %{$self->{EMPLOYEES}})
  {
    if (!$self->{EMPLOYEES}->{$key}->{data}->{main}->{active}->{checked}) { next; }
    my %f = $self->{EMPLOYEES}->{$key}->printForm(form => 'payroll', count => $count++);
    if ($self->{EMPLOYEES}->{$key}->error) { $self->error($self->{EMPLOYEES}->{$key}->errorMessage); }
    $foo .= $f{body};
    $last = $key;
  }
  my $headers = $self->{EMPLOYEES}->{$last}->{headers}->{payroll};
  $form{body} =~ s/HEADERS/$headers/;
  $form{body} =~ s/BODY/$foo/;
  my $rows = $count;
  $form{body} =~ s/COUNT/$rows/;
  return %form;
}

sub isNew { my $self = shift; return $self->{new}; }

sub populate
{
  my $self = shift;
  $self->{new} = 0;
  my %args = (rows => 0, @_);
  for (my $i = 0; $i < $args{rows}; $i++)
  {
    #create placeholders for each row so that it will be populated
    $self->{data}->{main}->{"id_$i"} = { -Type => 'text', -DataType => "text", };
    $self->{data}->{main}->{"gross_$i"} = { -Type => 'text', -DataType => "text", };
    #add profile entry here
  }
  return $self->SUPER::populate(@_);
}

=item integer update

  This does the actual processing of payroll data and creates journal entries.
  The journal Entries are cached in an array @{$self->{JE}}.
  
=cut
sub update
{
  my $self = shift;
  $self->getEmployees;
  if ($self->error) { return undef; }
  my $rows = $self->get('rows');
  my $count = 0;
#process each employee
#if row is empty skip it
#build up xml with each employee object
#run calculations through payroll modules
#return data hash
  use Business::Payroll;
  use Portal::Accounting::Objects::JournalEntry;
  my $bp = Business::Payroll->new;
  my $subEntity = $self->get('subEntity');
  my $today = $self->getCurrentDate;
  my $period = $self->get('period');
  my $date = $self->get('date');
  my $start = $self->get('start');
  my $end = $self->get('end');
  $start =~ s/-//g;
  $end =~ s/-//g;
  $date =~ s/-//g;
  my $xml = <<"ENDOF";
<?xml version="1.0" encoding="ISO-8859-1"?>
  <payroll type="raw" version="1.1" date="$date" period="$period" genSysId="XIWA $today" startPeriod="$start" endPeriod="$end">
ENDOF
  for (my $i = 0; $i < $self->get('rows'); $i++)
  {
    #check that this employee has the chosen state entries?
    my $id = $self->get("id_$i");
    my $gross = $self->get("gross_$i");
    next if ($gross eq '');
    if (!$self->{EMPLOYEES}->{$id}->checkEmployee(state => 'US'))
    {
      $self->error('<div class="error">No records have been configured for the US and employee "'.$self->{EMPLOYEES}->{$id}->get('name').'"</div>');
      return undef;
    }
    if (!$self->{EMPLOYEES}->{$id}->checkEmployee(state => $self->get('states')))
    {
      $self->error('<div class="error">No records have been configured for this state and employee "'.$self->{EMPLOYEES}->{$id}->get('name').'"</div>');
      return undef;
    }
    if ($self->get("gross_$i") ne '')
    {
      $xml .= $self->{EMPLOYEES}->{$id}->updatePayroll(state => $self->get('states'), date => $self->get('date'));
      if ($self->{EMPLOYEES}->{$id}->error) { $self->error($self->{EMPLOYEES}->{$id}->errorMessage); }
    }
  }
  $xml .= "  </payroll>\n";
  if ($self->error) { return undef; }
#$self->error("<textarea cols=160 rows=15>$xml</textarea>\n"); return undef;
  
  my $data = $bp->process(string => $xml);
  
#create JournalEntry (Check) from data for each person
  my $foo;
  my $medicare = $self->get('medicare');
  my $medicareExp = $self->get('medicareExp');
  my $fica = $self->get('fica');
  my $ficaExp = $self->get('ficaExp');
  for (my $p = 0; $p < scalar @{$data->{persons}}; $p++)
  {
    my @rows;
    $count++;
    my $je = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
    if ($je->error) { $self->error($je->errorMessage); return undef; }
    my $id = $data->{persons}[$p]{id};
    for (my $i = 0; $i < @{$data->{persons}[$p]{items}}; $i++ )
    {
      my $name = $data->{persons}[$p]{items}[$i]{name};
      my $value = $data->{persons}[$p]{items}[$i]{value};
      if ($name eq 'US FICA')
      {
        my $row1 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        my $row2 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        my $row3 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        #fix to use populate so d_amount will be set!!! but why didn't isValid catch this?~~~!!!!
        $row1->populate('account' => $fica,
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity,
        );
        if ($row1->error) { $self->error($row1->errorMessage); }
        #match with company
        $row2->populate('account' => $fica,
          'subAccount' => $self->{company},
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity,
        );
        if ($row2->error) { $self->error($row2->errorMessage); }
        #company matching is an expense
        $row3->populate('account' => $ficaExp,
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'true',
          'subEntity' => $subEntity,
        );
        if ($row3->error) { $self->error($row3->errorMessage); }
        push(@rows, $row1);
        push(@rows, $row2);
        push(@rows, $row3);
      }
      elsif ($name eq 'US Medicare')
      {
        my $row1 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        my $row2 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        my $row3 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        $row1->populate('account' => $medicare,
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity,
        );
        if ($row1->error) { $self->error($row1->errorMessage); }
        #match with company
        $row2->populate('account' => $medicare,
          'subAccount' => $self->{company},
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity,
        );
        if ($row2->error) { $self->error($row2->errorMessage); }
        #company matching is an expense
        $row3->populate('account' => $medicareExp,
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'true',
          'subEntity' => $subEntity,
        );
        if ($row3->error) { $self->error($row3->errorMessage); }
        push(@rows, $row1);
        push(@rows, $row2);
        push(@rows, $row3);
      }
      elsif ($name eq 'US Federal')
      {
        my $row1 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        $row1->populate('account' => $self->get('fitw'),
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity
        );
        if ($row1->error) { $self->error($row1->errorMessage); }
        push(@rows, $row1);
      }
      elsif ($name eq 'US MO' || $name eq 'US AZ')
      {
        my $row1 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        $row1->populate('account' => $self->get('sitw'),
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity
        );
        if ($row1->error) { $self->error($row1->errorMessage); }
        push(@rows, $row1);
      }
      elsif ($name eq 'gross')
      {
        my $row1 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        $row1->populate('account' => $self->get('salary'),
          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'true',
          'subEntity' => $subEntity
        );
        if ($row1->error) { $self->error($row1->errorMessage); }
        push(@rows, $row1);
      }
      elsif ($name eq 'net')
      {
        my $row1 = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
        $row1->populate('account' => $self->get('bank'),
#          'subAccount' => $id,
          'amount' => abs($value),
          'dorc' => 'false',
          'subEntity' => $subEntity
        );
        if ($row1->error) { $self->error($row1->errorMessage); }
        @rows = ($row1, @rows);
      }
    }
    $je->set('date', $self->get('date'));
    $je->set('payee', $data->{persons}[$p]{name});
    $je->set('journal', 'CP');
    $je->set('date', $self->get('date'));
    if ($je->error) { $self->error($je->errorMessage); return undef; }
    for (my $i = 0; $i < scalar @rows; $i++)
    {
      $foo .= "<div>$i<br>";
      if (!$je->addRow($rows[$i]))
      {
        $foo .= $je->errorMessage;
      }
      $foo .= "</div>";
    }
    $je->insert(commit => 0);
    if ($je->error) { $self->error($je->errorMessage); }
    else { push @{$self->{JE}}, $je->get('id'); }
#$self->error($je->printHTML);
  }
  
  if ($self->error) { $self->dbHandle->rollback; }
  else { $self->dbHandle->commit; }
  return $count;
}

=item string defaultPeriod(ledgerCode)

=cut
sub defaultPeriod
{
  my $self = shift;
  my $lc = shift;
  if ($lc eq '') { $self->error("<div>No ledgerCode sent to defaultPeriod</div>"); return ''; }
  my $sql = "SELECT payroll_period FROM ledger_tb WHERE code = ?";
  my $sth = $self->dbHandle->read(sql => $sql, plug => [$lc]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); }
  elsif (my @info = $sth->fetchrow_array)
  {
    return $info[0];
  }
  return '';
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
