# Reports::Editor - This program is the editor for the reporting system.
# It will either generate the create/edit report screens.
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Reports::Editor;
use strict;
use Portal::Accounting::Reports::Main;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

use HTMLObject::Normal;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Storable;  # the module that will allow me to dump data to disk and recover later.  #!!!! may need to convert to sending parameter.  was using a cookie for the file name
use POSIX qw(tmpnam); # the tmpnam function which will return a file name for a temporary file supposed to be unique.
require Exporter;

@ISA = qw(Portal::Accounting::Reports::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.23 $';

# global variables.

# modify to represthe commands this state provides.
my %commands = (
  display => "Report Editor",       # List all created reports to allow the user to select one to edit/delete
  'create_report' => 'Create Report',       # Start a new edit session that will create a report.
  'create_report2' => 'Create Report',      # Takes the default settings from 'create_report' and allows the user to specify the contents of the report in the top frame.  Creates the entry in report_type_tb.
  'copy_report' => 'Copy Report',         # Takes the id and new_name for the specified report and copies the report_type_tb and report_data_tb entries for the old report, naming it as the new report name.
  'update_report' => 'Update Report',       # Takes the new report or edited data and updates the entries in report_data_tb in the database.
  'edit_report' => 'Edit Report',         # Takes the selected report and allows the user to edit the date in report_type_tb.  Generates the frameset.
  'edit_report1' => 'Edit Report',        # displays the first screen in the frameset.
  'delete_report' => 'Delete Report',       # Takes the selected report and deletes it from the database.
  'edit_report2' => 'Edit Report',        # Updates the edited default data in report_type_tb and then allows the user to edit the contents of the report in the top frame.
  'edit_report2_top' => 'Edit Report',    # Creates the form and code in the top frame to actually work with the report contents.
  'saveReport' => 'Save Report',       # Takes the new report or edited data and updates the entries in report_data_tb in the database.  Returns ajax document
);

my %errors = (
  4000 => 'Eval of HTMLObject::FrameSet failed',
  4001 => 'Invalid command specified',
  4002 => 'Eval of HTMLObject::Normal failed',
  4003 => 'command must be specified',
  4004 => "Couldn't connect to Database",
  4005 => "Parameter requires a value",
  4006 => "Invalid Value for Parameter",
  4007 => "DBI Preparation of SQL Statement Failed",
  4008 => "DBI Execution of Statement Failed",
  4009 => "DBI Error Occurred",
  4010 => "Column Header Already Used",
  4011 => "Invalid Column Header",
  4012 => "Invalid use of Column Header",
  4013 => "Could not find Sub Account",
  4014 => "Could not find Sub Account Type",
  4015 => "Report Name already used",
  4016 => "Report ID already used",
  4017 => "Report ID is required",
  4018 => "Error opening file",
  4019 => "Error Committing Database changes",
  4020 => "Could not find Report ID",
  4021 => "Count not returned from Database",
  4022 => "Ledger Code required",
  4023 => "Could not find Ledger Database",
  4024 => "Could not find Ledger Name",
  4025 => "Could not find Ledger Code",
  4026 => "Ledger DB Name required",
  4027 => "Ledger Name required",
  4028 => "Database Handle required",
  4029 => "Memorized value required",
  4030 => "Invalid ID format",
  4031 => "Invalid Item in Row",
  4032 => "Item missing from Row",
  4033 => "Invalid Value for Row Item",
  4034 => "Parameter Required",
  4035 => "File Name required",
);

my %sql_statements = (
  1000 => "SELECT num, name FROM sub_acct_type_tb ORDER BY name",
  1001 => "SELECT sub_acc_number, sub_acc_name FROM sub_acc_tb WHERE type = ? ORDER BY sub_acc_number",
  1002 => "INSERT INTO report_type_tb (name, report_class, report_type, column_variable, column_variable_data, date_range, display_dates, max_columns, min_columns, text_font, text_size, data_font, data_size, total_column, underline_length, editable) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, 'true')",
  1003 => "SELECT id FROM report_type_tb WHERE name = ?",
  1004 => "UPDATE report_type_tb SET name = ?, report_class = ?, report_type = ?, column_variable = ?, column_variable_data = ?, date_range = ?, display_dates = ?, max_columns = ?, min_columns = ?, text_font = ?, text_size = ?, data_font = ?, data_size = ?, total_column = ?, underline_length = ? WHERE id = ?",
  1005 => "SELECT name FROM report_type_tb WHERE id = ?",
  1006 => "SELECT name, report_class, report_type, column_variable, column_variable_data, date_range, display_dates, max_columns, min_columns, text_font, text_size, data_font, data_size, total_column, underline_length, editable FROM report_type_tb WHERE id = ?",
  1008 => "DELETE FROM report_data_tb WHERE id = ?",
  1009 => "DELETE FROM report_type_tb WHERE id = ?",
  1014 => "SELECT count(name) FROM memorized_report_tb WHERE id = ?",
  1017 => "SELECT code FROM ledger_tb WHERE name = ?",
  1018 => "SELECT color FROM ledger_tb WHERE code = ?",
  1019 => "SELECT name, num FROM acct_type_tb ORDER BY name, num",
  1020 => "SELECT name, number, type, active FROM account_tb ORDER BY name, number, type, active",
  1021 => "SELECT sub_acc_name, sub_acc_number, type FROM sub_acc_tb ORDER BY sub_acc_name, sub_acc_number, type",
  1022 => "SELECT DISTINCT(report_type_tb.id), report_type_tb.name, report_type_tb.editable FROM report_type_tb, report_data_tb WHERE report_type_tb.report_type = 'd' AND report_type_tb.id = report_data_tb.id ORDER BY report_type_tb.name, report_type_tb.id",
  1025 => "DELETE FROM report_data_tb WHERE id = ?",
  1026 => "INSERT INTO report_data_tb (id, index, cell_text, cell_type, item_name, display_level, initial_state, negative_format, other_attributes) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
  1029 => "INSERT INTO temp_file_tb (timeof, fname) VALUES (now(), ?)",
  1030 => "DELETE FROM memorized_report_tb WHERE id = ? AND ledger_code = ? AND name = ?",
  1033 => "SELECT name, code FROM sub_entity_tb WHERE ledger_code = ?",
  1034 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
  1036 => "SELECT name, id, ledger_code FROM memorized_report_tb ORDER BY ledger_code, name, id",
  1037 => "SELECT name FROM memorized_report_tb WHERE id = ? AND name = ? AND ledger_code = ?",
  1038 => "SELECT editable FROM report_type_tb WHERE id = ?",
  1040 => "SELECT name, id, editable FROM report_type_tb WHERE report_type = 'r' ORDER BY name, id",
  1041 => "SELECT name, id, editable FROM report_type_tb WHERE report_type = 'd' ORDER BY name, id",
  1042 => "SELECT name FROM acct_type_tb WHERE num = ?",
  1043 => "SELECT debit_increases FROM acct_type_tb WHERE num = ?",
  1044 => "SELECT number FROM account_tb WHERE type = ? ORDER BY number",
  1045 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = true AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
  1046 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = false AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
  1047 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
  1048 => "SELECT name FROM account_tb WHERE number = ?",
  1049 => "SELECT num FROM acct_type_tb ##?## ORDER BY num",
  1054 => "SELECT sub_acc_name, sub_acc_number FROM sub_acc_tb WHERE type IN (##1?##) ORDER BY ##2?##",
  1055 => "SELECT sub_acct_type FROM acct_type_tb WHERE num = ?",
  1056 => "SELECT account_tb.type, acct_type_tb.debit_increases FROM account_tb, acct_type_tb WHERE account_tb.number = ? AND acct_type_tb.num = account_tb.type",
  1057 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
  1058 => "SELECT name FROM sub_acct_type_tb WHERE num = ?",
  1059 => "SELECT type FROM sub_acc_tb WHERE sub_acc_number = ?",
  1060 => "SELECT num, sub_acct_type FROM acct_type_tb",
  1064 => "SELECT report_type FROM report_type_tb WHERE id = ?",
  1067 => "SET datestyle TO 'ISO'",
  1072 => "SELECT working_year(?)",
);

my %report_classes = ( "custom" => "Custom", "cash" => "Cash Flow", "income" => "Income Statement", "balance" => "Balance Sheet", "acct_balance" => "Accounts Balance Sheet", "acct_activity" => "Accounts Activity Sheet" );
my @report_classes = ( "custom", "cash", "income", "balance", "acct_balance", "acct_activity" );
my %report_types = ( "r" => "Regular Report", "d" => "Detailed View" );
my @report_types = ( "r", "d" );
my @report_levels = ( "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "begin_sub_account_balance", "end_sub_account_balance" );
my @account_type_levels = ( "nothing", "accounts", "accounts, sub-accounts" );
my @account_levels = ( "nothing", "sub-accounts" );
my @accounts_of_levels = ( "nothing", "sub-accounts" );
my @sub_account_levels = ( "nothing", "accounts" );
my @sub_accounts_of_type_levels = ( "nothing", "accounts" );
my @sub_accounts_of_account_levels = ( "not available" );
my %report_levels = ( "account_type" => \@account_type_levels, "account" => \@account_levels, "sub_account" => \@sub_account_levels, "accounts_of" => \@accounts_of_levels, "sub_accounts_of_type" => \@sub_accounts_of_type_levels, "sub_accounts_of_account" => \@sub_accounts_of_account_levels, "begin_account_type_balance" => \@account_type_levels, "end_account_type_balance" => \@account_type_levels, "begin_account_balance" => \@account_levels, "end_account_balance" => \@account_levels, "begin_sub_account_balance" => \@sub_account_levels, "end_sub_account_balance" => \@sub_account_levels );
my %regular_cell_types = ( "text" => "Text", "total" => "Total", "compare" => "Compare Rows", "account_type" => "Account Type", "account" => "Account", "sub_account" => "Sub Account", "accounts_of" => "Accounts of Account Type", "sub_accounts_of_type" => "Sub Accounts of Account Type", "sub_accounts_of_account" => "Sub Accounts of Account", "begin_account_type_balance" => "Beginning Account Type Balance", "end_account_type_balance" => "Ending Account Type Balance", "begin_account_balance" => "Beginning Account Balance", "end_account_balance" => "Ending Account Balance", "begin_sub_account_balance" => "Beginning Sub Account Balance", "end_sub_account_balance" => "Ending Sub Account Balance", "line" => "Line" );
my @regular_cell_types = ( "text", "total", "compare", "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "line" ); # "begin_sub_account_balance", "end_sub_account_balance", (to be added later)
my %detailed_cell_types = ( "text" => "Text", "line" => "Line", "detail" => "Detail" );
my @detailed_cell_types = ( "text", "line", "detail" );
my @font_names = ( "helvetica" );
my %font_names = ( "helvetica" => "Helvetica" );
my @font_sizes = ( "+2", "+1", "+0", "-1", "-2" );
my %font_sizes = ( "+2" => "+2", "+1" => "+1", "+0" => "nothing", "-1" => "-1", "-2" => "-2" );
my %column_names = ( 0 => "Zero", 1 => "One", 2 => "Two", 3 => "Three", 4 => "Four", 5 => "Five", 6 => "Six", 7 => "Seven", 8 => "Eight", 9 => "Nine", 10 => "Ten", 11 => "Eleven", 12 => "Twelve", 13 => "Thirteen", 14 => "Fourteen", 15 => "Fifteen", 16 => "Sixteen", 17 => "Seventeen", 18 => "Eighteen", 19 => "Nineteen", 20 => "Twenty", 21 => "Twenty One", 22 => "Twenty Two", 23 => "Twenty Three", 24 => "Twenty Four", 25 => "Twenty Five", 26 => "Twenty Six", 27 => "Twenty Seven", 28 => "Twenty Eight", 29 => "Twenty Nine", 30 => "Thirty", 31 => "Thirty One", 32 => "Thirty Two", 33 => "Thirty Three", 34 => "Thirty Four", 35 => "Thirty Five", 36 => "Thirty Six" );
my @column_headers = ( "sub_entities", "sub_accounts", "sub_account_types" );
my %column_headers = ( "sub_entities" => "Sub Entities", "sub_accounts" => "Sub Accounts", "sub_account_types" => "Sub Account Types" );
my @negative_formats = ( "0", "1", "2" );
my %negative_formats = ( "0" => "<font color=\"red\">(xxx.xx)</font>", "1" => "<font color=\"red\">-</font>xxx.xx", "2" => "xxx.xx<font color=\"red\">-</font>" );
my $max_dates = 36;   # the maximum number of columns that we are currently supporting in the reporting system.
my $min_underline_length = 5;  # the min and max lengths that underlines can range within.
my $max_underline_length = 25;
my $default_underline_length = -1;
my %t_or_f = ( 0 => "false", 1 => "true", "true" => "true", "false" => "false" );  # a mapping that will allow us to map 0 to false and 1 to true for the sql statements.
my %true_false = ( "true" => 1, "false" => 0, 1 => 1, 0 => 0 );

# commonly used compiled regular expressions
my $editReportWindowRE = qr/^(edit_report_window)$/;
my $operatorsRE = qr/^(\+|-|\*|\/)$/;
my $openParenRE = qr/^\($/;
my $closeParenRE = qr/^\)$/;
my $tokenRE = qr/^R\d+|-?\d+\.\d+|-?\d+$/;
my $rowIdentifierRE = qr/^R\d+$/;
my $substitutionRE = qr/\#\#\?\#\#/;
my $firstSubstitutionRE = qr/\#\#1\?\#\#/;
my $secondSubstitutionRE = qr/\#\#2\?\#\#/;

# turn on debugging in Apache::DBI
#$Apache::DBI::DEBUG = 0;

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{ACCOUNT} = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($self->{ACCOUNT}->error) { $self->error($self->{ACCOUNT}->errorMessage); }
  $self->{SUBACCOUNT} = Portal::Accounting::Objects::SubAccount->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($self->{SUBACCOUNT}->error) { $self->error($self->{SUBACCOUNT}->errorMessage); }

  # do validation
  if (!$self->Portal::Accounting::Reports::Editor::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
#   if ($self->numInvalid() > 0 ,, $self->numMissing() > 0)
#   {
#     $self->postfixError($self->genErrorString("all"));
#     return 0;
#   }
  return 1;
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Reports/Editor.js");
  return $doc;
}

#****************************** Helper functions *****************************

# This function generates the display screen.
# Takes: error_string and/or status_string to display (optional)
sub c_display
{
  my $self = shift;
  my %args = ( error_string => '', status_string => '', @_, );
  my $error_string = $args{'error_string'};
  my $status_string = $args{'status_string'};
  my $dbh = $self->dbHandle;

  my $doc = $self->prepDoc();
#  my $doc = HTMLObject::Normal->new();
#  $doc->setTitle("Select Report to Edit");
  $doc->setFocus("body");
  $doc->setLinkDecorations();
  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>");
  }
  if (length $status_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Status</font>: &nbsp;$status_string</h2>");
  }

  $doc->print(<<"END_OF_BODY");
<form method="post" name="refresh_form">
  <input type="hidden" name="app" value="Accounting">
  <input type="hidden" name="state" value="Reports::Editor">
  <input type="hidden" name="command" value="display">
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
  <a href="" onMouseOver="window.status='Refresh'" onClick="document.refresh_form.command.value='display';document.refresh_form.submit();return false;">Refresh</a>
</form>
<div class="primary" style="text-align: center;">
  <form method="post" name="report_form">
    <input type="hidden" name="app" value="Accounting">
    <input type="hidden" name="state" value="Reports::Editor">
    <input type="hidden" name="command" value="">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="hidden" name="new_report_name" value="">
    <input type="hidden" name="windowName" value="">
    <input type="hidden" name="report_type" value="r" />
    <select name="report_group" onChange="report_group_change();" style="display: none;">
      <option value="regular" selected>Standard
    </select>
    <select name="id" size=6 width=250>
    </select>
    <br><br>
    <input type="button" name="edit" value="Edit" onClick="do_edit();">
    &nbsp;&nbsp;<input type="button" name="create" value="Create" onClick="do_create();">
    &nbsp;&nbsp;<input type="button" name="delete" value="Delete" onClick="do_delete();">
    &nbsp;&nbsp;<input type="button" name="copy" value="Copy" onClick="do_copy()">
  </form>
</div>
END_OF_BODY
  $doc->setFocus("javascript");
  my $counter = 0;
  # only gather the reports that are of type Regular to select from for editing
  my $sth = $dbh->read($sql_statements{1040});
  if (! defined $sth) { $self->error("<div>NO STH!!!$sql_statements{1040}</div>"); return undef; }
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  $doc->print("  reports[\"regular\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    my $editable = ($info[2] ? "" : " *");
    $doc->print("  reports[\"regular\"][" . $counter++ . "] = new report_entry(\"$info[1]\", \"$info[0] ($info[1])$editable\", \"$info[0]\", $t_or_f{$info[2]});\n");
  }

  # select detailed view reports
  $sth = $dbh->read( $sql_statements{1041} );
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  $counter = 0;
  $doc->print("  reports[\"detailed\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    my $editable = ($info[2] ? "" : " *");
    $doc->print("  reports[\"detailed\"][" . $counter++ . "] = new report_entry(\"$info[1]\", \"$info[0] ($info[1])$editable\", \"$info[0]\", $t_or_f{$info[2]});\n");
  }

  # select memorized reports
  $sth = $dbh->read( $sql_statements{1036} );
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  $counter = 0;
  $doc->print("  reports[\"memorized\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    $doc->print("  reports[\"memorized\"][" . $counter++ . "] = new report_entry(\"$info[1]|$info[0]|$info[2]\", \"$info[0] ($info[1] - $info[2])\", \"$info[0]\", true);\n");
  }


  my $onload_code = "";
  $onload_code = "document.report_form.report_group.selectedIndex = 0;\n";
  $onload_code .= "report_group_change();\n";
  $onload_code .= "initialize();\n";
  $doc->setOnload( code => $onload_code );

  return $doc;
}

sub c_saveReport
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my $doc = HTMLObject::Base->new;
  $doc->setContentType('text/plain');
  $doc->print("//<!DOCTYPE JSON>\n");

  # validate I was given a report to save.
  use vars qw( $id $rows $mode $report_type $windowName );
  my @arguments = ( "id", "rows", "mode", "report_type", ); #"windowName" );

  no strict 'refs';  # turn off the use of strict for symbolic references.
  foreach my $value (@arguments)
  {
    if (!defined $self->{input}->{$value})
    { # error!
      $doc->print($self->JER("missing parameter: $value"));
    }
    elsif (!$self->validateParameter($value, $self->{input}->{$value}))
    {
      $doc->print($self->JER("invalid parameter: $value = $self->{input}->{$value}"));
    }
    $$value = $self->{input}->{$value};  # assign the value to this variable via a symbolic reference :).
  }
  use strict 'refs';  # turn strict checking back on.

  # next check and make sure it still exists.
  my $sth = $dbh->read(sql => $sql_statements{1005}, plug => [ $id] );
  if ($dbh->error)
  {
    $doc->print($self->JER($dbh->errorMessage));
    return $doc;
  }
  my @info = $sth->fetchrow_array;
  if (!defined @info)
  {
    $doc->print("ERROR('Report ID = $id no longer exists! &nbsp;It may have been deleted.');");
    return $doc;
  }
  my $name = $info[0];

  # extract the rows from the input and parse apart
  my @rows = $self->parse_rows( rows => $rows, report_type => $report_type );
  if ($self->error)
  {
    $doc->print($self->JER($self->errorMessage));
    $self->resetError;
    return $doc;
  }
  # if it does, first delete all row entries from the database.
  my $sth = $dbh->write(sql => $sql_statements{1025}, plug => [ $id] );
  if ($dbh->error)
  {
    $doc->print($self->JER($dbh->errorMessage));
    return $doc;
  }

  # now we insert all the rows
  #!!! need to convert all array to hash to get data out
  for (my $i=0; $i < scalar @rows; $i++)
  {
    my $other_attributes = "";
    my $cell_type = $rows[$i]{cell_type};
    if ($cell_type eq "text")
    {
      $other_attributes = "bold=" . $true_false{$rows[$i]{bold}};
    }
    elsif ($cell_type eq "total")
    {
      $other_attributes = "bold=" . $true_false{$rows[$i]{bold}} . "|cell_formula=" . $rows[$i]{cell_formula} . "|show=" . $rows[$i]{show} . "|underline=" . $rows[$i]{underline};
    }
    elsif ($cell_type eq "line")
    {
      $other_attributes = "bold=" . $true_false{$rows[$i]{bold}};
    }
    elsif ($cell_type eq "compare")
    {
      $other_attributes = "bold=" . $true_false{$rows[$i]{bold}} . "|cell_formula=" . $rows[$i]{cell_formula} . "|result_type=" . $rows[$i]{result_type} . "|show=" . $rows[$i]{show} . "|underline=" . $rows[$i]{underline};
    }
    elsif ($cell_type eq "account_type" || $cell_type eq "accounts_of" || $cell_type eq "sub_accounts_of_type" || $cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance" ||
    $cell_type eq "account" || $cell_type eq "sub_accounts_of_account" || $cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance" ||
    $cell_type eq "sub_account")
    {
      $other_attributes = "bold=" . $true_false{$rows[$i]{bold}} . "|sub_levels=" . $rows[$i]{sub_levels} . "|drillable=" . $true_false{$rows[$i]{drillable}} . "|activity=" . $true_false{$rows[$i]{activity}} . "|query_type=" . $rows[$i]{query_type} . "|show=" . $rows[$i]{show};
    }
    elsif ($cell_type eq "detail")
    {
      $other_attributes = "bold=" . $true_false{$rows[$i]{bold}};
    }
    $sth = $dbh->write(sql => $sql_statements{1026} , plug => [ $id, $rows[$i]{index}, $rows[$i]{cell_text}, $rows[$i]{cell_type}, $rows[$i]{item_name}, $rows[$i]{display_level}, $rows[$i]{initial_state}, $rows[$i]{negative_format}, $other_attributes] );
    if ($dbh->error)
    {
      $doc->print($self->JER($dbh->errorMessage));
      return $doc;
    }
  }

  # commit the transaction to the database
  if ($self->error)
  {
    $doc->print("var REPORTSTATUS = 0;\n");
    $doc->print($self->JER($self->errorMessage));
    $dbh->rollback;
    $self->resetError;
  }
  else
  {
    $doc->print("var REPORTSTATUS = 1;\n");
    $dbh->commit;
  }

  return $doc;
}

sub c_create_report
{
  my $self = shift;
  my $report_type = "";
  if ((!exists $self->{input}->{'report_group'}) || (length $self->{input}->{'report_group'} == 0))
  {
    $self->my_die("Error: Report.cgi - create_report", 4017, "Report Group required at line " . __LINE__);  return undef;
  }
  if ($self->{input}->{'report_group'} eq "regular")
  {
    $report_type = "r";
  }
  else
  {
    $report_type = "d";
  }
  # check and see if we are regenerating the create report screen with the user specified values and an error message or just defaults.
  my %args = $self->get_default_data();
  $args{'mode'} = "create";
  $args{'report_type'} = $report_type;
  # call the first screen edit/create routine with the default values we want displayed.
  return $self->report_first_screen_edit_create( %args );
}

#validate input, makes db entry, and outputs second screen.
# outputs first screen again if error.
sub c_create_report2
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my $doc = $self->prepDoc;

  # check for the input from the first stage of creating a report and create the entry in report_type_tb before we proceed to the next stage.
  use vars qw( $name $id $mode $report_class $report_type $date_range $max_columns $min_columns $display_dates $text_font $data_font $text_size $data_size $total_column $underline_length $column_variable $column_variable_data $windowName );
  my @arguments = ( "name", "mode", "report_class", "report_type", "column_variable", "column_variable_data", "date_range", "display_dates", "max_columns", "min_columns", "text_size", "text_font", "data_size", "data_font", "total_column", "underline_length", );
  $max_columns = 4;
  $id = 0;  #initialize
  if (defined $self->{input}->{id}) { $id = $self->{input}->{id}; }
  my $doc2 = $self->prepDoc;

  no strict 'refs';  # turn off the use of strict for symbolic references.

  foreach my $value (@arguments)
  {
    if (!exists $self->{input}->{$value})
    { # error!
      $doc2->doRequiredParameterError('create_report2', $value);
    }
    if (length $self->{input}->{$value} == 0)
    {
      if ($value ne "column_variable" && $value ne "column_variable_data")
      { # error!
        $self->my_die("Error: Report.cgi - create_report2", 4005, "Parameter = '$value'");  return undef;
      }
    }
    elsif (!$self->validateParameter($value, $self->{input}->{$value}))
    {
      $self->my_die("Error: Report.cgi - create_report2", 4006, "$value = '$self->{input}->{$value}'");  return undef;
    }
    $$value = $self->{input}->{$value};  # assign the value to this variable via a symbolic reference :).
  }

  use strict 'refs';  # turn strict checking back on.

  # first verify that this report name is not already being used.
  my $sth = $dbh->read( sql => $sql_statements{1003} , plug => [ $name ] );
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  my @info = $sth->fetchrow_array;
  if (defined($info[0]))
  { # error!
    # call the first screen edit/create routine with the values of the selected report displayed.
    my %data = (
      name => "$name",
      id => "$id",
      mode => "$mode",
      report_class => "$report_class",
      report_type => "$report_type",
      column_variable => "$column_variable",
      column_variable_data => "$column_variable_data",
      date_range => "$date_range",
      display_dates => "$display_dates",
      max_columns => "$max_columns",
      min_columns => "$min_columns",
      text_size => "$text_size",
      text_font => "$text_font",
      data_size => "$data_size",
      data_font => "$data_font",
      total_column => "$total_column",
      underline_length => "$underline_length",
      error_string => "'$name' is already assigned to Report ID = '$info[0]'."
    );
    return $self->report_first_screen_edit_create( %data, report_type => $report_type);
  }
  else
  {
    # now create this entry in the database.
    $sth = $dbh->write(sql => $sql_statements{1002}, plug => [ $name, $report_class, $report_type, $column_variable, $column_variable_data, $date_range, $display_dates, $max_columns, $min_columns, $text_font, $text_size, $data_font, $data_size, $total_column, $underline_length] );
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }

    # commit the transaction to the database
    $dbh->commit;

    # get the id of the report we just created.
    $sth = $dbh->read(sql => $sql_statements{1003}, plug => [ $name ] );
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    @info = $sth->fetchrow_array;
    if (defined(@info))
    {
      $id = $info[0];
    }
  }

  return $self->report_second_screen_edit_create( mode => "create", id => "$id");
}

sub c_edit_report1
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my $doc;
  if ($self->error)
  {
    my %args = ();
    # check and see if we are regenerating the create report screen with the user specified values and an error message or just defaults.
    %args = $self->get_default_data();
    $args{'mode'} = "edit";
    $args{'report_type'} = $self->{input}->{'report_type'};
    $doc = $self->report_first_screen_edit_create( %args );
  }
  else
  {
    # check to make sure we were given the id of the report we are editing.
    my $id = $self->{input}->{'id'};
    if (length $id == 0)
    {
      $self->my_die("Error: Report.cgi - edit_report", 4017, "Report ID required at line " . __LINE__);  return undef;
    }

    # Gather the data from the database for the specified report.
    my $sth = $dbh->read(sql => $sql_statements{1006}, plug => [$id]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    my $info = $sth->fetchrow_hashref;
    if (!defined $info)
    {
      $self->my_die("Error: Report.cgi - edit_report1", 4020, "ID = '$id'");  return undef;
    }
    # call the first screen edit/create routine with the values of the selected report displayed.
    $doc = $self->report_first_screen_edit_create(
      mode => "edit",
      name => $info->{name},
      id => $id,
      report_class => $info->{report_class},
      report_type => $info->{report_type},
      column_variable => $info->{column_variable},
      column_variable_data => $info->{column_variable_data},
      date_range => "$t_or_f{$info->{date_range}}",
      display_dates => "$t_or_f{$info->{display_dates}}",
      max_columns => $info->{max_columns},
      min_columns => $info->{min_columns},
      text_size => "$info->{text_size}",
      text_font => $info->{text_font},
      data_size => "$info->{data_size}",
      data_font => $info->{data_font},
      total_column => "$t_or_f{$info->{total_column}}",
      underline_length => $info->{underline_length},
    );
  }
  return $doc;
}

sub c_edit_report2
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my $doc = HTMLObject::FrameSet->new;

  # check for the input from the first stage of editing the report and update the entry in report_type_tb before we proceed to the next stage.
  use vars qw( $name $id $original_name $report_class $report_type $date_range $max_columns $min_columns $display_dates $text_font $data_font $text_size $data_size $total_column $underline_length $column_variable $column_variable_data );

  my @arguments = ( "name", "original_name", "id", "report_class", "report_type", "column_variable", "column_variable_data", "date_range", "display_dates", "max_columns",  "min_columns", "text_size", "text_font", "data_size", "data_font", "total_column", "underline_length" );
  $max_columns = 4;
  my $doc2 = $self->prepDoc;
  no strict 'refs';  # turn off the use of strict for symbolic references.

  foreach my $value (@arguments)
  {
    if (!exists $self->{input}->{$value})
    { # error!
      $doc2->doRequiredParameterError('edit_report2', $value);
    }
    if (length $self->{input}->{$value} == 0)
    {
      if ($value ne "column_variable" && $value ne "column_variable_data")
      { # error!
        $self->my_die("Error: Report.cgi - edit_report2", 4005, "Parameter = '$value'");  return undef;
      }
    }
    elsif (!$self->validateParameter($value, $self->{input}->{$value}))
    {
      $self->my_die("Error: Report.cgi - edit_report2", 4006, "$value = '$self->{input}->{$value}'");  return undef;
    }
    $$value = $self->{input}->{$value};  # assign the value to this variable via a symbolic reference :).
  }

  use strict 'refs';  # turn strict checking back on.

  $doc->setTitle("Edit Report - $name");

  # first verify that the report in question exists.
  my $sth = $dbh->read(sql => $sql_statements{1005} , plug => [$id]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  my @info = $sth->fetchrow_array;


  if (!defined @info)
  {
    $self->my_die("Error: Report.cgi - edit_report2", 4020, "ID = '$id'");  return undef;
  }

  # see if they want to change the report name and if so verify that the new name is not already being used.
  if ($name ne $original_name)
  {
    $sth = $dbh->read(sql => $sql_statements{1003} , plug=>[$name]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    @info = $sth->fetchrow_array;
    if (defined($info[0]) && $info[0] != $id)
    { # error!
      my $found = 1;  # assume we are the same report until we know for sure.
      if (!$found)
      {
        # call the first screen edit/create routine with the values of the selected report displayed.
        my %data = (
          name => "$name",
          id => "$id",
          report_class => "$report_class",
          report_type => "$report_type",
          column_variable => "$column_variable",
          column_variable_data => "$column_variable_data",
          date_range => "$date_range",
          display_dates => "$display_dates",
          max_columns => "$max_columns",
          min_columns => "$min_columns",
          text_size => "$text_size",
          text_font => "$text_font",
          data_size => "$data_size",
          data_font => "$data_font",
          total_column => "$total_column",
          underline_length => "$underline_length",
          error_string => "'$name' is already assigned to Report ID = '$info[0]'.",
          original_name => "$original_name",
        );
        my $file_name = tmpnam(); # get the file name of the temp file to work with.
        # store the data object.
        store(\%data, $file_name);
        # $self->my_die("Error: Report.cgi - create_report2", 4015, "Report Name = '$name' is assigned to Report ID = '$info[0]' at Line " . __LINE__);  return undef;

        # now we need to generate the frameset, set the error_happened cookie with the file_name and then exit.
        return $self->c_edit_report( cookie_name => "error_happened", cookie_value => "$file_name", );
        # $self->my_die("Error: Report.cgi - edit_report2", 4015, "Report Name = '$name' is assigned to Report ID = '$info[0]'");  return undef;
      }
    }
  }
  # now update this entry in the database.
  $sth = $dbh->write(sql => $sql_statements{1004}, plug => [$name, $report_class, $report_type, $column_variable, $column_variable_data, $date_range, $display_dates, $max_columns, $min_columns, $text_font, $text_size, $data_font, $data_size, $total_column, $underline_length, $id]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }

  # commit the transaction to the database
  $dbh->commit;
  return $self->c_edit_report2_top(@_);
}

sub c_edit_report2_top
{
  my $self = shift;
  # check to make sure we were given the id of the report we are creating.
  my $id = $self->{input}->{'id'};
  if (length $id == 0)
  {
    $self->my_die("Error: Report.cgi - edit_report2_top", 4017, "Report ID required at line " . __LINE__);  return undef;
  }

  # call the report second screen create/edit function with the specified ID to work with.
  return $self->report_second_screen_edit_create( mode => "edit", id => "$id", );
}

# This function generates the frameset which causes a report to be edited (screen 1).
# Takes: cookie_name, cookie_value, windowName
# Result: Sets cookie, if specified, and generates the frameset.
sub c_edit_report
{
  my $self = shift;
  my %args = ( @_, );
  my $cookie_name = $args{'cookie_name'};
  my $cookie_value = $args{'cookie_value'};
  my $windowName = '';
  my $dbh = $self->dbHandle;
  my $report_type = "";

  # check to make sure we were given the id of the report we are editing.
  my $id = $self->{input}->{'id'};
  if (length $id == 0)
  {
    $self->my_die("Error: Report.cgi - c_edit_report", 4017, "Report ID required at line " . __LINE__);  return undef;
  }

  # first verify that the report in question exists.
  my $sth = $dbh->read(sql =>  $sql_statements{1005}, plug => [$id]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  my @info = $sth->fetchrow_array;
  if (!defined @info)
  {
    $self->my_die("Error: Report.cgi - edit_report", 4020, "ID = '$id'");  return undef;
  }

  if ($self->{input}->{'report_group'} eq "regular")
  {
    $report_type = "r";
  }
  else
  {
    $report_type = "d";
  }

  return $self->c_edit_report1(@_);
}

sub c_copy_report
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my $doc = $self->prepDoc;

  # validate I was given a report to delete.
  use vars qw( $id $report_group $new_report_name );
  my @arguments = ( "id", "report_group", "new_report_name" );
  no strict 'refs';  # turn off the use of strict for symbolic references.

  foreach my $value (@arguments)
  {
    if (!exists $self->{input}->{$value})
    { # error!
      $doc->doRequiredParameterError('copy_report', $value);
    }
    if (length $self->{input}->{$value} == 0)
    {
      $self->my_die("Error: Report.cgi - copy_report", 4005, "Parameter = '$value'");  return undef;
    }
    elsif (!$self->validateParameter($value, $self->{input}->{$value}))
    {
      $self->my_die("Error: Report.cgi - copy_report", 4006, "$value = '$self->{input}->{$value}'");  return undef;
    }
    $$value = $self->{input}->{$value};  # assign the value to this variable via a symbolic reference :).
  }

  use strict 'refs';  # turn strict checking back on.

  if ($report_group eq "memorized")
  {
    return $self->c_display( error_string => "Copy Failed! &nbsp;You can not copy a Memorized Report!" );
  }
  else
  {
    my $report_name = $self->DoesReportExist(dbh => $dbh, id => $id);
    if (length $report_name == 0)
    {
      return $self->c_display( error_string => "Copy Failed! &nbsp;Report with ID = '$id' does not exist!" );
    }
    elsif ($report_name eq $new_report_name)
    {
      return $self->c_display( error_string => "Copy Failed! &nbsp;You must specify a different name for Report '$report_name' = ($id)!" );
    }
    else
    {
      my $new_id = $self->FindReportByName(dbh => $dbh, name => $new_report_name);
      if ($new_id != -1)
      {
        return $self->c_display( error_string => "Copy Failed! &nbsp;Report '$new_report_name' = ($new_id) already exists!" );
      }
      else
      {
        my @report_type_data = $self->GetReportData(id => $id);
        my $report_class = $report_type_data[1];
        my $report_type = $report_type_data[2];
        my $column_variable = $report_type_data[3];
        my $column_variable_data = $report_type_data[4];
        my $date_range = $report_type_data[5];
        my $display_dates = $report_type_data[6];
        my $max_columns = $report_type_data[7];
        my $min_columns = $report_type_data[8];
        my $text_font = $report_type_data[9];
        my $text_size = $report_type_data[10];
        my $data_font = $report_type_data[11];
        my $data_size = $report_type_data[12];
        my $total_column = $report_type_data[13];
        my $underline_length = $report_type_data[14];

        # create the report_type_tb entry and gather the id of the report
        my $sth = $dbh->write(sql => $sql_statements{1002}, plug => [$new_report_name, $report_class, $report_type, $column_variable, $column_variable_data, $date_range, $display_dates, $max_columns, $min_columns, $text_font, $text_size, $data_font, $data_size, $total_column, $underline_length] );
        if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }

        # commit the transaction to the database
        $dbh->commit;

        # get the id of the report we just created.
        $sth = $dbh->read(sql => $sql_statements{1003}, plug => [ $new_report_name] );
        if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
        my @info = $sth->fetchrow_array;


        if (defined(@info))
        {
          $new_id = $info[0];
          my @rows = $self->GetReportRows(id => $id);

          # create the report_data_tb entries using the new id of the report.
          for (my $i=0; $i < scalar @rows; $i++)
          {
            $sth = $dbh->write(sql => $sql_statements{1026}, plug => [ $new_id, $rows[$i]{'index'}, $rows[$i]{'cell_text'}, $rows[$i]{'cell_type'}, $rows[$i]{'item_name'}, $rows[$i]{'display_level'}, $rows[$i]{'initial_state'}, $rows[$i]{'negative_format'}, $rows[$i]{'other_attributes'}] );
            if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
          }

          # commit the transaction to the database
          $dbh->commit;

          return $self->c_display( status_string => "Copy Succeeded! &nbsp;Report '$new_report_name' = ($new_id) created!" );
        }
        else
        {
          return $self->c_display( error_string => "Copy Failed! &nbsp;Creation of Report '$new_report_name' failed!&nbsp; New Report ID not defined!" );
        }
      }
    }
  }
  return $doc;
}

sub c_delete_report
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my $doc = $self->prepDoc;
  my $id = $self->{input}->{id};
  # validate I was given a report to delete.
  if (!defined $self->{input}->{id})
  { # error!
    $doc->doRequiredParameterError('delete_report', 'id');
  }

  # see if we are working with a normal report or a memorized report.  Memorized reports have 2 |'s in them.
  my @id_parts = split /\|/, $id;
  my $memorized;
  if (scalar @id_parts == 3)
  {
    $memorized = 1;
  }
  else
  {
    $memorized = 0;
  }

  # next check and make sure it still exists.
  if (!$memorized)
  {
    my $name = $self->DoesReportExist(id => $id);
    if ($self->error) { return undef; }
    if (length $name == 0)
    {
      return $self->c_display( error_string => "Report ID = '$id' no longer exists! &nbsp;It appears to have been deleted already.",);
    }

    my $foo = $self->IsReportEditable(id => $id);
    if ($self->error) { return undef; }
    if (!$foo)
    {
      return $self->c_display( error_string => "Report ID = '$id' is a System Report! &nbsp;You can not delete it." );
    }

    # if it does, first make sure there are no memorized reports associated with this report.  If so, blow an error.
#     my $sth = $dbh->read(sql => $sql_statements{1014}, plug => [$id]);
#     if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
#     my @info = $sth->fetchrow_array;
#     if (!defined @info)
#     {
#       $self->my_die("Error: Report.cgi - delete_report", 4021, "ID = '$id'");  return undef;
#     }
#     if ($info[0] > 0)
#     {
#       return $self->c_display( error_string => "You have '$info[0]' memorized reports defined for Report '$name' = $id." );
#     }
#
    # else we can go ahead and delete the data for the report and then the report itself.
    my $sth = $dbh->write(sql => $sql_statements{1008}, plug => [$id]);
    if ($dbh->error) { $self->error($dbh->errorMessage . $sql_statements{1008}); return undef; }

    $sth = $dbh->write( sql => $sql_statements{1009}, plug => [$id]);
    if ($dbh->error) { $self->error($dbh->errorMessage . $sql_statements{1009}); return undef; }

    # regenerate the $self->c_display screen
    $doc = $self->c_display( status_string => "Report '$name' = $id Successfully Deleted.", );
  }
  if ($self->error) { $dbh->rollback; }
  else { $dbh->commit; }
  return $doc;
}

# This function returns the data from the storable object (if specified via cookie) or defaults for creating a report.
# currently takes nothing.
sub get_default_data
{
  my $self = shift;
  my %values;
  my $value_ref; # reference to the hash.

  # check and see if the error cookie exists.
  if ($self->error)
  {
    my $file_name = $self->{input}->{file_name};
    $value_ref = retrieve($file_name);
    %values = %{$value_ref};
    #unlink $file_name;  # delete the file.
  }
  else
  {
    $values{'name'} = "";
    $values{'id'} = 0;
    $values{'report_class'} = "custom";
    $values{'report_type'} = "r";
    $values{'column_variable'} = "";
    $values{'column_variable_data'} = "";
    $values{'display_dates'} = "true";
    $values{'date_range'} = "true";
    $values{'max_columns'} = 36;
    $values{'min_columns'} = 1;
    $values{'text_size'} = "-1";
    $values{'text_font'} = "helvetica";
    $values{'data_size'} = "-1";
    $values{'data_font'} = "helvetica";
    $values{'total_column'} = "false";
    $values{'underline_length'} = $default_underline_length;
  }

  return %values;
}

sub validateParameter
{
  my $self = shift;
  # this function takes the name of the parameter and it's value and does the work of making sure that the value is valid for that parameter.
  my ($name, $value) = ( @_ );
  my $result = 1;

  if (length $name == 0)
  {
    $result = 0;
  }
  if (length $value == 0)
  {
    $result = 0;
  }

  if ($name eq "mode")
  {
    if ($value ne "edit" && $value ne "create")
    {
      $result = 0;
    }
  }
  elsif ($name eq "name")
  {
  }
  elsif ($name eq "original_name")
  {
  }
  elsif ($name eq "id")
  {
    if ($value < -1)
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_class")
  {
    if (!exists $report_classes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_type")
  {
    if ($value ne "r" && $value ne "d")
    {
      $result = 0;
    }
  }
  elsif ($name eq "column_variable")
  {
  }
  elsif ($name eq "column_variable_data")
  {
  }
  elsif ($name eq "date_range")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "display_dates")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "max_columns")
  {
    if ($value <= 0 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "min_columns")
  {
    if ($value <= 0 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "total_column")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "underline_length")
  {
    if ($value < $min_underline_length || $value > $max_underline_length)
    {
      $result = 0 if ($value != -1);
    }
  }
  elsif ($name eq "dbh")
  {
  }
  elsif ($name eq "error_string")
  {
  }
  elsif ($name eq "rows")
  {
    my @rows = split /\n/, $value;
    if (scalar @rows == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "file_name")
  {
  }
  elsif ($name eq "report_group")
  {
    if ($value ne "regular" && $value ne "detailed" && $value ne "memorized")
    {
      $result = 0;
    }
  }
  elsif ($name eq "new_report_name")
  {
  }
  elsif ($name eq "windowName")
  {
    $result = 0 if ($value !~ /$editReportWindowRE/);
  }
  else
  {
    $result = 0;  # signal that an error occurred as we do not know anything about this variable.
  }

  return $result;
}

sub get_all_detailed_reports
{
  my $self = shift;
  my %args = ( @_, );
  my $dbh = $self->dbHandle;
  my @data = ();

  my $sth = $dbh->read( $sql_statements{1022} );
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  my $ref = $sth->fetchall_arrayref;

  @data = @{$ref};

  return @data;
}

sub find_detailed_report
{
  my $self = shift;
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    $self->my_die("Error:  Report.cgi - find_detailed_report", 4005, "array is required");  return undef;
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    $self->my_die("Error:  Report.cgi - find_detailed_report", 4005, "num is required");  return undef;
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][0] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_level
{
  my $self = shift;
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $level = $args{'level'};
  my $found = -1;

  if (!exists $args{'array'})
  {
    $self->my_die("Error:  Report.cgi - find_detailed_report", 4005, "array is required");  return undef;
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'level'} || length $level == 0)
  {
    $self->my_die("Error:  Report.cgi - find_detailed_report", 4005, "level is required");  return undef;
  }

  for (my $i=0; $i < scalar @array && $found == -1; $i++)
  {
    if ($array[$i] eq $level)
    {
      $found = $i;
    }
  }

  return $found;
}

# parse_rows
# takes the string that represents the rows for the report from create/edit screen 2
# returns the array that represents the rows in their native format in format data[row]{field}
sub parse_rows
{
  my $self = shift;
  my $dbh = $self->dbHandle;
  my %args = ( @_, );
  my $rows = $args{'rows'};
  my $report_type = $args{'report_type'};
  my @rows = ();
  my %row_to_index = (
    "index" => "0",
    "cell_text" => "1",
    "cell_formula" => "2",
    "cell_type" => "3",
    "item_name" => "4",
    "bold" => "5",
    "sub_levels" => "6",
    "display_level" => "7",
    "initial_state" => "8",
    "drillable" => "9",
    "negative_format" => "10",
    "activity" => "11",
    "query_type" => "12",
    "show" => "13",
    "result_type" => "14",
    "underline" => "15",
  );

  if (length $rows == 0)
  {
    $self->error("Error:parse_rows, $errors{4005}, The Rows String to parse must be specified!");  return undef;
  }

  my %accounts;
  my %sub_accounts;
  my %account_types;
  if ($report_type eq "r")
  {
    %account_types = $self->{ACCOUNT}->selectTypeHash;
    %accounts = $self->{ACCOUNT}->selectHash;
    %sub_accounts = $self->{SUBACCOUNT}->selectHash;
  }

  my $rows_counter = 0;
  my @temp = split /\\n/, $rows;   # split apart on the \n
  foreach my $row (@temp)
  {
    my @values = split /\|/, $row;  # split apart on the |
    foreach my $item (@values)
    {
      my ($name, $value) = split /=/, $item, 2;  # only split on the first =
      if (!exists $row_to_index{$name})
      {
        $self->error("<div>Error:parse_rows, $errors{4031}, Item = '$name'.</div>");  return undef;
      }
      if (!$self->validate_row_item($name, $value, $report_type))
      {
        $self->error("<div>Error:parse_rows, $errors{4033}, Item = '$name', Value = '$value'.</div>");  return undef;
      }

      # store in array
      $rows[$rows_counter]{$name} = $value;
    }
    $rows_counter++;
  }

  # now walk the array and for the special items, validate that they are what they claim to be.
  for ($rows_counter=0; $rows_counter < scalar @rows; $rows_counter++)
  {
    my ($index, $cell_type, $item_name, $sub_levels, $drillable);
    $index        = $rows[$rows_counter]{index};
    $cell_type    = $rows[$rows_counter]{cell_type};
    $item_name    = $rows[$rows_counter]{item_name};
    $sub_levels   = $rows[$rows_counter]{sub_levels};
    $drillable    = $rows[$rows_counter]{drillable};

    if ($report_type eq "r")
    {
      if ($cell_type eq "text" || $cell_type eq "total" || $cell_type eq "line" || $cell_type eq "compare")
      {
        if ($item_name ne "---------------")
        {
          $self->error("Error:parse_rows, $errors{4033}, '$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");  return undef;
        }
        if ($sub_levels ne "---------------")
        {
          $self->error("Error:parse_rows, $errors{4033}, '$sub_levels' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");  return undef;
        }
        $rows[$rows_counter]{sub_levels} = -1;
        if ($cell_type ne "compare")
        {
          $rows[$rows_counter]{result_type} = ""; # The Result Type doesn't matter.
        }
      }
      else
      {
        if ($cell_type eq "account_type" || $cell_type eq "accounts_of" || $cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance" || $cell_type eq "sub_accounts_of_type")
        {
          #!!! is this validating that the account type is valid?  WTF??!!! it says send a number in the api for the functions but james used $item_name.  WTF?  I'm going to try to replace with number.
          if (! exists $account_types{$item_name} )
          {
            $self->error("Error:parse_rows, $errors{4033}, Item Name = '$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");  return undef;
          }
        }
        elsif ($cell_type eq "account" || $cell_type eq "sub_accounts_of_account" || $cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance")
        {
          if (! exists $accounts{$item_name} )
          {
            $self->error("Error:parse_rows, $errors{4033}, Item Name = '$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");  return undef;
          }
        }
        elsif ($cell_type eq "sub_account")
        {
          if (! exists $sub_accounts{$item_name})
          {
            $self->error("<div>Error:parse_rows, $errors{4033}, Item Name = $item_name is invalid for Cell Type = $cell_type!<br>Row = $index.</div>");
            return undef;
          }
        }

        my ($level);
        if (($level = $self->find_level( array => \@{$report_levels{$cell_type}}, level => "$sub_levels" )) == -1)
        {
          $self->error("Error:parse_rows, $errors{4033}, Display Sub Level = '$sub_levels' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");  return undef;
        }
        $rows[$rows_counter]{sub_levels} = $level;  # set the index
        $rows[$rows_counter]{result_type} = ""; # The Result Type doesn't matter.
        $rows[$rows_counter]{underline} = "0"; # The Underline Flag should always be false.
      } # end of else
    }
    else  # Detailed View Report
    {
      if ($cell_type eq "text" || $cell_type eq "line" || $cell_type eq "detail")
      {
        $rows[$rows_counter]{item_name} = "---------------";
        $rows[$rows_counter]{sub_levels} = -1;
        $rows[$rows_counter]{drillable} = "false";
        $rows[$rows_counter]{cell_formula} = "";
        $rows[$rows_counter]{display_level} = "1";
        $rows[$rows_counter]{initial_state} = "c";
        $rows[$rows_counter]{negative_format} = 0;
        $rows[$rows_counter]{activity} = "true";
        $rows[$rows_counter]{query_type} = "0";
        $rows[$rows_counter]{show} = "1";  # always show this row.
        $rows[$rows_counter]{result_type} = ""; # The Result Type doesn't matter.
        $rows[$rows_counter]{underline} = "0"; # The Underline Flag should always be false.
      }
    }
  }

  return @rows;
}

# validate_row_item
# takes  : name, value, report_type
# returns: 1 if ok, 0 if not
sub validate_row_item
{
  my $self = shift;
  my ($name, $value, $report_type);
  $name  = $_[0];
  $value = $_[1];
  $report_type = $_[2];

  my $result = 1;

  if (length $name == 0)
  {
    $self->my_die("Error:  Report.cgi - validate_row_item", 4005, "The name parameter is required!");  return undef;
  }

  if ($name eq "index")
  {
    $result = 0 if ($value < 0);
  }
  elsif ($name eq "cell_text")
  {
  }
  elsif ($name eq "cell_formula")
  {
    # this is where I need to parse apart the formula and validate it.
  }
  elsif ($name eq "cell_type")
  {
    if ($report_type eq "r")
    {
      $result = 0 if (!exists $regular_cell_types{$value});
    }
    elsif ($report_type eq "d")
    {
      $result = 0 if (!exists $detailed_cell_types{$value});
    }
  }
  elsif ($name eq "item_name")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "bold")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "sub_levels")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "display_level")
  {
    $result = 0 if ($value < 0 || $value > 2);
  }
  elsif ($name eq "initial_state")
  {
    $result = 0 if ($value ne "o" && $value ne "c");
  }
  elsif ($name eq "drillable")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "negative_format")
  {
    $result = 0 if (!exists $negative_formats{$value});
  }
  elsif ($name eq "activity")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "query_type")
  {
    $result = 0 if (!($value == 0 || $value == 1 || $value == 2));
  }
  elsif ($name eq "show")
  {
    $result = 0 if (!($value == 0 || $value == 1));
  }
  elsif ($name eq "underline")
  {
    $result = 0 if (!($value == 0 || $value == 1));
  }
  elsif ($name eq "result_type")
  {
  }
  else
  {
    $result = 0;  # signal an error occurred.
  }

  return $result;
}

# FindSubAccount
# takes: number and reference to array to search in.
# returns the Name of the Sub Account.
sub FindSubAccount
{
  my $self = shift;
  my ($number, @array);
  $number = $_[0];
  @array = @{$_[1]};

  if (length $number == 0)
  {
    $self->my_die("Error: Report.cgi - FindSubAccount", 4005, "The Sub Account Number must be specified!");  return undef;
  }
  if (scalar @array == 0)
  {
    $self->my_die("Error: Report.cgi - FindSubAccount", 4005, "The Array to search must be specified!");  return undef;
  }

  # now we walk through all SubAccountTypes and for those that have SubAccount arrays, we walk them till we find
  # the specified SubAccount number and then we can get it's name!
  my $name = "";
  my $found = 0;

  for (my $index=0; $index < scalar @array && !$found; $index++)
  {
    if (scalar @{$array[$index][2]} > 0)
    {
      for (my $index2=0; $index2 < scalar @{$array[$index][2]} && !$found; $index2++)
      {
        if ($array[$index][2][$index2][0] == $number)
        {
          $found = 1;
          $name = $array[$index][2][$index2][1];
        }
      }
    }
  }

  if (!$found)
  {
    $self->my_die("Error: Report.cgi - FindSubAccount", 4013, "The specified Sub Account = '$number' was not found!");  return undef;
  }

  return $name;
}

# FindSubAccountType
# takes: number and reference to array to search in.
# returns the Name of the Sub Account Type.
sub FindSubAccountType
{
  my $self = shift;
  my ($number, @array);
  $number = $_[0];
  @array = @{$_[1]};

  if (length $number == 0)
  {
    $self->my_die("Error: Report.cgi - FindSubAccount", 4005, "The Sub Account Type Number must be specified!");  return undef;
  }
  if (scalar @array == 0)
  {
    $self->my_die("Error: Report.cgi - FindSubAccount", 4005, "The Array to search must be specified!");  return undef;
  }

  # now we walk through all SubAccountTypes till we find
  # the specified SubAccountType number and then we can get it's name!
  my $name = "";
  my $found = 0;

  for (my $index=0; $index < scalar @array && !$found; $index++)
  {
    if ($array[$index][0] == $number)
    {
      $found = 1;
      $name = $array[$index][1];
    }
  }

  if (!$found)
  {
    $self->my_die("Error: Report.cgi - FindSubAccount", 4014, "The specified Sub Account Type = '$number' was not found!");  return undef;
  }

  return $name;
}

sub report_first_screen_edit_create
{
  # this function will take a list of parameters that defines the settings for the first screen of editing or creating a report.
  # mode = (edit|create), name, id=(-1 if creating), report_class, report_type, column_variable=(empty if creating), column_variable_data=(empty if creating)
  # date_range, display_dates, max_columns, min_columns, text_size, data_size, text_font, data_font, total_column, underline_length

  my $self = shift;
  my $dbh = $self->dbHandle;
  my %args = ( @_, );  # gather all arguments from the command line.

  # validate we have all parameters we need.
  my $mode = $args{'mode'};
  my $name = $args{'name'};
  my $id = $args{'id'};
  my $report_class = $args{'report_class'};
  my $report_type = $args{'report_type'};
  my $column_variable = $args{'column_variable'};
  my $column_variable_data = $args{'column_variable_data'};
  my $date_range = $args{'date_range'};
  my $display_dates = $args{'display_dates'};
  my $max_columns = $args{'max_columns'};
  my $min_columns = $args{'min_columns'};
  my $text_size = $args{'text_size'};
  my $text_font = $args{'text_font'};
  my $data_size = $args{'data_size'};
  my $data_font = $args{'data_font'};
  my $total_column = $args{'total_column'};
  my $underline_length = $args{'underline_length'};
  my $error_string = $args{'error_string'};

  my @arguments = ( "mode", "name", "id", "report_class", "report_type", "column_variable", "column_variable_data", "date_range", "display_dates", "max_columns",
        "min_columns", "text_size", "text_font", "data_size", "data_font", "total_column", "underline_length" );

  my $doc = $self->prepDoc;
  foreach my $value (@arguments)
  {
    if (!exists $args{$value})
    { # error!
      $doc->doRequiredParameterError('report_first_screen_edit_create', $value);
    }
    if (length $args{$value} == 0)
    {
      if (($value eq "name" && $mode eq "create"))
      { # do nothing, this is the default value.
      }
      elsif ($value ne "column_variable" && $value ne "column_variable_data")
      { # error!
        $self->my_die("Error: Report.cgi - report_first_screen_edit_create", 4005, "Parameter = '$value'");  return undef;
      }
    }
    elsif (!$self->validateParameter($value, $args{$value}))
    {
      $self->my_die("Error: Report.cgi - report_first_screen_edit_create", 4006, "$value = '$args{$value}'");  return undef;
    }
  }

  my $sth;
  my $rc;
  my $sth2;
  my @info = ();
  my $title = "";
  my $target_window = "";
  my $command = "";
  my $header = "";
  my %already_selected_column_headers = ();
  if ($mode eq "create")
  {
    $title = "Create Report";
    $header = "Create Report";
    $target_window = "create_report_window";
    $command = "create_report2";
  }
  elsif ($mode eq "edit")
  {
    $title = "Edit Report";
    $header = "Edit Report - ID = $id";
    $target_window = "edit_report_window";
    $command = "edit_report2";
  }

  if ($report_type eq "r")
  {
    $sth = $dbh->read( $sql_statements{1000} );
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    $doc->setFocus("javascript");  # point to the JavaScript section of this document.

    # walk through all sub-account-types returned and store them in the array.
    my $array_index = 0;  # start out at 0.
    my @SubAccountTypes = ();  # the perl array to store the SubAccountTypes and SubAccounts in.
    while (my @db_items = $sth->fetchrow_array)
    {
      $doc->print("  SubAccountTypes[$array_index] = new SubAccountType(\"$db_items[1]\", $db_items[0]);\n");
      my @temp = ();
      $temp[0] = $db_items[0];
      $temp[1] = $db_items[1];

      # Now we need to gather all SubAccounts for this SubAccountType.
      my $sth2 = $dbh->read(sql => $sql_statements{1001}, plug => [ $db_items[0] ] );
      if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
      my $sub_array_index = 0;  # start out at 0.
      my @temp2 = ();
      # create the array for the sub accounts - even if we don't have any sub accounts.
      $doc->print("  SubAccountTypes[$array_index].sub_accounts = new Array();\n");
      while (my @db_items2 = $sth2->fetchrow_array)
      {
        $doc->print("  SubAccountTypes[$array_index].sub_accounts[$sub_array_index] = new SubAccount(\"$db_items2[1]\", $db_items2[0]);\n");
        my @temp3 = ();
        $temp3[0] = $db_items2[0];
        $temp3[1] = $db_items2[1];
        $temp2[$sub_array_index++] = [ @temp3 ];
      }
      $temp[2] = [ @temp2 ];  # Assign the SubAccounts Array to the SubAccountType

      $SubAccountTypes[$array_index++] = [ @temp ];  # Store this SubAccountType in the overall Array.
    }
    # initialize the Selected Column Headers Array and all necessary data storage
    $doc->print(<<"END_OF_CODE");
    function InitializeSelectedColumnsArray()
    {
      SelectedColumns = new Array();
END_OF_CODE

    # split the column_variable string apart and put the found items in the already_selected_column_headers hash.
    $array_index = 0;  # start over at 0 again.
    my @column_items = split /\|/, $column_variable_data;

    foreach my $column ( split /,/, $column_variable )
    {
      if (!exists $column_headers{$column})
      {
        $self->my_die("Error:  Report.cgi - report_first_screen_edit_create", 4011, "Column Header = '$column' used in $mode of Report = '$id'");  return undef;
      }
      if (!exists $already_selected_column_headers{$column})
      {
        $already_selected_column_headers{$column} = 1;
        $doc->print("    SelectedColumns[$array_index] = new ColumnInfo(\"$column_headers{$column}\", \"$column\");\n");
        $doc->print("    SelectedColumns[$array_index].items = new Array();\n");

        # split the data from the column_variable_data string and assign it to the items array.
        my @items = split /,/, $column_items[$array_index];
        my $index2 = 0;  # start at 0.
        foreach my $item (@items)
        {
          my $value = $item;
          my $text = "";

          if ($column eq "sub_accounts")
          {
            $text = $self->FindSubAccount($value, \@SubAccountTypes);  # get the name of the SubAccount
          }
          elsif ($column eq "sub_account_types")
          {
            $text = $self->FindSubAccountType($value, \@SubAccountTypes);  # get the name of the SubAccountType
          }
          else
          {
            $self->my_die("Error:  Report.cgi - report_first_screen_edit_create", 4012, "Column Header = '$column' has items assigned to it when it should not in $mode of Report = '$id'");  return undef;
          }
          $doc->print("    SelectedColumns[$array_index].items[" . $index2++ . "] = new ColumnItemInfo(\"$text\", \"$value\");\n");
        }

        $array_index++;
      }
      else
      {
        $self->my_die("Error:  Report.cgi - report_first_screen_edit_create", 4010, "Column Header = '$column' already used in Edit of Report = '$id'");  return undef;
      }
    }
    $doc->print("  }\n\n");  # put space between the SelectedColumns variables and the rest of the code.
  }

  $target_window = "_top";  # temporarily send it to the same window unless they want it elsewhere later.
  $doc->setTitle($title);
  $doc->setFocus("body");
  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>\n");
  }
  my $original_name = $name;
  if (exists $args{'original_name'})
  {
    if (length $args{'original_name'} > 0)
    {
      $original_name = $args{'original_name'};
    }
  }
  $doc->print(<<"END_OF_BODY");
<div id="_top">
  <form name="report_form" method="post" onSubmit="return validate_form()">
    <input type="hidden" name="app" value="Accounting">
    <input type="hidden" name="state" value="Reports::Editor">
    <input type="hidden" name="command" value="$command">
    <input type="hidden" name="mode" value="$mode">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="hidden" name="id" value="$id">
    <input type="hidden" name="report_type" value="$report_type">
    <input type="hidden" name="windowName" value="">
END_OF_BODY
  if ($report_type eq "r")
  {
    $doc->print(<<"END_OF_BODY");
    <input type="hidden" name="column_variable" value="">
    <input type="hidden" name="column_variable_data" value="">
    <input type="hidden" name="original_name" value="$original_name">
    <table class="primary" style="border:1; text-align: center;">
      <tr >
        <th>Report Name</th><th>Report Class</th><th>Report Type</th><th>ID</th>
      </tr>
      <tr>
        <td align=center><input type=text name="name" value="$name"></td>
        <td align=center><select name="report_class">
END_OF_BODY
        for (my $i=0; $i < scalar @report_classes; $i++)
        {
          my $string = "          <option value=\"$report_classes[$i]\"";
          if ($report_classes[$i] eq $report_class)
          {
            $string .= " selected";
          }
          $string .= ">$report_classes{$report_classes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align="center">$report_types{$report_type}</td>
        <td align="center">$id</td>
      </tr>
      <tr >
        <th>Date Type</th><th>Max Columns to Show</th><th>Min Columns to Fill</th><th>Display Date Selector</th>
      </tr>
      <tr>
        <td align=center><select name="date_range">
END_OF_BODY
          my $string = "          <option value=\"true\"";
          if ($date_range eq "true")
          {
            $string .= " selected";
          }
          $string .= ">Date Range\n";
          $doc->print($string);

          $string = "          <option value=\"false\"";
          if ($date_range eq "false")
          {
            $string .= " selected";
          }
          $string .= ">Single Date\n";
          $doc->print($string);
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="max_columns">
END_OF_BODY
        for (my $i=1; $i <= $max_dates; $i++)
        {
          my $string = "          <option value=\"$i\"";
          if ($i eq $max_columns)
          {
            $string .= " selected";
          }
          $string .= ">$column_names{$i}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="min_columns">
END_OF_BODY
        for (my $i=1; $i <= $max_dates; $i++)
        {
          my $string = "          <option value=\"$i\"";
          if ($i eq $min_columns)
          {
            $string .= " selected";
          }
          $string .= ">$column_names{$i}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="display_dates">
END_OF_BODY
          my $string = "          <option value=\"true\"";
          if ($display_dates eq "true")
          {
            $string .= " selected";
          }
          $string .= ">Yes\n";
          $doc->print($string);

          $string = "          <option value=\"false\"";
          if ($display_dates eq "false")
          {
            $string .= " selected";
          }
          $string .= ">No\n";
          $doc->print($string);
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr >
        <th>Text Font</th><th>Increase Text Font Size By</th><th>Data Font</th><th>Increase Data Font Size By</th>
      </tr>
      <tr>
        <td align=center><select name="text_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $text_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="text_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $text_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="data_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $data_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="data_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $data_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr >
        <th>Display Total Column?</th><th>Length of Underlines</th>
      </tr>
      <tr>
        <td align=center>
          <select name="total_column">
END_OF_BODY
          my $string = "          <option value=\"true\"";
          if ($total_column eq "true")
          {
            $string .= " selected";
          }
          $string .= ">Yes\n";
          $doc->print($string);

          $string = "          <option value=\"false\"";
          if ($total_column eq "false")
          {
            $string .= " selected";
          }
          $string .= ">No\n";
          $doc->print($string);
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center>
          <select name="underline_length">
END_OF_BODY
        $doc->print("          <option value=\"-1\"" . ($underline_length == -1 ? " selected" : "") . ">User Default</option>\n");
        for (my $i=$min_underline_length; $i <= $max_underline_length; $i++)
        {
          my $string = "          <option value=\"$i\"";
          if ($i eq $underline_length)
          {
            $string .= " selected";
          }
          $string .= ">$column_names{$i}</option>\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr >
        <th>Available Column Headers</th><th>Selected Column Headers</th><th colspan=2>&nbsp;</th>
      </tr>
      <tr>
        <td align=center valign=center>
          <table border=0 align=center cellpadding=0 cellspacing=0>
            <tr>
              <td align=center width=85%><select name="available_columns" size="5" width="150">
                <option value="---------------">---------------
                </select>
              </td>
              <td align=center valign=center width=15%><input type="button" name="insert" value=">" onClick="InsertColumnHeader()"><br><input type="button" name="remove" value="<" onClick="RemoveColumnHeader()"></td>
            </tr>
          </table>
        </td>
        <td align=center><select name="selected_columns" size="5" width="150">
          <option value="---------------">---------------
          </select>
        </td>
        <td colspan=2 align=left>
          <input type="button" name="edit" value="Customize Column Header" onClick="DisplaySelectedColumnHeader(); var s = this.form.selected_columns; InitializeChildData(s[s.selectedIndex].value);"><br><br><br>
          <input type="submit" name="next" value="Next Step">&nbsp;&nbsp;
          <input type="button" name="reset_button" value="Reset Form" onclick="reset_form();">
        </td>
      </tr>
    </table>
END_OF_BODY
  }
  else  # This is a detailed view report
  {
    $doc->print(<<"END_OF_BODY");
    <input type="hidden" name="original_name" value="$original_name">
    <input type="hidden" name="report_class" value="custom">
    <input type="hidden" name="column_variable" value="">
    <input type="hidden" name="column_variable_data" value="">
    <input type="hidden" name="date_range" value="false">
    <input type="hidden" name="display_dates" value="false">
    <input type="hidden" name="max_columns" value="1">
    <input type="hidden" name="min_columns" value="1">
    <input type="hidden" name="total_column" value="false">
    <input type="hidden" name="underline_length" value="$default_underline_length">
    <table border=1 cellpadding=2 cellspacing=0 align=center>
      <tr >
        <th>Report Name</th><th> Report Type</th>
      </tr>
      <tr>
        <td align=center><input type=text name="name" value="$name"></td>
        <td align=center>$report_types{$report_type}</td>
      </tr>
      <tr >
        <th>Text Font</th><th>Increase Text Font Size By</th>
      </tr>
      <tr>
        <td align=center>
          <select name="text_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $text_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center>
          <select name="text_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $text_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr >
        <th>Data Font</th><th>Increase Data Font Size By</th>
      </tr>
      <tr>
        <td align=center>
          <select name="data_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $data_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="data_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $data_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
    </table>
END_OF_BODY
  }

  $doc->print(<<"END_OF_BODY");
  </form>

  <form name="edit_form" method="post">
    <input type="hidden" name="app" value="Accounting">
    <input type="hidden" name="state" value="Reports::Editor">
    <input type="hidden" name="command" value="display">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <div align=right><a href="" onClick="document.edit_form.submit();return false;" onMouseOver="window.status='Return to Edit/Create Report - Main Screen';">Edit/Create Report - Main Screen</a></div>
  </form>
</div>
<div id="_bottom"></div>
END_OF_BODY

  # turn off the link decorations
  $doc->setLinkDecorations();

  if ($report_type eq "r")
  {
    # set the onLoad code in place.
    $doc->setOnload(code => "InitializeAvailableColumns();\nInitializeSelectedColumns();\ninitialize();");
  }
  else
  {
    $doc->setOnload(code => "initialize();");
  }

  $doc->setFocus("javascript");
  if ($report_type eq "r")
  {
  $doc->print(<<"END_OF_CODE");
  function InitializeAvailableColumns()
  {
    var s = document.report_form.available_columns;
    // Clear the Select box
    s.options.length = 0;

    // fill in the options that are currently available.
END_OF_CODE
    for (my $i=0; $i < scalar @column_headers; $i++)
    {
      my $string = "    s.options[s.length] = new Option(\"$column_headers{$column_headers[$i]}\", \"$column_headers[$i]\");\n";
      if (!exists $already_selected_column_headers{$column_headers[$i]})
      {
        $doc->print($string);
      }
    }
  $doc->print(<<"END_OF_CODE");
  }

  function InitializeSelectedColumns()
  {
    var s = document.report_form.selected_columns;
    // Clear the Select box
    s.options.length = 0;

    // fill in the options that are already selected.
END_OF_CODE
    foreach my $column (split /,/, $column_variable)
    {
      my $string = "    s.options[s.length] = new Option(\"$column_headers{$column}\", \"$column\");\n";
      $doc->print($string);
    }
  $doc->print(<<"END_OF_CODE");

    // call the data array initialization function
    InitializeSelectedColumnsArray();

    // make sure the bottom window is empty.
    document.getElementById('_bottom').innerHTML = '';
  }

END_OF_CODE
  }
  else  # Detailed View Report
  {
  }

  return $doc;
}

# report_second_screen_edit_create
# takes: mode, id
# creates or edits the specified report's rows.
sub report_second_screen_edit_create
{
  my $self = shift;
  my %args = ( @_, );
  my $mode = $args{'mode'};
  my $id = $args{id};
  my $dbh = $self->dbHandle;
  my $name = "";

  my @arguments = ( "mode", "id");
  my $doc = $self->prepDoc;
  foreach my $value (@arguments)
  {
    if (!exists $args{$value})
    { # error!
      $doc->doRequiredParameterError('report_second_screen_edit_create', $value);
    }
    if (length $args{$value} == 0)
    {
      $self->my_die("Error: Report.cgi - report_second_screen_edit_create", 4005, "Parameter = '$value'");  return undef;
    }
    elsif (!$self->validateParameter($value, $args{$value}))
    {
      $self->my_die("Error: Report.cgi - report_second_screen_edit_create", 4006, "$value = '$args{$value}'");  return undef;
    }
  }

  my $title = "";
  my $target_window = "_top";
  my $command = "update_report";
  my $header = "";
  my ($sth, $rc, @info);

  # validate that the report exists in the report_type_tb.
  $name = $self->DoesReportExist(id => $id);
  if (length $name == 0)
  {
    $self->my_die("Error: Report.cgi - $mode". " report_second_screen_edit_create", 4020, "ID = '$id' at line " . __LINE__);  return undef;
  }

  # get the data for the Report we Are editing.
  my @report_info = $self->GetReportData(dbh => $dbh, id => $id);

  my $report_type = $report_info[2];
  my $date_range = $report_info[5];

  if ($mode eq "create")
  {
    $title = "Create Report Rows";
    $header = "Report '$name', ID = $id";
  }
  elsif ($mode eq "edit")
  {
    $title = "Edit Report Rows";
    $header = "Report - '$name', ID = $id";
  }

  my $doc = $self->prepDoc;
  my $select;
  if ($report_type eq "r")
  {
    foreach my $item (@regular_cell_types)
    {
      $select .= "          <option value=\"$item\"" . ($item eq "text" ? " selected" : "") . ">$regular_cell_types{$item}\n";
    }
  }
  else
  {
    foreach my $item (@detailed_cell_types)
    {
      $select .= "          <option value=\"$item\"" . ($item eq "text" ? " selected" : "") . ">$detailed_cell_types{$item}\n";
    }
  }
  $doc->setFocus("body");
  $doc->print(<<"END_OF_BODY");
<textarea id="debugDiv" style="display: none;" rows="10" cols="150" wrap="virtual">
</textarea>
<div id="formulaEditorDiv" style="display: none; text-align: right;" ></div>
<div id="formulaComparisonDiv" style="display: none; margin-bottom: .5em;" >
  <form name="formula_comparison">
    Initialize form with default values so Results Use Formulas will work even when data is empty.
    <input type="text" name="formula1" value="1" />
    <input type="text" name="formula2" value="1" />
    <input type="text" name="result_true" value="" />
    <input type="text" name="result_false" value="" />
  </form>
</div>
<div id="_top" style="float: left; ">
  <div>
    <form name="mainButton" method="post">
$self->{postInfo}
      <input type="hidden" name="command" value="display">
      <input type="submit" name="main" value="Edit/Create Report - Main Screen">
      <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    </form>
  </div>
  <form name="report_form" method="post" onSubmit="return validate_form();">
$self->{postInfo}
    <input type="hidden" name="command" value="$command">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="hidden" name="id" value="$id">
    <input type="hidden" name="mode" value="$mode">
    <input type="hidden" name="report_type" value="$report_type">
    <input type="hidden" id="rows" name="rows" value="" />
    <input type="hidden" name="windowName" value="">
    <input type="button" id="save" name="save" value="Save" onClick="if (validate_form()) { this.disabled = true; saveReport(this.form); }">
    <span id="reportStatus">Report Loaded</span>
    <input type="button" name="reset_report_button" value="Reset" onclick="reset_report();">&nbsp;
    <input type="checkbox" onclick="var foo = document.getElementById('debugDiv'); foo.style.display = 'inline'; serialize_rows(); foo.value = this.form.rows.value; ">Debug
  </form>
  <div style="padding-bottom: 1em; ">
    <span style="font-weight: bold;">$header</span>
  </div>

  <form name="top_work_form" id="top_work_form" method="post" onSubmit="return false;">
    <input type="hidden" name="app" value="Accounting">
    <input type="hidden" name="state" value="Reports::Editor">
    <input type="hidden" name="command" value="display">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <table class="primary">
      <tr>
        <th>Row #</th><th>Visible</th>
      </tr>
      <tr>
        <td align="center">
          <input type="hidden" name="real_index" value="0">
          <input type="text" name="index" value="0" size="2">
          <input type="button" value="Edit" onclick="edit_row(this.form.index.value);" />
        </td>
        <td align="center">
          <input type="checkbox" name="show" value="true" checked onClick="Checked=true;validate_show_row();">
        </td>
      </tr>
      <tr>
        <th colspan="2">Row Type</th>
      </tr>
      <tr>
        <td colspan="2" align="center">
          <select name="cell_type" onChange="display_middle_window(EditMode);">
          $select
        </select>
      </td>
    </tr>
    </table>
  </form>
</div>
<div id="_middle">
</div>
<hr style="clear:both;">
<div id="_bottom"></div>
END_OF_BODY

  # turn off the link decorations
  $doc->setLinkDecorations();

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");
SCREEN = 'second';
END_OF_CODE
  if ($report_type eq "r")
  {
    # gather the Account Types, Accounts, Sub Accounts and Detailed Reports

    my $counter = 0;
    # Account Types
    my @h = $self->{ACCOUNT}->selectTypeArray;
    $doc->print("// $self->{ACCOUNT}->{TYPEARRAY}\n");
    for (my $k = 1; $k < scalar @h; $k++)  #start at 1 because 0 is a lookup hash
    {
      $doc->print("  AccountTypesArray[".$counter++."] = new AccountTypeEntry('$h[$k]->{name}', '$h[$k]->{type}');\n");
    }

    # Accounts
    $counter = 0;
    foreach my $info ($self->{ACCOUNT}->selectArray)
    {
      $doc->print("  AccountsArray[".$counter++."] = new AccountEntry('".$info->get('name')."', '".$info->get('id')."', '".$info->get('type')."', '1');\n");
    }

    # Sub Accounts
    $counter = 0;
    foreach my $info ($self->{SUBACCOUNT}->selectArray)
    {
      $doc->print("  SubAccountsArray[".$counter++."] = new SubAccountEntry('".$info->get('name') ."','". $info->get('id') ."','". $info->get('type')."');\n");
    }

    # Detailed Reports
    my @detailed_reports = $self->get_all_detailed_reports( dbh => $dbh );

    for ($counter=0; $counter < @detailed_reports; $counter++)
    {
      $doc->print("  DetailedReportsArray[$counter] = new ReportEntry(\"$detailed_reports[$counter][1]\", \"$detailed_reports[$counter][0]\");\n");
    }

    foreach my $item (@report_levels)
    {
      $doc->print("  ReportDisplayLevels[\"$item\"] = new Array();\n");
      my $counter = 0;
      foreach my $level (@{$report_levels{$item}})
      {
        $doc->print("  ReportDisplayLevels[\"$item\"][" . $counter++ . "] = \"$level\";\n");
      }
    }

    foreach my $item (@font_names)
    {
      $doc->print("  Fonts[\"$item\"] = \"$font_names{$item}\";\n");
    }

    foreach my $item (@negative_formats)
    {
      $doc->print("  negative_formats[\"$item\"] = '$negative_formats{$item}';\n");
    }

    foreach my $item (@regular_cell_types)
    {
      $doc->print("  cell_types[\"$item\"] = '$regular_cell_types{$item}';\n");
    }

    $doc->print(<<"END_OF_CODE");
    Date_Range = $date_range;  // Indicates if we are doing an activity or balance report.

    // Helper functions

    function initialize_work_form()
    {
      Rows.length = 0;

END_OF_CODE

    # gather all rows from the database and populate the Rows array with them.
    my @rows = $self->GetReportRows(id => $id);
    my $activityFix = 0;

    for ($counter = 0; $counter < scalar @rows; $counter++)
    {
      my %info = %{$rows[$counter]};
      my $cell_type = $info{'cell_type'};
      $info{cell_formula} =~ s/\\/\\\\/g;
      $info{cell_formula} =~ s/\//\\\//g;
      $info{'cell_formula'} =~ s/"/\\\"/g;
      $info{cell_text} =~ s/\\/\\\\/g;
      $info{cell_text} =~ s/\//\\\//g;
      if ($cell_type !~ /^(text|line|total|compare)$/ && $info{activity} != $date_range)
      {
        $activityFix = 1;
        $info{activity} = $date_range;
      }
      my $sublevels = ($info{'sub_levels'} == -1 ? "---------------" : "$report_levels{$cell_type}[$info{'sub_levels'}]");
      my $drillable = ($info{'drillable'} ? 0 : 1);
      $doc->print(<<"END_OF_CODE");
Rows[$counter] = new rowEntry("$info{'index'}", "$info{'cell_type'}", "$info{'item_name'}", "$info{'cell_text'}", "$info{'cell_formula'}", "$t_or_f{$info{'activity'}}", "$info{'negative_format'}", "$t_or_f{$info{'bold'}}", "$sublevels", "$info{'initial_state'}", "$info{'display_level'}", "$drillable", "$info{'query_type'}", "$info{'show'}", "$info{'result_type'}", "$info{'underline'}");
END_OF_CODE
    }

    if ($activityFix)
    {
      my $onLoadCode = <<"END_OF_CODE";
    alert("You must save the report as the activity flag is now different.");
END_OF_CODE
      $doc->setOnload(code => $onLoadCode);
    }

    $doc->print(<<"END_OF_CODE");
      reset_row();
      //window.setTimeout('update_bottom_window(true);', 30);
      document.report_form.windowName.value = window.parent.name;
    }
END_OF_CODE
    my $on_unload_code = <<"END_OF_CODE";
    if (typeof(FormulaWindow) != "undefined")
    {
      // see if the window is closed.
      if (FormulaWindow.closed == false)
      {
        FormulaWindow.close();  // close the window.
      }
    }

    if (typeof(CompareFormulaWindow) != "undefined")
    {
      // see if the window is closed.
      if (CompareFormulaWindow.closed == false)
      {
        CompareFormulaWindow.close();  // close the window.
      }
    }
END_OF_CODE
    $doc->setOnunload(code => $on_unload_code);  # set the onUnload Event handler to make sure that the windows are cleaned up.
  }

  # generate the javascript code to be run when the page has been loaded.
  my $on_load_code = $doc->{javascriptOnload};  # make sure to preserve anything already specified.

  $on_load_code .= <<"END_OF_CODE";
  initialize_work_form();
END_OF_CODE

  $doc->setOnload(code => $on_load_code);
  return $doc;
}

# takes Title, Error Code, Custom Message
sub my_die
{
  my $self = shift;
  my ($title, $code, $message) = @_;
  $self->error("<div>$title</div><div>$code:$errors{$code}</div><div>$message</div>");
}

###################### Data Gathering Functions converted from reports.pm to DBI structure #############################

# Takes  : dbh, name of report to find
# Returns: -1 if not found, id otherwise
sub FindReportByName
{
  my $self = shift;
  my %args = ( @_ );
  my $dbh = $self->dbHandle;
  my $name = $args{'name'};
  my $result = -1;  # assume it is not found.

  if (length $dbh == 0)
  {
    $self->my_die("Error: Report.cgi - FindReportByName", 4028, "The DB Handle is required at line " . __LINE__);  return undef;
  }

  if (length $name == 0)
  {
    $self->my_die("Error: Report.cgi - FindReportByName", 4034, "The Report Name is required at line " . __LINE__);  return undef;
  }

  my $sth = $dbh->read(sql => $sql_statements{1003}, plug => [$name]);
  my @info = $sth->fetchrow_array;

  if (defined $info[0])
  {
    $result = $info[0];
  }


  return $result;
}

# Takes  : dbh, id of report
# Returns: array with the report_type_tb entry values.
sub GetReportData
{
  my $self = shift;
  my %args = ( @_ );
  my $dbh = $self->dbHandle;
  my $id = $args{'id'};

  if (length $dbh == 0)
  {
    $self->my_die("Error: Report.cgi - GetReportData", 4028, "The DB Handle is required at line " . __LINE__);  return undef;
  }

  if (length $id == 0)
  {
    $self->my_die("Error: Report.cgi - GetReportData", 4034, "The Report ID is required at line " . __LINE__);  return undef;
  }

  my $sth = $dbh->read(sql => $sql_statements{1006}, plug => [$id]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  my @info = $sth->fetchrow_array;



  return @info;
}

# Takes  :  id of report (I'm assuming the report exists!)
# Returns: 1 if editable, 0 if not
sub IsReportEditable
{
  my $self = shift;
  my %args = ( @_ );
  my $dbh = $self->dbHandle;
  my $id = $args{'id'};
  my $result = 0;  # assume it is not editable.

  if (length $id == 0)
  {
    $self->my_die("Error: Report.cgi - IsReportEditable", 4034, "The Report ID is required at line " . __LINE__);  return undef;
  }
  my $sth = $dbh->read(sql => $sql_statements{1038}, plug => [$id]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  if (my @info = $sth->fetchrow_array)
  {
    $result = $info[0];
  }
  return $result;
}

# Takes  : dbh, id of report
# Returns: report_type
sub TypeOfReport
{
  my $self = shift;
  my %args = ( @_ );
  my $dbh = $self->dbHandle;
  my $id = $args{'id'};
  my $result;

  if (length $dbh == 0)
  {
    $self->my_die("Error: Report.cgi - TypeOfReport", 4028, "The DB Handle is required at line " . __LINE__);  return undef;
  }

  if (length $id == 0)
  {
    $self->my_die("Error: Report.cgi - TypeOfReport", 4034, "The Report ID is required at line " . __LINE__);  return undef;
  }

  my $sth = $dbh->read(sql => $sql_statements{1064}, plug => [$id]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  my @info = $sth->fetchrow_array;

  $result = $info[0];


  return $result;
}


1;
__END__;