#$Id: Detailed.pm,v 1.26 2006/05/31 20:36:51 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

Reports::Objects::Detailed

=head1 SYNOPSIS

  use Reports::Objects::Detailed;
  my $obj = Reports::Objects::Detailed->new(langObj =>);
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

  # set object data by specifying the data field desired
  my $form = $obj->printForm();

=head1 DESCRIPTION

Reports::Objects::Detailed is a class that stores data, methods, and forms for a
Detailed View Report Customization screen.

=head1 Exported FUNCTIONS

=cut

package Portal::Accounting::Reports::Objects::Detailed;

use strict;
use Portal::Accounting::Objects::JournalEntry::Base;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::SubEntity;
use Date::Calc qw(Today);
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.26 $';

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => "", @_);
  my $act = Portal::Accounting::Objects::Account->new(@_);
  my $sact = Portal::Accounting::Objects::SubAccount->new(@_);
  my $se = Portal::Accounting::Objects::SubEntity->new(@_);
  my $je = Portal::Accounting::Objects::JournalEntry::Base->new(@_);
  if ($act->error) { $self->error($act->errorMessage); }
  if ($sact->error) { $self->error($sact->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }
  if ($je->error) { $self->error($je->errorMessage); }
  my $l = $self->{langObj};

  #used to create field list in selection screen
  $self->{fields} = {
    "account_name" => "Account Name",
    "account_code" => "Account Code",
    "account_type" => "Account Type",
    "id" => "ID",
    "journal" => "Journal",
    "date" => "Date",
    "payee_payer" => "Payee-Payer",
    "comment" => "Comment",
    "sub_e_code" => "Sub Entity",
    "sub_account_name" => "Sub Account Name",
    "sub_account_code" => "Sub Account Code",
    "sub_account_type" => "Sub Account Type",
    "stock" => "Stock",
    taxable => 'Taxable',
    bond => "Bond",
    asset => "Asset",
    supplier => "Supplier",
    employee => "Employee",
    inventory => "Inventory",
    partner => "Partner/Stockholder",
    "amount" => "Amount",
    "cleared" => "Cleared",
  };
  #used to turn some fields into other fields ex. asset -> sub_account_name
  #uses this list for validation of search parameter
  $self->{fieldMap} = {
    "account_num"  => "account_num",
    "account_name"  => "account_name",
    "account_code"  => "account_code",
    "account_type"  => "account_type_num",
    "id"            => "id",
    "journal"       => "journal",
    "date"          => "date",
    "payee_payer"   => "payee_payer",
    "comment"       => "comment",
    "sub_e_code"    => "sub_e_code",
    "stock"         => "stock",
    "taxable"       => "taxable",
    "amount"        => "amount",
    "cleared"       => "cleared",
    "sub_account_num" => "sub_account_num",
    "sub_account_code" => "sub_account_code",
    "sub_account_type" => "sub_account_type",
    sub_account_name => "sub_account_name",
    bond      => "sub_account_name",
    asset     => "sub_account_name",
    supplier  => "sub_account_name",
    employee  => "sub_account_name",
    inventory => "sub_account_name",
    partner   => "sub_account_name",
  };
  #these are the columns that can be sorted on
  #uses the values of key for search parameter
  #uses the values for grouping parameter
  #used for sorting parameter?
  $self->{columns} = {
    id                => 'db.parent_num',
    journal           => 'je.journal',
    date              => 'db.trans_date',
    payee_payer       => 'je.payee_payer',
    sub_e_code        => 'db.sub_e_code',
    account_num       => 'db.account_num',
    account_name      => 'a.name',
    account_code      => 'a.code',
    account_type_num  => 'a.type',
    account_type      => 'at.name',
    taxable           => 'a.taxable',
    sub_account_num   => 'db.sub_account_num',
  #Sub Accounts cannot be used in join because the bank accounts use numbers that would clash with names and the numbers that dont have names in sub_acc_tb get left out of queries.
    sub_account_name  => '(SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num)',
    sub_account_code  => '(SELECT code FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num)',
    sub_account_type_num => '(SELECT type FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num)',
    sub_account_type  => '(SELECT name FROM sub_acct_type_tb WHERE num = (SELECT type FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num))',
    cleared           => 'db.cleared',
    comment           => 'db.comment',
    amount            => 'db.d_amount',
  };
  #used for display on screen
my %fieldOptions = ( names => [], values => []);
  foreach my $key (sort keys %{$self->{fields}})
  {
    push (@{$fieldOptions{values}}, $key);
    push (@{$fieldOptions{names}}, $self->{fields}->{$key});
  }
  my %sortOptions = ( names => [], values => []);
  foreach my $key (sort keys %{$self->{columns}})
  {
    if (defined $self->{fields}->{$key})
    {
      push (@{$sortOptions{values}}, $key);
      push (@{$sortOptions{names}}, $self->{fields}->{$key});
    }
  }
  my @date = Today;
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Reports::Detailed" },
    "command" => { -Type => "hidden", -Value => "run_report" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "sorted_by" => {-Type => "hidden", -Value => "", -DataType => "string", },
    "sorted_order" => {-Type => "hidden", -Value => "a", -DataType => "string", },
    "subtotal" => { -Type => "checkbox", -DataType => 'int',
      -Value => 0, -Label => 'Subtotal', checked => 1,
    },
    "sql_statement" => {-Type => "hidden", -Value => "", -DataType => "string", },
    "report_name" => {-Type => "text", -Value => $args{ledgerCode}." ".$l->map("Transactions"), -DataType => "string", -Label => 'Report Name',},
    "field_name" => {-Type => "select", -Options => \%fieldOptions, -Label => 'Field',
      onchange => 'updateFillin();', -NoSelectByDefault => 1,
    },
    "datePicker" => { -Type => "date-picker", -DataType => "string", -year => $date[0], -Label => 'Date', onchange => 'this.form.value_text.value = this.value;',},
    "format" => { -Type => "select", -DataType => "string", -Options => { names => [qw(html csv)], values => [qw(html csv)]}, },
    #output hidden fields for default columns that should show up
    "date" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
    "payee_payer" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
    "account_name" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
    "sub_account_name" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
    "comment" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
    "debit" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
    "sub_e_code" => { -Type => "hidden", -DataType => 'int', -Value => 1, },
  };
  $self->{css}->{main} .= <<"ENDOF";
    .row { background: #ffffff; }
ENDOF
  my @slist = $se->selectArray;
  if ($se->error) { $self->error($se->errorMessage); }
  my $subE;
  if (scalar @slist > 0) { $subE = "#FIELD=sub_e_code#"; }
  $self->{template}->{main} = <<"ENDOF";
<div style="text-align: center;">
  <form method="post" name="main" target="_top">
    #FIELD=app#
    #FIELD=state#
    #FIELD=command#
    #FIELD=ledgerCode#
    #FIELD=sorted_by#
    #FIELD=sorted_order#
    #LABEL=report_name# #FIELD=report_name#
    <table width="100%" class="primary">
      <tr>
        <th>#LABEL=field_name#</th>
        <th>#LANG=Operator#</th>
        <th>#LANG=Value#</th>
        <th>&nbsp;</td>
      </tr>
      <tr>
        <td>
          #FIELD=field_name#
        </td>
        <td>
          <select name="operator">
            <option value="=">=</option>
            <option value="<">&lt;</option>
            <option value="<=">&lt;=</option>
            <option value=">">&gt;</option>
            <option value=">=">&gt;=</option>
            <option value="<>">&lt;&gt;</option>
            <option value="LIKE">LIKE</option>
          </select>
        </td>
        <td>
          <select id="fillin" name="fillin" onchange="handleFillinSelection();" style="display: none;">
            <option value="">----</option>
          </select>
          <span id="datePicker" style="display: none;">#LABEL=datePicker# #FIELD=datePicker#</span>
          <input type="text" id="value_text" name="value_text" value="" size="25" onchange="this.form.fillin.selectedIndex=0;" style="display: none;" />
        </td>
        <td>
          <input type="button" name="add_button" value="Add Row" onclick="add_statement('data');">
          <input type="button" name="clear_button" value="Clear" onclick="clear_statement(); document.main.field_name.focus();">
          <br />
          <select name="connector">
            <option value="AND">AND</option>
            <option value="OR">OR</option>
            <option value="AND (">AND (</option>
            <option value="OR (">OR (</option>
            <option value="(">(</option>
            <option value=")">)</option>
          </select>
          <input type="button" name="add_connector" value="Add Connector" onclick="add_statement('connector');">
        </td>
      </tr>
    </table>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <input type="checkbox" onclick="bDebug = !bDebug; updateDiv();">&nbsp;Debug&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <input type="submit" value="Run Report" onclick="return validate_form();">&nbsp;
    <input type="button" value="Reset" onclick="document.main.reset(); reset_form();">
    #FIELD=sql_statement#
    #FIELD=date#
    #FIELD=payee_payer#
    #FIELD=account_name#
    #FIELD=sub_account_name#
    #FIELD=comment#
    #FIELD=debit#
    $subE
  </form>
</div>
<div id="dataDiv" style="margin: 1em;">
</div>
ENDOF
  $self->{javascript}->{main} .= $je->PAYEES(days => 1000, journal => '') . "\n";
  $self->{javascript}->{main} .= $act->javascriptOverhead($args{ledgerCode});
  $self->{javascript}->{main} .= $sact->javascriptOverhead($args{ledgerCode});
  $self->{javascript}->{main} .= $se->javascriptOverhead($args{ledgerCode});
  if ($act->error) { $self->error($act->errorMessage); }
  if ($sact->error) { $self->error($sact->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }
  if ($je->error) { $self->error($je->errorMessage); }

  $self->{profile}->{main} = {
    required => [  ],
    optional => [  ],
    constraints => {
    },
  };
  $self->{defaultForm} = "main";

  $self->{data}->{display} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Reports::Detailed" },
    "command" => { -Type => "hidden", -Value => "run_report" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "sorted_by" => { -Type => "select", -DataType => "string", -Options => \%sortOptions, -NoSelectByDefault => 1, -Label => 'Sort on',
      onchange => "var field = this[this.selectedIndex].value; eval('this.form.'+field+'.checked = true;');",
    },
    "sorted_order" => {-Type => "select", -Value => "a", -DataType => "string",
      -Options => { names => [qw(Ascending Descending)], values => [qw(a d)]},
    },
    "format" => { -Type => "select", -DataType => "string", -Options => { names => [qw(html csv)], values => [qw(html csv)]}, -Label => 'Format',
      onchange => "if (this[this.selectedIndex].value == 'html') { this.form.target = ''; } else { this.form.target = '_new'; }",
    },
    "subtotal" => { -Type => "checkbox", -DataType => 'integer', -Value => 1, -Label => 'Subtotal', checked => 1, onclick => 'if (this.checked && document.display.sorted_by.selectedIndex < 0) { alert("You must select a field for sorting to subtotal."); this.checked = false; }',},
    "debits" => { -Type => "select", -DataType => "string",
      -Options => {values => [qw(debits positive)], names => ['Debits/Credits','Postive/Negative'] },
      onchange => 'debitsOrPositive(this[this.selectedIndex].value);',
    },
    "sql_statement" => {-Type => "hidden", -Value => "", -DataType => "string", },
    "report_name" => {-Type => "text", -Value => "Transaction Report", -DataType => "string", -Label => 'Report Name',
      -onload => 'setLevelsByClass("hyper"); hyperTables();', class => 'ledgerBorderColor',
#      -onload => 'hyperTables();', class => 'ledgerBorderColor',
    },
    "id" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'ID', checked => 0, onclick => 'toggleColumn(this);', },
    "date" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Date', checked => 0, onclick => 'toggleColumn(this);',  },
    "journal" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Journal', checked => 0, onclick => 'toggleColumn(this);',  },
    "payee_payer" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Payee/Description', checked => 1, onclick => 'toggleColumn(this);',  },
    "sub_e_code" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'SubEntity', checked => 1, onclick => 'toggleColumn(this);',  },
    "account_name" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Account Name',  checked => 1, onclick => 'toggleColumn(this);',  },
    "account_code" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Account Code',  checked => 0, onclick => 'toggleColumn(this);',  },
    "account_type" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Account Type',  checked => 0, onclick => 'toggleColumn(this);',  },
    "sub_account_code" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Sub Account Code', checked => 0, onclick => 'toggleColumn(this);',   },
    "sub_account_name" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Sub Account Name', checked => 1, onclick => 'toggleColumn(this);',   },
    "sub_account_type" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Sub Account Type',  checked => 0, onclick => 'toggleColumn(this);',  },
    "comment" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Comment', checked => 1, onclick => 'toggleColumn(this);',   },
    "cleared" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Cleared', checked => 0, onclick => 'toggleColumn(this);',   },
    "amount" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Amount', checked => 0, onclick => 'toggleColumn(this);',   },
    "debit" => { -Type => "checkbox", -DataType => 'int', -Value => 1, -Label => 'Debit and Credit', checked => 1, onclick => 'toggleColumn(this);',   },
  };
  $self->{template}->{display} = <<"ENDOF";
<div style="text-align: center;">
  <form method="post" name="display" target="" class="ledgerBGColor">
    <span class="printHide">#LABEL=report_name#</span>
    #FIELD=report_name#
    <span class="printHide">
    #FIELD=sql_statement#
    #FIELD=app#
    #FIELD=state#
    #FIELD=command#
    #FIELD=ledgerCode#
    #LABEL=sorted_by# #FIELD=sorted_by#
    #FIELD=sorted_order#
    #LABEL=subtotal# #FIELD=subtotal#
    &nbsp;&nbsp;&nbsp;#LABEL=format# #FIELD=format#
    <a href="javascript: document.display.submit();">Redisplay</a>
    <div id="smallDiv" onclick="this.style.display = 'none'; showDiv('largeDiv');">More Options &gt;&gt;</div>
    <div id="largeDiv" style="display: none;">
      <span onclick="showDiv('smallDiv'); hide('largeDiv');">&lt;&lt Hide</span>
      <table>
        <tr><td width="1px">Show</td><td>Column</td></tr>
        <tr><td>#FIELD=id#</td><td>#LABEL=id#</td></tr>
        <tr><td>#FIELD=journal#</td><td>#LABEL=journal#</td></tr>
        <tr><td>#FIELD=date#</td><td>#LABEL=date#</td></tr>
        <tr><td>#FIELD=payee_payer#</td><td>#LABEL=payee_payer#</td></tr>
        <tr><td>#FIELD=sub_e_code#</td><td>#LABEL=sub_e_code#</td></tr>
        <tr><td>#FIELD=account_name#</td><td>#LABEL=account_name#</td></tr>
        <tr><td>#FIELD=account_code#</td><td>#LABEL=account_code#</td></tr>
        <tr><td>#FIELD=account_type#</td><td>#LABEL=account_type#</td></tr>
        <tr><td>#FIELD=sub_account_name#</td><td>#LABEL=sub_account_name#</td></tr>
        <tr><td>#FIELD=sub_account_code#</td><td>#LABEL=sub_account_code#</td></tr>
        <tr><td>#FIELD=sub_account_type#</td><td>#LABEL=sub_account_type#</td></tr>
        <tr><td>#FIELD=comment#</td><td>#LABEL=comment#</td></tr>
        <tr><td>#FIELD=cleared#</td><td>#LABEL=cleared#</td></tr>
        <tr><td>#FIELD=debit#</td><td>#LABEL=debit#</td></tr>
        <tr><td>#FIELD=amount#</td><td>#LABEL=amount#</td></tr>
      </table>
    </div>
    </span>
  </form>
ENDOF
  $self->{profile}->{display} = {
    required => [ qw(sql_statement) ],
    optional => [  ],
    constraints => {
    },
  };
  $self->{css}->{display} = <<"ENDOF";
table { width: 100%; }
th { text-align: left; }
td { text-align: left; }
th.level { display: none; }
td.level { display: none; }
th.account_num { display: none; }
td.account_num { display: none; }
th.sub_account_num { display: none; }
td.sub_account_num { display: none; }
th.account_type_num { display: none; }
td.account_type_num { display: none; }
th.sub_account_type_num { display: none; }
td.sub_account_type_num { display: none; }
th.debit { text-align: right; }
td.debit { text-align: right; }
th.credit { text-align: right; }
td.credit { text-align: right; }

th.amount { text-align: right; }
td.amount { text-align: right; }

.level1 { font-size: 1em; }
.level2 { font-size: 1.1em; }
ENDOF
  return $self;
}

sub isNew { return 1; }

=item Array selectSQL

  Returns array with first element being sql statement and the args to send with it after.

=cut
sub selectSQL
{
  my $self = shift;
  my $sorted_by = $self->get('sorted_by');
  my $subtotal = $sorted_by;
  my $found = 0;
  my %operators = ( ">" => ">", "<" => "<", ">=" => ">=", "<=" => "<=", "=" => "=", "<>" => "<>", "LIKE" => "LIKE" );
  my @plug = (); #dont add ledgerCode until putting all of it together
  my $UNION;
  my $WHERE = <<"ENDOF";
  WHERE
    db.ledger_code = ?
    AND db.parent_num = je.ref_num
    AND db.account_num = a.number
    AND at.num = a.type
ENDOF
  #process data sent in for where clause
  my $sql_statement = $self->get('sql_statement');
  my @rows = split(';', $sql_statement);
  if (@rows == 0) { $self->error('<div class="error">No rows parsed out of : '.$sql_statement.'</div>'); return undef; }
  $WHERE .= " AND ( \n";
  foreach my $row (@rows)
  {
    my @fields = split(/\|/,$row);
    if ($fields[0] eq 'connector')
    {
      $WHERE .= " $fields[2] \n";
    }
    else
    {
      if (!defined $operators{$fields[1]}) { $self->error('<div class="error">Invalid operator "'.$fields[1].'" in <div>'.$sql_statement."</div></div>\n"); return undef; }
      #some fields are not sortable like amount.
      #other fields like stock get turned into sub_account_name.
      #use fieldMap to get the list of valids and to convert into proper field name in SQL where clause
      if (!exists $self->{fieldMap}->{$fields[0]})
      {
        $self->error('<div class="error">Invalid column name "'.$fields[0].'" in <div>'.$sql_statement."</div></div>\n");
        return undef;
      }
      $WHERE .= " $self->{columns}->{$self->{fieldMap}->{$fields[0]}} $fields[1] ?\n";
      push(@plug, $fields[2]);
    }
  }
  $WHERE .= " )\n";
  #  the case statements on sub_account name and code are to keep from grabbing data for check and deposit numbers on bank account entries.  Sales Invoices are supposed to be in sales revenue accounts.  The nosubs flag marks these in the acct_type_tb but the flag also marks Investments so that the user cannot enter data into an investment account outside of the investment journal.  So we use nosubs OR type = 3 (investment) to decide when to grab sub account data.
  # cannot join with sub_account tables because bank transactions and the like get left out since there is no name and code equivalent for check and deposit numbers.  Had to use subselects instead.
  my $sql = <<"ENDOF";
  SELECT
    1 AS level,
    db.parent_num AS id,
    je.journal,
    db.trans_date::text AS date,
    je.payee_payer AS payee_payer,
    db.sub_e_code AS sub_e_code,
    db.account_num AS account_num,
    a.name AS account_name,
    a.code  AS account_code,
    a.type AS account_type_num,
    at.name AS account_type,
    CASE WHEN a.taxable = true THEN 'true' ELSE 'false' END AS taxable,
    db.sub_account_num AS sub_account_num,
    CASE WHEN at.nosubs = false OR a.type = 3 THEN (SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num) WHEN db.sub_account_num <> -1 THEN db.sub_account_num::text ELSE NULL END AS sub_account_name,
    CASE WHEN at.nosubs = false OR a.type = 3 THEN (SELECT code FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num) WHEN db.sub_account_num <> -1 THEN db.sub_account_num::text ELSE NULL END AS sub_account_code,
    CASE WHEN at.nosubs = false OR a.type = 3 THEN (SELECT type FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num) ELSE NULL END AS sub_account_type_num,
    CASE WHEN at.nosubs = false OR a.type = 3 THEN (SELECT name FROM sub_acct_type_tb WHERE num = (SELECT type FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num) ) ELSE NULL END AS sub_account_type,
    db.cleared AS cleared,
    db.comment,
    db.d_amount AS amount,
    CASE WHEN db.dorc = true THEN db.amount ELSE NULL END AS debit,
    CASE WHEN db.dorc = false THEN db.amount ELSE NULL END AS credit
  FROM debitcredit_tb AS db, journal_entry_tb AS je, account_tb AS a, acct_type_tb AS at
  $WHERE
ENDOF
  my $ORDER = <<"ENDOF";
  ORDER BY
ENDOF
  if ($self->{data}->{display}->{subtotal}->{checked})
  {
    $UNION = <<"ENDOF";
  UNION SELECT 2 AS level
ENDOF
    #array of columns needed to order them properly
    my @columns = qw( id journal date payee_payer sub_e_code account_num account_name account_code account_type_num account_type taxable sub_account_num sub_account_name sub_account_code sub_account_type_num sub_account_type cleared comment);
    for (my $i = 0; $i < scalar @columns; $i++)
    {
      if ($columns[$i] ne $subtotal)
      {
        $UNION .= ", NULL AS $columns[$i] ";
      }
      else
      {
        $found = 1;
        $UNION .= ", $self->{columns}->{$columns[$i]} || ' Total' AS $columns[$i] ";
      }
    }
    if (!$found) { $self->error("<div class=\"error\">subtotal: $subtotal is not valid!</div>"); }
    $UNION .= <<"ENDOF";
    , SUM(db.d_amount) AS amount,
    SUM(CASE WHEN db.dorc = true THEN db.amount ELSE NULL END) AS debit,
    SUM(CASE WHEN db.dorc = false THEN db.amount ELSE NULL END) AS credit
  FROM debitcredit_tb AS db, journal_entry_tb AS je, account_tb AS a, acct_type_tb AS at
  $WHERE
  GROUP BY $subtotal
ENDOF
    @plug = (@plug, $self->get('ledgerCode'), @plug );
  }
  if ($sorted_by ne '')
  {
    my $ASC_DESC = $self->get('sorted_order');
    if ($ASC_DESC eq 'a') { $ASC_DESC = 'ASC'; }
    else { $ASC_DESC = 'DESC'; }
    $ORDER .= "$sorted_by $ASC_DESC, level, date"
  }
  else
  {
    $ORDER .= "level, date"
  }
  $self->{SELECT}[0] = $sql.$UNION.$ORDER;
  return ($sql.$UNION.$ORDER, $self->get('ledgerCode'), @plug );
}

sub printForm
{
  my $self = shift;
  my $css = '';
  my %args = (form => 'main', @_);
  my %f = $self->SUPER::printForm(@_);
  if ($args{form} eq 'display')
  {
    foreach my $field (qw(id journal date payee_payer sub_e_code account_name account_code account_type taxable sub_account_name sub_account_code sub_account_type comment cleared amount))
    {
      if ($self->{data}->{display}->{$field}->{checked} == 0)
      {
        $css .= "th.$field { display: none; }\ntd.$field { display: none; }\n";
      }
      else
      {
        $css .= "th.$field { }\ntd.$field { }\n";
      }
    }
    if ($self->{data}->{display}->{debit}->{checked} == 1)
    {
      $css .= "th.debit { }\ntd.debit { }\n";
      $css .= "th.credit { }\ntd.credit { }\n";
    }
    else
    {
      $css .= "th.debit { display: none; }\ntd.debit { display: none; }\n";
      $css .= "th.credit { display: none; }\ntd.credit { display: none; }\n";
    }
  }
  $f{css} .= $css;
  return %f;
}

sub printHTML
{
  my $self = shift;
  my @sql = $self->selectSQL;
  if ($self->error) { return undef; }
  my $sql = shift @sql;
  $self->{PLUG} = \@sql;
  my $html = $self->dbHandle->readHTML(
    sql => $sql,
    plug => $self->{PLUG},
    tableClass => 'hyper',
    displayNULLAs => '',
    headers => {
      %{$self->{fields}}, debit => 'Debit', credit => 'Credit',
    },
    footer => 1,
    @_,
  );
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  return $html;
}

1;
__END__

=pod

=head1 AUTHOR

JT Moree

=head1 SEE ALSO

Portal::Objects::Base, Portal::Accounting::Objects::BaseL

=cut