package Portal::Accounting::Reports::Customize;
use strict;
use Portal::Accounting::Reports::Main;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

use HTMLObject::Widgets;
use HTMLObject::FrameSet;
use HTMLObject::Normal;
use HTMLObject::ReadCookie; # the module used to read cookies sent to us from the client.
use Storable;  # the module that will allow me to dump data to disk and recover later.  The file names will be stored in the cookie.
use POSIX qw(tmpnam); # the tmpnam function which will return a file name for a temporary file supposed to be unique.
use Date::Calc qw(Today Add_Delta_Days);

require Exporter;

@ISA = qw(Portal::Accounting::Reports::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.27 $';

my %commands = (
  display => "Reports",
  run_report => "Reports",
);

my $delim = '|';
my %errors;
%errors = (
        4000 => 'Eval of HTMLObject::FrameSet failed',
        4001 => 'Invalid command specified',
        4002 => 'Eval of HTMLObject::Normal failed',
        4003 => 'command must be specified',
        4004 => "Couldn't connect to Database",
        4005 => "Parameter requires a value",
        4006 => "Invalid Value for Parameter",
        4007 => "DBI Preparation of SQL Statement Failed",
        4008 => "DBI Execution of Statement Failed",
        4009 => "DBI Error Occurred",
        4013 => "Could not find Sub Account",
        4014 => "Could not find Sub Account Type",
        4017 => "Report ID is required",
        4019 => "Error Committing Database changes",
        4022 => "Ledger Code required",
        4023 => "Could not find Ledger Database",
        4024 => "Could not find Ledger Name",
        4025 => "Could not find Ledger Code",
        4026 => "Ledger DB Name required",
        4027 => "Ledger Name required",
        4028 => "Database Handle required",
        4029 => "Memorized value required",
        4030 => "Invalid ID format",
        4034 => "Parameter Required",
        4035 => "File Name required",
        4037 => "Problem with Formula evaluation",
);

my %sql_statements;
%sql_statements = (
   1006 => "SELECT name, report_class, report_type, column_variable, column_variable_data, date_range, display_dates, max_columns, min_columns, text_font, text_size, data_font, data_size, total_column, underline_length, editable FROM report_type_tb WHERE id = ?",
   1023 => "SELECT name FROM report_type_tb WHERE id = ?",
   1033 => "SELECT name, code FROM sub_entity_tb WHERE ledger_code = ?",
   1034 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
   1035 => "SELECT name FROM sub_acct_type_tb WHERE num = ?",
   1043 => "SELECT debit_increases FROM acct_type_tb WHERE num = ?",
   1044 => "SELECT number FROM account_tb WHERE type = ? ORDER BY number",
   1045 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = true AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
   1046 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = false AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
   1047 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
   1050 => "SELECT db.amount, db.dorc FROM debitcredit_tb AS db, journal_entry_tb AS je WHERE db.ledger_code = je.ledger_code AND db.ledger_code = ? AND db.parent_num = je.ref_num AND db.account_num = ? AND ##1?####2?####?##",
   1060 => "SELECT num, sub_acct_type FROM acct_type_tb",
   1061 => <<"ENDOF",
  SELECT
    db.parent_num AS id,
    je.journal,
    db.trans_date AS date,
    je.payee_payer,
    db.sub_e_code,
    db.account_num,
    a.name AS account_name,
    a.code  AS account_code,
    a.type AS account_type,
    db.sub_account_num,
    (SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num) AS sub_account_name,
    (SELECT code FROM sub_acc_tb WHERE sub_acc_number = db.sub_account_num) AS sub_account_code,
    db.cleared,
    db.d_amount AS amount,
    CASE WHEN db.dorc = true THEN db.amount ELSE NULL END AS debit,
    CASE WHEN db.dorc = false THEN db.amount ELSE NULL END AS credit,
    db.comment
  FROM debitcredit_tb AS db, journal_entry_tb AS je, account_tb AS a
  WHERE
    db.ledger_code = ?
    AND db.parent_num = je.ref_num
    AND db.account_num = a.number
    AND (##?##)
  ORDER BY je.trans_date, db.account_num, db.sub_account_num, db.sub_e_code
ENDOF
   1062 => "SELECT balance, bal_date FROM acct_bal_tb WHERE ledger_code = ? AND acct_num = ? AND sub_acct_num = ? AND bal_date <= ? ORDER BY bal_date DESC",
   1063 => "SELECT db.amount, db.dorc FROM debitcredit_tb AS db WHERE db.ledger_code = ? AND (##?##db.trans_date <= ?) AND db.account_num = ?##1?##",
   1067 => "SET datestyle TO 'ISO'",
   1071 => "SELECT max(bal_date) FROM acct_bal_tb WHERE ledger_code = ? and bal_date <= ?",
   # account_type row SQL queries
    # account type activity using the stored procedures in the database for account_type row
    1073 => "SELECT account_type_activity(?, ?, ?, ?, ?)",
    1074 => "SELECT account_type_activity(?, ?, ?, ?, ?::text, ?)",
    1075 => "SELECT account_type_activity(?, ?, ?, ?, ?, ?, ?)",
    # account activity using the stored procedures in the database for account_type row
    1076 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num",
    1077 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num",
    1078 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
    1079 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
    1080 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? and sub_account_num = ? group by account_num",
    1081 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? and sub_account_num = ? group by account_num",
    1082 => "SELECT account_num, account_name(account_num), sub_account_type_activity(account_num, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? and sub_account_num = (select sub_acc_number from sub_acc_tb where sub_acc_number = sub_account_num and type = ?) group by account_num",
    1083 => "SELECT account_num, account_name(account_num), sub_account_type_activity(account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? and sub_account_num = (select sub_acc_number from sub_acc_tb where sub_acc_number = sub_account_num and type = ?) group by account_num",
    # account balance/activity using the stored procedures in the database for account_type row
    1084 => <<"ENDSQL",
  SELECT
    db.account_num AS account_num,
    a.name AS account_name,
    account_balance_plus_activity(db.account_num, db.ledger_code, ?, ?)
  FROM
    debitcredit_tb AS db, account_tb AS a
  WHERE
    a.number = db.account_num
    AND db.ledger_code = ?
    AND a.type = ?
    AND db.trans_date <= ?
  GROUP BY db.account_num, a.name, db.ledger_code
  UNION SELECT
    ab.acct_num,
    a.name AS account_name,
    account_balance_plus_activity(ab.acct_num, ab.ledger_code, ?, ?)
  FROM acct_bal_tb AS ab, account_tb AS a
  WHERE
    a.number = ab.acct_num
    AND ab.ledger_code = ?
    AND a.type = ?
    AND ab.bal_date <= ?
  GROUP BY ab.acct_num, a.name, ab.ledger_code
ENDSQL
    1085 => <<"ENDSQL",
  SELECT
    db.account_num,
    a.name AS account_name,
    account_balance_plus_activity(db.account_num, db.ledger_code, ?, ?)
  FROM debitcredit_tb AS db, account_tb AS a
  WHERE
    db.account_num = a.number
    AND db.ledger_code = ?
    AND a.type = ?
    AND a.taxable = ?
    AND trans_date <= ?
  GROUP BY db.acct_num, a.name, db.ledger_code
  UNION
  SELECT
    db.acct_num,
    a.name AS account_name,
    account_balance_plus_activity(db.acct_num, db.ledger_code, ?, ?)
  FROM acct_bal_tb AS db, account_tb AS a
  WHERE
    a.number = db.acct_num
    AND ledger_code = ?
    AND a.type = ?
    AND a.taxable = ?
    and db.bal_date <= ?
  GROUP BY db.acct_num, a.name, db.ledger_code
ENDSQL
    # sub-account activity using the stored procedures in the database for account_type row
    1086 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num, sub_account_num",
    1087 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num, sub_account_num",
    1088 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num, sub_account_num",
    1089 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num, sub_account_num",
    # sub-account balance/activity using the stored procedures in the database for account_type row
    1090 => <<"ENDSQL",
  SELECT
    db.account_num,
    a.name AS account_name,
    db.sub_account_num,
    sub_account_name(db.sub_account_num),
    sub_account_balance_plus_activity(db.account_num, db.sub_account_num, db.ledger_code, ?, ?)
  FROM debitcredit_tb AS db, account_tb AS a
  WHERE
    db.account_num = a.number
    AND db.ledger_code = ?
    AND a.type = ?
    AND db.trans_date <= ?
  GROUP BY db.account_num, a.name, db.sub_account_num, db.ledger_code
  UNION
  SELECT
    db.acct_num,
    a.name AS account_name,
    db.sub_acct_num,
    sub_account_name(db.sub_acct_num),
    sub_account_balance_plus_activity(db.acct_num, db.sub_acct_num, db.ledger_code, ?, ?)
  FROM acct_bal_tb AS db, account_tb AS a
  WHERE
    a.number = db.acct_num
    AND db.ledger_code = ?
    AND a.type = ?
    and db.bal_date <= ?
    and db.sub_acct_num <> -2
  GROUP BY db.acct_num, a.name, db.sub_acct_num, db.ledger_code
ENDSQL
    1091 => <<"ENDSQL",
  SELECT
    db.account_num,
    a.name AS account_name,
    db.sub_account_num,
    sub_account_name(db.sub_account_num),
    sub_account_balance_plus_activity(db.account_num, db.sub_account_num, ?, ?, ?)
  FROM debitcredit_tb AS db, account_tb AS a
  WHERE
    a.number = db.account_num
    AND db.ledger_code = ?
    AND a.type = ?
    AND a.taxable = ?
    AND db.trans_date <= ?
  GROUP BY db.account_num, a.name, db.sub_account_num
  UNION
  SELECT
    db.acct_num,
    a.name AS account_name,
    db.sub_acct_num,
    sub_account_name(db.sub_acct_num),
    sub_account_balance_plus_activity(db.acct_num, db.sub_acct_num, ?, ?, ?)
  FROM acct_bal_tb AS db, account_tb AS a
  WHERE
    a.number = db.acct_num
    AND db.ledger_code = ?
    AND a.type = ?
    AND a.taxable = ?
    AND db.bal_date <= ?
    AND db.sub_acct_num <> -2
  GROUP BY db.acct_num, a.name, db.sub_acct_num
ENDSQL
   # account row SQL queries
    # account row activity using the stored procedures in the database for account row
    1092 => "SELECT sub_account_num, sub_account_name(sub_account_num), sub_account_activity(?, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = ? and trans_date between ? and ? group by sub_account_num",
    1093 => "SELECT sub_account_num, sub_account_name(sub_account_num), sub_account_activity(?, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = ? and trans_date between ? and ? group by sub_account_num",
    # sub-account balance/activity using the stored procedures in the database for account row
    1094 => "SELECT sub_account_num, sub_account_name(sub_account_num), sub_account_balance_plus_activity(?, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = ? and trans_date <= ? group by sub_account_num UNION SELECT sub_acct_num, sub_account_name(sub_acct_num), sub_account_balance_plus_activity(?, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = ? and bal_date <= ? and sub_acct_num <> -2 group by sub_acct_num",
   # sub-account row SQL queries
    # sub-account row activity using the stored procedures in the database for sub-account row
    1095 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_account_num = ? and trans_date between ? and ? group by account_num",
    1096 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and sub_account_num = ? and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
    1097 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and sub_account_num = ? and trans_date between ? and ? group by account_num",
    1098 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and sub_account_num = ? and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
    # sub-account balance/activity using the stored procedures in the database for sub-account row
    1099 => "SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and trans_date <= ? group by account_num UNION SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from acct_bal_tb where ledger_code = ? and bal_date <= ? group by acct_num",
    1100 => "SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by account_num UNION SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by acct_num",
   # sub-accounts of account_type row SQL queries
    # activity
    1101 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
    1102 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
    1103 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
    1104 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?::bool, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
    # ----------
    1105 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
    1106 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
    1107 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
    1108 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::bool, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
    # balance
    1109 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num, account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), acct_num, account_name(acct_num), sub_account_balance_plus_activity(acct_num, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num, acct_num",
    1110 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num, account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), acct_num, account_name(acct_num), sub_account_balance_plus_activity(acct_num, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num, acct_num",
    # -----
    1111 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_act_type_balance_plus_activity(?, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), sub_account_act_type_balance_plus_activity(?, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num",
    1112 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_act_type_balance_plus_activity(?, sub_account_num, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), sub_account_act_type_balance_plus_activity(?, sub_acct_num, ?, ?, ?::bool, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num",
   # account type balances
    1113 => "SELECT account_type_balance_plus_activity(?, ?, ?, ?)",
   # more account type activity
    1114 => "SELECT account_type_activity(?, ?, ?, ?, ?::bool, ?)",
    1115 => "SELECT acct_type_sub_acct_type_activity(?, ?, ?, ?, ?, ?)",
    1116 => "SELECT acct_type_sub_acct_type_activity(?, ?, ?, ?, ?, ?:bool, ?)",
   # more account type balances
    1117 => "SELECT account_type_balance_plus_activity(?, ?, ?, ?::bool, ?)",
    1118 => "SELECT acct_type_sub_acct_type_bp(?, ?, ?, ?, ?)",
    1119 => "SELECT acct_type_sub_acct_type_bp(?, ?, ?, ?, ?::bool, ?)",
   # sub-account activity for account type
    1120 => "SELECT sub_account_account_type_activity(?, ?, ?, ?, ?, ?)",
    1121 => "SELECT sub_account_account_type_activity(?, ?, ?, ?, ?, ?::bool, ?)",
   # sub-account balances/activity for account type
    1122 => "SELECT sub_account_act_type_balance_plus_activity(?, ?, ?, ?, ?)",
    1123 => "SELECT sub_account_act_type_balance_plus_activity(?, ?, ?, ?, ?::bool, ?)",
);

my %report_classes;
%report_classes = ( "custom" => "Custom", "cash" => "Cash Flow", "income" => "Income Statement", "balance" => "Balance Sheet", "acct_balance" => "Accounts Balance Sheet", "acct_activity" => "Accounts Activity Sheet" );
my @report_classes;
@report_classes = ( "custom", "cash", "income", "balance", "acct_balance", "acct_activity" );
my %report_types;
%report_types = ( "r" => "Regular Report", "d" => "Detailed View" );
my @report_types;
@report_types = ( "r", "d" );
my @report_levels;
@report_levels = ( "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "begin_sub_account_balance", "end_sub_account_balance" );
my @account_type_levels;
@account_type_levels = ( "nothing", "accounts", "accounts, sub-accounts" );
my @account_levels;
@account_levels = ( "nothing", "sub-accounts" );
my @accounts_of_levels;
@accounts_of_levels = ( "nothing", "sub-accounts" );
my @sub_account_levels;
@sub_account_levels = ( "nothing", "accounts" );
my @sub_accounts_of_type_levels;
@sub_accounts_of_type_levels = ( "nothing", "accounts" );
my @sub_accounts_of_account_levels;
@sub_accounts_of_account_levels = ( "not available" );
my %report_levels;
%report_levels = ( "account_type" => \@account_type_levels, "account" => \@account_levels, "sub_account" => \@sub_account_levels, "accounts_of" => \@accounts_of_levels, "sub_accounts_of_type" => \@sub_accounts_of_type_levels, "sub_accounts_of_account" => \@sub_accounts_of_account_levels, "begin_account_type_balance" => \@account_type_levels, "end_account_type_balance" => \@account_type_levels, "begin_account_balance" => \@account_levels, "end_account_balance" => \@account_levels, "begin_sub_account_balance" => \@sub_account_levels, "end_sub_account_balance" => \@sub_account_levels );
my %regular_cell_types;
%regular_cell_types = ( "text" => "Text", "total" => "Total", "compare" => "Compare Rows", "account_type" => "Account Type", "account" => "Account", "sub_account" => "Sub Account", "accounts_of" => "Accounts of Account Type", "sub_accounts_of_type" => "Sub Accounts of Account Type", "sub_accounts_of_account" => "Sub Accounts of Account", "begin_account_type_balance" => "Beginning Account Type Balance", "end_account_type_balance" => "Ending Account Type Balance", "begin_account_balance" => "Beginning Account Balance", "end_account_balance" => "Ending Account Balance", "begin_sub_account_balance" => "Beginning Sub Account Balance", "end_sub_account_balance" => "Ending Sub Account Balance", "line" => "Line" );
my @regular_cell_types;
@regular_cell_types = ( "text", "total", "compare", "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "line" ); # "begin_sub_account_balance", "end_sub_account_balance", (to be added later)
my @font_names;
@font_names = ( "helvetica" );
my %font_names;
%font_names = ( "helvetica" => "Helvetica" );
my @font_sizes;
@font_sizes = ( "+2", "+1", "+0", "-1", "-2" );
my %font_sizes;
%font_sizes = ( "+2" => "+2", "+1" => "+1", "+0" => "nothing", "-1" => "-1", "-2" => "-2" );
my @column_headers;
@column_headers = ( "sub_entities", "sub_accounts", "sub_account_types" );
my %column_headers;
%column_headers = ( "sub_entities" => "Sub Entities", "sub_accounts" => "Sub Accounts", "sub_account_types" => "Sub Account Types" );
my @negative_formats;
@negative_formats = ( "0", "1", "2" );
my %negative_formats;
%negative_formats = ( "0" => "<font color=\"red\">(xxx.xx)</font>", "1" => "<font color=\"red\">-</font>xxx.xx", "2" => "xxx.xx<font color=\"red\">-</font>" );
my $max_dates;
$max_dates = 36;   # the maximum number of columns that we are currently supporting in the reporting system.
my @precision_formats;
@precision_formats = ( "1", "10", "100", "1000", "10000", "100000", "1000000", "1000000000" );
my %precision_formats;
%precision_formats = ( "1" => "1", "10" => "10", "100" => "100", "1000" => "1,000", "10000" => "10,000", "100000" => "100,000", "1000000" => "1,000,000", "1000000000" => "1,000,000,000" );
my %true_false;
%true_false = ( "true" => 1, "false" => 0, 1 => 1, 0 => 0 );
my %column_data_sql;
%column_data_sql = ( "sub_entities" => 1033, "sub_accounts" => 1034, "sub_account_types" => 1035 );

# commonly used compiled regular expressions
my $reportNumRE = qr/^(report_\d+)$/;
my $minusRE = qr/\-/;
my $accountRE = qr/\<ACCOUNT\>/;
my $subAccountRE = qr/\<SUBACCOUNT\>/;
my $subAccountTypeRE = qr/\<SUBACCOUNTTYPE\>/;
my $operatorRE = qr/^(\+|-|%|\*|\/)$/;
my $openParenRE = qr/^\($/;
my $closeParenRE = qr/^\)$/;
my $tokenRE = qr/^R\d+|-?\d+\.\d+|-?\d+$/;
my $rowIdentifierRE = qr/^R\d+$/;
my $rowIdentifier2RE = qr/^R(\d+)/;
my $substitutionRE = qr/\#\#\?\#\#/;
my $firstSubstitutionRE = qr/\#\#1\?\#\#/;
my $secondSubstitutionRE = qr/\#\#2\?\#\#/;

# regular view display settings values
my %hide_numbers;
%hide_numbers = {};
$hide_numbers{'account_type'} = {};
$hide_numbers{'account_type'}{'0'} = 1;

$hide_numbers{'accounts_of'} = {};
$hide_numbers{'accounts_of'}{'0'} = 1;

$hide_numbers{'begin_account_type_balance'} = {};
$hide_numbers{'begin_account_type_balance'}{'0'} = 1;

$hide_numbers{'end_account_type_balance'} = {};
$hide_numbers{'end_account_type_balance'}{'0'} = 1;

$hide_numbers{'sub_accounts_of_type'} = {};
$hide_numbers{'sub_accounts_of_type'}{'0'} = 1;
$hide_numbers{'sub_accounts_of_type'}{'1'} = 1;

# my $cookies;
# $cookies = HTMLObject::ReadCookie->new;
# my %ch;
# %ch = $cookies->getCookies(cookies => ['pcx_language']);

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{ACCOUNT} = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($self->{ACCOUNT}->error) { $self->error($self->{ACCOUNT}->errorMessage); }
  $self->{ACCOUNTS} = {$self->{ACCOUNT}->selectHash};
  if ($self->{ACCOUNT}->error) { $self->error($self->{ACCOUNT}->errorMessage); }
  $self->{ACCOUNTTYPES} = {$self->{ACCOUNT}->selectTypeHash};
  if ($self->{ACCOUNT}->error) { $self->error($self->{ACCOUNT}->errorMessage); }
  $self->{SUBACCOUNT} = Portal::Accounting::Objects::SubAccount->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($self->{SUBACCOUNT}->error) { $self->error($self->{SUBACCOUNT}->errorMessage); }
  $self->{SUBACCOUNTS} = {$self->{SUBACCOUNT}->selectHash};
  if ($self->{SUBACCOUNT}->error) { $self->error($self->{SUBACCOUNT}->errorMessage); }
  $self->{SUBACCOUNTTYPES} = {$self->{SUBACCOUNT}->selectTypes};
  if ($self->{SUBACCOUNT}->error) { $self->error($self->{SUBACCOUNT}->errorMessage); }
  if (defined $self->input->{ledgerCode}) { $self->{ledgerCode} = $self->{input}->{ledgerCode}; }
  else { $self->error('no ledgerCode.  Reports do not yet support running consolidated Ledgers');  }

  # do validation
  if (!$self->Portal::Accounting::Reports::Customize::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
#   if ($self->numInvalid() > 0 ,, $self->numMissing() > 0)
#   {
#     $self->postfixError($self->genErrorString("all"));
#     return 0;
#   }
  return 1;
}

sub c_debug
{
  my $self = shift;
  my $doc = $self->prepDoc;
  $doc->print('<h2>DEBUG AND TESTING FOR CUSTOMIZE</h2>');
  foreach my $foo (keys %{$self->{ACCOUNTS}})
  {
    $doc->print($foo."->".$self->{ACCOUNTS}->{$foo}->get('name')."<br>");
  }
  $doc->print("<hr>");
  foreach my $foo (keys %{$self->{SUBACCOUNTS}})
  {
    $doc->print($foo."->".$self->{SUBACCOUNTS}->{$foo}->get('name')."<br>");
  }
  return $doc;
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Reports/Customize.js");
  return $doc;
}

sub c_display
{
  my $self = shift;
  my $dbh = $self->{dbHandle};  # the handle of the database.
  my %input = %{$self->{input}};
  my $currentDate = POSIX::strftime('%Y-%m-%d', localtime);
  chomp $currentDate;
  my $doc;

  # force the database session to use ISO convention date styles
  my $sth = $dbh->write( $sql_statements{1067} );

  # validate the input values
  my $windowName = $input{windowName};
  my $id = $input{'id'};
  if (length $id == 0)
  {
    $self->error("Error:  display. $errors{4017} Report ID required at line " . __LINE__);
    return $doc;
  }

  my $report_name;
  # get the report name from the database
  my $sth = $dbh->read(sql => $sql_statements{1023}, plug => [$id] );
  if ($dbh->error)
  {
    $self->error("<div>Error:  display", 4008, "SQL Statement = <div>$sql_statements{1023}</div></div>".$dbh->errorMessage);
    return undef;
  }
  my @temp_row = $sth->fetchrow_array;

  my $ledgerCode = $self->{input}->{ledgerCode};
  my $ledger_name = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};
  my $ledger_color = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{color};
  $report_name = $ledger_name ." ". $temp_row[0] . "<br />run on " . $currentDate;

  # create generic structure to hold all values and their defaults in order to set them using memorized or pre-configured values
  my %report_attributes;
  $report_attributes{'id'} = $id;
  $report_attributes{'report_name'} = $report_name;

  $report_attributes{'ledger_name'} = $ledger_name;
  $report_attributes{'ledgerCode'} = $ledgerCode;
  $report_attributes{'ledger_color'} = $ledger_color;

  # get the report default values from the database
  my $sth = $dbh->read( sql => $sql_statements{1006}, plug => [$id] );
  if ($dbh->error)
  {
    $self->error($dbh->errorMessage."Error:  display", 4008, "SQL Statement = '$sql_statements{1006}'");
    return undef;
  }
  my @temp_row = $sth->fetchrow_array;
  $report_attributes{'report_class'} = $temp_row[1];
  $report_attributes{'report_type'} = $temp_row[2];
  $report_attributes{'column_variable'} = $temp_row[3];
  $report_attributes{'column_variable_data'} = $temp_row[4];
  # split the column_variable to add the -1 sub-account to the column_variable_data if sub_accounts exists
  my @temp_column_variable = split /,/, $report_attributes{'column_variable'};
  for (my $variable_count=0; $variable_count < scalar @temp_column_variable; $variable_count++)
  {
    if ($temp_column_variable[$variable_count] eq "sub_accounts")
    {
      # split the column_variable_data and insert "-1" at the beginning of the sub_accounts string
      my @temp_column_variable_data = split /\|/, $report_attributes{'column_variable_data'};
      $temp_column_variable_data[$variable_count] = "-1," . $temp_column_variable_data[$variable_count];
      # reconstruct the column_variable_data string
      $report_attributes{'column_variable_data'} = join "|", @temp_column_variable_data;
    }
  }
  $report_attributes{'date_range'} = $temp_row[5];
  $report_attributes{'display_dates'} = $temp_row[6];
  $report_attributes{'max_columns'} = $temp_row[7];
  $report_attributes{'min_columns'} = $temp_row[8];
  $report_attributes{'text_font'} = $temp_row[9];
  $report_attributes{'text_size'} = $temp_row[10];
  $report_attributes{'data_font'} = $temp_row[11];
  $report_attributes{'data_size'} = $temp_row[12];
  $report_attributes{'total_column'} = $temp_row[13];
  $report_attributes{'underline_length'} = ($temp_row[14] == -1 ? $self->{default_underline_length} : $temp_row[14]);

  # attributes that need default settings to start out with
  $report_attributes{reportWidth} = ($report_attributes{'report_type'} eq "r" ? "55" : "100");  # default to 55%

  # attributes not in report_type_tb but that are in memorized_report_tb and need to have defaults if this is not a memorized report
  $report_attributes{'account_format'} = "2";
  $report_attributes{'data_precision'} = "1";
  $report_attributes{'negative_format'} = "-1";
  $report_attributes{'dates_entered'} = ();
  $report_attributes{'label'} = ();
  $report_attributes{'column_data'} = ();
  $report_attributes{'sql_statement'} = "";
  $report_attributes{'headers'} = "";
#   for (my $column_count=0; $column_count < scalar @column_field_names; $column_count++)
#   {
#     $report_attributes{'headers'} .= $column_field_names[$column_count];
#     if ($column_count < scalar @column_field_names - 1)
#     {
#       $report_attributes{'headers'} .= "|";
#     }
#   }
  $report_attributes{'sorted_by'} = "Ref #";
  $report_attributes{'sorted_order'} = "a";
  $report_attributes{'subtotal'} = "false";
  $report_attributes{'subtotal_type'} = "d";
  $report_attributes{'border'} = "0";

  $report_attributes{windowName} = $windowName;

  $report_attributes{displayZeroRows} = 0;  # default to not showing zero rows.

  return $self->customize_report_display( %report_attributes );
}

sub c_run_report
{
  my $self = shift;
  my %report_attributes;
  my $dbh = $self->{dbHandle};
  my $doc = $self->prepDoc;
  my $args_ref;
  my %input = %{$self->{input}};
  my %report_attributes;
  if (exists $input{'file_name'} && length $input{'file_name'} > 0)
  {
    #rerunning report?
    # retrieve the already customized settings from the file of an already running report to regather the data using the
    # customization settings already selected
    $args_ref = retrieve($input{'file_name'});
    %report_attributes = %{@{$args_ref}[0]};
  }
  else
  {
    #  get default settings???
    %report_attributes = $self->get_report_attributes();
    if ($self->error)
    {
      $doc->print($self->errorMessage);
      foreach my $attr (("display_dates", "date_range", "total_column"))
      {
        $report_attributes{$attr} = $true_false{$report_attributes{$attr}};
      }
      $doc = $self->customize_report_display( %report_attributes);
      return $doc;
    }
  }
  # gather the data
  $doc = $self->gather_regular_report_data( %report_attributes );
  return $doc;
}

#****************************** Helper functions *****************************

# $self->gather_regular_report_data - does the work of gathering the data for a regular report and storing the data to disk in a
#          usable data structure
sub gather_regular_report_data
{
  my $self = shift;
  my %args = (
    'report_class' => "custom",
    'report_type' => "r",
    'column_variable' => "",
    'column_variable_data' => "",
    'display_dates' => "true",
    'date_range' => "true",
    'max_columns' => 36,
    'min_columns' => 1,
    'text_size' => "-1",
    'text_font' => "helvetica",
    'data_size' => "-1",
    'data_font' => "helvetica",
    'total_column' => "false",
    'underline_length' => $self->{default_underline_length},
    displayZeroRows => 0,
    @_);
  my $dbh = $self->{dbHandle};
  my @arguments = ( "id", "report_name", "report_class", "date_range", "column_variable", "column_variable_data", "column_data", "column_type", "dates_entered" );
  my $startTime = time();
  my $doc = $self->prepDoc;

  $args{'date_range'} = $true_false{$args{'date_range'}};
  foreach my $name (@arguments)
  {
    if (!exists $args{$name})
    {
      $doc->doRequiredParameterError('$self->gather_regular_report_data', $name);
      $self->error("<div>Error: $name</div>");
    }
  }
  if ($self->error) { $doc->print($self->errorMessage); return $doc; }

  # do the work of actually gathering the rows from report_data_tb, gathering each row's data, doing all calculations, etc. and then generate the JavaScript data and code to manipulate the data.
  my @report_data;

  my @dates_array = ();
  for (my $count=0; $count < scalar @{$args{dates_entered}}; $count++)
  {
    my @date = split /,/, $args{'dates_entered'}[$count], 2;
    my @temp_dates = ();
    if ($args{'date_range'})
    {
      @temp_dates = ($date[0], $date[1]);
    }
    else
    {
      @temp_dates = ($date[0]);
    }
    $dates_array[scalar @dates_array] = [ @temp_dates ];
    #$doc->print("  date_header_array[$count] = \"" . join("<br>- ", @temp_dates) . "\";\n");
  }
  $doc->print("\n");

  my @row_array = $self->GetReportRows( id => $args{'id'} );
  if ($self->error) { $doc->print($self->errorMessage); return $doc; }
  my $max_display_level = -1;
  for (my $count=0; $count < scalar @row_array; $count++)
  {
    my $temp_display_level = -1;
    my $index = $row_array[$count]{'index'};
    my $cell_text = $row_array[$count]{'cell_text'};
    my $cell_formula = $row_array[$count]{'cell_formula'};
    my $cell_type = $row_array[$count]{'cell_type'};
    my $item_name = $row_array[$count]{'item_name'};
    my $bold = $row_array[$count]{'bold'};
    my $sub_levels = $row_array[$count]{'sub_levels'};
    my $display_level = $row_array[$count]{'display_level'};
    my $initial_state = $row_array[$count]{'initial_state'};
    my $show = $row_array[$count]{'show'};
    my $drillable = $row_array[$count]{'drillable'};
    my $negative_format = $row_array[$count]{'negative_format'};
    my $activity = $row_array[$count]{'activity'};
    my $query_type = $row_array[$count]{'query_type'};
    my $comparison_result_type = $row_array[$count]{'result_type'};
    my $show_row = $row_array[$count]{'show'};
    my $underline = $row_array[$count]{'underline'};

    my %temp_row = (index => $index, cell_text => $cell_text, cell_formula => $cell_formula, cell_type => $cell_type,
          item_name => $item_name, bold => $bold, sub_levels => $sub_levels,
          display_level => $display_level, initial_state => $initial_state, drillable => $drillable,
          negative_format => $negative_format, activity => $activity,
          query_type => $query_type, result_type => $comparison_result_type);

    my %temp_hash = {};
    $temp_hash{'report_class'} = $args{'report_class'};
    $temp_hash{'cell_type'} = $cell_type;
    $temp_hash{'cell_text'} = $cell_text;
    $temp_hash{'item_name'} = $item_name;
    $temp_hash{'bold'} = $bold;
    $temp_hash{'display_level'} = $display_level;
    $temp_hash{'initial_state'} = $initial_state;
    $temp_hash{'show'} = $show;
    if ($show eq "1" || $show eq "")
    {
      $temp_hash{'visible'} = "true";
    }
    else
    {
      $temp_hash{'visible'} = "false";
    }
    $temp_hash{'date_range'} = $args{'date_range'};
    $temp_hash{'sub_levels'} = $sub_levels;
    $temp_hash{'negative_format'} = $negative_format;
    $temp_hash{'account_format'} = $args{'account_format'};
    $temp_hash{'text_font'} = $args{'text_font'};
    $temp_hash{'text_size'} = $args{'text_size'};
    $temp_hash{'data_font'} = $args{'data_font'};
    $temp_hash{'data_size'} = $args{'data_size'};
    $temp_hash{'drillable'} = $drillable;
    $temp_hash{'underline'} = $underline;
    $temp_hash{'result_type'} = $comparison_result_type;
    $temp_hash{'dates'} = \@dates_array;

    if ($cell_type eq "text")
    {
      $temp_hash{'sub_levels'} = 0;

      $temp_display_level = $temp_hash{'display_level'};
      #change out script tags here
      foreach my $word qw(head body script table tr td thead tbody th)
      {
        $temp_hash{cell_text} =~ s/<\s*$word\s*>/&lt;$word&gt;/;
        $temp_hash{cell_text} =~ s/<\s*\/\s*$word\s*>/&lt;\/$word&gt;/;
      }
    }
    elsif ($cell_type eq "line")
    {
      $temp_display_level = $temp_hash{'display_level'};
    }
    elsif ($cell_type eq "total")
    {
      $temp_hash{'sub_levels'} = 0;

      $temp_display_level = $temp_hash{'display_level'};
      $self->get_total_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row, report_data => \@report_data);
    }
    elsif ($cell_type eq "compare")
    {
      $temp_display_level = $temp_hash{'display_level'};
      $self->get_comparison_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row, report_data => \@report_data);
    }
    elsif ($cell_type eq "account_type" || $cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance")
    {
      if ($cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance")
      {
        $temp_row{'activity'} = 0;
      }

      $temp_display_level = $self->get_account_type_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "account" || $cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance")
    {
      if ($cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance")
      {
        $temp_row{'activity'} = 0;
      }

      $temp_display_level = $self->get_account_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "sub_account" || $cell_type eq "begin_sub_account_balance" || $cell_type eq "end_sub_account_balance")
    {
      if ($cell_type eq "begin_sub_account_balance" || $cell_type eq "end_sub_account_balance")
      {
        $temp_row{'activity'} = 0;
      }

      $temp_display_level = $self->get_sub_account_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "accounts_of")
    {
      $temp_row{'sub_levels'} += 1;

      $temp_display_level = $self->get_account_type_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "sub_accounts_of_type")
    {
      $temp_display_level = $self->get_sub_account_type_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "sub_accounts_of_account")
    {
      $temp_row{'sub_levels'} = 1;
      $temp_display_level = $self->get_account_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }

    $report_data[$count] = { %temp_hash };

    if ($temp_display_level > $max_display_level)
    {
      $max_display_level = $temp_display_level;
    }
  }

  $args{'max_display_level'} = $max_display_level;

  my $endTime = time();

  if ($self->error)
  {
    $doc = $self->prepDoc;
    $doc->print($self->error);
    return $doc;
  }
  #### **************************** ####
  my $file_name = tmpnam(); # get the file name of the temp file to work with.
  # store the data object.
  my @data = ();
  $data[0] = \%args;
  $data[1] = \@report_data;
  $data[2] = ($endTime - $startTime);  # number of seconds it took to gather the data.

  $Storable::forgive_me = 1;  # try to get around the CODE error.
  store(\@data, $file_name);

  # create the entry in the temp_file_tb
  $self->log_temp_file(file_name => $file_name );

  # create the frameset and call the next step of the process.
  my $s = $doc->formEncodeString('Reports::Display');
  $doc = HTMLObject::FrameSet->new();
  my $inner_content;
  $inner_content = $doc->createNoframe("You must have a frames capable browser!");
  $inner_content .= $doc->createFrame(src => "$self->{baseURL}?app=$self->{app}&state=$s&ledgerCode=$self->{input}->{ledgerCode}&command=run_report_top&id=$args{'id'}&file_name=$file_name&report_type=r", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => "_t", bordercolor => 'black');
  $inner_content .= $doc->createFrame(src => "$self->{baseURL}?app=$self->{app}&state=$s&ledgerCode=$self->{input}->{ledgerCode}&command=run_report_bottom&id=$args{'id'}&file_name=$file_name&report_type=r&report_command=display", frameborder => 'yes', scrolling => 'auto', name => "_b", marginwidth => '1', bordercolor => 'white');
  $doc->outerFrameset(rows => '40, *', content => $inner_content, border => '5', frameborder => 'yes');

  $doc->setTitle("Report - $args{'report_name'}");

  return $doc;
}

sub customize_report_display
{
  my $self = shift;
  my %args = ( error_string => '', @_, );
  my $error_string = $args{'error_string'};
  my $dbh = $self->{dbHandle};
  my %input = %{$self->{input}};
  my $langObj = $self->{langObj};
  my @known_dates;
  my $currentWorkingYear;

  use Portal::Accounting::Objects::Ledger;
  my $l = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  # check to make sure we were given the ledger_db name
  my $ledger = $input{'ledgerCode'};
  if (length $ledger == 0)
  {
    $self->error("Error:  customize_report.  $errors{4022}, The Ledger Code is required at line " . __LINE__);
    return undef;
  }
  $l->set('code', $ledger);
  $l->select;
  if ($l->error) { $self->error($l->errorMessage); return undef; }
  my $doc = $self->prepDoc;
  $doc->print(css => [
    ".padme { margin: .7em; }",
    ".columnLabel { width: 10em; }",
  ]);
  $doc->setFocus('body');

  my $date = POSIX::strftime('%Y-%m-%d', localtime);

  my $ledger_name = $l->get('name');
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $ledger_color = $l->get('color');
  my $reportWidth = $args{reportWidth};

  # retrieve values from the known_years_tb table
  @known_dates = $l->selectKnownYearsArrays();
  $currentWorkingYear = $l->workingYear();

  # create the header for the page.
  $doc->print(<<"END_OF_HEADER");
  <table border=0 cellpadding=0 cellspacing=0 width=100%>
    <tr>
      <td align=left width=75%>
END_OF_HEADER

  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>\n");
  }

  $doc->print(<<"END_OF_HEADER");
      </td>
    </tr>
  </table>
END_OF_HEADER

  # turn off the link decorations
  $doc->setLinkDecorations();
  $doc->print(css => [".padme { margin: .7em; }"]);

  # set the overall font size to be smaller
  if ($args{'report_type'} eq "r")
  { # regular view report
    my $date_range = ($args{'date_range'}) ? "true" : "false";
    my $display_dates = ($args{'display_dates'}) ? "true" : "false";
    my $s = '';
    my $colspan = "2";
    if ($args{'date_range'})
    {
      $s = 's';
      $colspan = "3";
    }
    my $dateSelect;
    my @today = Today();
    my @yesterday = Add_Delta_Days(@today, -1);
    my @begin = split("-", $l->start(fiscalYear => $yesterday[0], month => 1) );  #!!!this should grab the beginning of the same fiscal year from a ledger.
    $dateSelect .= "                <option value=\"$begin[0]-$begin[1]-$begin[2],$yesterday[0]-$yesterday[1]-$yesterday[2]\">YESTERDAY</option>\n";
    $dateSelect .= "                <option value=\"$begin[0]-$begin[1]-$begin[2],$today[0]-$today[1]-$today[2]\">TODAY</option>\n";
    for (my $count=scalar(@known_dates)-1; $count >= 0; $count--)
    {
      if ($args{'date_range'})
      {
        $dateSelect .= "                <option value=\"$known_dates[$count][1],$known_dates[$count][2]\">$known_dates[$count][0]</option>\n";
        if ($known_dates[$count][0] >= $currentWorkingYear - 1 && $known_dates[$count][0] <= $currentWorkingYear)
        {
          $dateSelect .= "                <option value=\"$known_dates[$count][3],$known_dates[$count][4]\">&nbsp;&nbsp;&nbsp;&nbsp;Q1</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][5],$known_dates[$count][6]\">&nbsp;&nbsp;&nbsp;&nbsp;Q2</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][7],$known_dates[$count][8]\">&nbsp;&nbsp;&nbsp;&nbsp;Q3</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][9],$known_dates[$count][10]\">&nbsp;&nbsp;&nbsp;&nbsp;Q4</option>\n";
        }
      }
      else
      {
        if ($known_dates[$count][0] >= $currentWorkingYear - 1 && $known_dates[$count][0] <= $currentWorkingYear)
        {
          $dateSelect .= "                <option value=\"$known_dates[$count][1]\">$known_dates[$count][0] Start</option>\n";
        }
        $dateSelect .= "                <option value=\"$known_dates[$count][2]\">$known_dates[$count][0] End</option>\n";
        if ($known_dates[$count][0] >= $currentWorkingYear - 1 && $known_dates[$count][0] <= $currentWorkingYear)
        {
          $dateSelect .= "                <option value=\"$known_dates[$count][3]\">&nbsp;&nbsp;&nbsp;&nbsp;Q1 Start</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][4]\">&nbsp;&nbsp;&nbsp;&nbsp;Q1 End</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][5]\">&nbsp;&nbsp;&nbsp;&nbsp;Q2 Start</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][6]\">&nbsp;&nbsp;&nbsp;&nbsp;Q2 End</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][7]\">&nbsp;&nbsp;&nbsp;&nbsp;Q3 Start</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][8]\">&nbsp;&nbsp;&nbsp;&nbsp;Q3 End</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][9]\">&nbsp;&nbsp;&nbsp;&nbsp;Q4 Start</option>\n";
          $dateSelect .= "                <option value=\"$known_dates[$count][10]\">&nbsp;&nbsp;&nbsp;&nbsp;Q4 End</option>\n";
        }
      }
    }
    my $dateSelectDiv = <<"END_OF_BODY";
          <div class="secondary" style="float: right;">
            Dates:<br>
            <select name="date_select" size="10" onchange="date_select_change();">
              $dateSelect
            </select>
          </div>
END_OF_BODY
    my $columnVariable = '<input type="hidden" name="column_type" value="nothing">';
    if ($args{'column_variable'} ne "")
    {
      $columnVariable = <<"END_OF_BODY";
  <div style="float: left; text-align: right; margin: 1em; ">
    <table>
      <tr>
        <th colspan="2">COLUMN VARIABLE</th>
      </tr>
      <tr>
        <th align="center" valign="top">Column Type</th>
        <th align="center" valign="top">Column Data</th>
      </tr>
      <tr>
        <td align="center" valign="top">
          <select name="column_type" onChange="column_type_change();">
END_OF_BODY
      $columnVariable .= "                <option value=\"nothing\">Nothing\n";
      my @temp_array = split /\,/, $args{'column_variable'};
      for (my $count=0; $count < scalar @temp_array; $count++)
      {
        $columnVariable .= "                <option value=\"$temp_array[$count]\"";
        if ($args{'column_type'} eq $temp_array[$count])
        {
          $columnVariable .= " selected";
        }
        $columnVariable .= ">$column_headers{$temp_array[$count]}\n";
      }
      $columnVariable .= <<"END_OF_BODY";
          </select><br><br>
          <input type=button value="All" onclick="select_all_column_data();"><br>
          <input type=button value="None" onclick="unselect_column_data();">
        </td>
        <td align="center" valign="top">
          <select name="column_data" size="5" multiple width="250">
          </select>
          <input type="hidden" name="column_data_names" value="">
        </td>
      </tr>
    </table>
  </div>
END_OF_BODY
    }

    my $balanceNotes = '';
    my $total_column = '';
    if ($args{report_class} ne 'balance')
    {
    $total_column = <<"T";
    <div class="padme">
      Total Column
      <select name="total_column">
T
    $total_column .= "            <option value=\"true\"" . ($args{total_column} ? " selected" : "") . ">Yes</option>\n";
    $total_column .= "            <option value=\"false\"" . (!$args{total_column} ? " selected" : "") . ">No</option>\n";
    $total_column .= <<"T";
      </select>
    </div>
T
    }
    else
    {
      $balanceNotes = '  <div class="help tertiary">Balance reports use date ranges to gather some detail during the period of the report.  The last date will normally be the date of the balances.</div>';
    }

    #start output of main form
    $doc->print(<<"END_OF_BODY");
<div style="text-align: center" class="primary">
  <form method="post" name="customize_form">
$self->{postInfo}
    <input type=hidden name="command" value="run_report">
    <input type=hidden name="date_range" value="$date_range">
    <input type=hidden name="display_dates" value="$display_dates">
    <input type=hidden name="id" value="$args{'id'}">
    <input type=hidden name="report_class" value="$args{'report_class'}">
    <input type=hidden name="column_variable" value="$args{'column_variable'}">
    <input type=hidden name="column_variable_data" value="$args{'column_variable_data'}">
    <input type=hidden name="report_type" value="$args{'report_type'}">
    <input type=hidden name="text_font" value="$args{'text_font'}">
    <input type=hidden name="data_font" value="$args{'data_font'}">
    <input type=hidden name="text_size" value="$args{'text_size'}">
    <input type=hidden name="data_size" value="$args{'data_size'}">
    <input type=hidden name="underline_length" value="$args{'underline_length'}">
    <input type=hidden name="account_format" value="$args{'account_format'}">
    <input type=hidden name="negative_format" value="$args{'negative_format'}">
    <input type=hidden name="border" value="$args{'border'}">
    <input type="hidden" name="windowName" value="$args{windowName}">
    <input type=hidden name="min_columns" value="$args{'min_columns'}">
    <input type=hidden name="max_columns" value="$args{'max_columns'}">
    <div>
      Report Name:
      <input type=text name="report_name" value="$args{'report_name'}" size="30">
    </div>
    <div style="float: left;">
      <table>
        <tr>
          <td align="center" colspan="$colspan">
            NOTE: A minimum of <b>$args{'min_columns'}</b> row(s) must be filled in.<br>
          </td>
        </tr>
        <tr>
          <th>
            Column
          </th>
          <th>
            Date$s
          </th>
          <th>
            Label
          </th>
        </tr>
END_OF_BODY
    for (my $count=0; $count < $args{'max_columns'}; $count++)
    {
      $doc->print("            <tr>\n");
      $doc->print('              <td nowrap="1">' . ($count + 1) .":</td>\n");
      (my $temp_start_date, my $temp_end_date) = split(/,/, $args{'dates_entered'}[$count]);
      $doc->print('              <td align="left" nowrap="1">');
      my $currYear = POSIX::strftime("%Y", localtime);
      my $widget = HTMLObject::Widgets->new();

      my $onChange = "setLabel(this.form.column$count"."_label, this.value);";
      if ($args{'date_range'})
      {
        $onChange = '';
      }
      my %datePicker1 = $widget->generatePickerCode(
        type => "date",
        doc => $doc,
        form => "customize_form",
        itemName => "column" . $count . "_start_date",
        itemValue => $temp_start_date,
        year => $currYear,
        phrase => $langObj->map("FromColon"),
        width => 400,
        height => 350,
        baseUrl => "",
        onChange => $onChange,
        class => 'date',  ####!!!! these two lines DONT WORK
        size => 8,
      );
      if ($widget->error)
      {
        $self->error($widget->errorMessage);
        return undef;
      }
      my %datePicker2 = $widget->generatePickerCode(
        type => "date",
        doc => $doc,
        form => "customize_form",
        itemName => "column" . $count . "_end_date",
        itemValue => $temp_end_date,
        year => $currYear,
        phrase => $langObj->map("ToColon"),
        width => 400,
        height => 350,
        baseUrl => "",
        onChange => "setLabel(this.form.column$count"."_label, this.value);",
        class => 'date',
        size => 8,
      );
      if ($widget->error)
      {
        $self->error($widget->errorMessage);
        return undef;
      }

      $doc->print(%datePicker1);
      if ($args{'date_range'})
      {
        $doc->print(%datePicker2);
      }
      $doc->print("            </td>\n");
      $doc->print('            <td align="left" nowrap="1">'."\n");
      $doc->print('              <input type="text" class="columnLabel" name="column'.$count.'_label" />');
      $doc->print("            </td>\n");
      $doc->print('            <td align="left" nowrap="1">'."\n");
      $doc->print('              <input type="button" value="X" onclick="document.customize_form.column'.$count."_label.value = ''; document.customize_form.column$count" . "_start_date.value = '';");
      if ($args{'date_range'})
      {
        $doc->print(" document.customize_form.column" . $count . "_end_date.value = '';");
      }
      $doc->print(" tweakReportWidth(); document.customize_form.column" . $count . "_start_date.focus();\" />\n");
      $doc->print("              </td>\n");
      $doc->print("            </tr>\n");
    }
    $doc->print(<<"END_OF_BODY");
      </table>
    </div>

  $dateSelectDiv

    <div style="clear: both;"></div>
  $columnVariable

    <div class="padme">
      Precision of Data
      <select name="data_precision">
END_OF_BODY
    for (my $count=0; $count < scalar @precision_formats; $count++)
    {
      $doc->print("          <option value=\"$precision_formats[$count]\"");
      if ($args{'data_precision'} eq $precision_formats[$count])
      {
        $doc->print(" selected");
      }
      $doc->print(">$precision_formats{$precision_formats[$count]}</option>\n");
    }
    $doc->print(<<"END_OF_BODY");
      </select>
    </div>
$total_column
    <div class="padme">
      Printed Report Width %
      <input type="text" name="reportWidth" value="$reportWidth" size="3" maxlength="3">
    </div>
    <input type="submit" name="custom_submit" value="Run Report" onclick="return validate_form();">&nbsp;<input type=button value="Reset" onclick="document.customize_form.reset(); reset_form();">
    <div style="clear: both;"></div>
  </form>
</div>
  $balanceNotes
END_OF_BODY

    my $onload_code;
    $onload_code = "    // onload code\n";
    if ($args{'column_variable'} ne "")
    {
      $onload_code = <<"END_OF_ONLOAD_CODE";
    var f = document.customize_form;

    column_type_change();

    for (var count=0; count < f.column_data.options.length; count++)
    {
      if (typeof column_defaults[f.column_data.options[count].value] == "undefined" && column_defaults[f.column_data.options[count].value] != 1)
      {
        f.column_data.options[count].selected = false;
      }
      else
      {
        f.column_data.options[count].selected = true;
      }
    }
END_OF_ONLOAD_CODE
    }

    $doc->setFocus("javascript");
    my @reset_split_array = split(/,/, $args{'column_data'});
    for (my $count=0; $count < scalar @reset_split_array; $count++)
    {
      $doc->print("  column_defaults[\"$reset_split_array[$count]\"] = 1;\n");
    }
    $doc->print(<<"END_OF_SCRIPT");

END_OF_SCRIPT
    my @temp_array = split /,/, $args{'column_variable'};
    my @temp_data_array = split /\|/, $args{'column_variable_data'};
    $doc->print("  column_data[\"nothing\"] = new Array();\n");
    for (my $count=0; $count < scalar @temp_array; $count++)
    {
      my @temp_data_split_array = split /,/, $temp_data_array[$count];
      my @temp_data_name_array;
      # retrieve values from the database based on the type of data for the current column_type
      my $sth;
      if ($temp_array[$count] eq "sub_entities")
      {
        my $sth = $dbh->read(sql =>  $sql_statements{$column_data_sql{$temp_array[$count]}}, plug => [$ledgerCode] );
        if ($dbh->error)
        {
          $self->error($dbh->errorMessage);
          return undef;
        }
        my $count2 = 0;
        # add the default of "" (nothing) to the sub-entities arrays
        $temp_data_split_array[$count2] = "<non_sub_entity>";
        $temp_data_name_array[$count2] = "<None>";
        $count2++;
        while (my @temp_row = $sth->fetchrow_array)
        {
          $temp_data_split_array[$count2] = $temp_row[1];
          $temp_data_name_array[$count2] = $temp_row[0];
          $count2++;
        }

        # add a comma-separated list of sub-entity codes to the appropriate array element and rebuild the column_variable_data
        # environment variable
        my $temp_data = "";
        for (my $count2=0; $count2 < scalar @temp_data_split_array; $count2++)
        {
          $temp_data .= $temp_data_split_array[$count2];
          if ($count2 < scalar @temp_data_split_array - 1)
          {
            $temp_data .= ",";
          }
        }
        $temp_data_array[$count] = $temp_data;
        my $temp_column_variable_data = "";
        for (my $count2=0; $count2 < scalar @temp_data_array; $count2++)
        {
          $temp_column_variable_data .= $temp_data_array[$count2];
          if ($count2 < scalar @temp_data_array - 1)
          {
            $temp_column_variable_data .= "|";
          }
        }
        $args{'column_variable_data'} = $temp_column_variable_data;
        $onload_code .= "\n    f.column_variable_data.value = \"$temp_column_variable_data\";\n";
      }
      else
      {
        for (my $count2=0; $count2 < scalar @temp_data_split_array; $count2++)
        {
          my $sth = $dbh->read(sql =>  $sql_statements{$column_data_sql{$temp_array[$count]}}, plug => [$temp_data_split_array[$count2]] );
          if ($dbh->error)
          {
            $self->error($dbh->errorMessage);
            return undef;
          }
          my @temp_row = $sth->fetchrow_array;
          $temp_data_name_array[$count2] = $temp_row[0];
        }
      }
      $doc->print("  column_data[\"$temp_array[$count]\"] = new Array();\n");
      for (my $count2=0; $count2 < scalar @temp_data_split_array; $count2++)
      {
        $doc->print("  column_data[\"$temp_array[$count]\"][$count2] = new create_column_object(\"$temp_data_split_array[$count2]\", \"$temp_data_name_array[$count2] - ");
        if ($temp_data_split_array[$count2] eq "-1" || $temp_data_split_array[$count2] eq "<non_sub_entity>")
        {
          $doc->print("(");
        }
        if ($temp_data_split_array[$count2] ne "<non_sub_entity>")
        {
          $doc->print("$temp_data_split_array[$count2]");
        }
        if ($temp_data_split_array[$count2] eq "-1" || $temp_data_split_array[$count2] eq "<non_sub_entity>")
        {
          $doc->print(")");
        }
        $doc->print("\");\n");
      }

      # clear out the name array for the next time through
      @temp_data_name_array = ();
    }
    # set the onload code for JavaScript
    $doc->setOnload( code => $onload_code );
  }

  return $doc;
}

sub get_account_type_data
{
  my $self = shift;
  my $dbh = $self->{dbHandle};
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledgerCode = $self->{input}->{'ledgerCode'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @local_accounts = ();
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $data_precision = $args{'data_precision'};
  my $max_display_level = -1;
  my @sub_entities = ();
  my @sub_accounts = ();

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    my $numEntries = scalar @column_data;
    for (my $count=0; $count < $numEntries; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the account type name
  my @temp_item_name = ();
  $temp_item_name[0] = $temp_row{'item_name'};
  my @temp_cell_text = $self->GetAccountTypeNames(account_types => \@temp_item_name);
  $temp_hash->{'cell_text'} = $temp_cell_text[0];

  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  my $numEntries = scalar @{$temp_hash->{dates}};
  for (my $count=0; $count < $numEntries; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'} && $temp_row{'cell_type'} ne "begin_account_type_balance" && $temp_row{'cell_type'} ne "end_account_type_balance")
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      if ($temp_row{'cell_type'} eq "begin_account_type_balance")
      {
        (my $temp_date = $temp_hash->{'dates'}[$count][0]) =~ s/$minusRE/\//g;
        my @temp_date = split("/", $temp_date);
        # make date_start and date_end one less day
        @temp_date = Add_Delta_Days(@temp_date, -1);
        $date_start = "$temp_date[0]-$temp_date[1]-$temp_date[2]";
        $date_end = $date_start;
      }
      elsif ($temp_row{'cell_type'} eq "end_account_type_balance")
      {
        $date_end = $temp_hash->{'dates'}[$count][1];
        $date_start = $date_end;
      }
      else
      {
        $date_start = $temp_hash->{'dates'}[$count][0];
        $date_end = $date_start;
      }
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    if ($temp_row{'sub_levels'} == 0)
    {
      my $sth;
      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sqlClause;
      my $error_string = 0;
      my $numEntries = scalar @column_data;
      for (my $column_count=0; $column_count < $numEntries; $column_count++)
      {
        if ($column_type eq "nothing")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1073};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1113};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_end, $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1114};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_end, "true", $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1114};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_end, "false", $temp_hash->{'report_class'});
            }
          }

          $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
        }
        elsif ($column_type eq "sub_entities")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1074};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1113};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_end, $temp_hash->{'report_class'});
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1075};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_end, "true", $temp_hash->{'report_class'});
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1075};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_end, "false", $temp_hash->{'report_class'});
              $error_string = 1;
            }
          }

          $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";
        }
        elsif ($column_type eq "sub_accounts")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1120};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1122};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_end, $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1121};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1123};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_end, "true", $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1121};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1123};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_end, "false", $temp_hash->{'report_class'});
            }
          }

          if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
          { # cash/bank account type or sales revenue account type
            $sqlClause = "invalid";
            $error_string = 1;
          }
          else
          {
            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
            my $tax = '';
            if ($temp_row{'query_type'} == 1)
            {
              $tax = 'true';
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              $tax = 'false';
            }
            if ($tax ne '')
            {
              $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
            }
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_num${delim}=${delim}$column_data[$column_count];";
          }
        }
        elsif ($column_type eq "sub_account_types")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1115};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1118};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_end, $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1116};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1119};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_end, "true", $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1116};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1119};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledgerCode, $date_end, "false", $temp_hash->{'report_class'});
            }
          }

          if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
          { # cash/bank account type or sales revenue account type
            $sqlClause = "invalid";
            #$error_string;
          }
          else
          {
            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
            my $tax = '';
            if ($temp_row{'query_type'} == 1)
            {
              $tax = 'true';
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              $tax = 'false';
            }
            if ($tax ne '')
            {
              $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
            }
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_type${delim}=${delim}$column_data[$column_count];";
          }
        }

        if ($error_string == 1 && ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10))
        { # cash/bank account type or sales revenue account type
          $temp_hash->{'values'}[$count][$column_count] = 0;
        }
        else
        {
          $sth = $dbh->read(sql => $tmp_sql_statement, plug => \@sql_arguments);
          if ($dbh->error)
          {
            $self->error($dbh->errorMessage);
            return undef;
          }
          my $row_array = $sth->fetchrow_arrayref();
          my $value = $row_array->[0];
          $temp_hash->{'values'}[$count][$column_count] = $value;
        }

        if ($error_string == 1)
        {
          $temp_hash->{'error'} = $error_string;
        }
        $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        #perhaps wrap if around total statements to check row type!!!
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my @temp_account_total;
      my $sth;
      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sqlClause;
      my $level_sql_clause;
      my $error_string = 0;
      # calculate the value (activity/balance) for the account type for the current date range using each account given the column qualifiers
      if ($temp_row{'sub_levels'} == 1 || ($temp_row{'sub_levels'} == 2 && ($column_type eq "sub_accounts" || $column_type eq "sub_account_types")))
      {
        my $numEntries = scalar @column_data;
        for (my $column_count = 0; $column_count < $numEntries; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1076};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1078};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1078};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "false", $date_end);
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}<ACCOUNT>;";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1077};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1079};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1079};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "false", $date_end);
                $error_string = 1;
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1080};
                @sql_arguments = ($column_data[$column_count], $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1081};
                @sql_arguments = ($column_data[$column_count], $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1081};
                @sql_arguments = ($column_data[$column_count], $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "false", $date_end);
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_num${delim}=${delim}$column_data[$column_count];";
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1082};
                @sql_arguments = ($column_data[$column_count], $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1083};
                @sql_arguments = ($column_data[$column_count], $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1083};
                @sql_arguments = ($column_data[$column_count], $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "true", $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, "false", $date_end);
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_type${delim}=${delim}$column_data[$column_count];";
          }
          $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => \@sql_arguments);
          if ($dbh->error)
          {
            $self->error($dbh->errorMessage);
            return undef;
          }
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $account_num = $row_array->[0];
            my $account_name = $row_array->[1];
            my $value = $row_array->[2];

            if ($account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$account_num})
              {
                $temp_hash->{'sub_rows'}{$account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$account_num};
              $subRow->{'cell_text'} = $account_name;
              $subRow->{'item_name'} = $account_num;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              my $numEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $numEntries; $date_count++)
              {
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  my $subNumEntries = scalar @column_data;
                  for (my $fill_count=0; $fill_count < $subNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  my $subNumEntries = scalar @column_data;
                  for (my $fill_count=0; $fill_count < $subNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow->{'error'} = $error_string;
              my $tmp_sql_clause = $sqlClause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] = $value;
              $subRow->{'total'} += $temp_hash->{'sub_rows'}{$account_num}{'values'}[$count][$column_count];
              $temp_account_total[$column_count] += $subRow->{'values'}[$count][$column_count];
            }
          }
        }
      }
      elsif ($temp_row{'sub_levels'} == 2)
      {
        my $numEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $numEntries; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1086};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1090};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1088};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1088};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end);
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";

            $level_sql_clause = "";
            if ($temp_row{'activity'})
            {
              $level_sql_clause = "date${delim}>=${delim}$date_start;";
              $level_sql_clause .= "connector${delim}=${delim}AND;";
            }
            $level_sql_clause .= "date${delim}<=${delim}$date_end;";
            $level_sql_clause .= "connector${delim}=${delim}AND;account_num${delim}=${delim}<ACCOUNT>;";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1087};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1090};
                @sql_arguments = ($date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1089};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1089};
                @sql_arguments = ($ledgerCode, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end);
                $error_string = 1;
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";

            $level_sql_clause = "";
            if ($temp_row{'activity'})
            {
              $level_sql_clause = "date${delim}>=${delim}$date_start;";
              $level_sql_clause .= "connector${delim}=${delim}AND;";
            }
            $level_sql_clause .= "date${delim}<=${delim}$date_end;";
            $level_sql_clause .= "connector${delim}=${delim}AND;account_num${delim}=${delim}<ACCOUNT>;";
            $level_sql_clause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
          }
          $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => \@sql_arguments);
          if ($dbh->error)
          {
            $self->error($dbh->errorMessage);
            return undef;
          }
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $account_num = $row_array->[0];
            my $account_name = $row_array->[1];
            my $sub_account_num = $row_array->[2];
            my $sub_account_name = $row_array->[3];
            my $value = $row_array->[4];

            if ($account_num ne "" && $sub_account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$account_num})
              {
                $temp_hash->{'sub_rows'}{$account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$account_num};
              if (not defined $subRow->{'sub_rows'})
              {
                $subRow->{'sub_rows'} = {};
              }
              $subRow->{'cell_text'} = $account_name;
              $subRow->{'item_name'} = $account_num;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              my $subNumEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
              {
                my $subSubNumEntries = scalar @column_data;
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if (not defined $subRow->{'sub_rows'}{$sub_account_num})
              {
                $subRow->{'sub_rows'}{$sub_account_num} = {};
              }
              my $subRow2 = $subRow->{sub_rows}{$sub_account_num};
              $subRow2->{'cell_text'} = $sub_account_name;
              $subRow2->{'item_name'} = $sub_account_num;
              $subRow2->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $subRow->{'sub_rows_visible'} = "true";
                $subRow2->{'visible'} = "true";
              }
              else
              {
                $subRow->{'sub_rows_visible'} = "false";
                $subRow2->{'visible'} = "false";
              }
              $subRow2->{'display_level'} = $temp_row{'display_level'} + 2;
              if ($temp_row{'display_level'} + 2 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 2;
              }
              $subRow2->{'sub_levels'} = $temp_row{'sub_levels'} - 2;
              if (not defined $subRow2->{'values'})
              {
                $subRow2->{'values'} = ();
              }
              if (not defined $subRow2->{'sql_clause'})
              {
                $subRow2->{'sql_clause'} = ();
              }
              $subNumEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
              {
                my $subSubNumEntries = scalar @column_data;
                if (not defined $subRow2->{'values'}[$date_count])
                {
                  $subRow2->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow2->{'values'}[$date_count][$fill_count])
                    {
                      $subRow2->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow2->{'sql_clause'}[$date_count])
                {
                  $subRow2->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow2->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow2->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow2->{'error'} = $error_string;
              my $tmp_sql_clause = $sqlClause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
              $subRow2->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow2->{'values'}[$count][$column_count] = $value;
              $subRow2->{'total'} += $subRow2->{'values'}[$count][$column_count];

              $tmp_sql_clause = $level_sql_clause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] += $subRow2->{'values'}[$count][$column_count];
              $subRow->{'total'} += $subRow2->{'values'}[$count][$column_count];
              $temp_account_total[$column_count] += $subRow2->{'values'}[$count][$column_count];
            }
          }
        }
      }

      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = $temp_account_total[0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "date${delim}>=${delim}$date_start;";
          $sqlClause .= "connector${delim}=${delim}AND;";
        }
        $sqlClause .= "date${delim}<=${delim}$date_end;";
        $sqlClause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
        my $tax = '';
        if ($temp_row{'query_type'} == 1)
        {
          $tax = 'true';
        }
        elsif ($temp_row{'query_type'} == 2)
        {
          $tax = 'false';
        }
        if ($tax ne '')
        {
          $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
        }
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        my $numEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $numEntries; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
            my $tax = '';
            if ($temp_row{'query_type'} == 1)
            {
              $tax = 'true';
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              $tax = 'false';
            }
            if ($tax ne '')
            {
              $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
            }
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= " sub_e_code${delim}=${delim}$column_data[$column_count];";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "date${delim}>=${delim}$date_start;";
                $sqlClause .= "connector${delim}=${delim}AND;";
              }
              $sqlClause .= "date${delim}<=${delim}$date_end;";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
              my $tax = '';
              if ($temp_row{'query_type'} == 1)
              {
                $tax = 'true';
              }
              elsif ($temp_row{'query_type'} == 2)
              {
                $tax = 'false';
              }
              if ($tax ne '')
              {
                $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
              }
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "sub_account_num${delim}=${delim}$column_data[$column_count];";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "date${delim}>=${delim}$date_start;";
                $sqlClause .= "connector${delim}=${delim}AND;";
              }
              $sqlClause .= "date${delim}<=${delim}$date_end;";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
              my $tax = '';
              if ($temp_row{'query_type'} == 1)
              {
                $tax = 'true';
              }
              elsif ($temp_row{'query_type'} == 2)
              {
                $tax = 'false';
              }
              if ($tax ne '')
              {
                $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
              }
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "sub_account_type${delim}=${delim}$column_data[$column_count];";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }

  return $max_display_level;
  # modifying the passed in reference to the hash should update the hash when the function returns
}

sub get_account_data
{
  my $self = shift;
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledgerCode = $self->{ledgerCode};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @sub_accounts = ();
  my @account_type = ();
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $max_display_level = -1;
  my $data_precision = $args{'data_precision'};

  if (length $ledgerCode == 0) { $self->error("<div>LedgerCode not sent to get_account_data</div>"); return undef; }
  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    my $numEntries = scalar @column_data;
    for (my $count=0; $count < $numEntries; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the account's name
  $temp_hash->{'cell_text'} = $self->{ACCOUNTS}->{$temp_row{'item_name'}}->get('name');
  # get the account's account type and debit_increases flag
  @account_type = (
    $self->{ACCOUNTS}->{$temp_row{'item_name'}}->get('type'),
    $self->{ACCOUNTS}->{$temp_row{'item_name'}}->get('debitIncreases')
  );

  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  my $numEntries = scalar @{$temp_hash->{dates}};
  for (my $count=0; $count < $numEntries; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'} && $temp_row{'cell_type'} ne "begin_account_balance" && $temp_row{'cell_type'} ne "end_account_balance")
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      if ($temp_row{'cell_type'} eq "begin_account_balance")
      {
        (my $temp_date = $temp_hash->{'dates'}[$count][0]) =~ s/$minusRE/\//g;
        my @temp_date = split("/", $temp_hash->{'dates'}[$count][0]);
        # make date_start and date_end one less day
        @temp_date = Add_Delta_Days(@temp_date, -1);
        $date_start = "$temp_date[0]-$temp_date[1]-$temp_date[2]";
        $date_end = $date_start;
      }
      elsif ($temp_row{'cell_type'} eq "end_account_balance")
      {
        $date_end = $temp_hash->{'dates'}[$count][1];
        $date_start = $date_end;
      }
      else
      {
        $date_start = $temp_hash->{'dates'}[$count][0];
        $date_end = $date_start;
      }
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    if ($temp_row{'sub_levels'} == 0 || ($column_type eq "sub_accounts" || $column_type eq "sub_account_types"))
    {
      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        my %options = ();
        $options{'begin_date'} = $date_start if ($temp_row{'activity'});
        $options{'end_date'} = $date_end;
        $options{'balance'} = 1 if (!$temp_row{'activity'});
        $options{'taxable'} = $temp_row{'query_type'};

        my @result;
        my @accounts = ();

        $accounts[0] = ();
        $accounts[0][0] = $temp_hash->{'item_name'};
        $accounts[0][1] = $account_type[1];
        $accounts[0][2] = $temp_hash->{'cell_text'};  #can I add account code here?!!! as [0][3]

        if ($options{'balance'})
        {
          @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
        }
        else
        {
          @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
        }
        $temp_hash->{'values'}[$count][0] = $result[0][0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "date${delim}>=${delim}$date_start;";
          $sqlClause .= "connector${delim}=${delim}AND;";
        }
        $sqlClause .= "date${delim}<=${delim}$date_end;";
        $sqlClause .= "connector${delim}=${delim}AND;";
        $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        my $subNumEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $subNumEntries; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            my %options = ();
            $options{'begin_date'} = $date_start if ($temp_row{'activity'});
            $options{'end_date'} = $date_end;
            $options{'balance'} = 1 if (!$temp_row{'activity'});
            $options{'taxable'} = $temp_row{'query_type'};

            my @result;
            my @accounts = ();
            my @sub_entity = ();

            $accounts[0] = ();
            $accounts[0][0] = $temp_hash->{'item_name'};
            $accounts[0][1] = $account_type[1];
            $accounts[0][2] = $temp_hash->{'cell_text'};

            $sub_entity[0] = $column_data[$column_count];
            $options{'sub_entity'} = [ @sub_entity ];

            if ($options{'balance'})
            {
              @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
            }
            else
            {
              @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
            }
            $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my %options = ();
              $options{'begin_date'} = $date_start if ($temp_row{'activity'});
              $options{'end_date'} = $date_end;
              $options{'balance'} = 1 if (!$temp_row{'activity'});
              $options{'taxable'} = $temp_row{'query_type'};

              my @result;
              my @accounts = ();
              my @sub_account = ();

              $accounts[0] = ();
              $accounts[0][0] = $temp_hash->{'item_name'};
              $accounts[0][1] = $account_type[1];
              $accounts[0][2] = $temp_hash->{'cell_text'};

              $sub_account[0] = $column_data[$column_count];
              $options{'sub_account'} = \@sub_account;

              if ($options{'balance'})
              {
                @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
              }
              else
              {
                @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
              }
              $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "date${delim}>=${delim}$date_start;";
                $sqlClause .= "connector${delim}=${delim}AND;";
              }
              $sqlClause .= "date${delim}<=${delim}$date_end;";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "sub_account_num${delim}=${delim}$column_data[$column_count];";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my @sub_accts_for_sub_acct_type = $self->GetSubAccountsForSubAccountType(sub_account_type => $column_data[$column_count]);
              my $total = "0.00";
              my $subSubNumEntries = scalar @sub_accts_for_sub_acct_type;
              for (my $z=0; $z < $subSubNumEntries; $z++)
              {
                my %options = ();
                $options{'begin_date'} = $date_start if ($temp_row{'activity'});
                $options{'end_date'} = $date_end;
                $options{'balance'} = 1 if (!$temp_row{'activity'});
                $options{'taxable'} = $temp_row{'query_type'};
                $options{'sub_account'} = \@{$sub_accts_for_sub_acct_type[$z]};

                my @result;
                my @accounts = ();

                $accounts[0] = ();
                $accounts[0][0] = $temp_hash->{'item_name'};
                $accounts[0][1] = $account_type[1];
                $accounts[0][2] = $temp_hash->{'cell_text'};

                if ($options{'balance'})
                {
                  @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
                }
                else
                {
                  @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
                }
                $total += $result[0][0];
              }
              $temp_hash->{'values'}[$count][$column_count] = $total;
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "date${delim}>=${delim}$date_start;";
                $sqlClause .= "connector${delim}=${delim}AND;";
              }
              $sqlClause .= "date${delim}<=${delim}$date_end;";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "sub_account_type${delim}=${delim}$column_data[$column_count];";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my @temp_sub_account_total;
      my $dbh = $self->{dbHandle};
      my $sth;
      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sqlClause;
      my $error_string = 0;
      # calculate the value (activity/balance) for the sub-account for each date range given the column qualifiers
      if (!($column_type eq "sub_accounts" || $column_type eq "sub_account_types"))
      {
        my $subNumEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $subNumEntries; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1092};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1094};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1093};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1094};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              $error_string = 1;
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";
          }
          $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => \@sql_arguments);
          if ($dbh->error)
          {
            $self->error($dbh->errorMessage);
            return undef;
          }
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $sub_account_num = $row_array->[0];
            my $sub_account_name = $row_array->[1];
            my $value = $row_array->[2];

            if ($sub_account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$sub_account_num})
              {
                $temp_hash->{'sub_rows'}{$sub_account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$sub_account_num};
              $subRow->{'item_name'} = $sub_account_num;
              $subRow->{'cell_text'} = $sub_account_name;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              my $subSubNumEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $subSubNumEntries; $date_count++)
              {
                my $numColumnData = scalar @column_data;
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow->{'error'} = $error_string;
              my $tmp_sql_clause = $sqlClause;
              $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] = $value;
              $subRow->{'total'} += $subRow->{'values'}[$count][$column_count];
              $temp_sub_account_total[$column_count] += $subRow->{'values'}[$count][$column_count];
            }
          }
        }
      }

      $temp_hash->{'values'}[$count] = ();
      if ($column_type eq "nothing")
      {
        if ($account_type[0] == 0 || $account_type[0] == 10)
        { # cash/bank account type or sales revenue account type
          my %options = ();
          $options{'begin_date'} = $date_start if ($temp_row{'activity'});
          $options{'end_date'} = $date_end;
          $options{'balance'} = 1 if (!$temp_row{'activity'});
          $options{'taxable'} = $temp_row{'query_type'};

          my @result;
          my @accounts = ();

          $accounts[0] = ();
          $accounts[0][0] = $temp_hash->{'item_name'};
          $accounts[0][1] = $account_type[1];
          $accounts[0][2] = $temp_hash->{'cell_text'};

          if ($options{'balance'})
          {
            @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
          }
          else
          {
            @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
          }
          $temp_hash->{'values'}[$count][0] = $result[0][0];
        }
        else
        {
          $temp_hash->{'values'}[$count][0] = $temp_sub_account_total[0];
        }
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "date${delim}>=${delim}$date_start;";
          $sqlClause .= "connector${delim}=${delim}AND;";
        }
        $sqlClause .= "date${delim}<=${delim}$date_end;";
        $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        my $subNumEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $subNumEntries; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              my %options = ();
              $options{'begin_date'} = $date_start if ($temp_row{'activity'});
              $options{'end_date'} = $date_end;
              $options{'balance'} = 1 if (!$temp_row{'activity'});
              $options{'taxable'} = $temp_row{'query_type'};

              my @result;
              my @accounts = ();
              my @sub_entity = ();

              $accounts[0] = ();
              $accounts[0][0] = $temp_hash->{'item_name'};
              $accounts[0][1] = $account_type[1];
              $accounts[0][2] = $temp_hash->{'cell_text'};

              $sub_entity[0] = $column_data[$column_count];
              $options{'sub_entity'} = \@sub_entity;

              if ($options{'balance'})
              {
                @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
              }
              else
              {
                @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
              }
              $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
            }
            else
            {
              $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_total[$column_count];
            }
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
            $sqlClause .= "connector${delim}=${delim}AND;";
            $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my %options = ();
              $options{'begin_date'} = $date_start if ($temp_row{'activity'});
              $options{'end_date'} = $date_end;
              $options{'balance'} = 1 if (!$temp_row{'activity'});
              $options{'taxable'} = $temp_row{'query_type'};

              my @result;
              my @accounts = ();
              my @sub_account = ();

              $accounts[0] = ();
              $accounts[0][0] = $temp_hash->{'item_name'};
              $accounts[0][1] = $account_type[1];
              $accounts[0][2] = $temp_hash->{'cell_text'};

              $sub_account[0] = $column_data[$column_count];
              $options{'sub_account'} = \@sub_account;

              if ($options{'balance'})
              {
                @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
              }
              else
              {
                @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
              }
              $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "date${delim}>=${delim}$date_start;";
                $sqlClause .= "connector${delim}=${delim}AND;";
              }
              $sqlClause .= "date${delim}<=${delim}$date_end;";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "sub_account_num${delim}=${delim}$column_data[$column_count];";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my @sub_accts_for_sub_acct_type = $self->GetSubAccountsForSubAccountType(sub_account_type => $column_data[$column_count]);
              my $total = "0.00";
              my $subSubNumEntries = scalar @sub_accts_for_sub_acct_type;
              for (my $z=0; $z < $subSubNumEntries; $z++)
              {
                my %options = ();
                $options{'begin_date'} = $date_start if ($temp_row{'activity'});
                $options{'end_date'} = $date_end;
                $options{'balance'} = 1 if (!$temp_row{'activity'});
                $options{'taxable'} = $temp_row{'query_type'};
                $options{'sub_account'} = \@{$sub_accts_for_sub_acct_type[$z]};

                my @result;
                my @accounts = ();

                $accounts[0] = ();
                $accounts[0][0] = $temp_hash->{'item_name'};
                $accounts[0][1] = $account_type[1];
                $accounts[0][2] = $temp_hash->{'cell_text'};

                if ($options{'balance'})
                {
                  @result = $self->AccountBalanceActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, end_date => $date_end, %options);
                }
                else
                {
                  @result = $self->AccountActivity(ledgerCode => $args{'ledgerCode'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
                }
                $total += $result[0][0];
              }
              $temp_hash->{'values'}[$count][$column_count] = $total;
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "date${delim}>=${delim}$date_start;";
                $sqlClause .= "connector${delim}=${delim}AND;";
              }
              $sqlClause .= "date${delim}<=${delim}$date_end;";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "account_num${delim}=${delim}$temp_hash->{'item_name'};";
              $sqlClause .= "connector${delim}=${delim}AND;";
              $sqlClause .= "sub_account_type${delim}=${delim}$column_data[$column_count];";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }

  return $max_display_level;
}

sub get_sub_account_type_data
{
  my $self = shift;
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledgerCode = $args{'ledgerCode'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @sub_accounts = ();
  my %sub_account_accounts = {};
  my @account_type = ();
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $max_display_level = -1;
  my $data_precision = $args{'data_precision'};

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    my $numEntries = scalar @column_data;
    for (my $count=0; $count < $numEntries; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the name of the account type
  my @temp_item_name = ();
  $temp_item_name[0] = $temp_row{'item_name'};
  my @temp_cell_text = $self->GetAccountTypeNames(account_types => \@temp_item_name);
  $temp_hash->{'cell_text'} = $temp_cell_text[0];

  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  my $numEntries = scalar @{$temp_hash->{dates}};
  for (my $count=0; $count < $numEntries; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'})
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $date_start;
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    my @temp_sub_account_type_total;
    my $dbh = $self->{dbHandle};
    my $sth;
    my $tmp_sql_statement = "";
    my @sql_arguments = ();
    my $sqlClause;
    my $level_sql_clause;
    my $sub_level_sql_clause;
    my $error_string = 0;
    # calculate the value (activity/balance) for the account type for the current date range using each account given the column qualifiers
    if ($temp_row{'sub_levels'} == 0 || ($column_type eq "sub_accounts" || $column_type eq "sub_account_types"))
    {
      my $subNumEntries = scalar @column_data;
      for (my $column_count = 0; $column_count < $subNumEntries; $column_count++)
      {
        if ($column_type eq "nothing")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1101};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1111};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1102};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1102};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end);
            }
          }

          $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "date${delim}>=${delim}$date_start;";
            $level_sql_clause .= "connector${delim}=${delim}AND;";
          }
          $level_sql_clause .= "date${delim}<=${delim}$date_end;";
          $level_sql_clause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $level_sql_clause .= "taxable${delim}=${delim}$tax;";
          }
          $level_sql_clause .= "connector${delim}=${delim}AND;sub_account_type${delim}=${delim}<SUBACCOUNTTYPE>;";
        }
        elsif ($column_type eq "sub_entities")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1103};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1111};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1104};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1104};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end);
              $error_string = 1;
            }
          }

          $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "date${delim}>=${delim}$date_start;";
            $level_sql_clause .= "connector${delim}=${delim}AND;";
          }
          $level_sql_clause .= "date${delim}<=${delim}$date_end;";
          $level_sql_clause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $level_sql_clause .= "taxable${delim}=${delim}$tax;";
          }
          $level_sql_clause .= "connector${delim}=${delim}AND;sub_account_type${delim}=${delim}<SUBACCOUNTTYPE>;";
          $level_sql_clause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
        }
        $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => \@sql_arguments);
        if ($dbh->error)
        {
          $self->error($dbh->errorMessage);
          return undef;
        }
        while (my $row_array = $sth->fetchrow_arrayref())
        {
          my $sub_account_type_num = $row_array->[0];
          my $sub_account_type_name = $row_array->[1];
          my $sub_account_num = $row_array->[2];
          my $sub_account_name = $row_array->[3];
          my $value = $row_array->[4];

          if ($sub_account_type_num ne "" && $sub_account_num ne "")
          {
            if (not defined $temp_hash->{'sub_rows'})
            {
              $temp_hash->{'sub_rows'} = {};
            }

            if (not defined $temp_hash->{'sub_rows'}{$sub_account_type_num})
            {
              $temp_hash->{'sub_rows'}{$sub_account_type_num} = {};
            }
            my $subRow = $temp_hash->{sub_rows}{$sub_account_type_num};
            $subRow->{'cell_text'} = $sub_account_type_name;
            $subRow->{'item_name'} = $sub_account_type_num;
            $subRow->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $temp_hash->{'sub_rows_visible'} = "true";
              $subRow->{'visible'} = "true";
            }
            else
            {
              $temp_hash->{'sub_rows_visible'} = "false";
              $subRow->{'visible'} = "false";
            }
            $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
            if ($temp_row{'display_level'} + 1 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 1;
            }
            $subRow->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow->{'values'})
            {
              $subRow->{'values'} = ();
            }
            if (not defined $subRow->{'sql_clause'})
            {
              $subRow->{'sql_clause'} = ();
            }
            my $subSubNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subSubNumEntries; $date_count++)
            {
              my $numColumnData = scalar @column_data;
              if (not defined $subRow->{'values'}[$date_count])
              {
                $subRow->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'values'}[$date_count][$fill_count])
                  {
                    $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow->{'sql_clause'}[$date_count])
              {
                $subRow->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if (not defined $subRow->{'sub_rows'})
            {
              $subRow->{'sub_rows'} = {};
            }
            if (not defined $subRow->{'sub_rows'}{$sub_account_num})
            {
              $subRow->{'sub_rows'}{$sub_account_num} = {};
            }
            my $subRow2 = $subRow->{sub_rows}{$sub_account_num};
            $subRow2->{'cell_text'} = $sub_account_name;
            $subRow2->{'item_name'} = $sub_account_num;
            $subRow2->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $subRow->{'sub_rows_visible'} = "true";
              $subRow2->{'visible'} = "true";
            }
            else
            {
              $subRow->{'sub_rows_visible'} = "false";
              $subRow2->{'visible'} = "false";
            }
            $subRow2->{'display_level'} = $temp_row{'display_level'} + 2;
            if ($temp_row{'display_level'} + 2 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 2;
            }
            $subRow2->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow2->{'values'})
            {
              $subRow2->{'values'} = ();
            }
            if (not defined $subRow2->{'sql_clause'})
            {
              $subRow2->{'sql_clause'} = ();
            }
            $subSubNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subSubNumEntries; $date_count++)
            {
              my $numColumnData = scalar @column_data;
              if (not defined $subRow2->{'values'}[$date_count])
              {
                $subRow2->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow2->{'values'}[$date_count][$fill_count])
                  {
                    $subRow2->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow2->{'sql_clause'}[$date_count])
              {
                $subRow2->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow2->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow2->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if ($error_string == 1)
            {
              $temp_hash->{'error'} = $error_string;
            }
            $subRow2->{'error'} = $error_string;
            my $tmp_sql_clause = $sqlClause;
            $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
            $subRow2->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow2->{'values'}[$count][$column_count] = $value;
            $subRow2->{'total'} += $subRow2->{'values'}[$count][$column_count];

            $subRow->{'error'} = $error_string;
            $tmp_sql_clause = $level_sql_clause;
            $tmp_sql_clause =~ s/$subAccountTypeRE/$sub_account_type_num/g;
            $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow->{'values'}[$count][$column_count] += $subRow2->{'values'}[$count][$column_count];
            $subRow->{'total'} += $subRow2->{'values'}[$count][$column_count];
            $temp_sub_account_type_total[$column_count] += $subRow2->{'values'}[$count][$column_count];
          }
        }
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my $numColumnData = scalar @column_data;
      for (my $column_count = 0; $column_count < $numColumnData; $column_count++)
      {
        if ($column_type eq "nothing")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1105};
              @sql_arguments = ($ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1109};
              @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1106};
              @sql_arguments = ($ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1106};
              @sql_arguments = ($ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end);
            }
          }

          $sub_level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sub_level_sql_clause = "date${delim}>=${delim}$date_start;";
            $sub_level_sql_clause .= "connector${delim}=${delim}AND;";
          }
          $sub_level_sql_clause .= "date${delim}<=${delim}$date_end;";
          $sub_level_sql_clause .= "connector${delim}=${delim}AND;account_num${delim}=${delim}<ACCOUNT>;";
          $sub_level_sql_clause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";

          $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "date${delim}>=${delim}$date_start;";
            $level_sql_clause .= "connector${delim}=${delim}AND;";
          }
          $level_sql_clause .= "date${delim}<=${delim}$date_end;";
          $level_sql_clause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $level_sql_clause .= "taxable${delim}=${delim}$tax;";
          }
          $level_sql_clause .= "connector${delim}=${delim}AND;sub_account_type${delim}=${delim}<SUBACCOUNTTYPE>;";
        }
        elsif ($column_type eq "sub_entities")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1107};
              @sql_arguments = ($ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1109};
              @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1108};
              @sql_arguments = ($ledgerCode, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1108};
              @sql_arguments = ($ledgerCode, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_end);
              $error_string = 1;
            }
          }

          $sub_level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sub_level_sql_clause = "date${delim}>=${delim}$date_start;";
            $sub_level_sql_clause .= "connector${delim}=${delim}AND;";
          }
          $sub_level_sql_clause .= "date${delim}<=${delim}$date_end;";
          $sub_level_sql_clause .= "connector${delim}=${delim}AND;account_num${delim}=${delim}<ACCOUNT>;";
          $sub_level_sql_clause .= "connector${delim}=${delim}AND;";
          $sub_level_sql_clause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";
          $sub_level_sql_clause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";

          $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "account_type${delim}=${delim}$temp_row{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_account_num${delim}=${delim}<SUBACCOUNT>;";
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_e_code${delim}=${delim}$column_data[$column_count];";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "date${delim}>=${delim}$date_start;";
            $level_sql_clause .= "connector${delim}=${delim}AND;";
          }
          $level_sql_clause .= "date${delim}<=${delim}$date_end;";
          $level_sql_clause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $level_sql_clause .= "taxable${delim}=${delim}$tax;";
          }
          $level_sql_clause .= "connector${delim}=${delim}AND;sub_account_type${delim}=${delim}<SUBACCOUNTTYPE>;";
          $level_sql_clause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
        }
        $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => \@sql_arguments);
        if ($dbh->error)
        {
          $self->error($dbh->errorMessage);
          return undef;
        }
        while (my $row_array = $sth->fetchrow_arrayref())
        {
          my $sub_account_type_num = $row_array->[0];
          my $sub_account_type_name = $row_array->[1];
          my $sub_account_num = $row_array->[2];
          my $sub_account_name = $row_array->[3];
          my $account_num = $row_array->[4];
          my $account_name = $row_array->[5];
          my $value = $row_array->[6];

          if ($sub_account_type_num ne "" && $sub_account_num ne "" && $account_num ne "")
          {
            if (not defined $temp_hash->{'sub_rows'})
            {
              $temp_hash->{'sub_rows'} = {};
            }

            if (not defined $temp_hash->{'sub_rows'}{$sub_account_type_num})
            {
              $temp_hash->{'sub_rows'}{$sub_account_type_num} = {};
            }
            my $subRow = $temp_hash->{sub_rows}{$sub_account_type_num};
            $subRow->{'cell_text'} = $sub_account_type_name;
            $subRow->{'item_name'} = $sub_account_type_num;
            $subRow->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $temp_hash->{'sub_rows_visible'} = "true";
              $subRow->{'visible'} = "true";
            }
            else
            {
              $temp_hash->{'sub_rows_visible'} = "false";
              $subRow->{'visible'} = "false";
            }
            $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
            if ($temp_row{'display_level'} + 1 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 1;
            }
            $subRow->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow->{'values'})
            {
              $subRow->{'values'} = ();
            }
            if (not defined $subRow->{'sql_clause'})
            {
              $subRow->{'sql_clause'} = ();
            }
            my $subNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
            {
              if (not defined $subRow->{'values'}[$date_count])
              {
                $subRow->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'values'}[$date_count][$fill_count])
                  {
                    $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow->{'sql_clause'}[$date_count])
              {
                $subRow->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if (not defined $subRow->{'sub_rows'})
            {
              $subRow->{'sub_rows'} = {};
            }
            if (not defined $subRow->{'sub_rows'}{$sub_account_num})
            {
              $subRow->{'sub_rows'}{$sub_account_num} = {};
            }
            my $subRow2 = $subRow->{sub_rows}{$sub_account_num};
            $subRow2->{'cell_text'} = $sub_account_name;
            $subRow2->{'item_name'} = $sub_account_num;
            $subRow2->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $subRow->{'sub_rows_visible'} = "true";
              $subRow2->{'visible'} = "true";
            }
            else
            {
              $subRow->{'sub_rows_visible'} = "false";
              $subRow2->{'visible'} = "false";
            }
            $subRow2->{'display_level'} = $temp_row{'display_level'} + 2;
            if ($temp_row{'display_level'} + 2 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 2;
            }
            $subRow2->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow2->{'values'})
            {
              $subRow2->{'values'} = ();
            }
            if (not defined $subRow2->{'sql_clause'})
            {
              $subRow2->{'sql_clause'} = ();
            }
            $subNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
            {
              if (not defined $subRow2->{'values'}[$date_count])
              {
                $subRow2->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow2->{'values'}[$date_count][$fill_count])
                  {
                    $subRow2->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow2->{'sql_clause'}[$date_count])
              {
                $subRow2->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow2->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow2->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if (not defined $subRow2->{'sub_rows'})
            {
              $subRow2->{'sub_rows'} = {};
            }
            if (not defined $subRow2->{'sub_rows'}{$account_num})
            {
              $subRow2->{'sub_rows'}{$account_num} = {};
            }
            my $subRow3 = $subRow2->{sub_rows}{$account_num};
            $subRow3->{'cell_text'} = $account_name;
            $subRow3->{'item_name'} = $account_num;
            $subRow3->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $subRow2->{'sub_rows_visible'} = "true";
              $subRow3->{'visible'} = "true";
            }
            else
            {
              $subRow2->{'sub_rows_visible'} = "false";
              $subRow3->{'visible'} = "false";
            }
            $subRow3->{'display_level'} = $temp_row{'display_level'} + 3;
            if ($temp_row{'display_level'} + 3 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 3;
            }
            $subRow3->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
            if (not defined $subRow3->{'values'})
            {
              $subRow3->{'values'} = ();
            }
            if (not defined $subRow3->{'sql_clause'})
            {
              $subRow3->{'sql_clause'} = ();
            }
            for (my $date_count=0; $date_count < scalar @{$temp_hash->{'dates'}}; $date_count++)
            {
              if (not defined $subRow3->{'values'}[$date_count])
              {
                $subRow3->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow3->{'values'}[$date_count][$fill_count])
                  {
                    $subRow3->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow3->{'sql_clause'}[$date_count])
              {
                $subRow3->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow3->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow3->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if ($error_string == 1)
            {
              $temp_hash->{'error'} = $error_string;
            }
            $subRow3->{'error'} = $error_string;
            my $tmp_sql_clause = $sub_level_sql_clause;
            $tmp_sql_clause =~ s/$accountRE/$account_num/g;
            $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
            $subRow3->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow3->{'values'}[$count][$column_count] = $value;
            $subRow3->{'total'} += $subRow3->{'values'}[$count][$column_count];
            $subRow2->{'error'} = $error_string;
            $tmp_sql_clause = $sqlClause;
            $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
            $subRow2->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow2->{'values'}[$count][$column_count] += $subRow3->{'values'}[$count][$column_count];
            $subRow2->{'total'} += $subRow3->{'values'}[$count][$column_count];
            $subRow->{'error'} = $error_string;
            $tmp_sql_clause = $level_sql_clause;
            $tmp_sql_clause =~ s/$subAccountTypeRE/$sub_account_type_num/g;
            $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow->{'values'}[$count][$column_count] += $subRow3->{'values'}[$count][$column_count];
            $subRow->{'total'} += $subRow3->{'values'}[$count][$column_count];
            $temp_sub_account_type_total[$column_count] += $subRow3->{'values'}[$count][$column_count];
          }
        }
      }
    }

    $temp_hash->{'values'}[$count] = ();
    $temp_hash->{'sql_clause'}[$count] = ();
    if ($column_type eq "nothing")
    {
      $temp_hash->{'values'}[$count][0] = $temp_sub_account_type_total[0];
      my $sqlClause = "";
      if ($temp_row{'activity'})
      {
        $sqlClause = "date${delim}>=${delim}$date_start;";
        $sqlClause .= "connector${delim}=${delim}AND;";
      }
      $sqlClause .= "date${delim}<=${delim}$date_end;";
      $sqlClause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
      my $tax = '';
      if ($temp_row{'query_type'} == 1)
      {
        $tax = 'true';
      }
      elsif ($temp_row{'query_type'} == 2)
      {
        $tax = 'false';
      }
      if ($tax ne '')
      {
        $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
      }
      $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
      $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
    }
    else
    {
      for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
      {
        if ($column_type eq "sub_entities")
        {
          $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_type_total[$column_count];
          my $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
          $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        }
        elsif ($column_type eq "sub_accounts")
        {
          $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_type_total[$column_count];
          my $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$column_data[$column_count];";
          $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        }
        elsif ($column_type eq "sub_account_types")
        {
          $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_type_total[$column_count];
          my $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "date${delim}>=${delim}$date_start;";
            $sqlClause .= "connector${delim}=${delim}AND;";
          }
          $sqlClause .= "date${delim}<=${delim}$date_end;";
          $sqlClause .= "connector${delim}=${delim}AND;account_type${delim}=${delim}$temp_hash->{'item_name'};";
          my $tax = '';
          if ($temp_row{'query_type'} == 1)
          {
            $tax = 'true';
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $tax = 'false';
          }
          if ($tax ne '')
          {
            $sqlClause .= "connector${delim}=${delim}AND;taxable${delim}=${delim}$tax;";
          }
          $sqlClause .= "connector${delim}=${delim}AND;";
          $sqlClause .= "sub_account_type${delim}=${delim}$column_data[$column_count];";
          $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        }
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
      }
    }
  }

  return $max_display_level;
}

sub get_sub_account_data
{
  my $self = shift;
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledgerCode = $args{'ledgerCode'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @local_accounts = ();
  my $ledger = $args{'ledgerCode'};
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $max_display_level = -1;
  my $data_precision = $args{'data_precision'};

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    for (my $count=0; $count < scalar @column_data; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the name of the sub-account
  $temp_hash->{'cell_text'} = $self->{SUBACCOUNTS}{$temp_hash->{'item_name'}};
  # get the accounts this sub-account was used with
  @local_accounts = $self->GetAccountsForSubAccount(sub_account => $temp_row{'item_name'}, ledgerCode => $ledgerCode, taxable => $args{'query_type'});

  $temp_hash->{'sub_rows'} = ();
  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'} && $temp_row{'cell_type'} ne "begin_sub_account_balance" && $temp_row{'cell_type'} ne "end_sub_account_balance")
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      if ($temp_row{'cell_type'} eq "begin_sub_account_balance")
      {
        (my $temp_date = $temp_hash->{'dates'}[$count][0]) =~ s/$minusRE/\//g;
        my @temp_date = split("/", $temp_hash->{'dates'}[$count][0]);
        # make date_start and date_end one less day
        @temp_date = Add_Delta_Days(@temp_date, -1);
        $date_start = "$temp_date[0]-$temp_date[1]-$temp_date[2]";
        $date_end = $date_start;
      }
      elsif ($temp_row{'cell_type'} eq "end_sub_account_balance")
      {
        $date_end = $temp_hash->{'dates'}[$count][1];
        $date_start = $date_end;
      }
      else
      {
        $date_start = $temp_hash->{'dates'}[$count][0];
        $date_end = $date_start;
      }
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    if ($temp_row{'sub_levels'} == 0 || $column_type eq "sub_accounts" || $column_type eq "sub_account_types")
    {
      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        my %options = ();
        $options{'begin_date'} = $date_start if ($temp_row{'activity'});
        $options{'end_date'} = $date_end;
        $options{'balance'} = 1 if (!$temp_row{'activity'});
        $options{'taxable'} = $temp_row{'query_type'};

        my @result;
        my @accounts = ();
        my @sub_accounts = ();

        $accounts[0] = ();
        for (my $count2=0; $count2 < scalar @local_accounts; $count2++)
        {
          $accounts[0][$count2] = [ @{$local_accounts[$count2]} ];
        }

        $sub_accounts[0] = ();
        $sub_accounts[0][0] = $temp_hash->{'item_name'};
        $sub_accounts[0][1] = $temp_hash->{'cell_text'};

        @result = $self->SubAccountActivity(ledgerCode => $args{'ledgerCode'}, sub_accounts => \@sub_accounts, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
        $temp_hash->{'values'}[$count][0] = $result[0][0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "date${delim}>=${delim}$date_start;";
          $sqlClause .= "connector${delim}=${delim}AND;";
        }
        $sqlClause .= "date${delim}<=${delim}$date_end;";
        $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$temp_hash->{item_name}";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            my %options = ();
            $options{'begin_date'} = $date_start if ($temp_row{'activity'});
            $options{'end_date'} = $date_end;
            $options{'balance'} = 1 if (!$temp_row{'activity'});
            $options{'taxable'} = $temp_row{'query_type'};

            my @result;
            my @accounts = ();
            my @sub_accounts = ();
            my @sub_entity = ();

            $accounts[0] = ();
            for (my $count2=0; $count2 < scalar @local_accounts; $count2++)
            {
              $accounts[0][$count2] = [ @{$local_accounts[$count2]} ];
            }

            $sub_accounts[0] = ();
            $sub_accounts[0][0] = $temp_hash->{'item_name'};
            $sub_accounts[0][1] = $temp_hash->{'cell_text'};

            $sub_entity[0] = $column_data[$column_count];
            $options{'sub_entity'} = \@sub_entity;

            @result = $self->SubAccountActivity(ledgerCode => $args{'ledgerCode'}, sub_accounts => \@sub_accounts, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
            $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$temp_hash->{item_name}";
            $sqlClause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = 0;
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          elsif ($column_type eq "sub_account_types")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = 0;
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my @temp_account_total;
      my $dbh = $self->{dbHandle};
      my $sth;
      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sqlClause;
      my $error_string = 0;
      # calculate the value (activity/balance) for the sub-account for each date range given the column qualifiers
      if ($column_type ne "sub_accounts" && $column_type ne "sub_account_types")
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1095};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1099};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1096};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "true", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1096};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledgerCode, $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "false", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "false", $date_end);
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$temp_hash->{item_name}";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1097};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1099};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1098};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "true", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "true", $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1098};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledgerCode, $column_data[$column_count], $temp_row{'item_name'}, $ledgerCode, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "false", $date_end, $temp_row{'item_name'}, $ledgerCode, $date_start, $temp_hash->{'report_class'}, $ledgerCode, $ledgerCode, "false", $date_end);
                $error_string = 1;
              }
            }

            $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;account_num${delim}=${delim}<ACCOUNT>;";
            $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$temp_hash->{item_name}";
            $sqlClause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
          }
          $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => \@sql_arguments);
          if ($dbh->error)
          {
            $self->error($dbh->errorMessage);
            return undef;
          }
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $account_num = $row_array->[0];
            my $account_name = $row_array->[1];
            my $value = $row_array->[2];

            if ($account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$account_num})
              {
                $temp_hash->{'sub_rows'}{$account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$account_num};
              $subRow->{'item_name'} = $account_num;
              $subRow->{'cell_text'} = $account_name;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              for (my $date_count=0; $date_count < scalar @{$temp_hash->{'dates'}}; $date_count++)
              {
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow->{'error'} = $error_string;
              my $tmp_sql_clause = $sqlClause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] = $value;
              $subRow->{'total'} += $subRow->{'values'}[$count][$column_count];
              $temp_account_total[$column_count] += $subRow->{'values'}[$count][$column_count];
            }
          }
        }
      }

      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = $temp_account_total[0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "date${delim}>=${delim}$date_start;";
          $sqlClause .= "connector${delim}=${delim}AND;";
        }
        $sqlClause .= "date${delim}<=${delim}$date_end;";
        $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$temp_hash->{item_name}";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "date${delim}>=${delim}$date_start;";
              $sqlClause .= "connector${delim}=${delim}AND;";
            }
            $sqlClause .= "date${delim}<=${delim}$date_end;";
            $sqlClause .= "connector${delim}=${delim}AND;sub_account_num${delim}=${delim}$temp_hash->{item_name}";
            $sqlClause .= "connector${delim}=${delim}AND;sub_e_code${delim}=${delim}$column_data[$column_count];";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          elsif ($column_type eq "sub_account_types")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }

  return $max_display_level;
}

# parse_formula - validates the specified formula and returns the tokens that make up the formula.
# Takes  : formula string
# Returns: Error string and token array.
sub parse_formula
{
  my $self = shift;
  my %args = ( @_, );
  my $formula = $args{'formula'};
  my $result = "";

  if (length $formula == 0)
  {
    $self->error("Error:   parse_formula. $errors{4035}, formula must be specified!");
    return undef;
  }

  my @elements = ($formula =~ m/(\(|R\d+\.\d+|R\d+|-?\d+\.\d+|-?\d+|\+|-|%|\*|\/|\)|\s+|.*)/g);
  my $i;

  # get rid of any empty or whitespace related elements.
  my @nodesToDelete;
  for ($i=0; $i < @elements; $i++)
  {
    if ($elements[$i] =~ /^(\s+|)$/)
    {
      push @nodesToDelete, $i;
    }
  }
  for ($i=0; $i < @nodesToDelete; $i++)
  {
    splice(@elements, $nodesToDelete[$i]-$i, 1);
  }

  if (scalar @elements == 0)
  {
    $result = "Error:  The formula didn't split out any tokens!\n";
    return ($result, @elements);
  }

  my $open_parens = 0;
  for ($i=0; $i < scalar @elements; $i++)
  {
    if ($i == 0) # first element only.
    { # make sure we are not starting out with an operator
      if ($elements[$i] =~ /$operatorRE/)
      {
        $result = "Error:  Formula can not start with an operator!\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /$openParenRE/)
      {
        $open_parens++;
      }
      elsif ($elements[$i] =~ /$closeParenRE/)
      {
        $result = "Error:  You can not start the Formula with an )!\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] !~ /$tokenRE/)
      {
        $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
        return ($result, @elements);
      }
    }
    else  # the rest of the elements.
    { # make sure that the last token was a row, number, or ) if we have an operator
      # else if we have a row make sure the last token was an (
      # and not another row.
      if ($elements[$i] =~ /$operatorRE/)
      {
        if ($elements[$i-1] !~ /$tokenRE/ && $elements[$i-1] !~ /$closeParenRE/)
        {
          $result = "Error:  Operator can not come after another Operator or (!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$openParenRE/)
      {
        $open_parens++;
        # make sure that the previous token was an ( or an operator.
        if ($elements[$i-1] !~ /$openParenRE/ && $elements[$i-1] !~ /$operatorRE/)
        {
          $result = "Error:  ( can not come after a Row, Number, or )!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$closeParenRE/)
      {
        if ($open_parens > 0)
        {
          $open_parens--;
        }
        else
        {
          $result = "Error:  Formula is invalid at token $i, '$elements[$i]'.  Close Paren when no Open Paren to match with!\n";
          return ($result, @elements);
        }

        # make sure that we are not following a ( or an operator.
        if ($elements[$i-1] =~ /$openParenRE/ || $elements[$i-1] =~ /$operatorRE/)
        {
          $result = "Error:  ) can not come after a ( or operator!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$rowIdentifierRE/ && $elements[$i-1] =~ /^-?\d+\.\d+|-?\d+$/)
      {
        $result = "Error:  You can not have a Number and a Row back to back without an intervening operator! Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /^-\d+\.\d+|-\d+$/ && $elements[$i-1] =~ /$tokenRE/)
      {
        $result = "Error:  You can not have Two Numbers or a Row and a Number back to back without an intervening operator!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /$tokenRE/ && $elements[$i-1] !~ /$rowIdentifierRE/)
      {
        if ($elements[$i] =~ /^R\d+\.\d+$/)
        {
          $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$rowIdentifierRE/ && $elements[$i-1] =~ /$rowIdentifierRE/)
      {
        $result = "Error:  You can not have 2 Rows back to back without an intervening operator! Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      else
      {
        $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
        return ($result, @elements);
      }
    }
  }

  # make sure the last token wasn't an operator.
  if ($elements[$i-1] =~ /$operatorRE/)
  {
    $result = "Error:  The Formula can not end with an operator!  Token ${\($i-1)}, '$elements[${\($i-1)}]'.\n";
    return ($result, @elements);
  }

  # make sure we have all ()'s matched up.
  if ($open_parens > 0)
  {
    $result = "Error:  You have unmatched ('s!\n";
    return ($result, @elements);
  }

  return ($result, [ @elements ]);  # signal it passed.
}

sub get_total_data
{
  my $self = shift;
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $dbh = $self->{dbHandle};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $data_precision = $args{'data_precision'};
  my @report_data = @{ $arguments{'report_data'} };
  my $formula = $temp_row{'cell_formula'};
  my @formula_tokens = $self->parse_formula(formula => $formula);
  my @original_tokens = $self->RetrieveSubArray(array => \@formula_tokens, index => 1);

  if ($column_type eq "sub_entities")
  {
    for (my $count=0; $count < scalar @column_data; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  if (length $formula_tokens[0] > 0) # error happened parsing the formula
  {
    $temp_hash->{'cell_text'} .= "<br>(<b>Invalid row formula!</b>)";
    $temp_hash->{'values'} = ();
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = 0;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          $temp_hash->{'values'}[$count][$column_count] = 0;
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }
  else
  {
    $temp_hash->{'values'} = ();
    # run over the dates and calculate the formula of the data for each column
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        my $result;
        my $temp_formula = "\$result = ";
        for (my $token_count=0; $token_count < scalar @original_tokens; $token_count++)
        {
          if ($original_tokens[$token_count] =~ /$rowIdentifier2RE/)
          {
            # replace the row value with the value of that row
            # added parentheses around the value in case it is negative
            $temp_formula .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
          }
          else
          {
            $temp_formula .= $original_tokens[$token_count];
          }
        }
        eval($temp_formula);
        if ($@)
        {
          $self->error('<div class="error">1 Error evaluating formula - get_total_data. '.$errors{4037}."</div><div>Formula string: '$formula'<br>Row: $temp_row{'index'}</div><div>$@</div>");
          return undef;
        }
        $temp_hash->{'values'}[$count][0] = $result;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          my $result;
          my $temp_formula = "\$result = ";
          for (my $token_count=0; $token_count < scalar @original_tokens; $token_count++)
          {
            if ($original_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $temp_formula .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
            }
            else
            {
              $temp_formula .= $original_tokens[$token_count];
            }
          }
          eval($temp_formula);
          if ($@)
          {
            $self->error('<div class="error">2 Error evaluating formula - get_total_data. '.$errors{4037}."</div><div>Formula string: '$formula'<br>Row: $temp_row{'index'}</div><div>$@</div>");
            return undef;
          }
          $temp_hash->{'values'}[$count][$column_count] = $result;
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }
}

sub get_comparison_data
{
  my $self = shift;
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $dbh = $self->{dbHandle};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $data_precision = $args{'data_precision'};
  my @report_data = @{ $arguments{'report_data'} };
  (my $formula = $temp_row{'cell_formula'}) =~ m/^\(\s*(.+)\s+(==|!=|<=|>=|<|>)\s+(.+)\s+\?\s*"(.*)"\s*:\s*"(.*)"\s*\)$/;
  my $formula1 = $1;
  my $formula2 = $3;
  my $operator = $2;
  my $result_true = $4;
  my $result_false = $5;
  my @formula1_tokens = $self->parse_formula(formula => $formula1);
  my @formula2_tokens = $self->parse_formula(formula => $formula2);
  my @original1_tokens = $self->RetrieveSubArray(array => \@formula1_tokens, index => 1);
  my @original2_tokens = $self->RetrieveSubArray(array => \@formula2_tokens, index => 1);
  my $result_formula = ($temp_row{'result_type'} eq "result_formula" ? 1 : 0);

  my @formula3_tokens = ();
  my @formula4_tokens = ();
  my @original3_tokens = ();
  my @original4_tokens = ();

  if ($result_formula)
  {
    @formula3_tokens = $self->parse_formula(formula => $result_true);
    @formula4_tokens = $self->parse_formula(formula => $result_false);
    @original3_tokens = $self->RetrieveSubArray(array => \@formula3_tokens, index => 1);
    @original4_tokens = $self->RetrieveSubArray(array => \@formula4_tokens, index => 1);
  }

  # support the Comparison Row as a Formula Result ----------------

  if ($column_type eq "sub_entities")
  {
    for (my $count=0; $count < scalar @column_data; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  $temp_hash->{'values'} = ();
  if (length $formula1_tokens[0] > 0 || length $formula2_tokens[0] > 0 || ($result_formula && (length $formula3_tokens[0] > 0 || length $formula4_tokens[0] > 0))) # error happened parsing the formula(s)
  {
    $temp_hash->{'cell_text'} .= "<br>(<b>Invalid row formula!</b>)";
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = 0;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          $temp_hash->{'values'}[$count][$column_count] = 0;
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }
  else
  {
    # for each date calculate the formula of the data for each column
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      my $result_true_string = $result_true;
      my $result_false_string = $result_false;

      if ($result_formula)
      {
        $result_true_string = $result_false_string = "(";
      }

      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        my $result = "";
        my $temp_formula = '$result = ( ( sprintf("%f", ';
        for (my $token_count=0; $token_count < scalar @original1_tokens; $token_count++)
        {
          if ($original1_tokens[$token_count] =~ /$rowIdentifier2RE/)
          {
            # replace the row value with the value of that row
            # added parentheses around the value in case it is negative
            $temp_formula .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
          }
          else
          {
            $temp_formula .= $original1_tokens[$token_count];
          }
        }
        $temp_formula .= ") ) " . $operator . ' ( sprintf("%f", ';
        for (my $token_count=0; $token_count < scalar @original2_tokens; $token_count++)
        {
          if ($original2_tokens[$token_count] =~ /$rowIdentifier2RE/)
          {
            # replace the row value with the value of that row
            # added parentheses around the value in case it is negative
            $temp_formula .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
          }
          else
          {
            $temp_formula .= $original2_tokens[$token_count];
          }
        }
        if ($result_formula)
        {
          for (my $token_count=0; $token_count < scalar @original3_tokens; $token_count++)
          {
            if ($original3_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $result_true_string .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
            }
            else
            {
              $result_true_string .= $original3_tokens[$token_count];
            }

          }
          for (my $token_count=0; $token_count < scalar @original4_tokens; $token_count++)
          {
            if ($original4_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $result_false_string .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
            }
            else
            {
              $result_false_string .= $original4_tokens[$token_count];
            }
          }
          $result_true_string .= ")";
          $result_false_string .= ")";
          $temp_formula .= ") ) ? $result_true_string : $result_false_string)";
        }
        else
        {
          $temp_formula .= ") ) ? \"" . $result_true_string . "\" : \"" . $result_false_string . "\")";
        }

        eval($temp_formula);
        if ($@)
        {
          $self->error("<div>3: Error evaluating formula - get_comparison_data. $errors{4037}</div><div>Formula string: '$formula'</div><div>Temp Formula: $temp_formula</div><div>Result True String = '$result_true_string'<br>Result False String = '$result_false_string'</div><div>Row: $temp_row{'index'}</div><div>$@</div>");
        }
        $temp_hash->{'values'}[$count][0] = $result;
        $temp_hash->{'debug_formula'}[$count][0] = $temp_formula;
        if ($result_formula)
        {
          $temp_hash->{total} += $result;
        }
        else
        {
          $temp_hash->{'total'} = "&nbsp;";
        }
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          my $result = "";
          my $temp_formula = '$result = ( ( sprintf("%f", ';
          my $result_true_string = $result_true;
          my $result_false_string = $result_false;

          if ($result_formula)
          {
            $result_true_string = $result_false_string = "(";
          }

          for (my $token_count=0; $token_count < scalar @original1_tokens; $token_count++)
          {
            if ($original1_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $temp_formula .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
            }
            else
            {
              $temp_formula .= $original1_tokens[$token_count];
            }
          }
          $temp_formula .= ") ) " . $operator . " (";
          if ($result_formula)
          {
            $temp_formula .= "sprintf(\"\%f\", ";
          }
          for (my $token_count=0; $token_count < scalar @original2_tokens; $token_count++)
          {
            if ($original2_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $temp_formula .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
            }
            else
            {
              $temp_formula .= $original2_tokens[$token_count];
            }
          }
          if ($result_formula)
          {
            for (my $token_count=0; $token_count < scalar @original3_tokens; $token_count++)
            {
              if ($original3_tokens[$token_count] =~ /$rowIdentifier2RE/)
              {
                # replace the row value with the value of that row
                # added parentheses around the value in case it is negative
                $result_true_string .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
              }
              else
              {
                $result_true_string .= $original3_tokens[$token_count];
              }
            }
            for (my $token_count=0; $token_count < scalar @original4_tokens; $token_count++)
            {
              if ($original4_tokens[$token_count] =~ /$rowIdentifier2RE/)
              {
                # replace the row value with the value of that row
                # added parentheses around the value in case it is negative
                $result_false_string .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
              }
              else
              {
                $result_false_string .= $original4_tokens[$token_count];
              }
            }
            $result_true_string .= ")";
            $result_false_string .= ")";
            $temp_formula .= ") ) ? $result_true_string : $result_false_string)";
          }
          else
          {
            $temp_formula .= ")  ? \"" . $result_true_string . "\" : \"" . $result_false_string . "\")";
            #$temp_formula .= ") ) ? \"" . $result_true_string . "\" : \"" . $result_false_string . "\")";
          }

          eval($temp_formula);
          if ($@)
          {
            $self->error("<div>4 Error evaluating formula - get_comparison_data. $errors{4037}</div><div>Formula string: '$formula'</div><div>Temp Formula: $temp_formula</div><div>Row: $temp_row{'index'}</div><div>$@</div>");
            return undef;
          }
          $temp_hash->{'values'}[$count][$column_count] = $result;
          $temp_hash->{'debug_formula'}[$count][$column_count] = $temp_formula;
          if ($result_formula)
          {
            $temp_hash->{total} += $result;
          }
          else
          {
            $temp_hash->{'total'} = "&nbsp;";
          }
        }
      }
    }
  }
}

# This function gathers the values from the client report customization screen and stores them in the hash after validation.
sub get_report_attributes
{
  my $self = shift;
  my %args = ( memorize => 0, @_, );
  my $dbh = $self->{dbHandle};
  my %input = (
    'report_class' => "custom",
    'report_type' => "r",
    'column_variable' => "",
    'column_variable_data' => "",
    'display_dates' => "true",
    'date_range' => "true",
    'max_columns' => 36,
    'min_columns' => 1,
    'text_size' => "-1",
    'text_font' => "helvetica",
    'data_size' => "-1",
    'data_font' => "helvetica",
    'total_column' => "false",
    'underline_length' => $self->{default_underline_length},
    data_precision => 1,
    displayZeroRows => 0,
    negative_format => -1,
    account_format => 2,
    "sorted_by" => 'Ref #',
    "sorted_order" => 'a',
    subtotal => 'false',
    subtotal_type => 'd',
    reportWidth => 100,
    %{$self->{input}}
  );
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $memorize = $args{'memorize'};
  my %attributes = ();
  my @arguments = ("id", "report_type", "report_name", "report_class", "date_range", "max_columns", "min_columns", "underline_length", "data_precision", "negative_format", "text_size", "text_font", "data_size", "data_font", "account_format", "column_variable", "column_variable_data", "total_column", "display_dates", "column_data", "column_data_names", "column_type", "sql_statement", "sorted_by", "sorted_order", "subtotal", "subtotal_type", "dates_entered", "border", "reportWidth", "displayZeroRows");
  #took out headers
  my $error_string = "";

  my $doc = HTMLObject::Base->new;
  LOOP: foreach my $name (@arguments)
  {
    next LOOP if ($name eq "sql_statement" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "headers" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "sorted_by" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "sorted_order" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "subtotal" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "subtotal_type" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "dates_entered");
    next LOOP if ($name eq "displayZeroRows");
    if (!exists $input{$name})
    {
      if ($name eq "column_data" || $name eq "column_data_names" || $name eq "column_type")
      {
      }
      else
      {
        $doc->doRequiredParameterError('get_report_attributes', $name);
      }
    }
    if (!$self->validate_customized_report_attribute(name => $name, value => $input{$name}, dbh => $dbh))
    {
      $error_string .= "<br>\n" if (length $error_string > 0);
      $error_string .= "'$name' = '$input{$name}' is invalid!";
      #$self->error("Error:   get_report_attributes $errors{4006}, Name = '$name', Value = '$input{$name}'");
    }
    $attributes{$name} = $input{$name};
    if ($name eq "ledgerCode")
    { # get the ledger_name, ledgerCode, and ledger_color values
      my $ledger_name = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};
      my $ledger_color = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{color};

      $attributes{'ledger_name'} = $ledger_name;
      $attributes{'ledgerCode'} = $ledgerCode;
      $attributes{'ledger_color'} = $ledger_color;
    }
    if ($name eq "column_data")
    {
      $attributes{$name} = join(",", split(/\0/, $input{$name}));
    }
  }

  my $report_type = $attributes{'report_type'};
  if ($report_type eq "r")
  {
    # validate the column_data
    my $column_type = $attributes{'column_type'};
    my $column_data = $attributes{'column_data'};
    my @column_variable = split /,/, $attributes{'column_variable'};
    my @column_variable_data = split /\|/, $attributes{'column_variable_data'};
    if ($column_type ne "nothing")
    {
      # first validate that the column type is valid for this report
      my $found = 0;
      my $i;
      for ($i=0; $i < scalar @column_variable && !$found; $i++)
      {
        if ($column_variable[$i] eq $column_type)
        {
          $found = 1;
        }
      }
      $i--;  # point to the correct index.
      if (!$found)
      {
        $error_string .= "<br>\n" if (length $error_string > 0);
        $error_string .= "Column Type = '$column_type' is invalid for this report!";
      }
      else
      {
        # now validate the values entered
        if (length $column_data > 0)
        {
          foreach my $tmp_value (split(/,/, $column_data))
          {
            $found = 0;
            my $j;
            my @tmp_column_variable_data = split /,/, $column_variable_data[$i];
            for ($j=0; $j < scalar @tmp_column_variable_data && !$found; $j++)
            {
              if ($tmp_column_variable_data[$j] eq $tmp_value)
              {
                $found = 1;
              }
            }
            if (!$found)
            {
              $error_string .= "<br>\n" if (length $error_string > 0);
              $error_string .= "Column Data = '$tmp_value' is invalid for Column Type = '$column_type'!";
            }
          }
        }
        elsif (length $column_data == 0 && $column_type eq "sub_entities")
        {
        }
        else
        {
          $error_string .= "<br>\n" if (length $error_string > 0);
          $error_string .= "You must select at least one Column Data value to work with for Column Type = '$column_type'!";
        }
      }
    }

    $attributes{label} = [];
    my $date_counter = 0;  # keeps track of the number of valid dates we have encountered.
    # now check for the dates.
    if ($attributes{'date_range'} eq "true")
    {
      @arguments = ();
      for (my $i=0; $i < $attributes{'max_columns'}; $i++)
      {
        push (@arguments, ("column$i"."_start_date", "column$i"."_end_date"));
      }
    }
    else # single dates
    {
      @arguments = ();
      for (my $i=0; $i < $attributes{'max_columns'}; $i++)
      {
        push (@arguments, ("column$i"."_start_date"));
      }
    }

    my @entered_dates = ();
    my @label = ();
    # now validate we got the minimum amount of required dates and that all dates input are valid.
    for (my $i=0; $i < scalar @arguments; $i++)
    {
      my $date = $arguments[$i];
      if (!exists $input{$date})
      {
        $doc->doRequiredParameterError('get_report_attributes', $date);
      }

      if (length $input{$date} > 0)
      {
        if ($attributes{'date_range'} eq "false")
        { # simple case
          $entered_dates[$date_counter++] = $input{$date};
        }
        else
        { # check the ending date
          my $old_date = $date;
          $date = $arguments[++$i];
          if (!exists $input{$date})
          {
            $doc->doRequiredParameterError('get_report_attributes', $date);
          }

          if (length $input{$date} > 0)
          {
            $entered_dates[$date_counter++] = "$input{$old_date},$input{$date}";
          }
        }
      }
    }

    for (my $d = 0; $d < scalar @entered_dates; $d++)
    {
      if (!defined $input{"column$d"."_label"})
      {
        $doc->doRequiredParameterError('get_report_attributes', "column$d"."_label");
      }
      else
      {
        $attributes{"column$d"."_label"} = $input{"column$d"."_label"};
      }
    }
    $attributes{'dates_entered'} = [ @entered_dates ];

    if ($date_counter < $attributes{'min_columns'})
    {
      $error_string .= "<br>\n" if (length $error_string > 0);
      $error_string .= "Only <b>$date_counter</b> Date(s) were entered!&nbsp;&nbsp;At least <b>$attributes{'min_columns'}</b> Date(s) must be entered.";
    }

    $attributes{displayZeroRows} = (exists $input{displayZeroRows} ? $input{displayZeroRows} : 0 );
  }

  if (length $error_string > 0) { $self->error($error_string); }
  return %attributes;
}

# This function validates that the attribute and value given it from the get_report_attributes function which gets them from the customize report screen are valid.
sub validate_customized_report_attribute
{
  my $self = shift;
  my %args = ( @_, );
  my $dbh = $self->{dbHandle};
  my $name = $args{'name'};
  my $value = $args{'value'};
  my $result = 1;

  # validate it is valid.
  if ($name eq "date_range" || $name eq "total_column" || $name eq "display_dates")
  {
    if (!exists $true_false{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "id")
  {
    if ($value < 0)
    {
      $result = 0;
    }
    # look the report up that has this id and make sure it exists.
    if (length $self->DoesReportExist(dbh => $dbh, id => $value) == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "column_data" || $name eq "column_type" || $name eq "column_variable" || $name eq "column_variable_data")
  { # these are checked later in the get_report_attributes function.
  }
  elsif ($name eq "column_data_names")
  { # ignore this value; it's just used to pass data through for the names of the column data numbers
  }
  elsif ($name eq "report_name")
  {
    if (length $value == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_class")
  {
    if (length $value == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "max_columns")
  {
    if ($value < 1 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "min_columns")
  {
    if ($value < 1 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "underline_length")
  {
    if ($value < $self->{min_underline_length} || $value > $self->{max_underline_length})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_precision")
  {
    if (!exists $precision_formats{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "negative_format")
  {
    if ($value != -1 && !exists $negative_formats{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_size" || $name eq "data_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_font" || $name eq "data_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "account_format")
  {
    if (!exists $self->{account_formats}->{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_type")
  {
    $result = 0 if (!exists $report_types{$value});
  }
  elsif ($name eq "sql_statement")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "headers")
  {
  }
  elsif ($name eq "sorted_by")
  {
  }
  elsif ($name eq "sorted_order")
  {
    $result = 0 if ($value != "a" && $value != "d");
  }
  elsif ($name eq "subtotal")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "subtotal_type")
  {
    $result = 0 if ($value != "d" && $value != "b");
  }
  elsif ($name eq "dates_entered")
  {
    $result = 0 if (length $value == 0);
  }
#   elsif ($name eq "label")
#   {
#     $result = 0 if (length $value == 0);
#   }
  elsif ($name eq "border")
  {
    $result = 0 if ($value != 0 && $value != 1);
  }
  elsif ($name eq "windowName")
  {
#    $result = 0 if ($value !~ /$reportNumRE/);
  }
  elsif ($name eq "reportWidth")
  {
    $result = 0 if ($value !~ /^(\d+)$/ && ($value < 25 || $value > 100));
  }
  else
  {
    $result = 0;  # unknown parameter name.
  }

  return $result;
}

# This function returns the data from the storable object (if specified via cookie) or defaults for creating a report.
# currently takes nothing.
sub get_default_data
{
  my $self = shift;
  my %values;
  my $value_ref; # reference to the hash.

  # check and see if the error cookie exists.
  my $cookies = HTMLObject::ReadCookie->new;
  my %ch = $cookies->getCookies(cookies => ['error_happened']);
  if (scalar keys %ch == 1)
  {
    my $file_name = $cookies->cookies->{'error_happened'};
    $value_ref = retrieve($file_name);
    %values = %{$value_ref};
    #unlink $file_name;  # delete the file.
  }
  else
  {
    $values{'name'} = "";
    $values{'id'} = 0;
    $values{'report_class'} = "custom";
    $values{'report_type'} = "r";
    $values{'column_variable'} = "";
    $values{'column_variable_data'} = "";
    $values{'display_dates'} = "true";
    $values{'date_range'} = "true";
    $values{'max_columns'} = 36;
    $values{'min_columns'} = 1;
    $values{'text_size'} = "-1";
    $values{'text_font'} = "helvetica";
    $values{'data_size'} = "-1";
    $values{'data_font'} = "helvetica";
    $values{'total_column'} = "false";
    $values{'underline_length'} = $self->{default_underline_length};
    $values{displayZeroRows} = 0;
  }

  return %values;
}

# GetAccountTypeNames
# Takes  : Array of Account Types (account_types)
# Returns: Array of Account Type Names in order given.
sub GetAccountTypeNames
{
  my $self = shift;
  my %args = ( @_ );
  my @account_types = @{$args{'account_types'}};
  my @account_type_names = ();
  if (scalar @account_types == 0)
  {
    $self->error("Error: GetAccountTypeNames.  The Array of Account Types is required at line " . __LINE__);
    return undef;
  }

  foreach my $account_type (@account_types)
  {
    push(@account_type_names, $self->{ACCOUNTTYPES}->{$account_type}->{name});
  }
  return @account_type_names;
}

# GetAccountsForSubAccount
# Takes   : sub_account, dbh, ledgerCode
# Optional: taxable, account_type
# Returns : Array of Local Accounts that are used with the given sub-account
sub GetAccountsForSubAccount
{
  my $self = shift;
  my %args = ( taxable => '0', @_ );
  my $dbh = $self->dbHandle;
  my $sub_account = $args{'sub_account'};
  my $ledgerCode = $args{'ledgerCode'};
  my $account_type = $args{'account_type'};
  my @account_type_list = ();
  my $taxable = $args{'taxable'};

  if (length $sub_account == 0)
  {
    $self->error("Error: GetAccountsForSubAccount. $errors{4034} The Sub Account is required at line " . __LINE__);
    return undef;
  }

  if (defined $account_type && length $account_type == 0)
  {
    $self->error("Error: GetAccountsForSubAccount. $errors{4034} The Account Type value must be defined at line " . __LINE__);
    return undef;
  }

  # get the sub-account type for the given sub-account
  if (!defined $account_type)
  {
    my $sub_account_type = $self->{SUBACCOUNTS}->{$sub_account}->get('type');
    # get a list of account types this sub-account type is assigned to
    my $sth = $dbh->read( $sql_statements{1060} );
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    while (my @temp_array = $sth->fetchrow_array)
    {
      my @temp_type = split /,/, $temp_array[1];
      for (my $count=0; $count < scalar @temp_type; $count++)
      {
        if ($sub_account_type == $temp_type[$count] || $sub_account == -1)
        {
          $account_type_list[scalar @account_type_list] = $temp_array[0];
        }
      }
    }
  }
  else
  {
    $account_type_list[0] = $account_type;
  }

  my @local_accounts = ();
  # get the local list of accounts for the account_type_list of account types
  if (scalar @account_type_list > 0)
  {
    my @local_accounts_list = $self->GetLocalAccounts(account_types => \@account_type_list, taxable => $taxable, ledgerCode => $ledgerCode);
    my $count3 = 0;
    for (my $count=0; $count < scalar @local_accounts_list; $count++)
    {
      for (my $count2=0; $count2 < scalar @{$local_accounts_list[$count]}; $count2++)
      {
        $local_accounts[$count3] = ();
        $local_accounts[$count3][0] = $local_accounts_list[$count][$count2][0];
        $local_accounts[$count3][1] = $local_accounts_list[$count][$count2][1];
        $local_accounts[$count3][2] = $local_accounts_list[$count][$count2][2];
        $count3++;
      }
    }
  }

  return @local_accounts;
}

# GetLocalAccounts
# Takes  : Array of Account Types (account_types), dbh, ledgerCode
# Takes  : Taxable Flag (Optional, if not specified assumed to be 0) (0=don't care, 1=taxable only, 2=non-taxable only)
# Returns: Array of Local Accounts for each Account Type with the Debit Increases Flag for each Account Type and the Account Name
#          Array [AccountType]
#          [AccountType] = [LocalAccounts]
#          [LocalAccounts] = Account Number, Debit Increases Flag, Account Name
sub GetLocalAccounts
{
  my $self = shift;
  my %args = ( taxable => '0', @_ );
  my $dbh = $self->dbHandle;
  my @account_types = @{$args{'account_types'}};
  my $taxable = $args{'taxable'};
  my $ledgerCode = $args{'ledgerCode'};
  my @local_accounts = ();

  if (scalar @account_types == 0)
  {
    $self->error("Error: Report.cgi - GetLocalAccounts, $errors{4034}, The Array of Account Types is required at line " . __LINE__);
    return undef;
  }

  if ($taxable < 0 || $taxable > 2)
  {
    $self->error("Error:  Report.cgi - GetLocalAccounts, $errors{4034}, The Taxable flag can only be 0, 1, or 2!&nbsp;&nbsp;'$taxable' is invalid!<br>\nAt Line " . __LINE__);
    return undef;
  }

  for(my $i=0; $i < scalar @account_types; $i++)
  {
    my $account_type = $account_types[$i];
    my @Info = ();

    my $sth = $dbh->read( sql => $sql_statements{1043}, plug => [$account_type]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    my @info = $sth->fetchrow_array;
    my $debit_increases = $info[0];

    $sth = $dbh->read( sql => $sql_statements{1044}, plug => [$account_type]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    @info = @{$sth->fetchall_arrayref};
    if (scalar @info > 0)
    {
      my ($acct_num_list, $j, $acct_number, $acct_name);
      for ($j=0; $j < scalar @info; $j++)
      {
        $acct_number = $info[$j][0];

        $acct_num_list .= ", " if ($j > 0);
        $acct_num_list .= $acct_number;
      }

      # now get the accounts that exist in the local_accounts_tb that match up
      if ($taxable == 1 && $account_type != 15)
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1045}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->read( sql => $tmp_sql_statement, plug => [$ledgerCode]);
        if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
      }
      elsif ($taxable == 2)
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1046}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => [$ledgerCode]);
      }
      else
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1047}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->read(sql =>  $tmp_sql_statement, plug => [$ledgerCode]);
        if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
      }
      my @accounts = @{$sth->fetchall_arrayref};


      # now we walk the list and create the array that will make up this Account Type's entry
      for ($j=0; $j < scalar @accounts; $j++)
      {
        my @element = ();
        $acct_number = $accounts[$j][0];

        # get this accounts name
        $acct_name = $self->{ACCOUNTS}->{$acct_number};

        $element[0] = $acct_number;
        $element[1] = $debit_increases;
        $element[2] = $acct_name;

        $Info[$j] = [ @element ];
      }
    }

    $local_accounts[$i] = [ @Info ];
  }

  return @local_accounts;
}

# SubAccountActivity
# Takes  : Array of SubAccounts (number, name)
# Takes  : Array of Accounts (number, debit increases, name).  For each Sub Account is an array of the accounts.
# Takes  : Begin and End Dates, Search Type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Returns: Array of Activity with a subarray for each sub account.
#    Array [Sub Accounts]
#    [Sub Accounts] = total activity, [Accounts Activity], total # entries
#    [Accounts Activity] = activity, # entries
sub SubAccountActivity
{
  my $self = shift;
  my %args = ( search_type => '1', @_ );
  my $dbh = $self->{dbHandle};
  my @sub_accounts = @{$args{'sub_accounts'}};
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledgerCode = $args{'ledgerCode'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my $balance = $args{'balance'};
  my @activity = ();
  my @sub_entity = ();

  if (scalar @sub_accounts == 0)
  {
    $self->error("Error: Report.cgi - SubAccountActivity. $errors{4034} The Array of SubAccounts is required at line " . __LINE__);
    return undef;
  }

  if (scalar @accounts == 0)
  {
    $self->error("Error: Report.cgi - SubAccountActivity. $errors{4034} The Array of Accounts is required at line " . __LINE__);
    return undef;
  }

  if ($search_type < 0 || $search_type > 2)
  {
    $self->error("Error:  Report.cgi - SubAccountActivity. $errors{4034} The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (length $begin_date == 0)
  {
    $self->error("Error:  Report.cgi - SubAccountActivity. $errors{4034} The begin_date must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (length $end_date == 0)
  {
    $self->error("Error:  Report.cgi - SubAccountActivity. $errors{4034} The end_date must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      $self->error("Error:  Report.cgi - SubAccountActivity. $errors{4034} The Sub Entity info can not be empty!<br>\nAt Line " . __LINE__);
      return undef;
    }
  }

  my ($i, $tmp_balance, $num_entries);
  # now we need to walk through the SubAccounts array and for each entry, get all Account Activity totals and sum them up.
  for ($i=0; $i < scalar @sub_accounts; $i++)
  {
    $tmp_balance = "0.00";
    $num_entries = 0;
    my @entry = ();

    my @tmp_accounts = $self->RetrieveSubArray(array => \@accounts, index => $i);

    if (scalar @tmp_accounts > 0)
    {
      my %options = ();
      if (scalar @sub_entity > 0)
      {
        $options{'sub_entity'} = \@sub_entity;
      }
      my @balances = ();
      my @tmp_sub_account = $self->RetrieveSubArray(array => \@sub_accounts, index => $i);
      if ($balance)
      {
        @balances = $self->AccountBalanceActivity(ledgerCode => $ledgerCode, accounts => \@tmp_accounts, search_type => $search_type, begin_date => $begin_date, end_date => $end_date, sub_account => \@tmp_sub_account, %options);
      }
      else
      {
        @balances = $self->AccountActivity(ledgerCode => $ledgerCode, accounts => \@tmp_accounts, search_type => $search_type, begin_date => $begin_date, end_date => $end_date, sub_account => \@tmp_sub_account, %options);
      }
      @entry = @balances;
      for (my $j=0; $j < scalar @balances; $j++)
      {
        $tmp_balance += $balances[$j][0];
        $num_entries += $balances[$j][1];
      }
    }
    else
    {
      @entry = [ ];
    }

    $tmp_balance = $tmp_balance;

    my @tmp_entry = ();
    $tmp_entry[0] = $tmp_balance;
    $tmp_entry[1] = [ @entry ];
    $tmp_entry[2] = $num_entries;

    $activity[$i] = [ @tmp_entry ];
  }

  return @activity;
}

# RetrieveSubArray - Extracts the specified sub array from a multi-dimensional array and returns a flat array.
# Takes  : array, index
# Returns: sub array from array at index.
sub RetrieveSubArray
{
  my $self = shift;
  my %args = ( @_ );
  my @array = @{$args{'array'}};
  my $index = $args{'index'};
  my @sub_array = @{$array[$index]};
  return @sub_array;
}

# AccountBalanceActivity
# Takes  : Array of Accounts with Debit Increases Flag and Account Name (accounts), dbh, ledgerCode
# Takes  : end_date, sub_account (optional, the Sub Account number and name), sub_entity (optional, the Code and Name)
# Returns: Array of Balances and the # of entries for each account
sub AccountBalanceActivity
{
  my $self = shift;
  my %args = ( search_type => '1', @_ );
  my $dbh = $self->{dbHandle};
  my @accounts = @{$args{'accounts'}};
  my $ledgerCode = $args{'ledgerCode'};
  my $end_date = $args{'end_date'};
  my @sub_account = ();
  my @sub_entity = ();
  my @activity = ();

  if (scalar @accounts == 0)
  {
    $self->error("Error: Report.cgi - AccountBalanceActivity. $errors{4034} The Array of Accounts is required at line " . __LINE__);
    return undef;
  }

  if (length $end_date == 0)
  {
    $self->error("Error:  Report.cgi - AccountBalanceActivity. $errors{4034} The end_date must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      $self->error("Error:  Report.cgi - AccountBalanceActivity. $errors{4034} The Sub Account info can not be empty!<br>\nAt Line " . __LINE__);
      return undef;
    }
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      $self->error("Error:  Report.cgi - AccountBalanceActivity. $errors{4034} The Sub Entity info can not be empty!<br>\nAt Line " . __LINE__);
      return undef;
    }
  }

  my ($i);
  for ($i=0; $i < scalar @accounts; $i++)
  {
    my %options = ();
    if (scalar @sub_entity > 0)
    {
      $options{'sub_entity'} = \@sub_entity;
    }
    if (scalar @sub_account > 0)
    {
      $options{'sub_account'} = \@sub_account;
    }
    my @result = $self->GetAccountBalance(account => \@{$accounts[$i]}, end_date => $end_date, ledgerCode => $ledgerCode, %options);

    $activity[$i] = [ @result ];
  }

  return @activity;
}

# GetAccountBalance
# Takes  : dbh, ledgerCode, SubEntity Array (Code, Name) (optional), Account Array (Number, Debit Increases, Name), end_date
# Takes  : sub_account Array (Number, Name) (optional)
# Returns: Array of the Balance and # of entries for the account specified.

# Note:  As SubEntities/Accounts do not have their balances stored when the system is initialized or closed out each year, the
# SubEntity value is currently being ignored, but allowed to be entered for future implementation.

# Note:  Since Revenues and Expenses can never have balances stored in the acct_bal_tb, when one of these accounts is passed in,
# the last close out date is gathered and used as the beginning date to gather activity for that account and use that as the
# balance value.
sub GetAccountBalance
{
  my $self = shift;
  my %args = ( @_ );
  my $dbh = $self->{dbHandle};
  my @sub_entity = ();
  my @account = @{$args{'account'}};
  my $ledgerCode = $args{'ledgerCode'};
  my $end_date = $args{'end_date'};
  my @balance = ();
  my @sub_account = ();

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      $self->error("Error: Report.cgi - GetAccountBalance. $errors{4034} The SubEntity can not be empty at line " . __LINE__);
      return undef;
    }
  }

  if (scalar @account == 0)
  {
    $self->error("Error: Report.cgi - GetAccountBalance. $errors{4034} The Account is required at line " . __LINE__);
    return undef;
  }

  if (length $end_date == 0)
  {
    $self->error("Error:  Report.cgi - GetAccountBalance. $errors{4034} The end_date must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      $self->error("Error:  Report.cgi - GetAccountBalance. $errors{4034} The Sub Account is required at line " . __LINE__);
      return undef;
    }
  }

  my $temp_bal = "0.00";
  my $temp_date = "Never";
  my $red_account = 0;
  my $acct_number = $account[0];
  my $acct_name = $account[2];
  my @account_type = (
    $self->{ACCOUNTS}->{$acct_number}->get('type'),
    $self->{ACCOUNTS}->{$acct_number}->get('debitIncreases'),
  );
  #$self->GetAccountType(acct_number => $acct_number);
  my $debit_increases = $account[1];
  my $tmp_sql_statement = $sql_statements{1063};
  my $other_date_info = "";
  my $sub_acct_num = -2;  # the default sub_account that indicates the total balance for this account and all it's sub-accounts.
  my $sub_entity = '';
  my ($sth, @info);

  if (scalar @sub_account > 0)
  {
    $sub_acct_num = $sub_account[0];
  }
  if (scalar @sub_entity > 0)
  {
    $sub_entity = $sub_entity[0];
  }

  if ($acct_number =~ /^(4|5)/ || $account_type[0] == 17) # this is a Revenue or Expense or Dividends Paid account
  {
    $red_account = 1;
    # gather the last close out date from the acct_bal_tb or the first date of the first fiscal year if never been closed out.
    $sth = $dbh->read(sql => $sql_statements{1071}, plug => [$ledgerCode, $end_date]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    @info = $sth->fetchrow_array;
    my $operator = ">";
    my $tmp_string = "";

    if (length $info[0] > 0)
    {
      $temp_date = $info[0];
      $tmp_string = "db.trans_date $operator '$temp_date' AND ";
    }

    $tmp_sql_statement =~ s/$substitutionRE/$tmp_string/;
    $other_date_info = " start_date = '$temp_date',";
  }
  else
  {
    $sth = $dbh->read(sql => $sql_statements{1062}, plug => [$ledgerCode, $acct_number, $sub_acct_num, $end_date]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    @info = $sth->fetchrow_array;
    if ($sth->rows > 0)
    {
      $temp_bal = $info[0];
      $temp_date = $info[1];

      if ($temp_date ne $end_date)  # there's more data to gather
      {
        my $tmp_string = "db.trans_date > '$temp_date' AND ";
        $tmp_sql_statement =~ s/$substitutionRE/$tmp_string/;
        $other_date_info = " start_date = '$temp_date',";
      }
    }
    else
    {
      $tmp_sql_statement =~ s/$substitutionRE//;
    }

  }

  # replace the ##1?## with the correct sub-account/sub-entity information
  if (scalar @sub_account > 0 && scalar @sub_entity > 0)
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE/AND db.sub_account_num = $sub_acct_num AND db.sub_e_code = '$sub_entity'/;
  }
  elsif (scalar @sub_account > 0)
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE/AND db.sub_account_num = $sub_acct_num/;
  }
  elsif (scalar @sub_entity > 0)
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE/AND db.sub_e_code = '$sub_entity'/;
  }
  else
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE//;
  }
  if ($tmp_sql_statement !~ /$substitutionRE/ && !($red_account && ($temp_date eq $end_date)))
  {
    $sth = $dbh->read(sql => $tmp_sql_statement, plug => [$ledgerCode, $end_date, $acct_number]);
  }

  while (@info = $sth->fetchrow_array)
  {
    my $amount = $info[0];
    my $dorc = $info[1];

    $temp_bal = $self->AddMoneyByDebitCredit(balance => $temp_bal, amount => $amount, debit_increases => $debit_increases, dorc => $dorc);
  }

  $balance[0] = $temp_bal;
  $balance[1] = $sth->rows;

  return @balance;
}

# AddMoneyByDebitCredit
# Takes  : Current Balance (balance), Debit Increases Flag (debit_increases) (1/0), DORC Flag (dorc) (1/0), Amount (amount)
# Returns: New Balance
sub AddMoneyByDebitCredit
{
  my $self = shift;
  my %args = ( @_ );
  my $balance = $args{'balance'};
  my $debit_increases = $args{'debit_increases'};
  my $dorc = $args{'dorc'};
  my $amount = $args{'amount'};

  if (length $balance == 0)
  {
    $self->error("Error: Report.cgi - AddMoneyByDebitCredit. $errors{4034} The Beginning Balance is required at line " . __LINE__);
    return undef;
  }

  if (length $debit_increases == 0)
  {
    $self->error("Error: Report.cgi - AddMoneyByDebitCredit. $errors{4034} The Debit Increases Flag is required at line " . __LINE__);
    return undef;
  }

  if (length $dorc == 0)
  {
    $self->error("Error:  Report.cgi - AddMoneyByDebitCredit. $errors{4034} The Dorc Flag must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (length $amount == 0)
  {
    $self->error("Error:  Report.cgi - AddMoneyByDebitCredit. $errors{4034} The Amount to Increment/Decrement must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if ($debit_increases && $dorc)
  {
    $balance += $amount;
  }
  elsif ($debit_increases && !$dorc)
  {
    $balance -= $amount;
  }
  elsif (!$debit_increases && $dorc)
  {
    $balance -= $amount;
  }
  elsif (!$debit_increases && !$dorc)
  {
    $balance += $amount;
  }

  $balance = $balance;

  return $balance;
}

# GetActivityData
# Takes  : dbh, ledgerCode, sql_statement (The Where clause in our SQL statement)
# Returns: Array of Entries from the debitcredit_tb that match the given data.
#          Array [Entries]
#          [Entries] = parent_num, sub_e_code, sub_e_name, account_num, account_name, sub_account_num, sub_account_name, amount, dorc, cleared, counter, trans_date, payee_payer, memo, journal, journal_name, remember
sub GetActivityData
{
  my $self = shift;
  my %args = ( @_ );
  my $dbh = $self->dbHandle;
  my $ledgerCode = $args{'ledgerCode'};
  my @activity = ();
  my $sql_statement = $args{'sql_statement'};

  if (length $ledgerCode == 0)
  {
    $self->error("Error:  Report.cgi - GetActivityData. $errors{4034} The ledgerCode must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (exists $args{'sql_statement'} && length $sql_statement == 0)
  {
    $self->error("Error:  Report.cgi - GetActivityData. $errors{4034} The SQL Statement is required at line " . __LINE__);
    return undef;
  }

  my $tmp_sql_statement = $sql_statements{1061};
  $tmp_sql_statement =~ s/$substitutionRE/$sql_statement/;

  my $sth = $dbh->read(sql => $tmp_sql_statement, plug => [$ledgerCode]);
  if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
  while (my $info = $sth->fetchrow_hashref)
  {
    my %entry = (
      id => $info->{id},
      sub_e_code => $info->{sub_e_code},
      debit => $info->{debit},
      credit => $info->{credit},
      amount => $info->{amount},
      cleared => $info->{cleared},
      date => $info->{date},
      payee_payer => $info->{payee_payer},
      comment => $info->{comment},
      journal => $info->{journal},
      sub_account_num => $info->{sub_account_num},
      sub_account_name => $info->{sub_account_name},
      sub_account_code => $info->{sub_account_code},
      account_num => $info->{account_num},
      account_name => $info->{account_name},
      account_code => $info->{account_code},
      account_type => $info->{account_type},
    );
    $activity[scalar @activity] = [ %entry ];
  }

  return @activity;
}

# AccountActivity
# Takes  : Array of Accounts with Debit Increases Flag and Account Name (accounts), dbh, ledgerCode
# Takes  : begin_date, end_date, search_type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Takes  : sub_account (optional, the Sub Account number and name), sub_entity (optional, the Code and Name)
# Returns: Array of Activity and the # of entries for each account
sub AccountActivity
{
  my $self = shift;
  my %args = ( search_type => '1', @_ );
  my $dbh = $self->dbHandle;
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledgerCode = $self->{'ledgerCode'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my @sub_account = ();
  my @sub_entity = ();
  my @activity = ();

  if (scalar @accounts == 0)
  {
    $self->error("Error: Report.cgi - AccountActivity. $errors{4034} The Array of Accounts is required at line " . __LINE__);
    return undef;
  }

  if ($search_type < 0 || $search_type > 2)
  {
    $self->error("Error:  Report.cgi - AccountActivity. $errors{4034} The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (length $ledgerCode == 0)
  {
    $self->error("Error:  Report.cgi - AccountActivity. $errors{4034} The ledgerCode must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (length $begin_date == 0)
  {
    $self->error("Error:  Report.cgi - AccountActivity. $errors{4034} The begin_date must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (length $end_date == 0)
  {
    $self->error("Error:  Report.cgi - AccountActivity. $errors{4034} The end_date must be specified!<br>\nAt Line " . __LINE__);
    return undef;
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      $self->error("Error:  Report.cgi - AccountActivity. $errors{4034} The Sub Account info can not be empty!<br>\nAt Line " . __LINE__);
      return undef;
    }
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      $self->error("Error:  Report.cgi - AccountActivity. $errors{4034} The Sub Entity info can not be empty!<br>\nAt Line " . __LINE__);
      return undef;
    }
  }

  my ($i, $acct_num, $debit_increases, $balance, $acct_name);
  my $date_search_string = "";
  if ($search_type == 1)
  {
    $date_search_string = "(je.trans_date BETWEEN '$begin_date' AND '$end_date')";
  }
  elsif ($search_type == 2)
  {
    $date_search_string = "(je.trans_date > '$begin_date' AND je.trans_date <= '$end_date')";
  }
  elsif ($search_type == 0)
  {
    $date_search_string = "(je.trans_date > '$begin_date' AND je.trans_date < '$end_date')";
  }

  my $tmp_sql_statement = "";
  $tmp_sql_statement = $sql_statements{1050};
  if (scalar @sub_account > 0)
  {
    my $temp_string = "db.sub_account_num = $sub_account[0] AND ";
    $tmp_sql_statement =~ s/$firstSubstitutionRE/$temp_string/;
  }
  else
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE//;
  }
  if (scalar @sub_entity > 0)
  {
    my $temp_string = "db.sub_e_code = '$sub_entity[0]' AND ";
    $tmp_sql_statement =~ s/$secondSubstitutionRE/$temp_string/;
  }
  else
  {
    $tmp_sql_statement =~ s/$secondSubstitutionRE//;
  }
  $tmp_sql_statement =~ s/$substitutionRE/$date_search_string/;
  for ($i=0; $i < scalar @accounts; $i++)
  {
    $acct_num = $accounts[$i][0];
    #because this is cash flow based.  reverse the debit_increases flag for cash accounts
    if ($self->{ACCOUNTS}->{$acct_num}->get('type') == 0)
    {
      $debit_increases = 1;
    }
    else
    {
      $debit_increases = 0;
    }
    $acct_name = $accounts[$i][2];
    $balance = "0.00";

    # the power of DBI!
    my $error_string = "";
    if (scalar @sub_account > 0)
    {
      $error_string .= ",&nbsp;Sub Account = '$sub_account[0]'";
    }
    if (scalar @sub_entity > 0)
    {
      $error_string .= ",&nbsp;Sub Entity = '$sub_entity[0]'";
    }
    my $sth = $dbh->read(sql => $tmp_sql_statement, plug => [$ledgerCode, $acct_num]);
    if ($dbh->error) { $self->error($dbh->errorMessage); return undef; }
    my @info = @{$sth->fetchall_arrayref};
    for (my $j=0; $j < scalar @info; $j++)
    {
      my $amount = $info[$j][0];
      my $dorc = $info[$j][1];
      $balance = $self->AddMoneyByDebitCredit(balance => $balance, debit_increases => $debit_increases, dorc => $dorc, amount => $amount);
    }
    $balance = $balance;
    my @entry = ();
    $entry[0] = $balance;
    $entry[1] = $sth->rows;

    $activity[$i] = [ @entry ];
  }

  return @activity;
}

1;
__END__;